@@ -978,42 +978,6 @@ void __init setup_arch(char **cmdline_p)
 #endif
 #endif
 
-	/* NetApp: force console and earlycon on SGA controller */
-	if (strstr(boot_command_line, "netapp_sga_set_console_by_cpu=yes")) {
-		/* Edit any existing console and earlycon settings out of the line */
-		char *p, *q;
-		while (((p = strstr(boot_command_line, "console="))) ||
-			   ((p = strstr(boot_command_line, "earlycon=")))) {
-			/* If there's something after this option, move the string up */
-			if ((q = strchr(p, ' '))) {
-				memmove(p, &q[1], strlen(&q[1]) + 1);
-			}
-			/* Otherwise, terminate the string, trying not to leave a trailing space */
-			else {
-				if (p != boot_command_line)
-					p--;
-				p[0] = '\0';
-			}
-		}
-		/* Append console setting based on CPU extended model number EAX bits 19:16 */
-		/* Sandy Bridge 2, Ivy Bridge and Haswell 3, Broadwell 4, Broadwell-DE 5 */
-		if ((boot_cpu_data.x86_model >> 4) >= 4) {
-			/* Assume Sandhawk or Quanta */
-			if (boot_cpu_data.x86_model == 0x55 || boot_cpu_data.x86_model == 0x6A) {
-				/* Quanta; add VGA TTY first */
-				strlcat(boot_command_line, " console=tty0", COMMAND_LINE_SIZE);
-			}
-			strlcat(boot_command_line, " console=ttyS0,115200n8", COMMAND_LINE_SIZE);
-			if (boot_cpu_data.x86_model == 0x6A) {
-			    /* add second console for solo to enable usb-serial port in the back */
-			    strlcat(boot_command_line, " console=ttyS1,115200n8", COMMAND_LINE_SIZE);
-			}
-		} else {
-			/* Assume Titan */
-			strlcat(boot_command_line, " console=uart,io,0x9000,6400n81", COMMAND_LINE_SIZE);
-		}
-	}
-
 	strscpy(command_line, boot_command_line, COMMAND_LINE_SIZE);
 	*cmdline_p = command_line;
 
