@@ -81,7 +81,6 @@ import io.dropwizard.assets.AssetsBundle;
 import io.dropwizard.client.HttpClientBuilder;
 import io.dropwizard.configuration.EnvironmentVariableSubstitutor;
 import io.dropwizard.configuration.SubstitutingSourceProvider;
-import io.dropwizard.db.DataSourceFactory;
 import io.dropwizard.jdbi.DBIFactory;
 import io.dropwizard.jetty.BiDiGzipHandler;
 import io.dropwizard.setup.Bootstrap;
@@ -89,7 +88,6 @@ import io.dropwizard.setup.Environment;
 import io.prometheus.client.CollectorRegistry;
 import io.prometheus.client.dropwizard.DropwizardExports;
 import io.prometheus.client.exporter.MetricsServlet;
-import sun.misc.Signal;
 
 public final class ReaperApplication extends Application<ReaperApplicationConfiguration> {
 
@@ -177,6 +175,12 @@ public final class ReaperApplication extends Application<ReaperApplicationConfig
                                  }
                                  sb.append("End of dump of all threads for inspection:\n");
                                  LOG.info(sb.toString());
+                                 if (RepairManager.getInstance() != null) {
+                                        LOG.info("Dump of repair runners: " + RepairManager.getInstance().repairRunners.size() +
+                                                        ": " + RepairManager.getInstance().repairRunners);
+                                 } else {
+                                        LOG.info("RepairManager.getInstance() was null");
+                                 }
                          } catch (Exception e) {
                                  // no problem
                          }
@@ -530,24 +534,6 @@ public final class ReaperApplication extends Application<ReaperApplicationConfig
       storage = new MemoryStorage();
     } else if ("cassandra".equalsIgnoreCase(config.getStorageType())) {
       storage = new CassandraStorage(context.reaperInstanceId, config, environment);
-    } else if ("postgres".equalsIgnoreCase(config.getStorageType())
-        || "h2".equalsIgnoreCase(config.getStorageType())
-        || "database".equalsIgnoreCase(config.getStorageType())) {
-      // create DBI instance
-      final DBIFactory factory = new DBIFactory();
-      if (StringUtils.isEmpty(config.getDataSourceFactory().getDriverClass())
-          && "postgres".equalsIgnoreCase(config.getStorageType())) {
-        config.getDataSourceFactory().setDriverClass("org.postgresql.Driver");
-      } else if (StringUtils.isEmpty(config.getDataSourceFactory().getDriverClass())
-          && "h2".equalsIgnoreCase(config.getStorageType())) {
-        config.getDataSourceFactory().setDriverClass("org.h2.Driver");
-      }
-      // instantiate store
-      storage = new PostgresStorage(
-          context.reaperInstanceId,
-          factory.build(environment, config.getDataSourceFactory(), "postgresql")
-      );
-      initDatabase(config);
     } else {
       LOG.error("invalid storageType: {}", config.getStorageType());
       throw new ReaperException("invalid storage type: " + config.getStorageType());
@@ -574,37 +560,13 @@ public final class ReaperApplication extends Application<ReaperApplicationConfig
   private void addSignalHandlers() {
     if (!System.getProperty("os.name").toLowerCase().contains("win")) {
       LOG.debug("adding signal handler for SIGHUP");
-      Signal.handle(new Signal("HUP"), signal -> {
+      /*Signal.handle(new Signal("HUP"), signal -> {
         LOG.info("received SIGHUP signal: {}", signal);
         reloadConfiguration();
-      });
+      });*/
     }
   }
 
-  private void initDatabase(ReaperApplicationConfiguration config) throws ReaperException {
-    Flyway flyway = new Flyway();
-    DataSourceFactory dsfactory = config.getDataSourceFactory();
-    flyway.setDataSource(
-        dsfactory.getUrl(),
-        dsfactory.getUser(),
-        dsfactory.getPassword());
-
-    if ("database".equals(config.getStorageType())) {
-      LOG.warn("!!!!!!!!!!    USAGE 'database' AS STORAGE TYPE IS NOW DEPRECATED   !!!!!!!!!!!!!!");
-      LOG.warn("!!!!!!!!!!    PLEASE USE EITHER 'postgres' OR 'h2' FROM NOW ON     !!!!!!!!!!!!!!");
-      if (config.getDataSourceFactory().getUrl().contains("h2")) {
-        flyway.setLocations("/db/h2");
-      } else {
-        flyway.setLocations("/db/postgres");
-      }
-    } else {
-      flyway.setLocations("/db/".concat(config.getStorageType().toLowerCase()));
-    }
-    flyway.setBaselineOnMigrate(true);
-    flyway.repair();
-    flyway.migrate();
-  }
-
   private void initializeJmxSeedsForAllClusters() {
     LOG.info("Initializing JMX seed list for all clusters...");
     try (JmxConnectionsInitializer jmxConnectionsIntializer = JmxConnectionsInitializer.create(context);
@@ -624,3 +586,4 @@ public final class ReaperApplication extends Application<ReaperApplicationConfig
     }
   }
 }
+
