@@ -17,7 +17,9 @@
 
 package io.cassandrareaper.jmx;
 
+import java.io.BufferedReader;
 import java.io.IOException;
+import java.io.InputStreamReader;
 import java.lang.reflect.UndeclaredThrowableException;
 import java.math.BigInteger;
 import java.net.InetSocketAddress;
@@ -35,6 +37,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.Optional;
 import java.util.Set;
+import java.util.UUID;
 import java.util.concurrent.ConcurrentMap;
 import java.util.concurrent.ExecutionException;
 import java.util.concurrent.ExecutorService;
@@ -91,9 +94,10 @@ import io.cassandrareaper.ReaperException;
 import io.cassandrareaper.core.Cluster;
 import io.cassandrareaper.core.Table;
 import io.cassandrareaper.service.RingRange;
+import io.cassandrareaper.service.RepairManager.MetricsManager;
 
 
-final class JmxProxyImpl implements JmxProxy {
+public final class JmxProxyImpl implements JmxProxy {
 
   private static final Logger LOG = LoggerFactory.getLogger(JmxProxy.class);
 
@@ -122,6 +126,8 @@ final class JmxProxyImpl implements JmxProxy {
   private final LastEventIdBroadcasterMBean lastEventIdProxy;
 
   private static Map<String, Integer> repairThreadPoolZombieDetector = new Hashtable();
+  private static JmxProxyImpl _instance = null;
+  private static boolean _cancellingARepair = false;
 
   private JmxProxyImpl(
       String host,
@@ -151,6 +157,11 @@ final class JmxProxyImpl implements JmxProxy {
     this.metricRegistry = metricRegistry;
     this.smProxy = smProxy;
     registerConnectionsGauge();
+    _instance = this;
+  }
+
+  public static JmxProxyImpl getInstance() {
+         return _instance;
   }
 
   /**
@@ -263,7 +274,7 @@ final class JmxProxyImpl implements JmxProxy {
       if (smProxy.isPresent()) {
         mbeanServerConn.addNotificationListener(ObjectNames.STREAM_MANAGER, proxy, null, null);
       }
-      LOG.debug("JMX connection to {} properly connected: {}", host, jmxUrl.toString());
+      LOG.info("JMX connection to {} properly connected: {}", host, jmxUrl.toString());
 
       return proxy;
     } catch (IOException | ExecutionException | TimeoutException | InstanceNotFoundException e) {
@@ -409,6 +420,9 @@ final class JmxProxyImpl implements JmxProxy {
        boolean post22 = isRepairRunningPost22();
        boolean isValidationCompactionRunning = isValidationCompactionRunning();
     boolean retVal = pre22 || post22 || isValidationCompactionRunning;
+    if (retVal) {
+       MetricsManager.getInstance().addStat("repair_running", 1);
+    }
     LOG.info("isRepairRunning see " + pre22 + " " + post22 + " " + isValidationCompactionRunning);
     return retVal;
   }
@@ -526,7 +540,7 @@ final class JmxProxyImpl implements JmxProxy {
                  ObjectName nameToUseCurrentlyActive = new ObjectName(gonnaUse + ",name=ActiveTasks");
                  ObjectName nameToUseCurrentlyPending = new ObjectName(gonnaUse + ",name=PendingTasks");
                  int currentlyActive = (Integer) mbeanServer.getAttribute(nameToUseCurrentlyActive, VALUE_ATTRIBUTE);
-                 long currentlyPending = (Long) mbeanServer.getAttribute(nameToUseCurrentlyPending, VALUE_ATTRIBUTE);      
+                 long currentlyPending = (Long) Long.parseLong("" + mbeanServer.getAttribute(nameToUseCurrentlyPending, VALUE_ATTRIBUTE));
                  int retVal = currentlyActive + (int) currentlyPending;
                  return retVal;
          } catch (Exception e) {
@@ -535,20 +549,104 @@ final class JmxProxyImpl implements JmxProxy {
          }
   }
 
+  private static String executeCommandSafe(String command) {
+         String retVal = "";
+        try {
+               Process sequenceProcess = Runtime.getRuntime().exec(command);
+               BufferedReader sequenceBr = new BufferedReader(new InputStreamReader(sequenceProcess.getInputStream()));
+               boolean ready = false;
+               int readCount = 0;
+               boolean go = false;
+               StringBuffer currentResult = new StringBuffer();
+               String line = null;
+               while ((line = sequenceBr.readLine()) != null){
+                       currentResult.append(line);
+                       currentResult.append("\n");
+               }
+               retVal = currentResult.toString();
+        } catch (Exception e) {
+                retVal = e.toString();
+        }
+        LOG.info("Executing " + command + " yielded " + retVal);
+        return retVal;
+  }
+
   @Override
   public void cancelAllRepairs() {
     Preconditions.checkNotNull(ssProxy, "Looks like the proxy is not connected");
+    boolean sawOneExplicitly = false;
     for (int i=0; i<10; i++) {
            try {
+                  if (_cancellingARepair) {
+                          LOG.warn("Requested terminate all when already within termination");
+                          return;
+                  }
+                  _cancellingARepair = true;
                Thread.sleep(10000*i);
                LOG.warn("Terminating all repairs try number " + i);
-               ssProxy.forceTerminateAllRepairSessions();
-               LOG.info("Terminating all repairs succeeded on try number " + i);
+               // We used to just call forceTerminateAllRepairSessions(), but this tended to leave
+               // incremental repair tables with a pendingRepair status.  This was theoretically cleaned up
+               // after a day, but lets not wait.
+               // First we will try nodetool repair_admin cleanup.  If we see something there - we clean it up.
+               String output = executeCommandSafe("nodetool repair_admin");
+               if (output.length() > 20) {
+                  LOG.warn("Repair_admin shows sessions in flight");
+                  sawOneExplicitly = true;
+                  MetricsManager.getInstance().addStat("storagegrid_private_repair_terminate_all_triggered{type=\"explicit\"}", 1, false);
+                  for (String line : output.split(System.lineSeparator())) {
+                          String trimmedLine = line.trim();
+                          String uuid = trimmedLine.substring(0, trimmedLine.indexOf(" "));
+                          try {
+                                         if (uuid.contains("id")) {
+                                                 continue;
+                                         }
+                                  UUID.fromString(uuid);
+                                  executeCommandSafe("nodetool repair_admin cancel --force -s " + uuid);
+                          } catch (Exception e) {
+                                  LOG.info(uuid + " was not a uuid - saw " + e);
+                          }
+                  }
+                  try {
+                          ssProxy.forceTerminateAllRepairSessions();
+                  } catch (Exception e) {
+                          LOG.warn("forceTerminateAllRepairSessions failed with " + e);
+                          e.printStackTrace();
+                  }
+                  executeCommandSafe("nodetool stop VALIDATION");
+                  LOG.info("Terminating all repairs succeeded on try number " + i);
+               } else {
+                  LOG.warn("Repair_admin does not show sessions in flight");
+                  MetricsManager.getInstance().addStat("storagegrid_private_repair_terminate_all_triggered{type=\"implicit\"}", 1, false);
+                      // If not, we will go with the old method
+                             try {
+                                     ssProxy.forceTerminateAllRepairSessions();
+                             } catch (Exception e) {
+                                     LOG.warn("forceTerminateAllRepairSessions failed with " + e);
+                                     e.printStackTrace();
+                             }
+                      executeCommandSafe("nodetool repair_admin cleanup --force");
+                      executeCommandSafe("nodetool stop VALIDATION");
+                      LOG.info("Terminating all repairs succeeded on try number " + i);
+                      LOG.warn("Waiting for everyone to force their cleanup");
+                      try {
+                          Thread.sleep(Long.parseLong(System.getProperty("repair_admin.cleanup.force.timeinmillis", "300000")));
+                      } catch (Throwable t) {
+
+                      }
+                      LOG.warn("Proceeding");
+               }
                return;
            } catch (Exception e) {
              // This can happen if the node is down (UndeclaredThrowableException),
              // in which case repairs will be cancelled anyway...
              LOG.warn("Failed to terminate all repair sessions on try number " + i, e);
+             if (!sawOneExplicitly) {
+                // if we didn't see an explicit repair, and we failed, lets just bag
+                // it and go home
+                break;
+             }
+           } finally {
+                  _cancellingARepair = false;
            }
     }
   }
@@ -649,6 +747,7 @@ final class JmxProxyImpl implements JmxProxy {
             associatedTokens,
             repairThreadCount);
       }
+      MetricsManager.getInstance().addStat("trigger_repair", 1);
       repairStatusExecutors.putIfAbsent(repairNo, Executors.newSingleThreadExecutor());
       repairStatusHandlers.putIfAbsent(repairNo, repairStatusHandler);
       return repairNo;
@@ -691,6 +790,7 @@ final class JmxProxyImpl implements JmxProxy {
     LOG.info("Triggering repair for ranges {}", options.get(RepairOption.RANGES_KEY));
 
     options.put(RepairOption.DATACENTERS_KEY, StringUtils.join(datacenters, ","));
+    MetricsManager.getInstance().addStat("trigger_repair", 1);
     // options.put(RepairOption.HOSTS_KEY, StringUtils.join(specificHosts, ","));
     return ssProxy.repairAsync(keyspace, options);
   }
@@ -899,7 +999,13 @@ final class JmxProxyImpl implements JmxProxy {
       LOG.debug("failed on removing notification listener", e);
     }
     try {
+      LOG.info("Attempting to close JMX Connection");
       jmxConnector.close();
+      try {
+                 throw new RuntimeException("Closed JMX Connection");
+         } catch (Exception e) {
+                 LOG.error("Closed JMX Connection", e);
+         }
     } catch (IOException e) {
       LOG.warn("failed closing a JMX connection", e);
     }
@@ -1080,3 +1186,4 @@ final class JmxProxyImpl implements JmxProxy {
     }
   }
 }
+
