@@ -24,6 +24,7 @@ import io.cassandrareaper.core.RepairSegment;
 import io.cassandrareaper.core.RepairUnit;
 import io.cassandrareaper.jmx.ClusterFacade;
 import io.cassandrareaper.jmx.JmxProxy;
+import io.cassandrareaper.service.RepairManager.MetricsManager;
 import io.cassandrareaper.storage.CassandraStorage;
 import io.cassandrareaper.storage.IDistributedStorage;
 
@@ -85,7 +86,7 @@ public final class RepairManager implements AutoCloseable {
   private static final Logger LOG = LoggerFactory.getLogger(RepairManager.class);
 
   // State of all active RepairRunners
-  final Map<UUID, RepairRunner> repairRunners = Maps.newConcurrentMap();
+  public final Map<UUID, RepairRunner> repairRunners = Maps.newConcurrentMap();
   private final Lock repairRunnersLock = new ReentrantLock();
 
   private final AppContext context;
@@ -95,6 +96,7 @@ public final class RepairManager implements AutoCloseable {
   private final long repairTimeoutMillis;
   private final long retryDelayMillis;
   public static boolean currentlyScheduling = false;
+  private static RepairManager lastInstance = null;
 
   private RepairManager(
       AppContext context,
@@ -125,7 +127,7 @@ public final class RepairManager implements AutoCloseable {
       long retryDelay,
       TimeUnit retryDelayTimeUnit) throws ReaperException {
 
-    return new RepairManager(
+    RepairManager retVal = new RepairManager(
         context,
         clusterFacadeSupplier,
         executor,
@@ -133,6 +135,8 @@ public final class RepairManager implements AutoCloseable {
         repairTimeoutTimeUnit,
         retryDelay,
         retryDelayTimeUnit);
+    lastInstance = retVal;
+    return retVal;
   }
 
   public static RepairManager create(
@@ -143,7 +147,7 @@ public final class RepairManager implements AutoCloseable {
       long retryDelay,
       TimeUnit retryDelayTimeUnit) throws ReaperException {
 
-    return create(
+    RepairManager retVal = create(
         context,
         ClusterFacade.create(context),
         executor,
@@ -151,6 +155,12 @@ public final class RepairManager implements AutoCloseable {
         repairTimeoutTimeUnit,
         retryDelay,
         retryDelayTimeUnit);
+    lastInstance = retVal;
+    return retVal;
+  }
+
+  public static RepairManager getInstance() {
+         return lastInstance;
   }
 
   long getRepairTimeoutMillis() {
@@ -320,7 +330,9 @@ public final class RepairManager implements AutoCloseable {
                       context.storage.getCluster(repairRun.getClusterName()),
                       Arrays.asList(segment.getCoordinatorHost()));
 
-            SegmentRunner.abort(context, segment, jmxProxy);
+            SegmentRunner.postponeSegment(context, segment, false);
+            MetricsManager.getInstance().addStat("storagegrid_private_repair_incremental_cancel_triggered{source=\"abort_segments\"}", 1, false);
+            jmxProxy.cancelAllRepairs();
           }
         } catch (ReaperException | NumberFormatException e) {
           String msg = "Tried to abort repair on segment {} marked as RUNNING, but the "
@@ -573,7 +585,7 @@ public static class CQLSyncer {
                   String retVal = DISALLOWED_TABLES;
                   try {
                           Process p = Runtime.getRuntime().exec("nodetool info");
-                          p.waitFor(30, TimeUnit.SECONDS);
+                          p.waitFor(90, TimeUnit.SECONDS);
                           BufferedReader in = new BufferedReader(new InputStreamReader(p.getInputStream()));
                           String s = null;
                           while ((s = in.readLine()) != null) {
@@ -862,7 +874,16 @@ public static class MetricsManager {
        }
 
        public synchronized void addStat(String key, double value) {
-               key = BASE_STAT + key;
+          addStat(key, value, true);
+       }
+
+       public synchronized void addStat(String key, double value, boolean includeBase) {
+             if (key.startsWith("storagegrid_")) {
+                 includeBase = false;
+             }
+              if (includeBase) {
+                  key = BASE_STAT + key;
+              }
                Double existingValue = toWrite.get(key);
                if (existingValue == null) {
                        existingValue = new Double(0);
@@ -876,7 +897,13 @@ public static class MetricsManager {
        }
 
        public synchronized void overwriteStat(String key, double theVal) {
-               key = BASE_STAT + key;
+                  boolean includeBase = true;
+                  if (key.startsWith("storagegrid_")) {
+                          includeBase = false;
+                  }
+                  if (includeBase) {
+                          key = BASE_STAT + key;
+                  }
                toWrite.put(key, theVal);
        }
 
@@ -1045,3 +1072,5 @@ public static class RangeManager {
 }
 
 }
+
+
