@@ -256,7 +256,7 @@ final class RepairRunner implements Runnable {
       }
 
       RepairRun.RunState state = repairRun.get().getRunState();
-      LOG.debug("run() called for repair run #{} with run state {}", repairRunId, state);
+      LOG.info("run() called for repair run #{} with run state {}", repairRunId, state);
       switch (state) {
         case NOT_STARTED:
           start();
@@ -276,7 +276,12 @@ final class RepairRunner implements Runnable {
     } catch (RuntimeException | ReaperException | InterruptedException e) {
       LOG.error("RepairRun FAILURE, scheduling retry", e);
       try { Thread.sleep(10000); } catch (Throwable t) { }
-      context.repairManager.scheduleRetry(this);
+      try {
+         context.repairManager.scheduleRetry(this);
+      } catch (Throwable t) {
+         // lets not let a failure to schedule bubble all the way out
+         LOG.error("Failure scheduling retry", t);
+      }
     }
     // Adding this here to catch a deadlock
     LOG.debug("run() exiting for repair run #{}", repairRunId);
@@ -372,88 +377,92 @@ final class RepairRunner implements Runnable {
     // We want to know whether a repair was started,
     // so that a rescheduling of this runner will happen.
     boolean repairStarted = false;
+    try {
 
-    for (int rangeIndex = 0; rangeIndex < currentlyRunningSegments.length(); rangeIndex++) {
+           for (int rangeIndex = 0; rangeIndex < currentlyRunningSegments.length(); rangeIndex++) {
 
-      if (currentlyRunningSegments.get(rangeIndex) != null) {
-        anythingRunningStill = true;
+             if (currentlyRunningSegments.get(rangeIndex) != null) {
+               anythingRunningStill = true;
 
-        // Just checking that no currently running segment runner is stuck.
-        RepairSegment supposedlyRunningSegment
-            = context.storage.getRepairSegment(repairRunId, currentlyRunningSegments.get(rangeIndex)).get();
-        DateTime startTime = supposedlyRunningSegment.getStartTime();
-        if (startTime != null && startTime.isBefore(DateTime.now().minusDays(1))) {
-          LOG.warn(
-              "Looks like segment #{} has been running more than a day. Start time: {}",
-              supposedlyRunningSegment.getId(),
-              supposedlyRunningSegment.getStartTime());
-        } else if (startTime != null && startTime.isBefore(DateTime.now().minusHours(1))) {
-          LOG.info(
-              "Looks like segment #{} has been running more than an hour. Start time: {}",
-              supposedlyRunningSegment.getId(),
-              supposedlyRunningSegment.getStartTime());
-        } else if (startTime != null && startTime.isBefore(DateTime.now().minusMinutes(2))) {
-          LOG.debug(
-              "Looks like segment #{} has been running more than two minutes. Start time: {}",
-              supposedlyRunningSegment.getId(),
-              supposedlyRunningSegment.getStartTime());
-        }
-        // No need to try starting new repair for already active slot.
-        continue;
-      }
+               // Just checking that no currently running segment runner is stuck.
+               RepairSegment supposedlyRunningSegment
+                   = context.storage.getRepairSegment(repairRunId, currentlyRunningSegments.get(rangeIndex)).get();
+               DateTime startTime = supposedlyRunningSegment.getStartTime();
+               if (startTime != null && startTime.isBefore(DateTime.now().minusDays(1))) {
+                 LOG.warn(
+                     "Looks like segment #{} has been running more than a day. Start time: {}",
+                     supposedlyRunningSegment.getId(),
+                     supposedlyRunningSegment.getStartTime());
+               } else if (startTime != null && startTime.isBefore(DateTime.now().minusHours(1))) {
+                 LOG.info(
+                     "Looks like segment #{} has been running more than an hour. Start time: {}",
+                     supposedlyRunningSegment.getId(),
+                     supposedlyRunningSegment.getStartTime());
+               } else if (startTime != null && startTime.isBefore(DateTime.now().minusMinutes(2))) {
+                 LOG.debug(
+                     "Looks like segment #{} has been running more than two minutes. Start time: {}",
+                     supposedlyRunningSegment.getId(),
+                     supposedlyRunningSegment.getStartTime());
+               }
+               // No need to try starting new repair for already active slot.
+               continue;
+             }
 
-      // We have an empty slot, so let's start new segment runner if possible.
-      // When in sidecar mode, filter on ranges that the local node is a replica for only.
-      LOG.info("Running segment for range {}", parallelRanges.get(rangeIndex));
-      Optional<RepairSegment> nextRepairSegment
-          = context.config.isInSidecarMode()
-              ? ((IDistributedStorage) context.storage)
-                  .getNextFreeSegmentForRanges(
-                      repairRunId, Optional.of(parallelRanges.get(rangeIndex)), localEndpointRanges)
-              : context.storage.getNextFreeSegmentInRange(
-                  repairRunId, Optional.of(parallelRanges.get(rangeIndex)));
+             // We have an empty slot, so let's start new segment runner if possible.
+             // When in sidecar mode, filter on ranges that the local node is a replica for only.
+             LOG.info("Running segment for range {}", parallelRanges.get(rangeIndex));
+             Optional<RepairSegment> nextRepairSegment
+                 = context.config.isInSidecarMode()
+                     ? ((IDistributedStorage) context.storage)
+                         .getNextFreeSegmentForRanges(
+                             repairRunId, Optional.of(parallelRanges.get(rangeIndex)), localEndpointRanges)
+                     : context.storage.getNextFreeSegmentInRange(
+                         repairRunId, Optional.of(parallelRanges.get(rangeIndex)));
 
-      if (!nextRepairSegment.isPresent()) {
-        LOG.debug("No repair segment available for range {}", parallelRanges.get(rangeIndex));
+             if (!nextRepairSegment.isPresent()) {
+               LOG.debug("No repair segment available for range {}", parallelRanges.get(rangeIndex));
 
-      } else {
-        LOG.info("Next segment to run : {}", nextRepairSegment.get().getId());
-        UUID segmentId = nextRepairSegment.get().getId();
-        boolean wasSet = currentlyRunningSegments.compareAndSet(rangeIndex, null, segmentId);
-        if (!wasSet) {
-          LOG.debug("Didn't set segment id `{}` to slot {} because it was busy", segmentId, rangeIndex);
-        } else {
-          LOG.debug("Did set segment id `{}` to slot {}", segmentId, rangeIndex);
-          scheduleRetry = repairSegment(
-                  rangeIndex,
-                  nextRepairSegment.get().getId(),
-                  nextRepairSegment.get().getTokenRange());
-          if (!scheduleRetry) {
-            break;
-          }
-          segmentsTotal = context.storage.getSegmentAmountForRepairRun(repairRunId);
-          repairStarted = true;
-        }
-      }
-    }
+             } else {
+               LOG.debug("Next segment to run : {}", nextRepairSegment.get().getId());
+               UUID segmentId = nextRepairSegment.get().getId();
+               boolean wasSet = currentlyRunningSegments.compareAndSet(rangeIndex, null, segmentId);
+               if (!wasSet) {
+                 LOG.debug("Didn't set segment id `{}` to slot {} because it was busy", segmentId, rangeIndex);
+               } else {
+                 LOG.debug("Did set segment id `{}` to slot {}", segmentId, rangeIndex);
+                 scheduleRetry = repairSegment(
+                         rangeIndex,
+                         nextRepairSegment.get().getId(),
+                         nextRepairSegment.get().getTokenRange());
+                 LOG.info("RepairSegment returned " + scheduleRetry);
+                 if (!scheduleRetry) {
+                   break;
+                 }
+                 segmentsTotal = context.storage.getSegmentAmountForRepairRun(repairRunId);
+                 repairStarted = true;
+               }
+             }
+           }
 
-    if (!repairStarted && !anythingRunningStill) {
-      segmentsDone = context.storage.getSegmentAmountForRepairRunWithState(repairRunId, RepairSegment.State.DONE);
-      segmentsTotal = context.storage.getSegmentAmountForRepairRun(repairRunId);
+           if (!repairStarted && !anythingRunningStill) {
+             segmentsDone = context.storage.getSegmentAmountForRepairRunWithState(repairRunId, RepairSegment.State.DONE);
+             segmentsTotal = context.storage.getSegmentAmountForRepairRun(repairRunId);
 
-      LOG.info("Repair amount done {}", segmentsDone);
-      repairProgress = segmentsDone / segmentsTotal;
+             LOG.info("Repair amount done {}", segmentsDone);
+             repairProgress = segmentsDone / segmentsTotal;
 
-      if (segmentsDone == segmentsTotal) {
-        endRepairRun();
-        scheduleRetry = false;
-      }
-    } else {
-      segmentsDone = context.storage.getSegmentAmountForRepairRunWithState(repairRunId, RepairSegment.State.DONE);
-    }
-
-    if (scheduleRetry) {
-      context.repairManager.scheduleRetry(this);
+             if (segmentsDone == segmentsTotal) {
+               endRepairRun();
+               scheduleRetry = false;
+             }
+           } else {
+             segmentsDone = context.storage.getSegmentAmountForRepairRunWithState(repairRunId, RepairSegment.State.DONE);
+           }
+    } finally {
+       LOG.info("RepairRunning.startNextSegment - exiting with scheduleRetry=" + scheduleRetry);
+           if (scheduleRetry) {
+             context.repairManager.scheduleRetry(this);
+           }
     }
   }
 
@@ -686,3 +695,4 @@ final class RepairRunner implements Runnable {
     return repairRun;
   }
 }
+
