@@ -17,6 +17,8 @@
 
 package io.cassandrareaper.service;
 
+import java.io.BufferedWriter;
+import java.io.FileWriter;
 import java.lang.management.ManagementFactory;
 import java.lang.management.OperatingSystemMXBean;
 import java.util.ArrayList;
@@ -24,6 +26,7 @@ import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Optional;
@@ -57,6 +60,7 @@ import org.slf4j.LoggerFactory;
 
 import com.codahale.metrics.MetricRegistry;
 import com.codahale.metrics.Timer;
+import com.fasterxml.jackson.databind.ObjectMapper;
 import com.google.common.base.Preconditions;
 import com.google.common.collect.Maps;
 import com.google.common.collect.Sets;
@@ -75,7 +79,9 @@ import io.cassandrareaper.jmx.EndpointSnitchInfoProxy;
 import io.cassandrareaper.jmx.JmxProxy;
 import io.cassandrareaper.jmx.RepairStatusHandler;
 import io.cassandrareaper.jmx.SnapshotProxy;
+import io.cassandrareaper.service.RepairManager.MetricsManager;
 import io.cassandrareaper.storage.IDistributedStorage;
+import io.cassandrareaper.storage.OpType;
 
 
 public final class SegmentRunner implements RepairStatusHandler, Runnable {
@@ -118,6 +124,9 @@ public final class SegmentRunner implements RepairStatusHandler, Runnable {
   private static UUID lastUsedLockId = null;
   private static SegmentDurationHistoryRecorder recorder = new SegmentDurationHistoryRecorder();
   private static HashMap<RepairUnit, Long> failingSegmentsForRepair = new HashMap();
+  private static Long lastReturnedLock = null;
+  public static UUID lastUsedLeaderElectionId = null;
+  private long observedFailCount = 0;
 
 
   private SegmentRunner(
@@ -150,6 +159,7 @@ public final class SegmentRunner implements RepairStatusHandler, Runnable {
     this.repairRunner = repairRunner;
     this.segmentFailed = new AtomicBoolean(false);
     this.leaderElectionId = repairUnit.getIncrementalRepair() ? repairRunner.getRepairRunId() : segmentId;
+    lastUsedLeaderElectionId = this.leaderElectionId;
     this.tablesToRepair = tablesToRepair;
   }
 
@@ -219,7 +229,8 @@ public final class SegmentRunner implements RepairStatusHandler, Runnable {
           long necessaryCount = Long.parseLong(System.getProperty("\"reaper.topology.change.failure.abort.threshold.change.notices", "0"));
           LOG.info("Segment failed.  Considering topology based restart with parameters of " + duration + " " + target + " " +
                           necessaryCount + " " + RepairRunner.repairRunsWithChangedTopologyCounts.get(segment.getRunId()));
-          if (duration > target && RepairRunner.repairRunsWithChangedTopologyCounts.get(segment.getRunId()) > necessaryCount) {
+          if (duration > target && RepairRunner.repairRunsWithChangedTopologyCounts.get(segment.getRunId()) != null &&
+                         RepairRunner.repairRunsWithChangedTopologyCounts.get(segment.getRunId()) > necessaryCount) {
                   LOG.info("Decided upon topology based restart of repair");
                   RepairRun run = context.storage.getRepairRun(segment.getRunId()).get();
                   run = run.with().lastEvent("Aborted based on topology changes").build(run.getId());
@@ -247,17 +258,22 @@ public final class SegmentRunner implements RepairStatusHandler, Runnable {
     }
   }
 
-  static void abort(AppContext context, RepairSegment segment, JmxProxy jmxConnection) {
+  void abort(AppContext context, RepairSegment segment, JmxProxy jmxConnection) {
     postpone(context, segment, context.storage.getRepairUnit(segment.getRepairUnitId()), true);
-    LOG.info("Aborting repair on segment with id {} on coordinator {}", segment.getId(), segment.getCoordinatorHost());
+    if (renewLead()) {
+           LOG.info("Aborting repair on segment with id {} on coordinator {}", segment.getId(), segment.getCoordinatorHost());
 
-    String metric = MetricRegistry.name(
-        SegmentRunner.class,
-        "abort",
-        Optional.ofNullable(segment.getCoordinatorHost()).orElse("null").replace('.', '-'));
+           String metric = MetricRegistry.name(
+               SegmentRunner.class,
+               "abort",
+               Optional.ofNullable(segment.getCoordinatorHost()).orElse("null").replace('.', '-'));
 
-    context.metricRegistry.counter(metric).inc();
-    jmxConnection.cancelAllRepairs();
+           context.metricRegistry.counter(metric).inc();
+           MetricsManager.getInstance().addStat("storagegrid_private_repair_incremental_cancel_triggered{source=\"segment_abort\"}", 1, false);
+           jmxConnection.cancelAllRepairs();
+    } else {
+       MetricsManager.getInstance().addStat("storagegrid_private_repair_incremental_cancel_skipped", 1, false);
+    }
   }
 
   private void abort(RepairSegment segment, JmxProxy jmxConnection) {
@@ -353,6 +369,28 @@ public final class SegmentRunner implements RepairStatusHandler, Runnable {
               }
             }
 
+               if (repairUnit.getIncrementalRepair()) {
+                       try {
+                               String asStoredOps = ((IDistributedStorage) context.storage).listOperations(MetricsService.BROADCAST_CLUSTER,
+                        OpType.OP_UPGRADE_DETECTED,
+                        MetricsService.BROADCAST_HOST);
+                               if (asStoredOps != null && asStoredOps.length() > 3) {
+                       long timeLastUpgradeDetected = Long.parseLong(asStoredOps);
+                       long durationSinceUpgradeDetected = System.currentTimeMillis() - timeLastUpgradeDetected;
+                       long postUpgradeCoolingPeriod = Long.parseLong(System.getProperty("upgrade.no.launch.cooling.period", "" + 30*60000));
+
+                               if (durationSinceUpgradeDetected < postUpgradeCoolingPeriod) {
+                                       LOG.info("Not even considering running a segment based on upgrade detected " + durationSinceUpgradeDetected  + " ago");
+                                       return false;
+                               }
+                               }
+                       } catch (Exception e) {
+                               LOG.error("Confusion on whether anyone is upgrading", e);
+                               // lets not launch if there is confusion - we will try again soon
+                               return false;
+                       }
+               }
+
             LOG.info("Preflight successful for segment {} for repair {}.", segmentId, segment.getRunId());
 
             segment = segment
@@ -371,7 +409,33 @@ public final class SegmentRunner implements RepairStatusHandler, Runnable {
                LOG.info("SegmentRunner-CQLSyncer finished it all - returning");
                all_done = true;
             } else {
+               // previously we would repair right away.  But if there's been trouble, lets clear
+               // the decks right here and make sure everyone is up to the task.
+               if (segment.getFailCount() > 0) {
+                       LOG.info("SegmentRunner - clearing everyone's repair status to make sure we are really ready to repair");
+                    try {
+                        LOG.info("Broadcasting request for everyone to pre-terminate in SegementRunner");
+                        MetricsManager.getInstance().addStat("storagegrid_private_repair_broadcast_terminate{source=\"pre_launch\"}", 1, false);
+                        ((IDistributedStorage) context.storage).storeOperations(MetricsService.BROADCAST_CLUSTER,
+                            OpType.OP_BROADCAST_TERMINATE_ALL,
+                            MetricsService.BROADCAST_HOST,
+                            new ObjectMapper().writeValueAsString(System.currentTimeMillis()));
+                    } catch (Exception e) {
+                        LOG.error("Error storing broadcast flag", e);
+                    }
+                    try {
+                           long endTime = System.currentTimeMillis() + Long.parseLong(System.getProperty("reaper.prelaunch.failure.delay", "400000"));
+                           while(System.currentTimeMillis() < endTime) {
+                               LOG.info("SegmentRunner - waiting to clear everyone's status");
+                               Thread.sleep(30000l);
+                               renewLead();
+                           }
+                    } catch (Throwable t) {
+                       t.printStackTrace();
+                    }
+               }
                LOG.info("SegmentRunner-Repairing " + tablesToRepair.size() + " tables");
+
                    repairNo = coordinator.triggerRepair(
                            segment.getStartToken(),
                            segment.getEndToken(),
@@ -386,9 +450,7 @@ public final class SegmentRunner implements RepairStatusHandler, Runnable {
             }
 
             if (0 != repairNo) {
-              releaseSegmentRunners();
-              segmentsLocked = false;
-              processTriggeredSegment(segment, coordinator, repairNo);
+              processTriggeredSegment(segment, coordinator, repairNo, getDCsByNodeForRepairSegment(coordinator, cluster, segment, keyspace));
             } else {
               LOG.info("Nothing to repair for segment {} in keyspace {}", segmentId, keyspace);
 
@@ -469,17 +531,20 @@ public final class SegmentRunner implements RepairStatusHandler, Runnable {
   private long getTimeoutToUse(long largestPossibleTimeout, RepairSegment segment) {
          // Lets use an exponential backoff algorithm.  First few timeouts are short in case of gremlins.  Then we start getting
          // bigger in case we're dealing with a giant segment
+
+            // Incremental repair uses much larger segments - so we need to go for longer timeouts to make progress.  The good news is that
+            // Cassandra itself is now keeping the notion of incremental progress - so if we abort one that's halfway done - we get to keep
+            // that halfway when we ask for it to be repaired again.
+            int numberOfMinutesForAbsoluteMaxTimeout = 120;
          long[] timeoutsBasedOnSegmentFailure = new long[] {
-                         5*60000,
-                         20*60000,
-                         40*60000,
-                         80*60000,
-                         160*60000,
-                         240*60000
+                         10*60000,
+                         60*60000,
+                         numberOfMinutesForAbsoluteMaxTimeout*60000
          };
-         int numberOfMinutesForAbsoluteMaxTimeout = 480;
+
          // failcount is getting double incremented - its too complex and risky to chase down - so lets just go with half
          int indexToUse = segment.getFailCount();
+         this.observedFailCount = indexToUse;
 
          // We can get large failure counts if a Cassandra node has been down.  In that case, we don't want to be stuck with
          // 4 hour timeouts.
@@ -506,12 +571,17 @@ public final class SegmentRunner implements RepairStatusHandler, Runnable {
   private static class SegmentDurationHistoryRecorder {
          private static Map<UUID, List<Long>> durations = new HashMap();
          private static Map<UUID, Long> lastLaunchTime = new HashMap();
+         private static volatile long inFlightSegmentCount = 0;
 
          public static synchronized void noteSegmentLaunched(UUID repairId) {
                  lastLaunchTime.put(repairId, System.currentTimeMillis());
+                 inFlightSegmentCount++;
+                 MetricsManager.getInstance().overwriteStat("in_flight_segments", inFlightSegmentCount);
          }
 
          public static synchronized void noteSegmentLanded(UUID repairId) {
+                       inFlightSegmentCount--;
+                       MetricsManager.getInstance().overwriteStat("in_flight_segments", inFlightSegmentCount);
                  Long launchTime = lastLaunchTime.get(repairId);
                  if (launchTime != null) {
                          List<Long> knownDurations = durations.get(repairId);
@@ -549,29 +619,87 @@ public final class SegmentRunner implements RepairStatusHandler, Runnable {
 
   }
 
-  private void processTriggeredSegment(final RepairSegment segment, final JmxProxy coordinator, int repairNo) {
+  private void processTriggeredSegment(final RepairSegment segment, final JmxProxy coordinator, int repairNo, Map<String, String> dcByNode) {
 
     repairRunner.updateLastEvent(
         String.format("Triggered repair of segment %s via host %s", segment.getId(), coordinator.getHost()));
 
     long timeoutToUse = getTimeoutToUse(timeoutMillis, segment);
+    long roundedTimeoutInMinutes = timeoutToUse / 60000;
+    long roundedTimeoutInMilliseconds = roundedTimeoutInMinutes * 60000;
     recorder.noteSegmentLaunched(this.repairRunner.getRepairRunId());
     {
       LOG.info("Repair for segment {} with failCount {} started, status wait will timeout in {} millis",
                  segmentId, segment.getFailCount(), timeoutToUse);
     }
 
+    final long startTime = System.currentTimeMillis();
+    boolean isDoneOrFailed = false;
     try {
-      final long startTime = System.currentTimeMillis();
+
       final long maxTime = startTime + timeoutToUse;
       final long waitTime = Math.min(timeoutToUse, 60000);
-
-      while (System.currentTimeMillis() < maxTime) {
-        boolean isDoneOrFailed = condition.await(waitTime, TimeUnit.MILLISECONDS);
+      boolean shouldContinue = true;
+      while (shouldContinue) {
+        isDoneOrFailed = condition.await(waitTime, TimeUnit.MILLISECONDS);
 
         isDoneOrFailed |= RepairSegment.State.DONE == context.storage
             .getRepairSegment(segment.getRunId(), segmentId).get().getState();
 
+        shouldContinue = System.currentTimeMillis() < maxTime;
+        // traditionally we gave up at the end of this timeout period.  But that left us with A LOT
+        // of repairs that were humming along smoothly and just couldn't finish in time.  This is a
+        // bigger deal with incremental repair when the segments cover so much more data.  So now we
+        // want to see if anyone is still making acceptable progress.  And if so, then we will give it
+        // more time.
+        // Get all of the involved nodes.  Get their acceptable progress.  Compare it to now.  If anyone made
+        // acceptable progress in the last 20 minutes, give this segment a life extension.
+        int mostRecentAcceptableProgress = 0;
+
+        try {
+               Collection<String> nodes = new HashSet(coordinator.getLiveNodes());
+               String dc = EndpointSnitchInfoProxy.create(coordinator).getDataCenter();
+               boolean requireAllHostMetrics = DatacenterAvailability.LOCAL != context.config.getDatacenterAvailability();
+               boolean allLocalDcHostsChecked = true;
+               boolean allHostsChecked = true;
+               Set<String> unreachableNodes = Sets.newHashSet();
+
+               List<Pair<String, Future<Optional<NodeMetrics>>>> nodeMetricsTasks = nodes.stream()
+                   .map(node -> getNodeMetrics(node, dc != null ? dc : "", dcByNode.get(node) != null ? dcByNode.get(node) : "", true))
+                   .map(pair -> Pair.of(pair.getLeft(), METRICS_GRABBER_EXECUTOR.submit(pair.getRight())))
+                   .collect(Collectors.toList());
+
+               for (Pair<String, Future<Optional<NodeMetrics>>> pair : nodeMetricsTasks) {
+                   Optional<NodeMetrics> result = pair.getRight().get();
+                   if (result.isPresent() && !result.get().isRequested()) {
+                     NodeMetrics metrics = result.get();
+                     long acceptableProgress = metrics.getActiveAnticompactions();
+                     MetricsManager.getInstance().overwriteStat("storagegrid_private_repair_segment_extension_activity_time{target_instance=\"" + pair.getLeft() + "\"}", acceptableProgress);
+                     LOG.info("See acceptable progress on " + pair.getLeft() + " of " + acceptableProgress);
+                     mostRecentAcceptableProgress = Math.max(metrics.getActiveAnticompactions(), mostRecentAcceptableProgress);
+                   }
+               }
+        } catch (Exception e) {
+               LOG.error("Error checking for reasonable progress based life extension", e);
+        }
+        if (!shouldContinue) {
+               long currentTime = System.currentTimeMillis();
+               long mostRecentAcceptableProgressLong = (long) mostRecentAcceptableProgress*1000;
+               long howLongAgo = currentTime - mostRecentAcceptableProgressLong;
+               long acceptableWait = Long.parseLong(System.getProperty("reaper.acceptable.progress.timer", "9000000"));
+               boolean decision = howLongAgo < acceptableWait;
+               LOG.info("Segment " + segment.getId() + " appears to have timed out.  Considering acceptable progress values of " + currentTime + ", " +
+                               mostRecentAcceptableProgressLong + ", " + howLongAgo + ", and " + acceptableWait + ".  Yielded " + decision);
+               long howLongAgoRounded = howLongAgo / 1000;
+               long decisionMetric = 1;
+               if (!decision) {
+                       decisionMetric = 0;
+               }
+               MetricsManager.getInstance().overwriteStat("storagegrid_private_repair_segment_extension_activity_seconds{timeout=\"" + roundedTimeoutInMilliseconds + "\"}", howLongAgoRounded);
+               MetricsManager.getInstance().overwriteStat("storagegrid_private_repair_segment_extension_decision{timeout=\"" + roundedTimeoutInMilliseconds + "\"}", decisionMetric);
+               shouldContinue = decision;
+        }
+
         if (isDoneOrFailed) {
           break;
         }
@@ -580,6 +708,7 @@ public final class SegmentRunner implements RepairStatusHandler, Runnable {
     } catch (InterruptedException e) {
       LOG.warn("Repair command {} on segment {} interrupted", this.repairNo, segmentId, e);
     } finally {
+
       recorder.noteSegmentLanded(this.repairRunner.getRepairRunId());
       coordinator.removeRepairStatusHandler(repairNo);
       RepairSegment resultingSegment
@@ -590,13 +719,17 @@ public final class SegmentRunner implements RepairStatusHandler, Runnable {
           this.repairNo,
           segmentId,
           resultingSegment.getState());
+      MetricsManager.getInstance().addStat("storagegrid_private_repair_segment_finish{timeout=\"" + roundedTimeoutInMilliseconds + "\", state=\"" + resultingSegment.getState() + "\"}", 1, false);
+      MetricsManager.getInstance().addStat("storagegrid_private_repair_segment_finish_duration_ms{timeout=\"" + roundedTimeoutInMilliseconds + "\", state=\"" + resultingSegment.getState() + "\"}", System.currentTimeMillis() - startTime, false);
+
+      boolean shouldAbort = false;
 
       switch (resultingSegment.getState()) {
         case STARTED:
         case RUNNING:
           LOG.info("Repair command {} on segment {} has been cancelled while running", this.repairNo, segmentId);
           segmentFailed.set(true);
-          abort(resultingSegment, coordinator);
+          shouldAbort = true;
           break;
 
         case DONE:
@@ -617,9 +750,48 @@ public final class SegmentRunner implements RepairStatusHandler, Runnable {
               this.repairNo,
               segmentId);
           segmentFailed.set(true);
-          abort(resultingSegment, coordinator);
+          shouldAbort = true;
       }
       // Repair is still running, we'll renew lead on the segment when using Cassandra as storage backend
+      if (RepairSegment.State.DONE != context.storage
+              .getRepairSegment(segment.getRunId(), segmentId).get().getState() || isDoneOrFailed) {
+                 // We must have failed.
+                 long minimumNextStartTime = System.currentTimeMillis() + Long.parseLong(System.getProperty("minimum.segment.failed.runtime", "600000"));
+                 // Lets throw the flag that says we have incremental repair conflict and everyone needs
+                 // to back out and terminate everything
+                 try {
+                        if (repairUnit.getIncrementalRepair()) {
+                                LOG.info("Broadcasting request for everyone to terminate in SegementRunner");
+                                MetricsManager.getInstance().addStat("storagegrid_private_repair_broadcast_terminate{source=\"coordinator_detection\"}", 1, false);
+                                ((IDistributedStorage) context.storage).storeOperations(MetricsService.BROADCAST_CLUSTER,
+                                    OpType.OP_BROADCAST_TERMINATE_ALL,
+                                    MetricsService.BROADCAST_HOST,
+                                    new ObjectMapper().writeValueAsString(System.currentTimeMillis()));
+                        } else {
+                                LOG.info("Skipping terminate all request - was full repair");
+                        }
+                 } catch (Exception e) {
+                        LOG.error("Error storing broadcast flag", e);
+                 }
+                 while (minimumNextStartTime > System.currentTimeMillis()) {
+                         try {
+                                        MetricsManager.getInstance().addStat("fail_sleep", 1);
+                                 long sleepDuration = 30000;
+                                 LOG.warn("Sleeping " + sleepDuration + " based on quickly failed segment");
+                                 if (renewLead()) {
+                                 } else {
+                                        LOG.warn("Wanted to sleep as leader while everything settled, but have somehow lost leader");
+                                 }
+                                 Thread.sleep(sleepDuration);
+
+                         } catch (Throwable t) {
+                                 t.printStackTrace();
+                         }
+                 }
+         }
+      if (shouldAbort) {
+         abort(resultingSegment, coordinator);
+      }
       renewLead();
     }
   }
@@ -707,6 +879,11 @@ public final class SegmentRunner implements RepairStatusHandler, Runnable {
                          + "of any ongoing repair involving the host. Sending command to abort all repairs "
                          + "on the host.",
                      hostName);
+                 MetricsManager.getInstance().addStat("storagegrid_private_repair_incremental_cancel_triggered{source=\"stuck_repair\"}", 1, false);
+                 // See if we can grab lead to just slow things down while there is carnage and cancellations going on.
+                 // Doesn't really matter if we can't get it.
+                 takeLead();
+                 renewLead();
                  hostProxy.cancelAllRepairs();
                }
         }
@@ -717,11 +894,15 @@ public final class SegmentRunner implements RepairStatusHandler, Runnable {
   }
 
   Pair<String, Callable<Optional<NodeMetrics>>> getNodeMetrics(String node, String localDc, String nodeDc) {
+         return getNodeMetrics(node, localDc, localDc, false);
+  }
+
+  Pair<String, Callable<Optional<NodeMetrics>>> getNodeMetrics(String node, String localDc, String nodeDc, boolean forceStorage) {
 
     return Pair.of(node, () -> {
       LOG.debug("getMetricsForHost {} / {} / {}", node, localDc, nodeDc);
 
-      if (clusterFacade.nodeIsAccessibleThroughJmx(nodeDc, node)) {
+      if (clusterFacade.nodeIsAccessibleThroughJmx(nodeDc, node) && !forceStorage) {
         try {
           JmxProxy nodeProxy = clusterFacade.connect(context.storage.getCluster(clusterName), Arrays.asList(node));
 
@@ -980,7 +1161,7 @@ public final class SegmentRunner implements RepairStatusHandler, Runnable {
           "received " + (status.isPresent() ? status.get() : progress.get()) + " on unstarted segment " + segmentId);
 
       // See status explanations at: https://wiki.apache.org/cassandra/RepairAsyncAPI
-      // Old repair API â€“ up to Cassandra-2.1.x
+      // Old repair API Ã¢â‚¬â€œ up to Cassandra-2.1.x
       if (status.isPresent()) {
         failOutsideSynchronizedBlock = handleJmxNotificationForCassandra21(
             status,
@@ -990,7 +1171,7 @@ public final class SegmentRunner implements RepairStatusHandler, Runnable {
             progress,
             jmxProxy);
       }
-      // New repair API â€“ Cassandra-2.2 onwards
+      // New repair API Ã¢â‚¬â€œ Cassandra-2.2 onwards
       if (progress.isPresent()) {
         failOutsideSynchronizedBlock = handleJmxNotificationForCassandra22(
             progress,
@@ -1004,8 +1185,30 @@ public final class SegmentRunner implements RepairStatusHandler, Runnable {
     if (failOutsideSynchronizedBlock) {
       if (takeLead() || renewLead()) {
         try {
-          postponeCurrentSegment(true);
-          tryClearSnapshots(message);
+               // at this point we know there's been a failure.  We know that we have lead.  We'd like to hold
+               // some period of time to let everyone settle before we try again.  Lets sleep a minimum of
+               // X minutes plus number of failed segments.
+               long sleepTime = Long.parseLong(System.getProperty("reaper.segment.fail.sleep.time", "600000")) +
+                               (this.observedFailCount*60000);
+               MetricsManager.getInstance().addStat("reaper_segment_detention_time", sleepTime);
+               long endTime = System.currentTimeMillis() + sleepTime;
+               LOG.info("Writing repair conflict file to call off all other reapers");
+               try {
+                       BufferedWriter bw = new BufferedWriter(new FileWriter(Heart.REPAIR_CONFLICT_FILE));
+                       bw.write(System.currentTimeMillis() + "");
+                       bw.close();
+                       LOG.info("File written");
+               } catch (Throwable t) {
+                       t.printStackTrace();
+               }
+               while(System.currentTimeMillis() < endTime) {
+                       LOG.info("Sleeping for failed segment detention");
+                       try { Thread.sleep(30000); } catch (Throwable t) { }
+                       renewLead();
+               }
+
+               postponeCurrentSegment(true);
+               tryClearSnapshots(message);
         } finally {
           // if someone else does hold the lease, ie renewLead(..) was true,
           // then their writes to repair_run table and any call to releaseLead(..) will throw an exception
@@ -1024,6 +1227,10 @@ public final class SegmentRunner implements RepairStatusHandler, Runnable {
       boolean failOutsideSynchronizedBlock,
       JmxProxy jmxProxy) {
 
+         LOG.info("Got JMX notification of '{}' for segment with id '{}' and repair number '{}'",
+                         progress.get(), repairRunner.getRepairRunId(), segmentId);
+
+         MetricsManager.getInstance().addStat("progress_update{type=\"" + progress.get() + "\"}", 1);
     switch (progress.get()) {
       case START:
         try {
@@ -1060,33 +1267,39 @@ public final class SegmentRunner implements RepairStatusHandler, Runnable {
                 "Got SUCCESS for segment with id '{}' and repair number '{}', but it had already timed out",
                 segmentId,
                 repairNumber);
-          } else if (renewLead()) {
-            LOG.info(
-                "repair session succeeded for segment with id '{}' and repair number '{}'",
-                segmentId,
-                repairNumber);
+          } else {
+                 if (renewLead()) {
+                   LOG.info(
+                       "repair session succeeded for segment with id '{}' and repair number '{}'",
+                       segmentId,
+                       repairNumber);
 
-            context.storage.updateRepairSegment(
-                currentSegment
-                    .with()
-                    .withState(RepairSegment.State.DONE)
-                    .withEndTime(DateTime.now())
-                    .withId(segmentId)
-                    .build());
-            failingSegmentsForRepair.remove(repairUnit);
+                   context.storage.updateRepairSegment(
+                       currentSegment
+                           .with()
+                           .withState(RepairSegment.State.DONE)
+                           .withEndTime(DateTime.now())
+                           .withId(segmentId)
+                           .build());
+                   failingSegmentsForRepair.remove(repairUnit);
 
-            successOrFailedNotified.set(true);
-            // Since we can get out of order notifications,
-            // we need to exit if we already got the COMPLETE notification.
-            if (completeNotified.get()) {
-              condition.signalAll();
-              jmxProxy.removeRepairStatusHandler(repairNumber);
-            }
-            break;
+                   successOrFailedNotified.set(true);
+                   // Since we can get out of order notifications,
+                   // we need to exit if we already got the COMPLETE notification.
+                   if (completeNotified.get()) {
+                     condition.signalAll();
+                     jmxProxy.removeRepairStatusHandler(repairNumber);
+                   }
+                   break;
+                 } else {
+                         LOG.info("Could not renew lead - as such - cannot update repair segment");
+                         break;
+                 }
           }
         } catch (AssertionError er) {
           // ignore. segment repair has since timed out.
         }
+        LOG.info("Setting segment failed to true in fall through path");
         segmentFailed.set(true);
         break;
 
@@ -1132,6 +1345,7 @@ public final class SegmentRunner implements RepairStatusHandler, Runnable {
         }
         break;
       default:
+
         LOG.info(
             "Unidentified progressStatus {} for segment with id '{}' and repair number '{}'",
             progress.get(),
@@ -1148,7 +1362,7 @@ public final class SegmentRunner implements RepairStatusHandler, Runnable {
       boolean failOutsideSynchronizedBlock,
       Optional<ProgressEventType> progress,
       JmxProxy jmxProxy) {
-
+    MetricsManager.getInstance().addStat("progress_update{type=\"" + progress.get() + "\"}", 1);
     switch (status.get()) {
       case STARTED:
         try {
@@ -1341,9 +1555,6 @@ public final class SegmentRunner implements RepairStatusHandler, Runnable {
   }
 
   private boolean lockSegmentRunners() {
-    if (this.repairUnit.getIncrementalRepair() || !context.config.getDatacenterAvailability().isInCollocatedMode()) {
-      return true;
-    }
     UUID lockId = getLockId();
     if (lastUsedLockId != null && !lockId.equals(lastUsedLockId)) {
        LOG.info("Rotating lock from " + lastUsedLockId + " to " + lockId);
@@ -1367,10 +1578,15 @@ public final class SegmentRunner implements RepairStatusHandler, Runnable {
 
   public static long getRoundedTime() {
        long time = System.currentTimeMillis();
-       long roundingFactor = Long.parseLong(System.getProperty("reaper.lock.rotation.in.millis", "10000000"));  // 10,000 seconds is about 3 hours
+       long roundingFactor = Long.parseLong(System.getProperty("reaper.lock.rotation.in.millis", "30000000"));  // 30,000 seconds is about 9 hours
        double roundedTimeDouble = time / roundingFactor;
        long roundedTimeLong = Math.round(roundedTimeDouble);
-       return roundedTimeLong * roundingFactor;
+       long retVal = roundedTimeLong * roundingFactor;
+       if (lastReturnedLock != null && !lastReturnedLock.equals(retVal)) {
+          MetricsManager.getInstance().addStat("lock_rotation", 1);
+       }
+       lastReturnedLock = new Long(retVal);
+       return retVal;
   }
 
   private void releaseSegmentRunners() {
@@ -1460,3 +1676,5 @@ public final class SegmentRunner implements RepairStatusHandler, Runnable {
 
 }
 
+
+
