@@ -1038,6 +1038,8 @@ public final class CassandraStorage implements IStorage, IDistributedStorage {
 
     BatchStatement updateRepairSegmentBatch = new BatchStatement(BatchStatement.Type.UNLOGGED);
 
+    LOG.info("Updating " + segment.getId() + " to be state " + segment.getState());
+
     updateRepairSegmentBatch.add(
         updateRepairSegmentPrepStmt.bind(
             segment.getRunId(),
@@ -1492,6 +1494,18 @@ public final class CassandraStorage implements IStorage, IDistributedStorage {
     return takeLead(leaderId, LEAD_DURATION);
   }
 
+  private void dumpStackTrace(String s) {
+         if("true".equals(System.getProperty("dump.stack.trace.debug"))) {
+                try {
+                        throw new RuntimeException(s);
+                } catch (Exception e) {
+                        LOG.error(s, e);
+                }
+         } else {
+                 LOG.debug(s);
+         }
+  }
+
   @Override
   public boolean takeLead(UUID leaderId, int ttl) {
     LOG.debug("Trying to take lead on segment {}", leaderId);
@@ -1500,7 +1514,8 @@ public final class CassandraStorage implements IStorage, IDistributedStorage {
 
     if (lwtResult.wasApplied()) {
        MetricsManager.getInstance().addStat("takelead_succeeded", 1);
-      LOG.debug("Took lead on segment {}", leaderId);
+       LOG.info("Took lead on segment {}", leaderId);
+       dumpStackTrace("takeLead-" + leaderId);
       return true;
     }
 
@@ -1526,7 +1541,9 @@ public final class CassandraStorage implements IStorage, IDistributedStorage {
             reaperInstanceId));
 
     if (lwtResult.wasApplied()) {
-      LOG.debug("Renewed lead on segment {}", leaderId);
+      LOG.info("Renewed lead on segment {}", leaderId);
+      dumpStackTrace("renewLead-" + leaderId);
+      MetricsManager.getInstance().addStat("takelead_succeeded", 1);
       return true;
     }
     assert false : "Could not renew lead on segment " + leaderId;
@@ -1549,10 +1566,12 @@ public final class CassandraStorage implements IStorage, IDistributedStorage {
     ResultSet lwtResult = session.execute(releaseLeadPrepStmt.bind(leaderId, reaperInstanceId));
     LOG.info("Trying to release lead on segment {} for instance {}", leaderId, reaperInstanceId);
     if (lwtResult.wasApplied()) {
-      LOG.info("Released lead on segment {}", leaderId);
+       MetricsManager.getInstance().addStat("releaselead_succeeded", 1);
+       dumpStackTrace("releaseLead-" + leaderId);
+       LOG.info("Released lead on segment {}", leaderId);
     } else {
-      assert false : "Could not release lead on segment " + leaderId;
-      LOG.error("Could not release lead on segment {}", leaderId);
+       assert false : "Could not release lead on segment " + leaderId;
+       LOG.error("Could not release lead on segment {}", leaderId);
     }
   }
 
@@ -2040,3 +2059,5 @@ public final class CassandraStorage implements IStorage, IDistributedStorage {
 
 }
 
+
+
