@@ -1,16 +1,5 @@
 // Copyright (c) 2020 The Jaeger Authors.
-//
-// Licensed under the Apache License, Version 2.0 (the "License");
-// you may not use this file except in compliance with the License.
-// You may obtain a copy of the License at
-//
-// http://www.apache.org/licenses/LICENSE-2.0
-//
-// Unless required by applicable law or agreed to in writing, software
-// distributed under the License is distributed on an "AS IS" BASIS,
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-// See the License for the specific language governing permissions and
-// limitations under the License.
+// SPDX-License-Identifier: Apache-2.0
 
 package app
 
@@ -28,8 +17,10 @@ import (
 	"github.com/jaegertracing/jaeger/cmd/collector/app/flags"
 	"github.com/jaegertracing/jaeger/cmd/collector/app/handler"
 	"github.com/jaegertracing/jaeger/cmd/collector/app/processor"
-	"github.com/jaegertracing/jaeger/cmd/collector/app/sampling/strategystore"
+	"github.com/jaegertracing/jaeger/cmd/collector/app/sampling/samplingstrategy"
 	"github.com/jaegertracing/jaeger/cmd/collector/app/server"
+	"github.com/jaegertracing/jaeger/internal/safeexpvar"
+	"github.com/jaegertracing/jaeger/model"
 	"github.com/jaegertracing/jaeger/pkg/healthcheck"
 	"github.com/jaegertracing/jaeger/pkg/metrics"
 	"github.com/jaegertracing/jaeger/pkg/tenancy"
@@ -44,16 +35,16 @@ const (
 // Collector returns the collector as a manageable unit of work
 type Collector struct {
 	// required to start a new collector
-	serviceName    string
-	logger         *zap.Logger
-	metricsFactory metrics.Factory
-	spanWriter     spanstore.Writer
-	strategyStore  strategystore.StrategyStore
-	aggregator     strategystore.Aggregator
-	hCheck         *healthcheck.HealthCheck
-	spanProcessor  processor.SpanProcessor
-	spanHandlers   *SpanHandlers
-	tenancyMgr     *tenancy.Manager
+	serviceName        string
+	logger             *zap.Logger
+	metricsFactory     metrics.Factory
+	spanWriter         spanstore.Writer
+	samplingProvider   samplingstrategy.Provider
+	samplingAggregator samplingstrategy.Aggregator
+	hCheck             *healthcheck.HealthCheck
+	spanProcessor      processor.SpanProcessor
+	spanHandlers       *SpanHandlers
+	tenancyMgr         *tenancy.Manager
 
 	// state, read only
 	hServer                    *http.Server
@@ -67,27 +58,27 @@ type Collector struct {
 
 // CollectorParams to construct a new Jaeger Collector.
 type CollectorParams struct {
-	ServiceName    string
-	Logger         *zap.Logger
-	MetricsFactory metrics.Factory
-	SpanWriter     spanstore.Writer
-	StrategyStore  strategystore.StrategyStore
-	Aggregator     strategystore.Aggregator
-	HealthCheck    *healthcheck.HealthCheck
-	TenancyMgr     *tenancy.Manager
+	ServiceName        string
+	Logger             *zap.Logger
+	MetricsFactory     metrics.Factory
+	SpanWriter         spanstore.Writer
+	SamplingProvider   samplingstrategy.Provider
+	SamplingAggregator samplingstrategy.Aggregator
+	HealthCheck        *healthcheck.HealthCheck
+	TenancyMgr         *tenancy.Manager
 }
 
 // New constructs a new collector component, ready to be started
 func New(params *CollectorParams) *Collector {
 	return &Collector{
-		serviceName:    params.ServiceName,
-		logger:         params.Logger,
-		metricsFactory: params.MetricsFactory,
-		spanWriter:     params.SpanWriter,
-		strategyStore:  params.StrategyStore,
-		aggregator:     params.Aggregator,
-		hCheck:         params.HealthCheck,
-		tenancyMgr:     params.TenancyMgr,
+		serviceName:        params.ServiceName,
+		logger:             params.Logger,
+		metricsFactory:     params.MetricsFactory,
+		spanWriter:         params.SpanWriter,
+		samplingProvider:   params.SamplingProvider,
+		samplingAggregator: params.SamplingAggregator,
+		hCheck:             params.HealthCheck,
+		tenancyMgr:         params.TenancyMgr,
 	}
 }
 
@@ -102,8 +93,10 @@ func (c *Collector) Start(options *flags.CollectorOptions) error {
 	}
 
 	var additionalProcessors []ProcessSpan
-	if c.aggregator != nil {
-		additionalProcessors = append(additionalProcessors, handleRootSpan(c.aggregator, c.logger))
+	if c.samplingAggregator != nil {
+		additionalProcessors = append(additionalProcessors, func(span *model.Span, _ /* tenant */ string) {
+			c.samplingAggregator.HandleRootSpan(span, c.logger)
+		})
 	}
 
 	c.spanProcessor = handlerBuilder.BuildSpanProcessor(additionalProcessors...)
@@ -113,7 +106,7 @@ func (c *Collector) Start(options *flags.CollectorOptions) error {
 		HostPort:                options.GRPC.HostPort,
 		Handler:                 c.spanHandlers.GRPCHandler,
 		TLSConfig:               options.GRPC.TLS,
-		SamplingStore:           c.strategyStore,
+		SamplingProvider:        c.samplingProvider,
 		Logger:                  c.logger,
 		MaxReceiveMessageLength: options.GRPC.MaxReceiveMessageLength,
 		MaxConnectionAge:        options.GRPC.MaxConnectionAge,
@@ -125,13 +118,13 @@ func (c *Collector) Start(options *flags.CollectorOptions) error {
 	c.grpcServer = grpcServer
 
 	httpServer, err := server.StartHTTPServer(&server.HTTPServerParams{
-		HostPort:       options.HTTP.HostPort,
-		Handler:        c.spanHandlers.JaegerBatchesHandler,
-		TLSConfig:      options.HTTP.TLS,
-		HealthCheck:    c.hCheck,
-		MetricsFactory: c.metricsFactory,
-		SamplingStore:  c.strategyStore,
-		Logger:         c.logger,
+		HostPort:         options.HTTP.HostPort,
+		Handler:          c.spanHandlers.JaegerBatchesHandler,
+		TLSConfig:        options.HTTP.TLS,
+		HealthCheck:      c.hCheck,
+		MetricsFactory:   c.metricsFactory,
+		SamplingProvider: c.samplingProvider,
+		Logger:           c.logger,
 	})
 	if err != nil {
 		return fmt.Errorf("could not start HTTP server: %w", err)
@@ -150,7 +143,6 @@ func (c *Collector) Start(options *flags.CollectorOptions) error {
 			return fmt.Errorf("could not start Zipkin receiver: %w", err)
 		}
 		c.zipkinReceiver = zipkinReceiver
-
 	}
 
 	if options.OTLP.Enabled {
@@ -166,10 +158,10 @@ func (c *Collector) Start(options *flags.CollectorOptions) error {
 	return nil
 }
 
-func (c *Collector) publishOpts(cOpts *flags.CollectorOptions) {
-	internalFactory := c.metricsFactory.Namespace(metrics.NSOptions{Name: "internal"})
-	internalFactory.Gauge(metrics.Options{Name: metricNumWorkers}).Update(int64(cOpts.NumWorkers))
-	internalFactory.Gauge(metrics.Options{Name: metricQueueSize}).Update(int64(cOpts.QueueSize))
+func (*Collector) publishOpts(cOpts *flags.CollectorOptions) {
+	safeexpvar.SetInt(metricNumWorkers, int64(cOpts.NumWorkers))
+	//nolint: gosec // G115
+	safeexpvar.SetInt(metricQueueSize, int64(cOpts.QueueSize))
 }
 
 // Close the component and all its underlying dependencies
@@ -211,16 +203,22 @@ func (c *Collector) Close() error {
 	}
 
 	// aggregator does not exist for all strategy stores. only Close() if exists.
-	if c.aggregator != nil {
-		if err := c.aggregator.Close(); err != nil {
+	if c.samplingAggregator != nil {
+		if err := c.samplingAggregator.Close(); err != nil {
 			c.logger.Error("failed to close aggregator.", zap.Error(err))
 		}
 	}
 
 	// watchers actually never return errors from Close
-	_ = c.tlsGRPCCertWatcherCloser.Close()
-	_ = c.tlsHTTPCertWatcherCloser.Close()
-	_ = c.tlsZipkinCertWatcherCloser.Close()
+	if c.tlsGRPCCertWatcherCloser != nil {
+		_ = c.tlsGRPCCertWatcherCloser.Close()
+	}
+	if c.tlsHTTPCertWatcherCloser != nil {
+		_ = c.tlsHTTPCertWatcherCloser.Close()
+	}
+	if c.tlsZipkinCertWatcherCloser != nil {
+		_ = c.tlsZipkinCertWatcherCloser.Close()
+	}
 
 	return nil
 }
