@@ -1,17 +1,6 @@
 // Copyright (c) 2019 The Jaeger Authors.
 // Copyright (c) 2017 Uber Technologies, Inc.
-//
-// Licensed under the Apache License, Version 2.0 (the "License");
-// you may not use this file except in compliance with the License.
-// You may obtain a copy of the License at
-//
-// http://www.apache.org/licenses/LICENSE-2.0
-//
-// Unless required by applicable law or agreed to in writing, software
-// distributed under the License is distributed on an "AS IS" BASIS,
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-// See the License for the specific language governing permissions and
-// limitations under the License.
+// SPDX-License-Identifier: Apache-2.0
 
 package app
 
@@ -81,8 +70,9 @@ func TestBySvcMetrics(t *testing.T) {
 		}
 	}
 
-	for _, test := range tests {
+	testFn := func(t *testing.T, test TestCase) {
 		mb := metricstest.NewFactory(time.Hour)
+		defer mb.Backend.Stop()
 		logger := zap.NewNop()
 		serviceMetrics := mb.Namespace(metrics.NSOptions{Name: "service", Tags: nil})
 		hostMetrics := mb.Namespace(metrics.NSOptions{Name: "host", Tags: nil})
@@ -159,6 +149,11 @@ func TestBySvcMetrics(t *testing.T) {
 		}
 		mb.AssertCounterMetrics(t, expected...)
 	}
+	for _, test := range tests {
+		t.Run(fmt.Sprintf("%v", test), func(t *testing.T) {
+			testFn(t, test)
+		})
+	}
 }
 
 func isSpanAllowed(span *model.Span) bool {
@@ -191,7 +186,7 @@ func (n *fakeSpanWriter) WriteSpan(ctx context.Context, span *model.Span) error
 	return n.err
 }
 
-func (n *fakeSpanWriter) Close() error {
+func (*fakeSpanWriter) Close() error {
 	return nil
 }
 
@@ -258,6 +253,7 @@ func TestSpanProcessorErrors(t *testing.T) {
 		err: fmt.Errorf("some-error"),
 	}
 	mb := metricstest.NewFactory(time.Hour)
+	defer mb.Backend.Stop()
 	serviceMetrics := mb.Namespace(metrics.NSOptions{Name: "service", Tags: nil})
 	p := NewSpanProcessor(w,
 		nil,
@@ -295,7 +291,7 @@ type blockingWriter struct {
 	inWriteSpan atomic.Int32
 }
 
-func (w *blockingWriter) WriteSpan(ctx context.Context, span *model.Span) error {
+func (w *blockingWriter) WriteSpan(context.Context, *model.Span) error {
 	w.inWriteSpan.Add(1)
 	w.Lock()
 	defer w.Unlock()
@@ -342,6 +338,7 @@ func TestSpanProcessorBusy(t *testing.T) {
 
 func TestSpanProcessorWithNilProcess(t *testing.T) {
 	mb := metricstest.NewFactory(time.Hour)
+	defer mb.Backend.Stop()
 	serviceMetrics := mb.Namespace(metrics.NSOptions{Name: "service", Tags: nil})
 
 	w := &fakeSpanWriter{}
@@ -438,8 +435,9 @@ func TestSpanProcessorCountSpan(t *testing.T) {
 	}
 
 	for _, tt := range tests {
-		t.Run(tt.name, func(t *testing.T) {
+		t.Run(tt.name, func(_ *testing.T) {
 			mb := metricstest.NewFactory(time.Hour)
+			defer mb.Backend.Stop()
 			m := mb.Namespace(metrics.NSOptions{})
 
 			w := &fakeSpanWriter{}
@@ -465,7 +463,7 @@ func TestSpanProcessorCountSpan(t *testing.T) {
 					time.Second,
 					time.Millisecond,
 				)
-				assert.Greater(t, p.spansProcessed.Load(), uint64(0))
+				assert.Positive(t, p.spansProcessed.Load())
 			}
 
 			for i := 0; i < 10000; i++ {
@@ -598,14 +596,14 @@ func TestStartDynQueueSizeUpdater(t *testing.T) {
 
 	// we wait up to 50 milliseconds
 	for i := 0; i < 5; i++ {
-		if p.queue.Capacity() == 100 {
-			time.Sleep(10 * time.Millisecond)
-		} else {
+		if p.queue.Capacity() != 100 {
 			break
 		}
+		time.Sleep(10 * time.Millisecond)
 	}
 
 	assert.EqualValues(t, 104857, p.queue.Capacity())
+	require.NoError(t, p.Close())
 }
 
 func TestAdditionalProcessors(t *testing.T) {
@@ -626,7 +624,7 @@ func TestAdditionalProcessors(t *testing.T) {
 
 	// additional processor is called
 	count := 0
-	f := func(s *model.Span, t string) {
+	f := func(_ *model.Span, _ string) {
 		count++
 	}
 	p = NewSpanProcessor(w, []ProcessSpan{f}, Options.QueueSize(1))
