@@ -1,16 +1,5 @@
 // Copyright (c) 2018 The Jaeger Authors.
-//
-// Licensed under the Apache License, Version 2.0 (the "License");
-// you may not use this file except in compliance with the License.
-// You may obtain a copy of the License at
-//
-// http://www.apache.org/licenses/LICENSE-2.0
-//
-// Unless required by applicable law or agreed to in writing, software
-// distributed under the License is distributed on an "AS IS" BASIS,
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-// See the License for the specific language governing permissions and
-// limitations under the License.
+// SPDX-License-Identifier: Apache-2.0
 
 package offset
 
@@ -54,7 +43,7 @@ func TestHandleReset(t *testing.T) {
 func TestCache(t *testing.T) {
 	offset := int64(1498)
 
-	fakeMarker := func(offset int64) {
+	fakeMarker := func(_ /* offset */ int64) {
 		assert.Fail(t, "Shouldn't mark cached offset")
 	}
 	manager := NewManager(offset, fakeMarker, "test_topic", 1, metrics.NullFactory)
