@@ -1,17 +1,6 @@
 // Copyright (c) 2019 The Jaeger Authors.
 // Copyright (c) 2017 Uber Technologies, Inc.
-//
-// Licensed under the Apache License, Version 2.0 (the "License");
-// you may not use this file except in compliance with the License.
-// You may obtain a copy of the License at
-//
-// http://www.apache.org/licenses/LICENSE-2.0
-//
-// Unless required by applicable law or agreed to in writing, software
-// distributed under the License is distributed on an "AS IS" BASIS,
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-// See the License for the specific language governing permissions and
-// limitations under the License.
+// SPDX-License-Identifier: Apache-2.0
 
 package app
 
@@ -30,6 +19,7 @@ import (
 	"github.com/gorilla/mux"
 	"go.opentelemetry.io/contrib/instrumentation/net/http/otelhttp"
 	"go.opentelemetry.io/otel/propagation"
+	"go.opentelemetry.io/otel/trace"
 	"go.uber.org/zap"
 
 	"github.com/jaegertracing/jaeger/cmd/query/app/querysvc"
@@ -63,7 +53,7 @@ type HTTPHandler interface {
 }
 
 type structuredResponse struct {
-	Data   interface{}       `json:"data"`
+	Data   any               `json:"data"`
 	Total  int               `json:"total"`
 	Limit  int               `json:"limit"`
 	Offset int               `json:"offset"`
@@ -90,7 +80,7 @@ type APIHandler struct {
 	basePath            string
 	apiPrefix           string
 	logger              *zap.Logger
-	tracer              *jtracer.JTracer
+	tracer              trace.TracerProvider
 }
 
 // NewAPIHandler returns an APIHandler
@@ -114,7 +104,7 @@ func NewAPIHandler(queryService *querysvc.QueryService, tm *tenancy.Manager, opt
 		aH.logger = zap.NewNop()
 	}
 	if aH.tracer == nil {
-		aH.tracer = jtracer.NoOp()
+		aH.tracer = jtracer.NoOp().OTEL
 	}
 	return aH
 }
@@ -141,7 +131,7 @@ func (aH *APIHandler) handleFunc(
 	router *mux.Router,
 	f func(http.ResponseWriter, *http.Request),
 	routeFmt string,
-	args ...interface{},
+	args ...any,
 ) *mux.Route {
 	route := aH.formatRoute(routeFmt, args...)
 	var handler http.Handler = http.HandlerFunc(f)
@@ -151,12 +141,12 @@ func (aH *APIHandler) handleFunc(
 	traceMiddleware := otelhttp.NewHandler(
 		otelhttp.WithRouteTag(route, traceResponseHandler(handler)),
 		route,
-		otelhttp.WithTracerProvider(aH.tracer.OTEL))
+		otelhttp.WithTracerProvider(aH.tracer))
 	return router.HandleFunc(route, traceMiddleware.ServeHTTP)
 }
 
-func (aH *APIHandler) formatRoute(route string, args ...interface{}) string {
-	args = append([]interface{}{aH.apiPrefix}, args...)
+func (aH *APIHandler) formatRoute(route string, args ...any) string {
+	args = append([]any{aH.apiPrefix}, args...)
 	return fmt.Sprintf("/%s"+route, args...)
 }
 
@@ -393,7 +383,7 @@ func (aH *APIHandler) convertModelToUI(trace *model.Trace, adjust bool) (*ui.Tra
 	return uiTrace, uiError
 }
 
-func (aH *APIHandler) deduplicateDependencies(dependencies []model.DependencyLink) []ui.DependencyLink {
+func (*APIHandler) deduplicateDependencies(dependencies []model.DependencyLink) []ui.DependencyLink {
 	type Key struct {
 		parent string
 		child  string
@@ -412,7 +402,7 @@ func (aH *APIHandler) deduplicateDependencies(dependencies []model.DependencyLin
 	return result
 }
 
-func (aH *APIHandler) filterDependenciesByService(
+func (*APIHandler) filterDependenciesByService(
 	dependencies []model.DependencyLink,
 	service string,
 ) []model.DependencyLink {
@@ -516,7 +506,7 @@ func (aH *APIHandler) handleError(w http.ResponseWriter, err error, statusCode i
 	return true
 }
 
-func (aH *APIHandler) writeJSON(w http.ResponseWriter, r *http.Request, response interface{}) {
+func (aH *APIHandler) writeJSON(w http.ResponseWriter, r *http.Request, response any) {
 	prettyPrintValue := r.FormValue(prettyPrintParam)
 	prettyPrint := prettyPrintValue != "" && prettyPrintValue != "false"
 
