@@ -1,17 +1,6 @@
 // Copyright (c) 2019 The Jaeger Authors.
 // Copyright (c) 2017 Uber Technologies, Inc.
-//
-// Licensed under the Apache License, Version 2.0 (the "License");
-// you may not use this file except in compliance with the License.
-// You may obtain a copy of the License at
-//
-// http://www.apache.org/licenses/LICENSE-2.0
-//
-// Unless required by applicable law or agreed to in writing, software
-// distributed under the License is distributed on an "AS IS" BASIS,
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-// See the License for the specific language governing permissions and
-// limitations under the License.
+// SPDX-License-Identifier: Apache-2.0
 
 package app
 
@@ -38,7 +27,8 @@ import (
 	sdktrace "go.opentelemetry.io/otel/sdk/trace"
 	"go.opentelemetry.io/otel/sdk/trace/tracetest"
 	"go.uber.org/zap"
-	"go.uber.org/zap/zapcore"
+	"go.uber.org/zap/zaptest"
+	"go.uber.org/zap/zaptest/observer"
 
 	"github.com/jaegertracing/jaeger/cmd/query/app/querysvc"
 	"github.com/jaegertracing/jaeger/model"
@@ -102,8 +92,9 @@ type structuredTraceResponse struct {
 	Errors []structuredError `json:"errors"`
 }
 
-func initializeTestServerWithHandler(queryOptions querysvc.QueryServiceOptions, options ...HandlerOption) *testServer {
+func initializeTestServerWithHandler(t *testing.T, queryOptions querysvc.QueryServiceOptions, options ...HandlerOption) *testServer {
 	return initializeTestServerWithOptions(
+		t,
 		&tenancy.Manager{},
 		queryOptions,
 		append(
@@ -119,23 +110,33 @@ func initializeTestServerWithHandler(queryOptions querysvc.QueryServiceOptions,
 	)
 }
 
-func initializeTestServerWithOptions(tenancyMgr *tenancy.Manager, queryOptions querysvc.QueryServiceOptions, options ...HandlerOption) *testServer {
+func initializeTestServerWithOptions(
+	t *testing.T,
+	tenancyMgr *tenancy.Manager,
+	queryOptions querysvc.QueryServiceOptions,
+	options ...HandlerOption,
+) *testServer {
+	options = append(options, HandlerOptions.Logger(zaptest.NewLogger(t)))
 	readStorage := &spanstoremocks.Reader{}
 	dependencyStorage := &depsmocks.Reader{}
 	qs := querysvc.NewQueryService(readStorage, dependencyStorage, queryOptions)
 	r := NewRouter()
 	handler := NewAPIHandler(qs, tenancyMgr, options...)
 	handler.RegisterRoutes(r)
-	return &testServer{
+	ts := &testServer{
 		server:           httptest.NewServer(tenancy.ExtractTenantHTTPHandler(tenancyMgr, r)),
 		spanReader:       readStorage,
 		dependencyReader: dependencyStorage,
 		handler:          handler,
 	}
+	t.Cleanup(func() {
+		ts.server.Close()
+	})
+	return ts
 }
 
-func initializeTestServer(options ...HandlerOption) *testServer {
-	return initializeTestServerWithHandler(querysvc.QueryServiceOptions{}, options...)
+func initializeTestServer(t *testing.T, options ...HandlerOption) *testServer {
+	return initializeTestServerWithHandler(t, querysvc.QueryServiceOptions{}, options...)
 }
 
 type testServer struct {
@@ -145,15 +146,13 @@ type testServer struct {
 	server           *httptest.Server
 }
 
-func withTestServer(doTest func(s *testServer), queryOptions querysvc.QueryServiceOptions, options ...HandlerOption) {
-	ts := initializeTestServerWithOptions(&tenancy.Manager{}, queryOptions, options...)
-	defer ts.server.Close()
+func withTestServer(t *testing.T, doTest func(s *testServer), queryOptions querysvc.QueryServiceOptions, options ...HandlerOption) {
+	ts := initializeTestServerWithOptions(t, &tenancy.Manager{}, queryOptions, options...)
 	doTest(ts)
 }
 
 func TestGetTraceSuccess(t *testing.T) {
-	ts := initializeTestServer()
-	defer ts.server.Close()
+	ts := initializeTestServer(t)
 	ts.spanReader.On("GetTrace", mock.AnythingOfType("*context.valueCtx"), mock.AnythingOfType("model.TraceID")).
 		Return(mockTrace, nil).Once()
 
@@ -163,61 +162,37 @@ func TestGetTraceSuccess(t *testing.T) {
 	assert.Empty(t, response.Errors)
 }
 
-type logData struct {
-	e zapcore.Entry
-	f []zapcore.Field
-}
-
-type testLogger struct {
-	logs *[]logData
-}
-
-func (testLogger) Enabled(zapcore.Level) bool          { return true }
-func (l testLogger) With([]zapcore.Field) zapcore.Core { return l }
-func (testLogger) Sync() error                         { return nil }
-func (l testLogger) Check(e zapcore.Entry, ce *zapcore.CheckedEntry) *zapcore.CheckedEntry {
-	return ce.AddCore(e, l)
-}
-
-func (l testLogger) Write(e zapcore.Entry, f []zapcore.Field) error {
-	*l.logs = append(*l.logs, logData{e: e, f: f})
-	return nil
-}
-
 func TestLogOnServerError(t *testing.T) {
-	l := &testLogger{
-		logs: &[]logData{},
-	}
+	zapCore, logs := observer.New(zap.InfoLevel)
+	logger := zap.New(zapCore)
 	readStorage := &spanstoremocks.Reader{}
 	dependencyStorage := &depsmocks.Reader{}
 	qs := querysvc.NewQueryService(readStorage, dependencyStorage, querysvc.QueryServiceOptions{})
-	apiHandlerOptions := []HandlerOption{
-		HandlerOptions.Logger(zap.New(l)),
-	}
-	h := NewAPIHandler(qs, &tenancy.Manager{}, apiHandlerOptions...)
+	h := NewAPIHandler(qs, &tenancy.Manager{}, HandlerOptions.Logger(logger))
 	e := errors.New("test error")
 	h.handleError(&httptest.ResponseRecorder{}, e, http.StatusInternalServerError)
-	require.Len(t, *l.logs, 1)
-	assert.Equal(t, "HTTP handler, Internal Server Error", (*l.logs)[0].e.Message)
-	assert.Len(t, (*l.logs)[0].f, 1)
-	assert.Equal(t, e, (*l.logs)[0].f[0].Interface)
+	require.Len(t, logs.All(), 1)
+	log := logs.All()[0]
+	assert.Equal(t, "HTTP handler, Internal Server Error", log.Message)
+	require.Len(t, log.Context, 1)
+	assert.Equal(t, e, log.Context[0].Interface)
 }
 
 // httpResponseErrWriter implements the http.ResponseWriter interface that returns an error on Write.
 type httpResponseErrWriter struct{}
 
-func (h *httpResponseErrWriter) Write([]byte) (int, error) {
+func (*httpResponseErrWriter) Write([]byte) (int, error) {
 	return 0, fmt.Errorf("failed to write")
 }
-func (h *httpResponseErrWriter) WriteHeader(statusCode int) {}
-func (h *httpResponseErrWriter) Header() http.Header {
+func (*httpResponseErrWriter) WriteHeader(int /* statusCode */) {}
+func (*httpResponseErrWriter) Header() http.Header {
 	return http.Header{}
 }
 
 func TestWriteJSON(t *testing.T) {
 	testCases := []struct {
 		name         string
-		data         interface{}
+		data         any
 		param        string
 		output       string
 		httpWriteErr bool
@@ -321,8 +296,7 @@ func TestGetTrace(t *testing.T) {
 			jTracer := jtracer.JTracer{OTEL: tracerProvider}
 			defer tracerProvider.Shutdown(context.Background())
 
-			ts := initializeTestServer(HandlerOptions.Tracer(&jTracer))
-			defer ts.server.Close()
+			ts := initializeTestServer(t, HandlerOptions.Tracer(jTracer.OTEL))
 
 			ts.spanReader.On("GetTrace", mock.AnythingOfType("*context.valueCtx"), model.NewTraceID(0, 0x123456abc)).
 				Return(makeMockTrace(t), nil).Once()
@@ -343,8 +317,7 @@ func TestGetTrace(t *testing.T) {
 }
 
 func TestGetTraceDBFailure(t *testing.T) {
-	ts := initializeTestServer()
-	defer ts.server.Close()
+	ts := initializeTestServer(t)
 	ts.spanReader.On("GetTrace", mock.AnythingOfType("*context.valueCtx"), mock.AnythingOfType("model.TraceID")).
 		Return(nil, errStorage).Once()
 
@@ -354,8 +327,7 @@ func TestGetTraceDBFailure(t *testing.T) {
 }
 
 func TestGetTraceNotFound(t *testing.T) {
-	ts := initializeTestServer()
-	defer ts.server.Close()
+	ts := initializeTestServer(t)
 	ts.spanReader.On("GetTrace", mock.AnythingOfType("*context.valueCtx"), mock.AnythingOfType("model.TraceID")).
 		Return(nil, spanstore.ErrTraceNotFound).Once()
 
@@ -366,13 +338,13 @@ func TestGetTraceNotFound(t *testing.T) {
 
 func TestGetTraceAdjustmentFailure(t *testing.T) {
 	ts := initializeTestServerWithHandler(
+		t,
 		querysvc.QueryServiceOptions{
 			Adjuster: adjuster.Func(func(trace *model.Trace) (*model.Trace, error) {
 				return trace, errAdjustment
 			}),
 		},
 	)
-	defer ts.server.Close()
 	ts.spanReader.On("GetTrace", mock.AnythingOfType("*context.valueCtx"), mock.AnythingOfType("model.TraceID")).
 		Return(mockTrace, nil).Once()
 
@@ -384,8 +356,7 @@ func TestGetTraceAdjustmentFailure(t *testing.T) {
 }
 
 func TestGetTraceBadTraceID(t *testing.T) {
-	ts := initializeTestServer()
-	defer ts.server.Close()
+	ts := initializeTestServer(t)
 
 	var response structuredResponse
 	err := getJSON(ts.server.URL+`/api/traces/chumbawumba`, &response)
@@ -393,8 +364,7 @@ func TestGetTraceBadTraceID(t *testing.T) {
 }
 
 func TestSearchSuccess(t *testing.T) {
-	ts := initializeTestServer()
-	defer ts.server.Close()
+	ts := initializeTestServer(t)
 	ts.spanReader.On("FindTraces", mock.AnythingOfType("*context.valueCtx"), mock.AnythingOfType("*spanstore.TraceQueryParameters")).
 		Return([]*model.Trace{mockTrace}, nil).Once()
 
@@ -405,8 +375,7 @@ func TestSearchSuccess(t *testing.T) {
 }
 
 func TestSearchByTraceIDSuccess(t *testing.T) {
-	ts := initializeTestServer()
-	defer ts.server.Close()
+	ts := initializeTestServer(t)
 	ts.spanReader.On("GetTrace", mock.AnythingOfType("*context.valueCtx"), mock.AnythingOfType("model.TraceID")).
 		Return(mockTrace, nil).Twice()
 
@@ -419,10 +388,9 @@ func TestSearchByTraceIDSuccess(t *testing.T) {
 
 func TestSearchByTraceIDSuccessWithArchive(t *testing.T) {
 	archiveReadMock := &spanstoremocks.Reader{}
-	ts := initializeTestServerWithOptions(&tenancy.Manager{}, querysvc.QueryServiceOptions{
+	ts := initializeTestServerWithOptions(t, &tenancy.Manager{}, querysvc.QueryServiceOptions{
 		ArchiveSpanReader: archiveReadMock,
 	})
-	defer ts.server.Close()
 	ts.spanReader.On("GetTrace", mock.AnythingOfType("*context.valueCtx"), mock.AnythingOfType("model.TraceID")).
 		Return(nil, spanstore.ErrTraceNotFound).Twice()
 	archiveReadMock.On("GetTrace", mock.AnythingOfType("*context.valueCtx"), mock.AnythingOfType("model.TraceID")).
@@ -436,8 +404,7 @@ func TestSearchByTraceIDSuccessWithArchive(t *testing.T) {
 }
 
 func TestSearchByTraceIDNotFound(t *testing.T) {
-	ts := initializeTestServer()
-	defer ts.server.Close()
+	ts := initializeTestServer(t)
 	ts.spanReader.On("GetTrace", mock.AnythingOfType("*context.valueCtx"), mock.AnythingOfType("model.TraceID")).
 		Return(nil, spanstore.ErrTraceNotFound).Once()
 
@@ -449,11 +416,10 @@ func TestSearchByTraceIDNotFound(t *testing.T) {
 }
 
 func TestSearchByTraceIDFailure(t *testing.T) {
-	ts := initializeTestServer()
-	defer ts.server.Close()
-	whatsamattayou := "https://youtu.be/WrKFOCg13QQ"
+	ts := initializeTestServer(t)
+	whatsamattayou := "whatsamattayou"
 	ts.spanReader.On("GetTrace", mock.AnythingOfType("*context.valueCtx"), mock.AnythingOfType("model.TraceID")).
-		Return(nil, fmt.Errorf(whatsamattayou)).Once()
+		Return(nil, errors.New(whatsamattayou)).Once()
 
 	var response structuredResponse
 	err := getJSON(ts.server.URL+`/api/traces?traceID=1`, &response)
@@ -462,6 +428,7 @@ func TestSearchByTraceIDFailure(t *testing.T) {
 
 func TestSearchModelConversionFailure(t *testing.T) {
 	ts := initializeTestServerWithOptions(
+		t,
 		&tenancy.Manager{},
 		querysvc.QueryServiceOptions{
 			Adjuster: adjuster.Func(func(trace *model.Trace) (*model.Trace, error) {
@@ -469,7 +436,6 @@ func TestSearchModelConversionFailure(t *testing.T) {
 			}),
 		},
 	)
-	defer ts.server.Close()
 	ts.spanReader.On("FindTraces", mock.AnythingOfType("*context.valueCtx"), mock.AnythingOfType("*spanstore.TraceQueryParameters")).
 		Return([]*model.Trace{mockTrace}, nil).Once()
 	var response structuredResponse
@@ -480,8 +446,7 @@ func TestSearchModelConversionFailure(t *testing.T) {
 }
 
 func TestSearchDBFailure(t *testing.T) {
-	ts := initializeTestServer()
-	defer ts.server.Close()
+	ts := initializeTestServer(t)
 	ts.spanReader.On("FindTraces", mock.AnythingOfType("*context.valueCtx"), mock.AnythingOfType("*spanstore.TraceQueryParameters")).
 		Return(nil, fmt.Errorf("whatsamattayou")).Once()
 
@@ -510,8 +475,7 @@ func TestSearchFailures(t *testing.T) {
 }
 
 func testIndividualSearchFailures(t *testing.T, urlStr, errMsg string) {
-	ts := initializeTestServer()
-	defer ts.server.Close()
+	ts := initializeTestServer(t)
 	ts.spanReader.On("Query", mock.AnythingOfType("spanstore.TraceQueryParameters")).
 		Return([]*model.Trace{}, nil).Once()
 
@@ -521,8 +485,7 @@ func testIndividualSearchFailures(t *testing.T, urlStr, errMsg string) {
 }
 
 func TestGetServicesSuccess(t *testing.T) {
-	ts := initializeTestServer()
-	defer ts.server.Close()
+	ts := initializeTestServer(t)
 	expectedServices := []string{"trifle", "bling"}
 	ts.spanReader.On("GetServices", mock.AnythingOfType("*context.valueCtx")).Return(expectedServices, nil).Once()
 
@@ -530,15 +493,14 @@ func TestGetServicesSuccess(t *testing.T) {
 	err := getJSON(ts.server.URL+"/api/services", &response)
 	require.NoError(t, err)
 	actualServices := make([]string, len(expectedServices))
-	for i, s := range response.Data.([]interface{}) {
+	for i, s := range response.Data.([]any) {
 		actualServices[i] = s.(string)
 	}
 	assert.Equal(t, expectedServices, actualServices)
 }
 
 func TestGetServicesStorageFailure(t *testing.T) {
-	ts := initializeTestServer()
-	defer ts.server.Close()
+	ts := initializeTestServer(t)
 	ts.spanReader.On("GetServices", mock.AnythingOfType("*context.valueCtx")).Return(nil, errStorage).Once()
 
 	var response structuredResponse
@@ -547,8 +509,7 @@ func TestGetServicesStorageFailure(t *testing.T) {
 }
 
 func TestGetOperationsSuccess(t *testing.T) {
-	ts := initializeTestServer()
-	defer ts.server.Close()
+	ts := initializeTestServer(t)
 	expectedOperations := []spanstore.Operation{{Name: ""}, {Name: "get", SpanKind: "server"}}
 	ts.spanReader.On(
 		"GetOperations",
@@ -577,17 +538,14 @@ func TestGetOperationsSuccess(t *testing.T) {
 }
 
 func TestGetOperationsNoServiceName(t *testing.T) {
-	ts := initializeTestServer()
-	defer ts.server.Close()
-
+	ts := initializeTestServer(t)
 	var response structuredResponse
 	err := getJSON(ts.server.URL+"/api/operations", &response)
 	require.Error(t, err)
 }
 
 func TestGetOperationsStorageFailure(t *testing.T) {
-	ts := initializeTestServer()
-	defer ts.server.Close()
+	ts := initializeTestServer(t)
 	ts.spanReader.On(
 		"GetOperations",
 		mock.AnythingOfType("*context.valueCtx"),
@@ -599,8 +557,7 @@ func TestGetOperationsStorageFailure(t *testing.T) {
 }
 
 func TestGetOperationsLegacySuccess(t *testing.T) {
-	ts := initializeTestServer()
-	defer ts.server.Close()
+	ts := initializeTestServer(t)
 	expectedOperationNames := []string{"", "get"}
 	expectedOperations := []spanstore.Operation{
 		{Name: ""},
@@ -617,12 +574,11 @@ func TestGetOperationsLegacySuccess(t *testing.T) {
 	err := getJSON(ts.server.URL+"/api/services/abc%2Ftrifle/operations", &response)
 
 	require.NoError(t, err)
-	assert.ElementsMatch(t, expectedOperationNames, response.Data.([]interface{}))
+	assert.ElementsMatch(t, expectedOperationNames, response.Data.([]any))
 }
 
 func TestGetOperationsLegacyStorageFailure(t *testing.T) {
-	ts := initializeTestServer()
-	defer ts.server.Close()
+	ts := initializeTestServer(t)
 	ts.spanReader.On(
 		"GetOperations",
 		mock.AnythingOfType("*context.valueCtx"),
@@ -634,14 +590,14 @@ func TestGetOperationsLegacyStorageFailure(t *testing.T) {
 
 func TestTransformOTLPSuccess(t *testing.T) {
 	reformat := func(in []byte) []byte {
-		obj := new(interface{})
+		obj := new(any)
 		require.NoError(t, json.Unmarshal(in, obj))
 		// format json similar to `jq .`
 		out, err := json.MarshalIndent(obj, "", "  ")
 		require.NoError(t, err)
 		return out
 	}
-	withTestServer(func(ts *testServer) {
+	withTestServer(t, func(ts *testServer) {
 		inFile, err := os.Open("./fixture/otlp2jaeger-in.json")
 		require.NoError(t, err)
 
@@ -661,7 +617,7 @@ func TestTransformOTLPSuccess(t *testing.T) {
 }
 
 func TestTransformOTLPReadError(t *testing.T) {
-	withTestServer(func(ts *testServer) {
+	withTestServer(t, func(ts *testServer) {
 		bytesReader := &IoReaderMock{}
 		bytesReader.On("Read", mock.AnythingOfType("[]uint8")).Return(0, errors.New("Mocked error"))
 		_, err := ts.server.Client().Post(ts.server.URL+"/api/transform", "application/json", bytesReader)
@@ -670,8 +626,8 @@ func TestTransformOTLPReadError(t *testing.T) {
 }
 
 func TestTransformOTLPBadPayload(t *testing.T) {
-	withTestServer(func(ts *testServer) {
-		response := new(interface{})
+	withTestServer(t, func(ts *testServer) {
+		response := new(any)
 		request := "Bad Payload"
 		err := postJSON(ts.server.URL+"/api/transform", request, response)
 		require.ErrorContains(t, err, "cannot unmarshal OTLP")
@@ -683,8 +639,7 @@ func TestGetMetricsSuccess(t *testing.T) {
 	apiHandlerOptions := []HandlerOption{
 		HandlerOptions.MetricsQueryService(mr),
 	}
-	ts := initializeTestServer(apiHandlerOptions...)
-	defer ts.server.Close()
+	ts := initializeTestServer(t, apiHandlerOptions...)
 	expectedLabel := &metrics.Label{
 		Name:  "service_name",
 		Value: "emailservice",
@@ -766,18 +721,14 @@ func TestGetMetricsSuccess(t *testing.T) {
 
 func TestMetricsReaderError(t *testing.T) {
 	metricsReader := &metricsmocks.Reader{}
-	apiHandlerOptions := []HandlerOption{
-		HandlerOptions.MetricsQueryService(metricsReader),
-	}
-	ts := initializeTestServer(apiHandlerOptions...)
-	defer ts.server.Close()
+	ts := initializeTestServer(t, HandlerOptions.MetricsQueryService(metricsReader))
 
 	for _, tc := range []struct {
 		name                       string
 		urlPath                    string
 		mockedQueryMethod          string
 		mockedQueryMethodParamType string
-		mockedResponse             interface{}
+		mockedResponse             any
 		wantErrorMessage           string
 	}{
 		{
@@ -801,7 +752,7 @@ func TestMetricsReaderError(t *testing.T) {
 				tc.mockedQueryMethod,
 				mock.AnythingOfType("*context.valueCtx"),
 				mock.AnythingOfType(tc.mockedQueryMethodParamType),
-			).Return(tc.mockedResponse, fmt.Errorf(tc.wantErrorMessage)).Once()
+			).Return(tc.mockedResponse, errors.New(tc.wantErrorMessage)).Once()
 
 			// Test
 			var response metrics.MetricFamily
@@ -818,10 +769,7 @@ func TestMetricsQueryDisabled(t *testing.T) {
 	disabledReader, err := disabled.NewMetricsReader()
 	require.NoError(t, err)
 
-	apiHandlerOptions := []HandlerOption{
-		HandlerOptions.MetricsQueryService(disabledReader),
-	}
-	ts := initializeTestServer(apiHandlerOptions...)
+	ts := initializeTestServer(t, HandlerOptions.MetricsQueryService(disabledReader))
 	defer ts.server.Close()
 
 	for _, tc := range []struct {
@@ -842,7 +790,7 @@ func TestMetricsQueryDisabled(t *testing.T) {
 	} {
 		t.Run(tc.name, func(t *testing.T) {
 			// Test
-			var response interface{}
+			var response any
 			err := getJSON(ts.server.URL+tc.urlPath, &response)
 
 			// Verify
@@ -854,10 +802,7 @@ func TestMetricsQueryDisabled(t *testing.T) {
 
 func TestGetMinStep(t *testing.T) {
 	metricsReader := &metricsmocks.Reader{}
-	apiHandlerOptions := []HandlerOption{
-		HandlerOptions.MetricsQueryService(metricsReader),
-	}
-	ts := initializeTestServer(apiHandlerOptions...)
+	ts := initializeTestServer(t, HandlerOptions.MetricsQueryService(metricsReader))
 	defer ts.server.Close()
 	// Prepare
 	metricsReader.On(
@@ -872,15 +817,15 @@ func TestGetMinStep(t *testing.T) {
 
 	// Verify
 	require.NoError(t, err)
-	assert.Equal(t, float64(5), response.Data)
+	assert.InDelta(t, float64(5), response.Data, 0.01)
 }
 
 // getJSON fetches a JSON document from a server via HTTP GET
-func getJSON(url string, out interface{}) error {
+func getJSON(url string, out any) error {
 	return getJSONCustomHeaders(url, make(map[string]string), out)
 }
 
-func getJSONCustomHeaders(url string, additionalHeaders map[string]string, out interface{}) error {
+func getJSONCustomHeaders(url string, additionalHeaders map[string]string, out any) error {
 	req, err := http.NewRequest(http.MethodGet, url, nil)
 	if err != nil {
 		return err
@@ -889,7 +834,7 @@ func getJSONCustomHeaders(url string, additionalHeaders map[string]string, out i
 }
 
 // postJSON submits a JSON document to a server via HTTP POST and parses response as JSON.
-func postJSON(url string, req interface{}, out interface{}) error {
+func postJSON(url string, req any, out any) error {
 	buf := &bytes.Buffer{}
 	encoder := json.NewEncoder(buf)
 	if err := encoder.Encode(req); err != nil {
@@ -903,7 +848,7 @@ func postJSON(url string, req interface{}, out interface{}) error {
 }
 
 // execJSON executes an http request against a server and parses response as JSON
-func execJSON(req *http.Request, additionalHeaders map[string]string, out interface{}) error {
+func execJSON(req *http.Request, additionalHeaders map[string]string, out any) error {
 	req.Header.Add("Accept", "application/json")
 	for k, v := range additionalHeaders {
 		req.Header.Add(k, v)
@@ -946,10 +891,9 @@ func TestSearchTenancyHTTP(t *testing.T) {
 	tenancyOptions := tenancy.Options{
 		Enabled: true,
 	}
-	ts := initializeTestServerWithOptions(
+	ts := initializeTestServerWithOptions(t,
 		tenancy.NewManager(&tenancyOptions),
 		querysvc.QueryServiceOptions{})
-	defer ts.server.Close()
 	ts.spanReader.On("GetTrace", mock.AnythingOfType("*context.valueCtx"), mock.AnythingOfType("model.TraceID")).
 		Return(mockTrace, nil).Twice()
 
@@ -973,10 +917,7 @@ func TestSearchTenancyRejectionHTTP(t *testing.T) {
 	tenancyOptions := tenancy.Options{
 		Enabled: true,
 	}
-	ts := initializeTestServerWithOptions(
-		tenancy.NewManager(&tenancyOptions),
-		querysvc.QueryServiceOptions{})
-	defer ts.server.Close()
+	ts := initializeTestServerWithOptions(t, tenancy.NewManager(&tenancyOptions), querysvc.QueryServiceOptions{})
 	ts.spanReader.On("GetTrace", mock.AnythingOfType("*context.valueCtx"), mock.AnythingOfType("model.TraceID")).
 		Return(mockTrace, nil).Twice()
 
@@ -986,12 +927,14 @@ func TestSearchTenancyRejectionHTTP(t *testing.T) {
 	// We don't set tenant header
 	resp, err := httpClient.Do(req)
 	require.NoError(t, err)
+	defer resp.Body.Close()
 	assert.Equal(t, http.StatusUnauthorized, resp.StatusCode)
 
 	tm := tenancy.NewManager(&tenancyOptions)
 	req.Header.Set(tm.Header, "acme")
 	resp, err = http.DefaultClient.Do(req)
 	require.NoError(t, err)
+	defer resp.Body.Close()
 	require.Equal(t, http.StatusOK, resp.StatusCode)
 	// Skip unmarshal of response; it is enough that it succeeded
 }
@@ -1000,18 +943,15 @@ func TestSearchTenancyFlowTenantHTTP(t *testing.T) {
 	tenancyOptions := tenancy.Options{
 		Enabled: true,
 	}
-	ts := initializeTestServerWithOptions(
-		tenancy.NewManager(&tenancyOptions),
-		querysvc.QueryServiceOptions{})
-	defer ts.server.Close()
-	ts.spanReader.On("GetTrace", mock.MatchedBy(func(v interface{}) bool {
+	ts := initializeTestServerWithOptions(t, tenancy.NewManager(&tenancyOptions), querysvc.QueryServiceOptions{})
+	ts.spanReader.On("GetTrace", mock.MatchedBy(func(v any) bool {
 		ctx, ok := v.(context.Context)
 		if !ok || tenancy.GetTenant(ctx) != "acme" {
 			return false
 		}
 		return true
 	}), mock.AnythingOfType("model.TraceID")).Return(mockTrace, nil).Twice()
-	ts.spanReader.On("GetTrace", mock.MatchedBy(func(v interface{}) bool {
+	ts.spanReader.On("GetTrace", mock.MatchedBy(func(v any) bool {
 		ctx, ok := v.(context.Context)
 		if !ok || tenancy.GetTenant(ctx) != "megacorp" {
 			return false
