@@ -213,6 +213,7 @@ pacifyWatchdog (void* unused)
     return 0;
 }
 
+#if 0 /* UNUSED */
 /**
  * FPGA processor reset function
  *
@@ -227,6 +228,7 @@ static void processorReset(void* arg)
     EXT_DBG( "resetting processor\n");
     WRITE_REG(REG_OFFSET_REBOOT, REBOOT_PROC);
 }
+#endif
 
 /**
  * FPGA perform the shadow register write sequence
@@ -635,8 +637,8 @@ static int fpgaMmRealDevInit(const char* name)
         {
             fpgaMmDev.phys_regs_base = 0xFC800000;
             fpgaMmDev.regs_size = 0xD000;
-            fpgaMmDev.virt_regs_base = (unsigned long)ioremap_nocache(fpgaMmDev.phys_regs_base,
-                                                                      fpgaMmDev.regs_size);
+            fpgaMmDev.virt_regs_base = (unsigned long)ioremap(fpgaMmDev.phys_regs_base,
+                                                              fpgaMmDev.regs_size);
             if (fpgaMmDev.virt_regs_base != 0)
             {
                 return 0;
