@@ -1,5 +1,3 @@
-import React from 'react';
-
 import { DataSourceJsonData, DataSourcePluginOptionsEditorProps } from '@grafana/data';
 
 import { InlineSwitch } from '../../components/Switch/Switch';
@@ -19,7 +17,12 @@ export function AlertingSettings<T extends AlertingConfig>({ options, onOptionsC
       <div className="gf-form-group">
         <div className="gf-form-inline">
           <div className="gf-form">
-            <InlineField labelWidth={26} label="Manage alerts via Alerting UI" disabled={options.readOnly}>
+            <InlineField
+              labelWidth={29}
+              label="Manage alert rules in Alerting UI"
+              disabled={options.readOnly}
+              tooltip="Manage alert rules for this data source. To manage other alerting resources, add an Alertmanager data source."
+            >
               <InlineSwitch
                 value={options.jsonData.manageAlerts !== false}
                 onChange={(event) =>
