@@ -4,7 +4,6 @@ import (
 	"html/template"
 
 	"github.com/grafana/grafana/pkg/services/navtree"
-	"github.com/grafana/grafana/pkg/setting"
 )
 
 type IndexViewData struct {
@@ -19,7 +18,7 @@ type IndexViewData struct {
 	NavTree                             *navtree.NavTreeRoot
 	BuildVersion                        string
 	BuildCommit                         string
-	Theme                               string
+	ThemeType                           string
 	NewGrafanaVersionExists             bool
 	NewGrafanaVersion                   string
 	AppName                             string
@@ -27,9 +26,40 @@ type IndexViewData struct {
 	FavIcon                             template.URL
 	AppleTouchIcon                      template.URL
 	AppTitle                            string
-	Sentry                              *setting.Sentry
-	ContentDeliveryURL                  string
 	LoadingLogo                         template.URL
+	CSPContent                          string
+	CSPEnabled                          bool
+	IsDevelopmentEnv                    bool
 	// Nonce is a cryptographic identifier for use with Content Security Policy.
-	Nonce string
+	Nonce           string
+	NewsFeedEnabled bool
+	Assets          *EntryPointAssets // Includes CDN info
+}
+
+type EntryPointAssets struct {
+	ContentDeliveryURL string            `json:"cdn,omitempty"`
+	JSFiles            []EntryPointAsset `json:"jsFiles"`
+	Dark               string            `json:"dark"`
+	Light              string            `json:"light"`
+	Swagger            []EntryPointAsset `json:"swagger"`
+}
+
+type EntryPointAsset struct {
+	FilePath  string `json:"filePath"`
+	Integrity string `json:"integrity"`
+}
+
+func (a *EntryPointAssets) SetContentDeliveryURL(prefix string) {
+	if prefix == "" {
+		return
+	}
+	a.ContentDeliveryURL = prefix
+	a.Dark = prefix + a.Dark
+	a.Light = prefix + a.Light
+	for i, p := range a.JSFiles {
+		a.JSFiles[i].FilePath = prefix + p.FilePath
+	}
+	for i, p := range a.Swagger {
+		a.Swagger[i].FilePath = prefix + p.FilePath
+	}
 }
