@@ -10,13 +10,6 @@ import (
 	"testing"
 	"time"
 
-	"github.com/grafana/grafana/pkg/services/accesscontrol"
-	"github.com/grafana/grafana/pkg/services/notifications"
-	"github.com/grafana/grafana/pkg/services/secrets/fakes"
-	tempuser "github.com/grafana/grafana/pkg/services/temp_user"
-	"github.com/grafana/grafana/pkg/services/temp_user/tempuserimpl"
-	"github.com/grafana/grafana/pkg/web/webtest"
-
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 	"golang.org/x/oauth2"
@@ -27,35 +20,48 @@ import (
 	"github.com/grafana/grafana/pkg/components/simplejson"
 	"github.com/grafana/grafana/pkg/infra/db"
 	"github.com/grafana/grafana/pkg/infra/db/dbtest"
-	"github.com/grafana/grafana/pkg/infra/usagestats"
+	"github.com/grafana/grafana/pkg/infra/remotecache"
+	"github.com/grafana/grafana/pkg/infra/tracing"
+	"github.com/grafana/grafana/pkg/login/social"
+	"github.com/grafana/grafana/pkg/login/social/socialtest"
+	"github.com/grafana/grafana/pkg/services/accesscontrol"
+	"github.com/grafana/grafana/pkg/services/accesscontrol/acimpl"
 	acmock "github.com/grafana/grafana/pkg/services/accesscontrol/mock"
+	"github.com/grafana/grafana/pkg/services/auth/idtest"
+	"github.com/grafana/grafana/pkg/services/authz/zanzana"
 	contextmodel "github.com/grafana/grafana/pkg/services/contexthandler/model"
+	"github.com/grafana/grafana/pkg/services/featuremgmt"
 	"github.com/grafana/grafana/pkg/services/login"
-	"github.com/grafana/grafana/pkg/services/login/authinfoservice"
-	authinfostore "github.com/grafana/grafana/pkg/services/login/authinfoservice/database"
-	"github.com/grafana/grafana/pkg/services/login/logintest"
+	"github.com/grafana/grafana/pkg/services/login/authinfoimpl"
+	"github.com/grafana/grafana/pkg/services/login/authinfotest"
+	"github.com/grafana/grafana/pkg/services/notifications"
 	"github.com/grafana/grafana/pkg/services/org/orgimpl"
 	"github.com/grafana/grafana/pkg/services/quota/quotatest"
 	"github.com/grafana/grafana/pkg/services/searchusers"
 	"github.com/grafana/grafana/pkg/services/searchusers/filters"
 	"github.com/grafana/grafana/pkg/services/secrets/database"
+	"github.com/grafana/grafana/pkg/services/secrets/fakes"
 	secretsManager "github.com/grafana/grafana/pkg/services/secrets/manager"
+	"github.com/grafana/grafana/pkg/services/sqlstore"
 	"github.com/grafana/grafana/pkg/services/supportbundles/supportbundlestest"
+	tempuser "github.com/grafana/grafana/pkg/services/temp_user"
+	"github.com/grafana/grafana/pkg/services/temp_user/tempuserimpl"
 	"github.com/grafana/grafana/pkg/services/user"
 	"github.com/grafana/grafana/pkg/services/user/userimpl"
 	"github.com/grafana/grafana/pkg/services/user/usertest"
 	"github.com/grafana/grafana/pkg/setting"
+	"github.com/grafana/grafana/pkg/web/webtest"
 )
 
-const newEmail = "newEmail@localhost"
+const newEmail = "newemail@localhost"
 
 func TestUserAPIEndpoint_userLoggedIn(t *testing.T) {
 	settings := setting.NewCfg()
-	sqlStore := db.InitTestDB(t)
+	sqlStore := db.InitTestDB(t, sqlstore.InitTestDBOpt{Cfg: settings})
 	hs := &HTTPServer{
 		Cfg:           settings,
 		SQLStore:      sqlStore,
-		AccessControl: acmock.New(),
+		AccessControl: acimpl.ProvideAccessControl(featuremgmt.WithFeatures(), zanzana.NewNoopClient()),
 	}
 
 	mockResult := user.SearchUserQueryResult{
@@ -67,19 +73,20 @@ func TestUserAPIEndpoint_userLoggedIn(t *testing.T) {
 	}
 	mock := dbtest.NewFakeDB()
 	userMock := usertest.NewUserServiceFake()
+
 	loggedInUserScenario(t, "When calling GET on", "api/users/1", "api/users/:id", func(sc *scenarioContext) {
 		fakeNow := time.Date(2019, 2, 11, 17, 30, 40, 0, time.UTC)
 		secretsService := secretsManager.SetupTestService(t, database.ProvideSecretsStore(sqlStore))
-		authInfoStore := authinfostore.ProvideAuthInfoStore(sqlStore, secretsService, userMock)
-		srv := authinfoservice.ProvideAuthInfoService(
-			&authinfoservice.OSSUserProtectionImpl{},
-			authInfoStore,
-			&usagestats.UsageStatsMock{},
-		)
+		authInfoStore := authinfoimpl.ProvideStore(sqlStore, secretsService)
+		srv := authinfoimpl.ProvideService(
+			authInfoStore, remotecache.NewFakeCacheStorage(), secretsService)
 		hs.authInfoService = srv
-		orgSvc, err := orgimpl.ProvideService(sqlStore, sqlStore.Cfg, quotatest.New(false, nil))
+		orgSvc, err := orgimpl.ProvideService(sqlStore, settings, quotatest.New(false, nil))
 		require.NoError(t, err)
-		userSvc, err := userimpl.ProvideService(sqlStore, orgSvc, sc.cfg, nil, nil, quotatest.New(false, nil), supportbundlestest.NewFakeBundleService())
+		userSvc, err := userimpl.ProvideService(
+			sqlStore, orgSvc, sc.cfg, nil, nil, tracing.InitializeTracerForTest(),
+			quotatest.New(false, nil), supportbundlestest.NewFakeBundleService(),
+		)
 		require.NoError(t, err)
 		hs.userService = userSvc
 
@@ -91,6 +98,7 @@ func TestUserAPIEndpoint_userLoggedIn(t *testing.T) {
 		}
 		usr, err := userSvc.Create(context.Background(), &createUserCmd)
 		require.NoError(t, err)
+		theUserUID := usr.UID
 
 		sc.handlerFunc = hs.GetUserByID
 
@@ -101,7 +109,7 @@ func TestUserAPIEndpoint_userLoggedIn(t *testing.T) {
 			TokenType:    "Bearer",
 		}
 		idToken := "testidtoken"
-		token = token.WithExtra(map[string]interface{}{"id_token": idToken})
+		token = token.WithExtra(map[string]any{"id_token": idToken})
 		userlogin := "loginuser"
 		query := &login.GetUserByAuthInfoQuery{AuthModule: "test", AuthId: "test", UserLookupParams: login.UserLookupParams{Login: &userlogin}}
 		cmd := &login.UpdateAuthInfoCommand{
@@ -112,11 +120,12 @@ func TestUserAPIEndpoint_userLoggedIn(t *testing.T) {
 		}
 		err = srv.UpdateAuthInfo(context.Background(), cmd)
 		require.NoError(t, err)
-		avatarUrl := dtos.GetGravatarUrl("@test.com")
+		avatarUrl := dtos.GetGravatarUrl(hs.Cfg, "@test.com")
 		sc.fakeReqWithParams("GET", sc.url, map[string]string{"id": fmt.Sprintf("%v", usr.ID)}).exec()
 
 		expected := user.UserProfileDTO{
 			ID:             1,
+			UID:            theUserUID, // from original request
 			Email:          "user@test.com",
 			Name:           "user",
 			Login:          "loginuser",
@@ -145,9 +154,12 @@ func TestUserAPIEndpoint_userLoggedIn(t *testing.T) {
 			Login:   "admin",
 			IsAdmin: true,
 		}
-		orgSvc, err := orgimpl.ProvideService(sqlStore, sqlStore.Cfg, quotatest.New(false, nil))
+		orgSvc, err := orgimpl.ProvideService(sqlStore, sc.cfg, quotatest.New(false, nil))
 		require.NoError(t, err)
-		userSvc, err := userimpl.ProvideService(sqlStore, orgSvc, sc.cfg, nil, nil, quotatest.New(false, nil), supportbundlestest.NewFakeBundleService())
+		userSvc, err := userimpl.ProvideService(
+			sqlStore, orgSvc, sc.cfg, nil, nil, tracing.InitializeTracerForTest(),
+			quotatest.New(false, nil), supportbundlestest.NewFakeBundleService(),
+		)
 		require.NoError(t, err)
 		_, err = userSvc.Create(context.Background(), &createUserCmd)
 		require.Nil(t, err)
@@ -169,7 +181,7 @@ func TestUserAPIEndpoint_userLoggedIn(t *testing.T) {
 	loggedInUserScenario(t, "When calling GET on", "/api/users", "/api/users", func(sc *scenarioContext) {
 		userMock.ExpectedSearchUsers = mockResult
 
-		searchUsersService := searchusers.ProvideUsersService(filters.ProvideOSSSearchUserFilter(), userMock)
+		searchUsersService := searchusers.ProvideUsersService(sc.cfg, filters.ProvideOSSSearchUserFilter(), userMock)
 		sc.handlerFunc = searchUsersService.SearchUsers
 		sc.fakeReqWithParams("GET", sc.url, map[string]string{}).exec()
 
@@ -182,7 +194,7 @@ func TestUserAPIEndpoint_userLoggedIn(t *testing.T) {
 	loggedInUserScenario(t, "When calling GET with page and limit querystring parameters on", "/api/users", "/api/users", func(sc *scenarioContext) {
 		userMock.ExpectedSearchUsers = mockResult
 
-		searchUsersService := searchusers.ProvideUsersService(filters.ProvideOSSSearchUserFilter(), userMock)
+		searchUsersService := searchusers.ProvideUsersService(sc.cfg, filters.ProvideOSSSearchUserFilter(), userMock)
 		sc.handlerFunc = searchUsersService.SearchUsers
 		sc.fakeReqWithParams("GET", sc.url, map[string]string{"perpage": "10", "page": "2"}).exec()
 
@@ -195,7 +207,7 @@ func TestUserAPIEndpoint_userLoggedIn(t *testing.T) {
 	loggedInUserScenario(t, "When calling GET on", "/api/users/search", "/api/users/search", func(sc *scenarioContext) {
 		userMock.ExpectedSearchUsers = mockResult
 
-		searchUsersService := searchusers.ProvideUsersService(filters.ProvideOSSSearchUserFilter(), userMock)
+		searchUsersService := searchusers.ProvideUsersService(sc.cfg, filters.ProvideOSSSearchUserFilter(), userMock)
 		sc.handlerFunc = searchUsersService.SearchUsersWithPaging
 		sc.fakeReqWithParams("GET", sc.url, map[string]string{}).exec()
 
@@ -211,7 +223,7 @@ func TestUserAPIEndpoint_userLoggedIn(t *testing.T) {
 	loggedInUserScenario(t, "When calling GET with page and perpage querystring parameters on", "/api/users/search", "/api/users/search", func(sc *scenarioContext) {
 		userMock.ExpectedSearchUsers = mockResult
 
-		searchUsersService := searchusers.ProvideUsersService(filters.ProvideOSSSearchUserFilter(), userMock)
+		searchUsersService := searchusers.ProvideUsersService(sc.cfg, filters.ProvideOSSSearchUserFilter(), userMock)
 		sc.handlerFunc = searchUsersService.SearchUsersWithPaging
 		sc.fakeReqWithParams("GET", sc.url, map[string]string{"perpage": "10", "page": "2"}).exec()
 
@@ -223,14 +235,133 @@ func TestUserAPIEndpoint_userLoggedIn(t *testing.T) {
 	}, mock)
 }
 
+func Test_GetUserByID(t *testing.T) {
+	testcases := []struct {
+		name                         string
+		authModule                   string
+		allowAssignGrafanaAdmin      bool
+		authEnabled                  bool
+		skipOrgRoleSync              bool
+		expectedIsGrafanaAdminSynced bool
+	}{
+		{
+			name:                         "Should return IsGrafanaAdminExternallySynced = false for an externally synced OAuth user if Grafana Admin role is not synced",
+			authModule:                   login.GenericOAuthModule,
+			authEnabled:                  true,
+			allowAssignGrafanaAdmin:      false,
+			skipOrgRoleSync:              false,
+			expectedIsGrafanaAdminSynced: false,
+		},
+		{
+			name:                         "Should return IsGrafanaAdminExternallySynced = false for an externally synced OAuth user if OAuth provider is not enabled",
+			authModule:                   login.GenericOAuthModule,
+			authEnabled:                  false,
+			allowAssignGrafanaAdmin:      true,
+			skipOrgRoleSync:              false,
+			expectedIsGrafanaAdminSynced: false,
+		},
+		{
+			name:                         "Should return IsGrafanaAdminExternallySynced = false for an externally synced OAuth user if org roles are not being synced",
+			authModule:                   login.GenericOAuthModule,
+			authEnabled:                  true,
+			allowAssignGrafanaAdmin:      true,
+			skipOrgRoleSync:              true,
+			expectedIsGrafanaAdminSynced: false,
+		},
+		{
+			name:                         "Should return IsGrafanaAdminExternallySynced = true for an externally synced OAuth user",
+			authModule:                   login.GenericOAuthModule,
+			authEnabled:                  true,
+			allowAssignGrafanaAdmin:      true,
+			skipOrgRoleSync:              false,
+			expectedIsGrafanaAdminSynced: true,
+		},
+		{
+			name:                         "Should return IsGrafanaAdminExternallySynced = false for an externally synced JWT user if Grafana Admin role is not synced",
+			authModule:                   login.JWTModule,
+			authEnabled:                  true,
+			allowAssignGrafanaAdmin:      false,
+			skipOrgRoleSync:              false,
+			expectedIsGrafanaAdminSynced: false,
+		},
+		{
+			name:                         "Should return IsGrafanaAdminExternallySynced = false for an externally synced JWT user if JWT provider is not enabled",
+			authModule:                   login.JWTModule,
+			authEnabled:                  false,
+			allowAssignGrafanaAdmin:      true,
+			skipOrgRoleSync:              false,
+			expectedIsGrafanaAdminSynced: false,
+		},
+		{
+			name:                         "Should return IsGrafanaAdminExternallySynced = false for an externally synced JWT user if org roles are not being synced",
+			authModule:                   login.JWTModule,
+			authEnabled:                  true,
+			allowAssignGrafanaAdmin:      true,
+			skipOrgRoleSync:              true,
+			expectedIsGrafanaAdminSynced: false,
+		},
+		{
+			name:                         "Should return IsGrafanaAdminExternallySynced = true for an externally synced JWT user",
+			authModule:                   login.JWTModule,
+			authEnabled:                  true,
+			allowAssignGrafanaAdmin:      true,
+			skipOrgRoleSync:              false,
+			expectedIsGrafanaAdminSynced: true,
+		},
+	}
+	for _, tc := range testcases {
+		t.Run(tc.name, func(t *testing.T) {
+			userAuth := &login.UserAuth{AuthModule: tc.authModule}
+			authInfoService := &authinfotest.FakeService{ExpectedUserAuth: userAuth}
+			socialService := &socialtest.FakeSocialService{}
+			userService := &usertest.FakeUserService{ExpectedUserProfileDTO: &user.UserProfileDTO{}}
+			cfg := setting.NewCfg()
+
+			switch tc.authModule {
+			case login.GenericOAuthModule:
+				socialService.ExpectedAuthInfoProvider = &social.OAuthInfo{AllowAssignGrafanaAdmin: tc.allowAssignGrafanaAdmin, Enabled: tc.authEnabled, SkipOrgRoleSync: tc.skipOrgRoleSync}
+			case login.JWTModule:
+				cfg.JWTAuth.Enabled = tc.authEnabled
+				cfg.JWTAuth.SkipOrgRoleSync = tc.skipOrgRoleSync
+				cfg.JWTAuth.AllowAssignGrafanaAdmin = tc.allowAssignGrafanaAdmin
+			}
+
+			hs := &HTTPServer{
+				Cfg:             cfg,
+				authInfoService: authInfoService,
+				SocialService:   socialService,
+				userService:     userService,
+			}
+
+			sc := setupScenarioContext(t, "/api/users/1")
+			sc.defaultHandler = routing.Wrap(func(c *contextmodel.ReqContext) response.Response {
+				sc.context = c
+				return hs.GetUserByID(c)
+			})
+
+			sc.m.Get("/api/users/:id", sc.defaultHandler)
+			sc.fakeReqWithParams("GET", sc.url, map[string]string{}).exec()
+
+			var resp user.UserProfileDTO
+			require.Equal(t, http.StatusOK, sc.resp.Code)
+			err := json.Unmarshal(sc.resp.Body.Bytes(), &resp)
+			require.NoError(t, err)
+
+			assert.Equal(t, tc.expectedIsGrafanaAdminSynced, resp.IsGrafanaAdminExternallySynced)
+		})
+	}
+}
+
 func TestHTTPServer_UpdateUser(t *testing.T) {
 	settings := setting.NewCfg()
+	settings.SAMLAuthEnabled = true
 	sqlStore := db.InitTestDB(t)
 
 	hs := &HTTPServer{
 		Cfg:           settings,
 		SQLStore:      sqlStore,
 		AccessControl: acmock.New(),
+		SocialService: &socialtest.FakeSocialService{ExpectedAuthInfoProvider: &social.OAuthInfo{Enabled: true}},
 	}
 
 	updateUserCommand := user.UpdateUserCommand{
@@ -246,14 +377,166 @@ func TestHTTPServer_UpdateUser(t *testing.T) {
 		routePattern: "/api/users/:id",
 		cmd:          updateUserCommand,
 		fn: func(sc *scenarioContext) {
-			sc.authInfoService.ExpectedUserAuth = &login.UserAuth{}
+			sc.authInfoService.ExpectedUserAuth = &login.UserAuth{AuthModule: login.SAMLAuthModule}
+
 			sc.fakeReqWithParams("PUT", sc.url, map[string]string{"id": "1"}).exec()
 			assert.Equal(t, 403, sc.resp.Code)
 		},
 	}, hs)
 }
 
+func setupUpdateEmailTests(t *testing.T, cfg *setting.Cfg) (*user.User, *HTTPServer, *notifications.NotificationServiceMock) {
+	t.Helper()
+
+	sqlStore := db.InitTestDB(t, sqlstore.InitTestDBOpt{Cfg: cfg})
+
+	tempUserService := tempuserimpl.ProvideService(sqlStore, cfg)
+	orgSvc, err := orgimpl.ProvideService(sqlStore, cfg, quotatest.New(false, nil))
+	require.NoError(t, err)
+	userSvc, err := userimpl.ProvideService(
+		sqlStore, orgSvc, cfg, nil, nil, tracing.InitializeTracerForTest(),
+		quotatest.New(false, nil), supportbundlestest.NewFakeBundleService(),
+	)
+	require.NoError(t, err)
+
+	// Create test user
+	createUserCmd := user.CreateUserCommand{
+		Email:   "testuser@localhost",
+		Name:    "testuser",
+		Login:   "loginuser",
+		Company: "testCompany",
+		IsAdmin: true,
+	}
+	usr, err := userSvc.Create(context.Background(), &createUserCmd)
+	require.NoError(t, err)
+
+	nsMock := notifications.MockNotificationService()
+	verifier := userimpl.ProvideVerifier(cfg, userSvc, tempUserService, nsMock, &idtest.MockService{})
+
+	hs := &HTTPServer{
+		Cfg:                 cfg,
+		SQLStore:            sqlStore,
+		userService:         userSvc,
+		tempUserService:     tempUserService,
+		NotificationService: nsMock,
+		userVerifier:        verifier,
+	}
+	return usr, hs, nsMock
+}
+
 func TestUser_UpdateEmail(t *testing.T) {
+	cases := []struct {
+		Name  string
+		Field user.UpdateEmailActionType
+	}{
+		{
+			Name:  "Updating Email field",
+			Field: user.EmailUpdateAction,
+		},
+		{
+			Name:  "Updating Login (username) field",
+			Field: user.LoginUpdateAction,
+		},
+	}
+
+	for _, tt := range cases {
+		t.Run(tt.Name, func(t *testing.T) {
+			t.Run("With verification disabled should update without verifying", func(t *testing.T) {
+				tests := []struct {
+					name               string
+					smtpConfigured     bool
+					verifyEmailEnabled bool
+				}{
+					{
+						name:               "SMTP not configured",
+						smtpConfigured:     false,
+						verifyEmailEnabled: true,
+					},
+					{
+						name:               "config verify_email_enabled = false",
+						smtpConfigured:     true,
+						verifyEmailEnabled: false,
+					},
+					{
+						name:               "config verify_email_enabled = false and SMTP not configured",
+						smtpConfigured:     false,
+						verifyEmailEnabled: false,
+					},
+				}
+				for _, ttt := range tests {
+					settings := setting.NewCfg()
+					settings.Smtp.Enabled = ttt.smtpConfigured
+					settings.VerifyEmailEnabled = ttt.verifyEmailEnabled
+
+					usr, hs, nsMock := setupUpdateEmailTests(t, settings)
+
+					updateUserCommand := user.UpdateUserCommand{
+						Email:  usr.Email,
+						Name:   "newName",
+						Login:  usr.Login,
+						UserID: usr.ID,
+					}
+
+					switch tt.Field {
+					case user.LoginUpdateAction:
+						updateUserCommand.Login = newEmail
+					case user.EmailUpdateAction:
+						updateUserCommand.Email = newEmail
+					}
+
+					fn := func(sc *scenarioContext) {
+						// User is internal
+						sc.authInfoService.ExpectedError = user.ErrUserNotFound
+
+						sc.fakeReqWithParams("PUT", sc.url, nil).exec()
+						assert.Equal(t, http.StatusOK, sc.resp.Code)
+
+						// Verify that no email has been sent after update
+						require.False(t, nsMock.EmailVerified)
+
+						userQuery := user.GetUserByIDQuery{ID: usr.ID}
+						updatedUsr, err := hs.userService.GetByID(context.Background(), &userQuery)
+						require.NoError(t, err)
+
+						// Verify fields have been updated
+						require.NotEqual(t, usr.Name, updatedUsr.Name)
+						require.Equal(t, updateUserCommand.Name, updatedUsr.Name)
+
+						switch tt.Field {
+						case user.LoginUpdateAction:
+							require.Equal(t, usr.Email, updatedUsr.Email)
+							require.NotEqual(t, usr.Login, updatedUsr.Login)
+							require.Equal(t, updateUserCommand.Login, updatedUsr.Login)
+						case user.EmailUpdateAction:
+							require.Equal(t, usr.Login, updatedUsr.Login)
+							require.NotEqual(t, usr.Email, updatedUsr.Email)
+							require.Equal(t, updateUserCommand.Email, updatedUsr.Email)
+						}
+
+						// Verify other fields have been kept
+						require.Equal(t, usr.Company, updatedUsr.Company)
+					}
+
+					updateUserScenario(t, updateUserContext{
+						desc:         ttt.name,
+						url:          fmt.Sprintf("/api/users/%d", usr.ID),
+						routePattern: "/api/users/:id",
+						cmd:          updateUserCommand,
+						fn:           fn,
+					}, hs)
+
+					updateSignedInUserScenario(t, updateUserContext{
+						desc:         ttt.name,
+						url:          "/api/user",
+						routePattern: "/api/user",
+						cmd:          updateUserCommand,
+						fn:           fn,
+					}, hs)
+				}
+			})
+		})
+	}
+
 	doReq := func(req *http.Request, usr *user.User) (*http.Response, error) {
 		r := webtest.RequestWithSignedInUser(
 			req,
@@ -324,23 +607,25 @@ func TestUser_UpdateEmail(t *testing.T) {
 	}
 
 	setupScenario := func(cfg *setting.Cfg) (*webtest.Server, user.Service, tempuser.Service, *notifications.NotificationServiceMock) {
-		setting.VerifyEmailEnabled = true
 		settings := setting.NewCfg()
 		settings.Smtp.Enabled = true
 		settings.VerificationEmailMaxLifetime = 1 * time.Hour
+		settings.VerifyEmailEnabled = true
 
 		if cfg != nil {
 			settings = cfg
 		}
 
 		nsMock := notifications.MockNotificationService()
-		sqlStore := db.InitTestDB(t)
-		sqlStore.Cfg = settings
+		sqlStore := db.InitTestDB(t, sqlstore.InitTestDBOpt{Cfg: settings})
 
-		tempUserSvc := tempuserimpl.ProvideService(sqlStore)
+		tempUserSvc := tempuserimpl.ProvideService(sqlStore, settings)
 		orgSvc, err := orgimpl.ProvideService(sqlStore, settings, quotatest.New(false, nil))
 		require.NoError(t, err)
-		userSvc, err := userimpl.ProvideService(sqlStore, orgSvc, settings, nil, nil, quotatest.New(false, nil), supportbundlestest.NewFakeBundleService())
+		userSvc, err := userimpl.ProvideService(
+			sqlStore, orgSvc, settings, nil, nil, tracing.InitializeTracerForTest(),
+			quotatest.New(false, nil), supportbundlestest.NewFakeBundleService(),
+		)
 		require.NoError(t, err)
 
 		server := SetupAPITestServer(t, func(hs *HTTPServer) {
@@ -351,8 +636,9 @@ func TestUser_UpdateEmail(t *testing.T) {
 			hs.tempUserService = tempUserSvc
 			hs.NotificationService = nsMock
 			hs.SecretsService = fakes.NewFakeSecretsService()
+			hs.userVerifier = userimpl.ProvideVerifier(settings, userSvc, tempUserSvc, nsMock, &idtest.MockService{})
 			// User is internal
-			hs.authInfoService = &logintest.AuthInfoServiceFake{ExpectedError: user.ErrUserNotFound}
+			hs.authInfoService = &authinfotest.FakeService{ExpectedError: user.ErrUserNotFound}
 		})
 
 		return server, userSvc, tempUserSvc, nsMock
@@ -371,99 +657,6 @@ func TestUser_UpdateEmail(t *testing.T) {
 		return usr
 	}
 
-	disabledCases := []struct {
-		Name  string
-		Field user.UpdateEmailActionType
-	}{
-		{
-			Name:  "Updating Email field",
-			Field: user.EmailUpdateAction,
-		},
-		{
-			Name:  "Updating Login (username) field",
-			Field: user.LoginUpdateAction,
-		},
-	}
-
-	for _, tt := range disabledCases {
-		t.Run(tt.Name, func(t *testing.T) {
-			t.Run("With verification disabled should update without verifying", func(t *testing.T) {
-				tests := []struct {
-					name               string
-					smtpConfigured     bool
-					verifyEmailEnabled bool
-				}{
-					{
-						name:               "SMTP not configured",
-						smtpConfigured:     false,
-						verifyEmailEnabled: true,
-					},
-					{
-						name:               "config verify_email_enabled = false",
-						smtpConfigured:     true,
-						verifyEmailEnabled: false,
-					},
-					{
-						name:               "config verify_email_enabled = false and SMTP not configured",
-						smtpConfigured:     false,
-						verifyEmailEnabled: false,
-					},
-				}
-				for _, ttt := range tests {
-					var body string
-					newName := "newName"
-
-					settings := setting.NewCfg()
-					settings.Smtp.Enabled = ttt.smtpConfigured
-					server, userSvc, _, nsMock := setupScenario(settings)
-
-					// Override after calling setupScenario()
-					setting.VerifyEmailEnabled = ttt.verifyEmailEnabled
-
-					originalUsr := createUser(userSvc, "name", "email@localhost", "login")
-
-					// Verify that no email has been sent yet
-					require.False(t, nsMock.EmailVerified)
-
-					// Start email update
-					switch tt.Field {
-					case user.LoginUpdateAction:
-						body = fmt.Sprintf(`{"login": "%s", "name": "%s"}`, newEmail, newName)
-					case user.EmailUpdateAction:
-						body = fmt.Sprintf(`{"email": "%s", "login": "%s", "name": "%s"}`, newEmail, originalUsr.Login, newName)
-					}
-					sendUpdateReq(server, originalUsr, body)
-
-					// Verify that email has not been sent
-					require.False(t, nsMock.EmailVerified)
-
-					// Verify Email has been updated
-					userQuery := user.GetUserByIDQuery{ID: originalUsr.ID}
-					updatedUsr, err := userSvc.GetByID(context.Background(), &userQuery)
-					require.NoError(t, err)
-
-					switch tt.Field {
-					case user.LoginUpdateAction:
-						require.Equal(t, originalUsr.Email, updatedUsr.Email)
-						require.NotEqual(t, originalUsr.Login, updatedUsr.Login)
-						require.Equal(t, newEmail, updatedUsr.Login)
-					case user.EmailUpdateAction:
-						require.Equal(t, originalUsr.Login, updatedUsr.Login)
-						require.NotEqual(t, originalUsr.Email, updatedUsr.Email)
-						require.Equal(t, newEmail, updatedUsr.Email)
-					}
-
-					// Verify name has been updated
-					require.NotEqual(t, originalUsr.Name, updatedUsr.Name)
-					require.Equal(t, newName, updatedUsr.Name)
-
-					// Fields unchanged
-					require.Equal(t, originalUsr.Company, updatedUsr.Company)
-				}
-			})
-		})
-	}
-
 	t.Run("Update Email and disregard other fields", func(t *testing.T) {
 		server, userSvc, tempUserSvc, nsMock := setupScenario(nil)
 
@@ -473,7 +666,7 @@ func TestUser_UpdateEmail(t *testing.T) {
 		require.False(t, nsMock.EmailVerified)
 
 		// Start email update
-		newName := "newName"
+		newName := "newname"
 		body := fmt.Sprintf(`{"email": "%s", "name": "%s"}`, newEmail, newName)
 		sendUpdateReq(server, originalUsr, body)
 
@@ -575,8 +768,8 @@ func TestUser_UpdateEmail(t *testing.T) {
 		require.False(t, nsMock.EmailVerified)
 
 		// Start email update
-		newLogin := "newLogin"
-		newName := "newName"
+		newLogin := "newlogin"
+		newName := "newname"
 		body := fmt.Sprintf(`{"login": "%s", "name": "%s"}`, newLogin, newName)
 		sendUpdateReq(server, originalUsr, body)
 
@@ -628,7 +821,7 @@ func TestUser_UpdateEmail(t *testing.T) {
 		require.False(t, nsMock.EmailVerified)
 
 		// Start email update
-		newLogin := "newEmail2@localhost"
+		newLogin := "newemail2@localhost"
 		body := fmt.Sprintf(`{"email": "%s", "login": "%s"}`, newEmail, newLogin)
 		sendUpdateReq(server, originalUsr, body)
 
@@ -663,7 +856,7 @@ func TestUser_UpdateEmail(t *testing.T) {
 		require.False(t, nsMock.EmailVerified)
 
 		// Start email update
-		newLogin := "newEmail2@localhost"
+		newLogin := "newemail2@localhost"
 		body := fmt.Sprintf(`{"email": "%s", "login": "%s"}`, newEmail, newLogin)
 		sendUpdateReq(server, originalUsr, body)
 
@@ -694,6 +887,7 @@ func TestUser_UpdateEmail(t *testing.T) {
 		cfg := setting.NewCfg()
 		cfg.Smtp.Enabled = true
 		cfg.VerificationEmailMaxLifetime = 0 // Expire instantly
+		cfg.VerifyEmailEnabled = true
 
 		server, userSvc, tempUserSvc, nsMock := setupScenario(cfg)
 
@@ -734,14 +928,14 @@ func TestUser_UpdateEmail(t *testing.T) {
 		require.False(t, nsMock.EmailVerified)
 
 		// First email verification
-		firstNewEmail := "newEmail1@localhost"
+		firstNewEmail := "newemail1@localhost"
 		body := fmt.Sprintf(`{"email": "%s"}`, firstNewEmail)
 		sendUpdateReq(server, originalUsr, body)
 		verifyEmailData(tempUserSvc, nsMock, originalUsr, firstNewEmail)
 		firstCode := nsMock.EmailVerification.Code
 
 		// Second email verification
-		secondNewEmail := "newEmail2@localhost"
+		secondNewEmail := "newemail2@localhost"
 		body = fmt.Sprintf(`{"email": "%s"}`, secondNewEmail)
 		sendUpdateReq(server, originalUsr, body)
 		verifyEmailData(tempUserSvc, nsMock, originalUsr, secondNewEmail)
@@ -886,7 +1080,7 @@ func updateUserScenario(t *testing.T, ctx updateUserContext, hs *HTTPServer) {
 	t.Run(fmt.Sprintf("%s %s", ctx.desc, ctx.url), func(t *testing.T) {
 		sc := setupScenarioContext(t, ctx.url)
 
-		sc.authInfoService = &logintest.AuthInfoServiceFake{}
+		sc.authInfoService = &authinfotest.FakeService{}
 		hs.authInfoService = sc.authInfoService
 
 		sc.defaultHandler = routing.Wrap(func(c *contextmodel.ReqContext) response.Response {
@@ -908,11 +1102,13 @@ func updateUserScenario(t *testing.T, ctx updateUserContext, hs *HTTPServer) {
 func TestHTTPServer_UpdateSignedInUser(t *testing.T) {
 	settings := setting.NewCfg()
 	sqlStore := db.InitTestDB(t)
+	settings.SAMLAuthEnabled = true
 
 	hs := &HTTPServer{
 		Cfg:           settings,
 		SQLStore:      sqlStore,
 		AccessControl: acmock.New(),
+		SocialService: &socialtest.FakeSocialService{},
 	}
 
 	updateUserCommand := user.UpdateUserCommand{
@@ -928,7 +1124,7 @@ func TestHTTPServer_UpdateSignedInUser(t *testing.T) {
 		routePattern: "/api/users/",
 		cmd:          updateUserCommand,
 		fn: func(sc *scenarioContext) {
-			sc.authInfoService.ExpectedUserAuth = &login.UserAuth{}
+			sc.authInfoService.ExpectedUserAuth = &login.UserAuth{AuthModule: login.SAMLAuthModule}
 			sc.fakeReqWithParams("PUT", sc.url, map[string]string{"id": "1"}).exec()
 			assert.Equal(t, 403, sc.resp.Code)
 		},
@@ -939,7 +1135,7 @@ func updateSignedInUserScenario(t *testing.T, ctx updateUserContext, hs *HTTPSer
 	t.Run(fmt.Sprintf("%s %s", ctx.desc, ctx.url), func(t *testing.T) {
 		sc := setupScenarioContext(t, ctx.url)
 
-		sc.authInfoService = &logintest.AuthInfoServiceFake{}
+		sc.authInfoService = &authinfotest.FakeService{}
 		hs.authInfoService = sc.authInfoService
 
 		sc.defaultHandler = routing.Wrap(func(c *contextmodel.ReqContext) response.Response {
