@@ -22,6 +22,8 @@ import (
 	"github.com/grafana/grafana/pkg/infra/tracing"
 	"github.com/grafana/grafana/pkg/services/accesscontrol"
 	"github.com/grafana/grafana/pkg/services/accesscontrol/acimpl"
+	"github.com/grafana/grafana/pkg/services/accesscontrol/permreg"
+	"github.com/grafana/grafana/pkg/services/authz/zanzana"
 	"github.com/grafana/grafana/pkg/services/featuremgmt"
 	"github.com/grafana/grafana/pkg/services/quota/quotaimpl"
 	"github.com/grafana/grafana/pkg/services/sqlstore"
@@ -33,7 +35,7 @@ import (
 	"github.com/grafana/grafana/pkg/setting"
 )
 
-func initConflictCfg(cmd *utils.ContextCommandLine) (*setting.Cfg, error) {
+func initConflictCfg(cmd *utils.ContextCommandLine) (*setting.Cfg, tracing.Tracer, featuremgmt.FeatureToggles, error) {
 	configOptions := strings.Split(cmd.String("configOverrides"), " ")
 	configOptions = append(configOptions, cmd.Args().Slice()...)
 	cfg, err := setting.NewCfgFromArgs(setting.CommandLineArgs{
@@ -43,17 +45,33 @@ func initConflictCfg(cmd *utils.ContextCommandLine) (*setting.Cfg, error) {
 	})
 
 	if err != nil {
-		return nil, err
+		return nil, nil, nil, err
 	}
-	return cfg, nil
+
+	features, err := featuremgmt.ProvideManagerService(cfg)
+	if err != nil {
+		return nil, nil, nil, err
+	}
+
+	tracingCfg, err := tracing.ProvideTracingConfig(cfg)
+	if err != nil {
+		return nil, nil, nil, fmt.Errorf("%v: %w", "failed to initialize tracer config", err)
+	}
+
+	tracer, err := tracing.ProvideService(tracingCfg)
+	if err != nil {
+		return nil, nil, nil, fmt.Errorf("%v: %w", "failed to initialize tracer service", err)
+	}
+
+	return cfg, tracer, features, err
 }
 
 func initializeConflictResolver(cmd *utils.ContextCommandLine, f Formatter, ctx *cli.Context) (*ConflictResolver, error) {
-	cfg, err := initConflictCfg(cmd)
+	cfg, tracer, features, err := initConflictCfg(cmd)
 	if err != nil {
 		return nil, fmt.Errorf("%v: %w", "failed to load configuration", err)
 	}
-	s, err := getSqlStore(cfg)
+	s, err := getSqlStore(cfg, tracer, features)
 	if err != nil {
 		return nil, fmt.Errorf("%v: %w", "failed to get to sql", err)
 	}
@@ -62,16 +80,13 @@ func initializeConflictResolver(cmd *utils.ContextCommandLine, f Formatter, ctx
 		return nil, fmt.Errorf("%v: %w", "failed to get users with conflicting logins", err)
 	}
 	quotaService := quotaimpl.ProvideService(s, cfg)
-	userService, err := userimpl.ProvideService(s, nil, cfg, nil, nil, quotaService, supportbundlestest.NewFakeBundleService())
+	userService, err := userimpl.ProvideService(s, nil, cfg, nil, nil, tracer, quotaService, supportbundlestest.NewFakeBundleService())
 	if err != nil {
 		return nil, fmt.Errorf("%v: %w", "failed to get user service", err)
 	}
 	routing := routing.ProvideRegister()
-	featMgmt, err := featuremgmt.ProvideManagerService(cfg, nil)
-	if err != nil {
-		return nil, fmt.Errorf("%v: %w", "failed to get feature management service", err)
-	}
-	acService, err := acimpl.ProvideService(cfg, s, routing, nil, nil, featMgmt)
+
+	acService, err := acimpl.ProvideService(cfg, s, routing, nil, nil, nil, features, tracer, zanzana.NewNoopClient(), permreg.ProvidePermissionRegistry())
 	if err != nil {
 		return nil, fmt.Errorf("%v: %w", "failed to get access control", err)
 	}
@@ -80,13 +95,9 @@ func initializeConflictResolver(cmd *utils.ContextCommandLine, f Formatter, ctx
 	return &resolver, nil
 }
 
-func getSqlStore(cfg *setting.Cfg) (*sqlstore.SQLStore, error) {
-	tracer, err := tracing.ProvideService(cfg)
-	if err != nil {
-		return nil, fmt.Errorf("%v: %w", "failed to initialize tracer service", err)
-	}
+func getSqlStore(cfg *setting.Cfg, tracer tracing.Tracer, features featuremgmt.FeatureToggles) (*sqlstore.SQLStore, error) {
 	bus := bus.ProvideBus(tracer)
-	return sqlstore.ProvideService(cfg, nil, &migrations.OSSMigrations{}, bus, tracer)
+	return sqlstore.ProvideService(cfg, features, &migrations.OSSMigrations{}, bus, tracer)
 }
 
 func runListConflictUsers() func(context *cli.Context) error {
@@ -101,9 +112,9 @@ func runListConflictUsers() func(context *cli.Context) error {
 			logger.Info(color.GreenString("No Conflicting users found.\n\n"))
 			return nil
 		}
-		logger.Infof("\n\nShowing conflicts\n\n")
-		logger.Infof(r.ToStringPresentation())
-		logger.Infof("\n")
+		logger.Info("\n\nShowing conflicts\n\n")
+		logger.Info(r.ToStringPresentation())
+		logger.Info("\n")
 		if len(r.DiscardedBlocks) != 0 {
 			r.logDiscardedUsers()
 		}
@@ -238,10 +249,10 @@ func generateConflictUsersFile(r *ConflictResolver) (*os.File, error) {
 	if err != nil {
 		return nil, err
 	}
-	if _, err := tmpFile.Write([]byte(getDocumentationForFile())); err != nil {
+	if _, err := tmpFile.WriteString(getDocumentationForFile()); err != nil {
 		return nil, err
 	}
-	if _, err := tmpFile.Write([]byte(r.ToStringPresentation())); err != nil {
+	if _, err := tmpFile.WriteString(r.ToStringPresentation()); err != nil {
 		return nil, err
 	}
 	return tmpFile, nil
@@ -280,7 +291,7 @@ func getValidConflictUsers(r *ConflictResolver, b []byte) error {
 			continue
 		}
 
-		entryRow := matchingExpression.Match([]byte(row))
+		entryRow := matchingExpression.MatchString(row)
 		// not an entry row -> is a conflict block row
 		if !entryRow {
 			// check for malformed row
@@ -439,7 +450,8 @@ func (r *ConflictResolver) showChanges() {
 			}
 		}
 		b.WriteString("Keep the following user.\n")
-		b.WriteString(fmt.Sprintf("%s\n", block))
+		b.WriteString(block)
+		b.WriteByte('\n')
 		b.WriteString(color.GreenString(fmt.Sprintf("id: %s, email: %s, login: %s\n", mainUser.ID, mainUser.Email, mainUser.Login)))
 		for _, r := range fmt.Sprintf("%s%s", mainUser.Email, mainUser.Login) {
 			if unicode.IsUpper(r) {
@@ -460,12 +472,12 @@ func (r *ConflictResolver) showChanges() {
 		b.WriteString("\n\n")
 	}
 	logger.Info("\n\nChanges that will take place\n\n")
-	logger.Infof(b.String())
+	logger.Info(b.String())
 }
 
 // Formatter make it possible for us to write to terminal and to a file
 // with different formats depending on the usecase
-type Formatter func(format string, a ...interface{}) string
+type Formatter func(format string, a ...any) string
 
 func shouldDiscardBlock(seenUsersInBlock map[string]string, block string, user ConflictingUser) bool {
 	// loop through users to see if we should skip this block
@@ -768,7 +780,7 @@ ORDER BY
 
 func notServiceAccount(ss *sqlstore.SQLStore) string {
 	return fmt.Sprintf("is_service_account = %s",
-		ss.Dialect.BooleanStr(false))
+		ss.GetDialect().BooleanStr(false))
 }
 
 // confirm function asks for user input
