@@ -5,29 +5,36 @@ import (
 	"testing"
 
 	"github.com/stretchr/testify/assert"
+	"github.com/stretchr/testify/mock"
 	"github.com/stretchr/testify/require"
 
+	"github.com/grafana/grafana/pkg/apimachinery/identity"
 	"github.com/grafana/grafana/pkg/components/simplejson"
 	"github.com/grafana/grafana/pkg/infra/db"
+	"github.com/grafana/grafana/pkg/services/accesscontrol"
+	"github.com/grafana/grafana/pkg/services/accesscontrol/actest"
 	accesscontrolmock "github.com/grafana/grafana/pkg/services/accesscontrol/mock"
-	"github.com/grafana/grafana/pkg/services/alerting"
-	"github.com/grafana/grafana/pkg/services/alerting/models"
 	"github.com/grafana/grafana/pkg/services/dashboards"
 	"github.com/grafana/grafana/pkg/services/dashboards/database"
 	"github.com/grafana/grafana/pkg/services/featuremgmt"
+	"github.com/grafana/grafana/pkg/services/folder"
 	"github.com/grafana/grafana/pkg/services/folder/folderimpl"
 	"github.com/grafana/grafana/pkg/services/folder/foldertest"
 	"github.com/grafana/grafana/pkg/services/guardian"
 	"github.com/grafana/grafana/pkg/services/org"
 	"github.com/grafana/grafana/pkg/services/quota/quotatest"
 	"github.com/grafana/grafana/pkg/services/tag/tagimpl"
-	"github.com/grafana/grafana/pkg/services/team/teamtest"
 	"github.com/grafana/grafana/pkg/services/user"
 	"github.com/grafana/grafana/pkg/setting"
+	"github.com/grafana/grafana/pkg/tests/testsuite"
 )
 
 const testOrgID int64 = 1
 
+func TestMain(m *testing.M) {
+	testsuite.Run(m)
+}
+
 func TestIntegrationIntegratedDashboardService(t *testing.T) {
 	if testing.Short() {
 		t.Skip("skipping integration test")
@@ -39,7 +46,7 @@ func TestIntegrationIntegratedDashboardService(t *testing.T) {
 			func(t *testing.T, sc *permissionScenarioContext) {
 				cmd := dashboards.SaveDashboardCommand{
 					OrgID: testOrgID,
-					Dashboard: simplejson.NewFromAny(map[string]interface{}{
+					Dashboard: simplejson.NewFromAny(map[string]any{
 						"id":    float64(123412321),
 						"title": "Expect error",
 					}),
@@ -58,7 +65,7 @@ func TestIntegrationIntegratedDashboardService(t *testing.T) {
 				true, func(t *testing.T, sc *permissionScenarioContext) {
 					cmd := dashboards.SaveDashboardCommand{
 						OrgID: otherOrgId,
-						Dashboard: simplejson.NewFromAny(map[string]interface{}{
+						Dashboard: simplejson.NewFromAny(map[string]any{
 							"id":    sc.savedDashInFolder.ID,
 							"title": "Expect error",
 						}),
@@ -74,7 +81,7 @@ func TestIntegrationIntegratedDashboardService(t *testing.T) {
 					const otherOrgId int64 = 2
 					cmd := dashboards.SaveDashboardCommand{
 						OrgID: otherOrgId,
-						Dashboard: simplejson.NewFromAny(map[string]interface{}{
+						Dashboard: simplejson.NewFromAny(map[string]any{
 							"uid":   sc.savedDashInFolder.UID,
 							"title": "Dash with existing uid in other org",
 						}),
@@ -100,7 +107,7 @@ func TestIntegrationIntegratedDashboardService(t *testing.T) {
 					sqlStore := db.InitTestDB(t)
 					cmd := dashboards.SaveDashboardCommand{
 						OrgID: testOrgID,
-						Dashboard: simplejson.NewFromAny(map[string]interface{}{
+						Dashboard: simplejson.NewFromAny(map[string]any{
 							"title": "Dash",
 						}),
 						UserID:    10000,
@@ -110,19 +117,22 @@ func TestIntegrationIntegratedDashboardService(t *testing.T) {
 					err := callSaveWithError(t, cmd, sqlStore)
 					assert.Equal(t, dashboards.ErrDashboardUpdateAccessDenied, err)
 
+					userID, err := identity.IntIdentifier(sc.dashboardGuardianMock.User.GetID())
+					require.NoError(t, err)
+
 					assert.Equal(t, "", sc.dashboardGuardianMock.DashUID)
 					assert.Equal(t, cmd.OrgID, sc.dashboardGuardianMock.OrgID)
-					assert.Equal(t, cmd.UserID, sc.dashboardGuardianMock.User.UserID)
+					assert.Equal(t, cmd.UserID, userID)
 				})
 
 			permissionScenario(t, "When creating a new dashboard in other folder, it should create dashboard guardian for other folder with correct arguments and rsult in access denied error",
 				canSave, func(t *testing.T, sc *permissionScenarioContext) {
 					cmd := dashboards.SaveDashboardCommand{
 						OrgID: testOrgID,
-						Dashboard: simplejson.NewFromAny(map[string]interface{}{
+						Dashboard: simplejson.NewFromAny(map[string]any{
 							"title": "Dash",
 						}),
-						FolderID:  sc.otherSavedFolder.ID,
+						FolderUID: sc.otherSavedFolder.UID,
 						UserID:    10000,
 						Overwrite: true,
 					}
@@ -130,19 +140,22 @@ func TestIntegrationIntegratedDashboardService(t *testing.T) {
 					err := callSaveWithError(t, cmd, sc.sqlStore)
 					require.Equal(t, dashboards.ErrDashboardUpdateAccessDenied, err)
 
+					userID, err := identity.IntIdentifier(sc.dashboardGuardianMock.User.GetID())
+					require.NoError(t, err)
+
 					assert.Equal(t, sc.otherSavedFolder.ID, sc.dashboardGuardianMock.DashID)
 					assert.Equal(t, cmd.OrgID, sc.dashboardGuardianMock.OrgID)
-					assert.Equal(t, cmd.UserID, sc.dashboardGuardianMock.User.UserID)
+					assert.Equal(t, cmd.UserID, userID)
 				})
 
 			permissionScenario(t, "When creating a new dashboard by existing title in folder, it should create dashboard guardian for dashboard with correct arguments and result in access denied error",
 				canSave, func(t *testing.T, sc *permissionScenarioContext) {
 					cmd := dashboards.SaveDashboardCommand{
 						OrgID: testOrgID,
-						Dashboard: simplejson.NewFromAny(map[string]interface{}{
+						Dashboard: simplejson.NewFromAny(map[string]any{
 							"title": sc.savedDashInFolder.Title,
 						}),
-						FolderID:  sc.savedFolder.ID,
+						FolderUID: sc.savedFolder.UID,
 						UserID:    10000,
 						Overwrite: true,
 					}
@@ -150,20 +163,23 @@ func TestIntegrationIntegratedDashboardService(t *testing.T) {
 					err := callSaveWithError(t, cmd, sc.sqlStore)
 					require.Equal(t, dashboards.ErrDashboardUpdateAccessDenied, err)
 
+					userID, err := identity.IntIdentifier(sc.dashboardGuardianMock.User.GetID())
+					require.NoError(t, err)
+
 					assert.Equal(t, sc.savedDashInFolder.UID, sc.dashboardGuardianMock.DashUID)
 					assert.Equal(t, cmd.OrgID, sc.dashboardGuardianMock.OrgID)
-					assert.Equal(t, cmd.UserID, sc.dashboardGuardianMock.User.UserID)
+					assert.Equal(t, cmd.UserID, userID)
 				})
 
 			permissionScenario(t, "When creating a new dashboard by existing UID in folder, it should create dashboard guardian for dashboard with correct arguments and result in access denied error",
 				canSave, func(t *testing.T, sc *permissionScenarioContext) {
 					cmd := dashboards.SaveDashboardCommand{
 						OrgID: testOrgID,
-						Dashboard: simplejson.NewFromAny(map[string]interface{}{
+						Dashboard: simplejson.NewFromAny(map[string]any{
 							"uid":   sc.savedDashInFolder.UID,
 							"title": "New dash",
 						}),
-						FolderID:  sc.savedFolder.ID,
+						FolderUID: sc.savedFolder.UID,
 						UserID:    10000,
 						Overwrite: true,
 					}
@@ -171,20 +187,23 @@ func TestIntegrationIntegratedDashboardService(t *testing.T) {
 					err := callSaveWithError(t, cmd, sc.sqlStore)
 					require.Equal(t, dashboards.ErrDashboardUpdateAccessDenied, err)
 
+					userID, err := identity.IntIdentifier(sc.dashboardGuardianMock.User.GetID())
+					require.NoError(t, err)
+
 					assert.Equal(t, sc.savedDashInFolder.UID, sc.dashboardGuardianMock.DashUID)
 					assert.Equal(t, cmd.OrgID, sc.dashboardGuardianMock.OrgID)
-					assert.Equal(t, cmd.UserID, sc.dashboardGuardianMock.User.UserID)
+					assert.Equal(t, cmd.UserID, userID)
 				})
 
 			permissionScenario(t, "When updating a dashboard by existing id in the General folder, it should create dashboard guardian for dashboard with correct arguments and result in access denied error",
 				canSave, func(t *testing.T, sc *permissionScenarioContext) {
 					cmd := dashboards.SaveDashboardCommand{
 						OrgID: testOrgID,
-						Dashboard: simplejson.NewFromAny(map[string]interface{}{
+						Dashboard: simplejson.NewFromAny(map[string]any{
 							"id":    sc.savedDashInGeneralFolder.ID,
 							"title": "Dash",
 						}),
-						FolderID:  sc.savedDashInGeneralFolder.FolderID,
+						FolderUID: sc.savedDashInGeneralFolder.FolderUID,
 						UserID:    10000,
 						Overwrite: true,
 					}
@@ -192,20 +211,23 @@ func TestIntegrationIntegratedDashboardService(t *testing.T) {
 					err := callSaveWithError(t, cmd, sc.sqlStore)
 					assert.Equal(t, dashboards.ErrDashboardUpdateAccessDenied, err)
 
+					userID, err := identity.IntIdentifier(sc.dashboardGuardianMock.User.GetID())
+					require.NoError(t, err)
+
 					assert.Equal(t, sc.savedDashInGeneralFolder.UID, sc.dashboardGuardianMock.DashUID)
 					assert.Equal(t, cmd.OrgID, sc.dashboardGuardianMock.OrgID)
-					assert.Equal(t, cmd.UserID, sc.dashboardGuardianMock.User.UserID)
+					assert.Equal(t, cmd.UserID, userID)
 				})
 
 			permissionScenario(t, "When updating a dashboard by existing id in other folder, it should create dashboard guardian for dashboard with correct arguments and result in access denied error",
 				canSave, func(t *testing.T, sc *permissionScenarioContext) {
 					cmd := dashboards.SaveDashboardCommand{
 						OrgID: testOrgID,
-						Dashboard: simplejson.NewFromAny(map[string]interface{}{
+						Dashboard: simplejson.NewFromAny(map[string]any{
 							"id":    sc.savedDashInFolder.ID,
 							"title": "Dash",
 						}),
-						FolderID:  sc.savedDashInFolder.FolderID,
+						FolderUID: sc.savedDashInFolder.FolderUID,
 						UserID:    10000,
 						Overwrite: true,
 					}
@@ -213,20 +235,23 @@ func TestIntegrationIntegratedDashboardService(t *testing.T) {
 					err := callSaveWithError(t, cmd, sc.sqlStore)
 					require.Equal(t, dashboards.ErrDashboardUpdateAccessDenied, err)
 
+					userID, err := identity.IntIdentifier(sc.dashboardGuardianMock.User.GetID())
+					require.NoError(t, err)
+
 					assert.Equal(t, sc.savedDashInFolder.UID, sc.dashboardGuardianMock.DashUID)
 					assert.Equal(t, cmd.OrgID, sc.dashboardGuardianMock.OrgID)
-					assert.Equal(t, cmd.UserID, sc.dashboardGuardianMock.User.UserID)
+					assert.Equal(t, cmd.UserID, userID)
 				})
 
 			permissionScenario(t, "When moving a dashboard by existing ID to other folder from General folder, it should create dashboard guardian for dashboard with correct arguments and result in access denied error",
 				canSave, func(t *testing.T, sc *permissionScenarioContext) {
 					cmd := dashboards.SaveDashboardCommand{
 						OrgID: testOrgID,
-						Dashboard: simplejson.NewFromAny(map[string]interface{}{
+						Dashboard: simplejson.NewFromAny(map[string]any{
 							"id":    sc.savedDashInGeneralFolder.ID,
 							"title": "Dash",
 						}),
-						FolderID:  sc.otherSavedFolder.ID,
+						FolderUID: sc.otherSavedFolder.UID,
 						UserID:    10000,
 						Overwrite: true,
 					}
@@ -234,20 +259,23 @@ func TestIntegrationIntegratedDashboardService(t *testing.T) {
 					err := callSaveWithError(t, cmd, sc.sqlStore)
 					require.Equal(t, dashboards.ErrDashboardUpdateAccessDenied, err)
 
+					userID, err := identity.IntIdentifier(sc.dashboardGuardianMock.User.GetID())
+					require.NoError(t, err)
+
 					assert.Equal(t, sc.savedDashInGeneralFolder.UID, sc.dashboardGuardianMock.DashUID)
 					assert.Equal(t, cmd.OrgID, sc.dashboardGuardianMock.OrgID)
-					assert.Equal(t, cmd.UserID, sc.dashboardGuardianMock.User.UserID)
+					assert.Equal(t, cmd.UserID, userID)
 				})
 
 			permissionScenario(t, "When moving a dashboard by existing id to the General folder from other folder, it should create dashboard guardian for dashboard with correct arguments and result in access denied error",
 				canSave, func(t *testing.T, sc *permissionScenarioContext) {
 					cmd := dashboards.SaveDashboardCommand{
 						OrgID: testOrgID,
-						Dashboard: simplejson.NewFromAny(map[string]interface{}{
+						Dashboard: simplejson.NewFromAny(map[string]any{
 							"id":    sc.savedDashInFolder.ID,
 							"title": "Dash",
 						}),
-						FolderID:  0,
+						FolderUID: "",
 						UserID:    10000,
 						Overwrite: true,
 					}
@@ -255,20 +283,23 @@ func TestIntegrationIntegratedDashboardService(t *testing.T) {
 					err := callSaveWithError(t, cmd, sc.sqlStore)
 					assert.Equal(t, dashboards.ErrDashboardUpdateAccessDenied, err)
 
+					userID, err := identity.IntIdentifier(sc.dashboardGuardianMock.User.GetID())
+					require.NoError(t, err)
+
 					assert.Equal(t, sc.savedDashInFolder.UID, sc.dashboardGuardianMock.DashUID)
 					assert.Equal(t, cmd.OrgID, sc.dashboardGuardianMock.OrgID)
-					assert.Equal(t, cmd.UserID, sc.dashboardGuardianMock.User.UserID)
+					assert.Equal(t, cmd.UserID, userID)
 				})
 
 			permissionScenario(t, "When moving a dashboard by existing uid to other folder from General folder, it should create dashboard guardian for dashboard with correct arguments and result in access denied error",
 				canSave, func(t *testing.T, sc *permissionScenarioContext) {
 					cmd := dashboards.SaveDashboardCommand{
 						OrgID: testOrgID,
-						Dashboard: simplejson.NewFromAny(map[string]interface{}{
+						Dashboard: simplejson.NewFromAny(map[string]any{
 							"uid":   sc.savedDashInGeneralFolder.UID,
 							"title": "Dash",
 						}),
-						FolderID:  sc.otherSavedFolder.ID,
+						FolderUID: sc.otherSavedFolder.UID,
 						UserID:    10000,
 						Overwrite: true,
 					}
@@ -276,20 +307,23 @@ func TestIntegrationIntegratedDashboardService(t *testing.T) {
 					err := callSaveWithError(t, cmd, sc.sqlStore)
 					require.Equal(t, dashboards.ErrDashboardUpdateAccessDenied, err)
 
+					userID, err := identity.IntIdentifier(sc.dashboardGuardianMock.User.GetID())
+					require.NoError(t, err)
+
 					assert.Equal(t, sc.savedDashInGeneralFolder.UID, sc.dashboardGuardianMock.DashUID)
 					assert.Equal(t, cmd.OrgID, sc.dashboardGuardianMock.OrgID)
-					assert.Equal(t, cmd.UserID, sc.dashboardGuardianMock.User.UserID)
+					assert.Equal(t, cmd.UserID, userID)
 				})
 
 			permissionScenario(t, "When moving a dashboard by existing UID to the General folder from other folder, it should create dashboard guardian for dashboard with correct arguments and result in access denied error",
 				canSave, func(t *testing.T, sc *permissionScenarioContext) {
 					cmd := dashboards.SaveDashboardCommand{
 						OrgID: testOrgID,
-						Dashboard: simplejson.NewFromAny(map[string]interface{}{
+						Dashboard: simplejson.NewFromAny(map[string]any{
 							"uid":   sc.savedDashInFolder.UID,
 							"title": "Dash",
 						}),
-						FolderID:  0,
+						FolderUID: "",
 						UserID:    10000,
 						Overwrite: true,
 					}
@@ -297,9 +331,12 @@ func TestIntegrationIntegratedDashboardService(t *testing.T) {
 					err := callSaveWithError(t, cmd, sc.sqlStore)
 					require.Equal(t, dashboards.ErrDashboardUpdateAccessDenied, err)
 
+					userID, err := identity.IntIdentifier(sc.dashboardGuardianMock.User.GetID())
+					require.NoError(t, err)
+
 					assert.Equal(t, sc.savedDashInFolder.UID, sc.dashboardGuardianMock.DashUID)
 					assert.Equal(t, cmd.OrgID, sc.dashboardGuardianMock.OrgID)
-					assert.Equal(t, cmd.UserID, sc.dashboardGuardianMock.User.UserID)
+					assert.Equal(t, cmd.UserID, userID)
 				})
 		})
 
@@ -313,11 +350,11 @@ func TestIntegrationIntegratedDashboardService(t *testing.T) {
 					canSave, func(t *testing.T, sc *permissionScenarioContext) {
 						cmd := dashboards.SaveDashboardCommand{
 							OrgID: testOrgID,
-							Dashboard: simplejson.NewFromAny(map[string]interface{}{
+							Dashboard: simplejson.NewFromAny(map[string]any{
 								"id":    nil,
 								"title": sc.savedDashInFolder.Title,
 							}),
-							FolderID:  0,
+							FolderUID: "",
 							Overwrite: shouldOverwrite,
 						}
 
@@ -336,11 +373,11 @@ func TestIntegrationIntegratedDashboardService(t *testing.T) {
 					canSave, func(t *testing.T, sc *permissionScenarioContext) {
 						cmd := dashboards.SaveDashboardCommand{
 							OrgID: testOrgID,
-							Dashboard: simplejson.NewFromAny(map[string]interface{}{
+							Dashboard: simplejson.NewFromAny(map[string]any{
 								"id":    nil,
 								"title": sc.savedDashInGeneralFolder.Title,
 							}),
-							FolderID:  sc.savedFolder.ID,
+							FolderUID: sc.savedFolder.UID,
 							Overwrite: shouldOverwrite,
 						}
 
@@ -360,7 +397,7 @@ func TestIntegrationIntegratedDashboardService(t *testing.T) {
 					canSave, func(t *testing.T, sc *permissionScenarioContext) {
 						cmd := dashboards.SaveDashboardCommand{
 							OrgID: testOrgID,
-							Dashboard: simplejson.NewFromAny(map[string]interface{}{
+							Dashboard: simplejson.NewFromAny(map[string]any{
 								"id":    nil,
 								"title": sc.savedDashInFolder.Title,
 							}),
@@ -385,7 +422,7 @@ func TestIntegrationIntegratedDashboardService(t *testing.T) {
 					canSave, func(t *testing.T, sc *permissionScenarioContext) {
 						cmd := dashboards.SaveDashboardCommand{
 							OrgID: testOrgID,
-							Dashboard: simplejson.NewFromAny(map[string]interface{}{
+							Dashboard: simplejson.NewFromAny(map[string]any{
 								"title": "Dash without id and uid",
 							}),
 							Overwrite: shouldOverwrite,
@@ -407,7 +444,7 @@ func TestIntegrationIntegratedDashboardService(t *testing.T) {
 					func(t *testing.T, sc *permissionScenarioContext) {
 						cmd := dashboards.SaveDashboardCommand{
 							OrgID: testOrgID,
-							Dashboard: simplejson.NewFromAny(map[string]interface{}{
+							Dashboard: simplejson.NewFromAny(map[string]any{
 								"id":    0,
 								"title": "Dash with zero id",
 							}),
@@ -428,26 +465,26 @@ func TestIntegrationIntegratedDashboardService(t *testing.T) {
 					func(t *testing.T, sc *permissionScenarioContext) {
 						cmd := dashboards.SaveDashboardCommand{
 							OrgID: testOrgID,
-							Dashboard: simplejson.NewFromAny(map[string]interface{}{
+							Dashboard: simplejson.NewFromAny(map[string]any{
 								"title": "Expect error",
 							}),
-							FolderID:  123412321,
+							FolderUID: "123412321",
 							Overwrite: shouldOverwrite,
 						}
 
 						err := callSaveWithError(t, cmd, sc.sqlStore)
-						assert.Equal(t, dashboards.ErrDashboardFolderNotFound, err)
+						assert.Equal(t, dashboards.ErrFolderNotFound, err)
 					})
 
 				permissionScenario(t, "When updating an existing dashboard by id without current version", canSave,
 					func(t *testing.T, sc *permissionScenarioContext) {
 						cmd := dashboards.SaveDashboardCommand{
 							OrgID: 1,
-							Dashboard: simplejson.NewFromAny(map[string]interface{}{
+							Dashboard: simplejson.NewFromAny(map[string]any{
 								"id":    sc.savedDashInGeneralFolder.ID,
 								"title": "test dash 23",
 							}),
-							FolderID:  sc.savedFolder.ID,
+							FolderUID: sc.savedFolder.UID,
 							Overwrite: shouldOverwrite,
 						}
 
@@ -459,12 +496,12 @@ func TestIntegrationIntegratedDashboardService(t *testing.T) {
 					func(t *testing.T, sc *permissionScenarioContext) {
 						cmd := dashboards.SaveDashboardCommand{
 							OrgID: 1,
-							Dashboard: simplejson.NewFromAny(map[string]interface{}{
+							Dashboard: simplejson.NewFromAny(map[string]any{
 								"id":      sc.savedDashInGeneralFolder.ID,
 								"title":   "Updated title",
 								"version": sc.savedDashInGeneralFolder.Version,
 							}),
-							FolderID:  sc.savedFolder.ID,
+							FolderUID: sc.savedFolder.UID,
 							Overwrite: shouldOverwrite,
 						}
 
@@ -483,11 +520,11 @@ func TestIntegrationIntegratedDashboardService(t *testing.T) {
 					func(t *testing.T, sc *permissionScenarioContext) {
 						cmd := dashboards.SaveDashboardCommand{
 							OrgID: 1,
-							Dashboard: simplejson.NewFromAny(map[string]interface{}{
+							Dashboard: simplejson.NewFromAny(map[string]any{
 								"uid":   sc.savedDashInFolder.UID,
 								"title": "test dash 23",
 							}),
-							FolderID:  0,
+							FolderUID: "",
 							Overwrite: shouldOverwrite,
 						}
 
@@ -499,12 +536,12 @@ func TestIntegrationIntegratedDashboardService(t *testing.T) {
 					func(t *testing.T, sc *permissionScenarioContext) {
 						cmd := dashboards.SaveDashboardCommand{
 							OrgID: 1,
-							Dashboard: simplejson.NewFromAny(map[string]interface{}{
+							Dashboard: simplejson.NewFromAny(map[string]any{
 								"uid":     sc.savedDashInFolder.UID,
 								"title":   "Updated title",
 								"version": sc.savedDashInFolder.Version,
 							}),
-							FolderID:  0,
+							FolderUID: "",
 							Overwrite: shouldOverwrite,
 						}
 
@@ -522,11 +559,11 @@ func TestIntegrationIntegratedDashboardService(t *testing.T) {
 					canSave, func(t *testing.T, sc *permissionScenarioContext) {
 						cmd := dashboards.SaveDashboardCommand{
 							OrgID: testOrgID,
-							Dashboard: simplejson.NewFromAny(map[string]interface{}{
+							Dashboard: simplejson.NewFromAny(map[string]any{
 								"id":    nil,
 								"title": sc.savedDashInFolder.Title,
 							}),
-							FolderID:  sc.savedDashInFolder.FolderID,
+							FolderUID: sc.savedDashInFolder.FolderUID,
 							Overwrite: shouldOverwrite,
 						}
 
@@ -538,11 +575,11 @@ func TestIntegrationIntegratedDashboardService(t *testing.T) {
 					canSave, func(t *testing.T, sc *permissionScenarioContext) {
 						cmd := dashboards.SaveDashboardCommand{
 							OrgID: testOrgID,
-							Dashboard: simplejson.NewFromAny(map[string]interface{}{
+							Dashboard: simplejson.NewFromAny(map[string]any{
 								"id":    nil,
 								"title": sc.savedDashInGeneralFolder.Title,
 							}),
-							FolderID:  sc.savedDashInGeneralFolder.FolderID,
+							FolderUID: sc.savedDashInGeneralFolder.FolderUID,
 							Overwrite: shouldOverwrite,
 						}
 
@@ -554,7 +591,7 @@ func TestIntegrationIntegratedDashboardService(t *testing.T) {
 					func(t *testing.T, sc *permissionScenarioContext) {
 						cmd := dashboards.SaveDashboardCommand{
 							OrgID: testOrgID,
-							Dashboard: simplejson.NewFromAny(map[string]interface{}{
+							Dashboard: simplejson.NewFromAny(map[string]any{
 								"id":    nil,
 								"title": sc.savedFolder.Title,
 							}),
@@ -574,11 +611,11 @@ func TestIntegrationIntegratedDashboardService(t *testing.T) {
 					func(t *testing.T, sc *permissionScenarioContext) {
 						cmd := dashboards.SaveDashboardCommand{
 							OrgID: 1,
-							Dashboard: simplejson.NewFromAny(map[string]interface{}{
+							Dashboard: simplejson.NewFromAny(map[string]any{
 								"id":    sc.savedDashInGeneralFolder.ID,
 								"title": "Updated title",
 							}),
-							FolderID:  sc.savedFolder.ID,
+							FolderUID: sc.savedFolder.UID,
 							Overwrite: shouldOverwrite,
 						}
 
@@ -596,11 +633,11 @@ func TestIntegrationIntegratedDashboardService(t *testing.T) {
 					func(t *testing.T, sc *permissionScenarioContext) {
 						cmd := dashboards.SaveDashboardCommand{
 							OrgID: 1,
-							Dashboard: simplejson.NewFromAny(map[string]interface{}{
+							Dashboard: simplejson.NewFromAny(map[string]any{
 								"uid":   sc.savedDashInFolder.UID,
 								"title": "Updated title",
 							}),
-							FolderID:  0,
+							FolderUID: "",
 							Overwrite: shouldOverwrite,
 						}
 
@@ -618,7 +655,7 @@ func TestIntegrationIntegratedDashboardService(t *testing.T) {
 					func(t *testing.T, sc *permissionScenarioContext) {
 						cmd := dashboards.SaveDashboardCommand{
 							OrgID: 1,
-							Dashboard: simplejson.NewFromAny(map[string]interface{}{
+							Dashboard: simplejson.NewFromAny(map[string]any{
 								"id":    sc.savedDashInFolder.ID,
 								"uid":   "new-uid",
 								"title": sc.savedDashInFolder.Title,
@@ -642,7 +679,7 @@ func TestIntegrationIntegratedDashboardService(t *testing.T) {
 					func(t *testing.T, sc *permissionScenarioContext) {
 						cmd := dashboards.SaveDashboardCommand{
 							OrgID: 1,
-							Dashboard: simplejson.NewFromAny(map[string]interface{}{
+							Dashboard: simplejson.NewFromAny(map[string]any{
 								"id":    sc.savedDashInFolder.ID,
 								"uid":   sc.savedDashInGeneralFolder.UID,
 								"title": sc.savedDashInFolder.Title,
@@ -658,11 +695,11 @@ func TestIntegrationIntegratedDashboardService(t *testing.T) {
 					func(t *testing.T, sc *permissionScenarioContext) {
 						cmd := dashboards.SaveDashboardCommand{
 							OrgID: testOrgID,
-							Dashboard: simplejson.NewFromAny(map[string]interface{}{
+							Dashboard: simplejson.NewFromAny(map[string]any{
 								"id":    nil,
 								"title": sc.savedDashInFolder.Title,
 							}),
-							FolderID:  sc.savedDashInFolder.FolderID,
+							FolderUID: sc.savedDashInFolder.FolderUID,
 							Overwrite: shouldOverwrite,
 						}
 
@@ -682,11 +719,11 @@ func TestIntegrationIntegratedDashboardService(t *testing.T) {
 					func(t *testing.T, sc *permissionScenarioContext) {
 						cmd := dashboards.SaveDashboardCommand{
 							OrgID: testOrgID,
-							Dashboard: simplejson.NewFromAny(map[string]interface{}{
+							Dashboard: simplejson.NewFromAny(map[string]any{
 								"id":    nil,
 								"title": sc.savedDashInGeneralFolder.Title,
 							}),
-							FolderID:  sc.savedDashInGeneralFolder.FolderID,
+							FolderUID: sc.savedDashInGeneralFolder.FolderUID,
 							Overwrite: shouldOverwrite,
 						}
 
@@ -706,7 +743,7 @@ func TestIntegrationIntegratedDashboardService(t *testing.T) {
 					func(t *testing.T, sc *permissionScenarioContext) {
 						cmd := dashboards.SaveDashboardCommand{
 							OrgID: 1,
-							Dashboard: simplejson.NewFromAny(map[string]interface{}{
+							Dashboard: simplejson.NewFromAny(map[string]any{
 								"id":    sc.savedFolder.ID,
 								"title": "new title",
 							}),
@@ -722,7 +759,7 @@ func TestIntegrationIntegratedDashboardService(t *testing.T) {
 					func(t *testing.T, sc *permissionScenarioContext) {
 						cmd := dashboards.SaveDashboardCommand{
 							OrgID: 1,
-							Dashboard: simplejson.NewFromAny(map[string]interface{}{
+							Dashboard: simplejson.NewFromAny(map[string]any{
 								"id":    sc.savedDashInFolder.ID,
 								"title": "new folder title",
 							}),
@@ -738,7 +775,7 @@ func TestIntegrationIntegratedDashboardService(t *testing.T) {
 					func(t *testing.T, sc *permissionScenarioContext) {
 						cmd := dashboards.SaveDashboardCommand{
 							OrgID: 1,
-							Dashboard: simplejson.NewFromAny(map[string]interface{}{
+							Dashboard: simplejson.NewFromAny(map[string]any{
 								"uid":   sc.savedFolder.UID,
 								"title": "new title",
 							}),
@@ -754,7 +791,7 @@ func TestIntegrationIntegratedDashboardService(t *testing.T) {
 					func(t *testing.T, sc *permissionScenarioContext) {
 						cmd := dashboards.SaveDashboardCommand{
 							OrgID: 1,
-							Dashboard: simplejson.NewFromAny(map[string]interface{}{
+							Dashboard: simplejson.NewFromAny(map[string]any{
 								"uid":   sc.savedDashInFolder.UID,
 								"title": "new folder title",
 							}),
@@ -770,7 +807,7 @@ func TestIntegrationIntegratedDashboardService(t *testing.T) {
 					func(t *testing.T, sc *permissionScenarioContext) {
 						cmd := dashboards.SaveDashboardCommand{
 							OrgID: 1,
-							Dashboard: simplejson.NewFromAny(map[string]interface{}{
+							Dashboard: simplejson.NewFromAny(map[string]any{
 								"title": sc.savedFolder.Title,
 							}),
 							IsFolder:  false,
@@ -785,7 +822,7 @@ func TestIntegrationIntegratedDashboardService(t *testing.T) {
 					func(t *testing.T, sc *permissionScenarioContext) {
 						cmd := dashboards.SaveDashboardCommand{
 							OrgID: 1,
-							Dashboard: simplejson.NewFromAny(map[string]interface{}{
+							Dashboard: simplejson.NewFromAny(map[string]any{
 								"title": sc.savedDashInGeneralFolder.Title,
 							}),
 							IsFolder:  true,
@@ -815,57 +852,61 @@ type permissionScenarioFunc func(t *testing.T, sc *permissionScenarioContext)
 func permissionScenario(t *testing.T, desc string, canSave bool, fn permissionScenarioFunc) {
 	t.Helper()
 
-	mock := &guardian.FakeDashboardGuardian{
+	guardianMock := &guardian.FakeDashboardGuardian{
 		CanSaveValue: canSave,
 	}
 
 	t.Run(desc, func(t *testing.T) {
+		features := featuremgmt.WithFeatures()
 		cfg := setting.NewCfg()
-		cfg.RBACEnabled = false
-		cfg.IsFeatureToggleEnabled = featuremgmt.WithFeatures().IsEnabled
 		sqlStore := db.InitTestDB(t)
 		quotaService := quotatest.New(false, nil)
-		dashboardStore, err := database.ProvideDashboardStore(sqlStore, cfg, featuremgmt.WithFeatures(), tagimpl.ProvideService(sqlStore, cfg), quotaService)
+		ac := actest.FakeAccessControl{ExpectedEvaluate: true}
+		dashboardStore, err := database.ProvideDashboardStore(sqlStore, cfg, features, tagimpl.ProvideService(sqlStore), quotaService)
 		require.NoError(t, err)
 		folderStore := folderimpl.ProvideDashboardFolderStore(sqlStore)
-		service := ProvideDashboardServiceImpl(
-			cfg, dashboardStore, folderStore, &dummyDashAlertExtractor{},
+		folderPermissions := accesscontrolmock.NewMockedPermissionsService()
+		folderPermissions.On("SetPermissions", mock.Anything, mock.Anything, mock.Anything, mock.Anything).Return([]accesscontrol.ResourcePermission{}, nil)
+		dashboardPermissions := accesscontrolmock.NewMockedPermissionsService()
+		dashboardService, err := ProvideDashboardServiceImpl(
+			cfg, dashboardStore, folderStore,
 			featuremgmt.WithFeatures(),
-			accesscontrolmock.NewMockedPermissionsService(),
-			accesscontrolmock.NewMockedPermissionsService(),
-			accesscontrolmock.New(),
+			folderPermissions,
+			dashboardPermissions,
+			ac,
 			foldertest.NewFakeService(),
+			folder.NewFakeStore(),
+			nil,
 		)
-		guardian.InitLegacyGuardian(cfg, sqlStore, service, &teamtest.FakeService{})
+		require.NoError(t, err)
+		guardian.InitAccessControlGuardian(cfg, ac, dashboardService)
 
 		savedFolder := saveTestFolder(t, "Saved folder", testOrgID, sqlStore)
-		savedDashInFolder := saveTestDashboard(t, "Saved dash in folder", testOrgID, savedFolder.ID, sqlStore)
-		saveTestDashboard(t, "Other saved dash in folder", testOrgID, savedFolder.ID, sqlStore)
-		savedDashInGeneralFolder := saveTestDashboard(t, "Saved dashboard in general folder", testOrgID, 0, sqlStore)
+		savedDashInFolder := saveTestDashboard(t, "Saved dash in folder", testOrgID, savedFolder.UID, sqlStore)
+		saveTestDashboard(t, "Other saved dash in folder", testOrgID, savedFolder.UID, sqlStore)
+		savedDashInGeneralFolder := saveTestDashboard(t, "Saved dashboard in general folder", testOrgID, "", sqlStore)
 		otherSavedFolder := saveTestFolder(t, "Other saved folder", testOrgID, sqlStore)
 
 		require.Equal(t, "Saved folder", savedFolder.Title)
 		require.Equal(t, "saved-folder", savedFolder.Slug)
 		require.NotEqual(t, int64(0), savedFolder.ID)
 		require.True(t, savedFolder.IsFolder)
-		require.Equal(t, int64(0), savedFolder.FolderID)
 		require.NotEmpty(t, savedFolder.UID)
 
 		require.Equal(t, "Saved dash in folder", savedDashInFolder.Title)
 		require.Equal(t, "saved-dash-in-folder", savedDashInFolder.Slug)
 		require.NotEqual(t, int64(0), savedDashInFolder.ID)
 		require.False(t, savedDashInFolder.IsFolder)
-		require.Equal(t, savedFolder.ID, savedDashInFolder.FolderID)
 		require.NotEmpty(t, savedDashInFolder.UID)
 
 		origNewDashboardGuardian := guardian.New
 		t.Cleanup(func() {
 			guardian.New = origNewDashboardGuardian
 		})
-		guardian.MockDashboardGuardian(mock)
+		guardian.MockDashboardGuardian(guardianMock)
 
 		sc := &permissionScenarioContext{
-			dashboardGuardianMock:    mock,
+			dashboardGuardianMock:    guardianMock,
 			sqlStore:                 sqlStore,
 			savedDashInFolder:        savedDashInFolder,
 			otherSavedFolder:         otherSavedFolder,
@@ -881,22 +922,29 @@ func permissionScenario(t *testing.T, desc string, canSave bool, fn permissionSc
 func callSaveWithResult(t *testing.T, cmd dashboards.SaveDashboardCommand, sqlStore db.DB) *dashboards.Dashboard {
 	t.Helper()
 
+	features := featuremgmt.WithFeatures()
 	dto := toSaveDashboardDto(cmd)
 	cfg := setting.NewCfg()
-	cfg.RBACEnabled = false
-	cfg.IsFeatureToggleEnabled = featuremgmt.WithFeatures().IsEnabled
 	quotaService := quotatest.New(false, nil)
-	dashboardStore, err := database.ProvideDashboardStore(sqlStore, cfg, featuremgmt.WithFeatures(), tagimpl.ProvideService(sqlStore, cfg), quotaService)
+	dashboardStore, err := database.ProvideDashboardStore(sqlStore, cfg, features, tagimpl.ProvideService(sqlStore), quotaService)
 	require.NoError(t, err)
 	folderStore := folderimpl.ProvideDashboardFolderStore(sqlStore)
-	service := ProvideDashboardServiceImpl(
-		cfg, dashboardStore, folderStore, &dummyDashAlertExtractor{},
+	folderPermissions := accesscontrolmock.NewMockedPermissionsService()
+	folderPermissions.On("SetPermissions", mock.Anything, mock.Anything, mock.Anything, mock.Anything).Return([]accesscontrol.ResourcePermission{}, nil)
+
+	dashboardPermissions := accesscontrolmock.NewMockedPermissionsService()
+	dashboardPermissions.On("SetPermissions", mock.Anything, mock.Anything, mock.Anything, mock.Anything).Return([]accesscontrol.ResourcePermission{}, nil)
+	service, err := ProvideDashboardServiceImpl(
+		cfg, dashboardStore, folderStore,
 		featuremgmt.WithFeatures(),
-		accesscontrolmock.NewMockedPermissionsService(),
-		accesscontrolmock.NewMockedPermissionsService(),
-		accesscontrolmock.New(),
+		folderPermissions,
+		dashboardPermissions,
+		actest.FakeAccessControl{},
 		foldertest.NewFakeService(),
+		folder.NewFakeStore(),
+		nil,
 	)
+	require.NoError(t, err)
 	res, err := service.SaveDashboard(context.Background(), &dto, false)
 	require.NoError(t, err)
 
@@ -904,34 +952,36 @@ func callSaveWithResult(t *testing.T, cmd dashboards.SaveDashboardCommand, sqlSt
 }
 
 func callSaveWithError(t *testing.T, cmd dashboards.SaveDashboardCommand, sqlStore db.DB) error {
+	features := featuremgmt.WithFeatures()
 	dto := toSaveDashboardDto(cmd)
 	cfg := setting.NewCfg()
-	cfg.RBACEnabled = false
-	cfg.IsFeatureToggleEnabled = featuremgmt.WithFeatures().IsEnabled
 	quotaService := quotatest.New(false, nil)
-	dashboardStore, err := database.ProvideDashboardStore(sqlStore, cfg, featuremgmt.WithFeatures(), tagimpl.ProvideService(sqlStore, cfg), quotaService)
+	dashboardStore, err := database.ProvideDashboardStore(sqlStore, cfg, features, tagimpl.ProvideService(sqlStore), quotaService)
 	require.NoError(t, err)
 	folderStore := folderimpl.ProvideDashboardFolderStore(sqlStore)
-	service := ProvideDashboardServiceImpl(
-		cfg, dashboardStore, folderStore, &dummyDashAlertExtractor{},
+	service, err := ProvideDashboardServiceImpl(
+		cfg, dashboardStore, folderStore,
 		featuremgmt.WithFeatures(),
 		accesscontrolmock.NewMockedPermissionsService(),
 		accesscontrolmock.NewMockedPermissionsService(),
-		accesscontrolmock.New(),
+		actest.FakeAccessControl{},
 		foldertest.NewFakeService(),
+		folder.NewFakeStore(),
+		nil,
 	)
+	require.NoError(t, err)
 	_, err = service.SaveDashboard(context.Background(), &dto, false)
 	return err
 }
 
-func saveTestDashboard(t *testing.T, title string, orgID, folderID int64, sqlStore db.DB) *dashboards.Dashboard {
+func saveTestDashboard(t *testing.T, title string, orgID int64, folderUID string, sqlStore db.DB) *dashboards.Dashboard {
 	t.Helper()
 
 	cmd := dashboards.SaveDashboardCommand{
-		OrgID:    orgID,
-		FolderID: folderID,
-		IsFolder: false,
-		Dashboard: simplejson.NewFromAny(map[string]interface{}{
+		OrgID:     orgID,
+		FolderUID: folderUID,
+		IsFolder:  false,
+		Dashboard: simplejson.NewFromAny(map[string]any{
 			"id":    nil,
 			"title": title,
 		}),
@@ -945,23 +995,27 @@ func saveTestDashboard(t *testing.T, title string, orgID, folderID int64, sqlSto
 			OrgRole: org.RoleAdmin,
 		},
 	}
-
+	features := featuremgmt.WithFeatures()
 	cfg := setting.NewCfg()
-	cfg.RBACEnabled = false
-	cfg.IsFeatureToggleEnabled = featuremgmt.WithFeatures().IsEnabled
 	quotaService := quotatest.New(false, nil)
-	dashboardStore, err := database.ProvideDashboardStore(sqlStore, cfg, featuremgmt.WithFeatures(), tagimpl.ProvideService(sqlStore, cfg), quotaService)
+	dashboardStore, err := database.ProvideDashboardStore(sqlStore, cfg, features, tagimpl.ProvideService(sqlStore), quotaService)
 	require.NoError(t, err)
 	folderStore := folderimpl.ProvideDashboardFolderStore(sqlStore)
-	service := ProvideDashboardServiceImpl(
-		cfg, dashboardStore, folderStore, &dummyDashAlertExtractor{},
-		featuremgmt.WithFeatures(),
+	dashboardPermissions := accesscontrolmock.NewMockedPermissionsService()
+	dashboardPermissions.On("SetPermissions", mock.Anything, mock.Anything, mock.Anything, mock.Anything).Return([]accesscontrol.ResourcePermission{}, nil)
+	service, err := ProvideDashboardServiceImpl(
+		cfg, dashboardStore, folderStore,
+		features,
 		accesscontrolmock.NewMockedPermissionsService(),
-		accesscontrolmock.NewMockedPermissionsService(),
-		accesscontrolmock.New(),
+		dashboardPermissions,
+		actest.FakeAccessControl{},
 		foldertest.NewFakeService(),
+		folder.NewFakeStore(),
+		nil,
 	)
+	require.NoError(t, err)
 	res, err := service.SaveDashboard(context.Background(), &dto, false)
+
 	require.NoError(t, err)
 
 	return res
@@ -970,10 +1024,10 @@ func saveTestDashboard(t *testing.T, title string, orgID, folderID int64, sqlSto
 func saveTestFolder(t *testing.T, title string, orgID int64, sqlStore db.DB) *dashboards.Dashboard {
 	t.Helper()
 	cmd := dashboards.SaveDashboardCommand{
-		OrgID:    orgID,
-		FolderID: 0,
-		IsFolder: true,
-		Dashboard: simplejson.NewFromAny(map[string]interface{}{
+		OrgID:     orgID,
+		FolderUID: "",
+		IsFolder:  true,
+		Dashboard: simplejson.NewFromAny(map[string]any{
 			"id":    nil,
 			"title": title,
 		}),
@@ -983,26 +1037,34 @@ func saveTestFolder(t *testing.T, title string, orgID int64, sqlStore db.DB) *da
 		OrgID:     orgID,
 		Dashboard: cmd.GetDashboardModel(),
 		User: &user.SignedInUser{
+			OrgID:   orgID,
 			UserID:  1,
 			OrgRole: org.RoleAdmin,
+			Permissions: map[int64]map[string][]string{
+				orgID: {dashboards.ActionFoldersWrite: {dashboards.ScopeFoldersAll}, dashboards.ActionDashboardsWrite: {dashboards.ScopeDashboardsAll}},
+			},
 		},
 	}
 
+	features := featuremgmt.WithFeatures()
 	cfg := setting.NewCfg()
-	cfg.RBACEnabled = false
-	cfg.IsFeatureToggleEnabled = featuremgmt.WithFeatures().IsEnabled
 	quotaService := quotatest.New(false, nil)
-	dashboardStore, err := database.ProvideDashboardStore(sqlStore, cfg, featuremgmt.WithFeatures(), tagimpl.ProvideService(sqlStore, cfg), quotaService)
+	dashboardStore, err := database.ProvideDashboardStore(sqlStore, cfg, features, tagimpl.ProvideService(sqlStore), quotaService)
 	require.NoError(t, err)
 	folderStore := folderimpl.ProvideDashboardFolderStore(sqlStore)
-	service := ProvideDashboardServiceImpl(
-		cfg, dashboardStore, folderStore, &dummyDashAlertExtractor{},
+	folderPermissions := accesscontrolmock.NewMockedPermissionsService()
+	folderPermissions.On("SetPermissions", mock.Anything, mock.Anything, mock.Anything, mock.Anything).Return([]accesscontrol.ResourcePermission{}, nil)
+	service, err := ProvideDashboardServiceImpl(
+		cfg, dashboardStore, folderStore,
 		featuremgmt.WithFeatures(),
+		folderPermissions,
 		accesscontrolmock.NewMockedPermissionsService(),
-		accesscontrolmock.NewMockedPermissionsService(),
-		accesscontrolmock.New(),
+		actest.FakeAccessControl{},
 		foldertest.NewFakeService(),
+		folder.NewFakeStore(),
+		nil,
 	)
+	require.NoError(t, err)
 	res, err := service.SaveDashboard(context.Background(), &dto, false)
 	require.NoError(t, err)
 
@@ -1020,14 +1082,3 @@ func toSaveDashboardDto(cmd dashboards.SaveDashboardCommand) dashboards.SaveDash
 		Overwrite: cmd.Overwrite,
 	}
 }
-
-type dummyDashAlertExtractor struct {
-}
-
-func (d *dummyDashAlertExtractor) GetAlerts(ctx context.Context, dashAlertInfo alerting.DashAlertInfo) ([]*models.Alert, error) {
-	return nil, nil
-}
-
-func (d *dummyDashAlertExtractor) ValidateAlerts(ctx context.Context, dashAlertInfo alerting.DashAlertInfo) error {
-	return nil
-}
