@@ -3,18 +3,25 @@ package folderimpl
 import (
 	"context"
 	"fmt"
+	"path"
+	"slices"
+	"sort"
 	"testing"
 
 	"github.com/google/go-cmp/cmp"
+	"github.com/google/uuid"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 
-	"github.com/grafana/grafana/pkg/services/featuremgmt"
+	"github.com/grafana/grafana/pkg/infra/db"
+	"github.com/grafana/grafana/pkg/services/accesscontrol"
 	"github.com/grafana/grafana/pkg/services/folder"
 	"github.com/grafana/grafana/pkg/services/org"
 	"github.com/grafana/grafana/pkg/services/org/orgimpl"
 	"github.com/grafana/grafana/pkg/services/quota/quotatest"
 	"github.com/grafana/grafana/pkg/services/sqlstore"
+	"github.com/grafana/grafana/pkg/services/user"
+	"github.com/grafana/grafana/pkg/setting"
 	"github.com/grafana/grafana/pkg/util"
 )
 
@@ -26,10 +33,10 @@ func TestIntegrationCreate(t *testing.T) {
 		t.Skip("skipping integration test")
 	}
 
-	db := sqlstore.InitTestDB(t)
-	folderStore := ProvideStore(db, db.Cfg, featuremgmt.WithFeatures(featuremgmt.FlagNestedFolders))
+	db, cfg := sqlstore.InitTestDB(t)
+	folderStore := ProvideStore(db)
 
-	orgID := CreateOrg(t, db)
+	orgID := CreateOrg(t, db, cfg)
 
 	t.Run("creating a folder without providing a UID should fail", func(t *testing.T) {
 		_, err := folderStore.Create(context.Background(), folder.CreateFolderCommand{
@@ -62,15 +69,16 @@ func TestIntegrationCreate(t *testing.T) {
 		require.NoError(t, err)
 
 		t.Cleanup(func() {
-			err := folderStore.Delete(context.Background(), f.UID, orgID)
+			err := folderStore.Delete(context.Background(), []string{f.UID}, orgID)
 			require.NoError(t, err)
 		})
 
 		assert.Equal(t, folderTitle, f.Title)
 		assert.Equal(t, folderDsc, f.Description)
-		assert.NotEmpty(t, f.ID)
+		assert.NotEmpty(t, f.UID)
 		assert.Equal(t, uid, f.UID)
 		assert.Empty(t, f.ParentUID)
+		assert.NotEmpty(t, f.URL)
 
 		ff, err := folderStore.Get(context.Background(), folder.GetFolderQuery{
 			UID:   &f.UID,
@@ -80,6 +88,7 @@ func TestIntegrationCreate(t *testing.T) {
 		assert.Equal(t, folderTitle, ff.Title)
 		assert.Equal(t, folderDsc, ff.Description)
 		assert.Empty(t, ff.ParentUID)
+		assert.NotEmpty(t, ff.URL)
 
 		assertAncestorUIDs(t, folderStore, f, []string{folder.GeneralFolderUID})
 	})
@@ -93,11 +102,12 @@ func TestIntegrationCreate(t *testing.T) {
 		})
 		require.NoError(t, err)
 		require.Equal(t, "parent", parent.Title)
-		require.NotEmpty(t, parent.ID)
+		require.NotEmpty(t, parent.UID)
 		assert.Equal(t, parentUID, parent.UID)
+		assert.NotEmpty(t, parent.URL)
 
 		t.Cleanup(func() {
-			err := folderStore.Delete(context.Background(), parent.UID, orgID)
+			err := folderStore.Delete(context.Background(), []string{parent.UID}, orgID)
 			require.NoError(t, err)
 		})
 		assertAncestorUIDs(t, folderStore, parent, []string{folder.GeneralFolderUID})
@@ -112,15 +122,16 @@ func TestIntegrationCreate(t *testing.T) {
 		})
 		require.NoError(t, err)
 		t.Cleanup(func() {
-			err := folderStore.Delete(context.Background(), f.UID, orgID)
+			err := folderStore.Delete(context.Background(), []string{f.UID}, orgID)
 			require.NoError(t, err)
 		})
 
 		assert.Equal(t, folderTitle, f.Title)
 		assert.Equal(t, folderDsc, f.Description)
-		assert.NotEmpty(t, f.ID)
+		assert.NotEmpty(t, f.UID)
 		assert.Equal(t, uid, f.UID)
 		assert.Equal(t, parentUID, f.ParentUID)
+		assert.NotEmpty(t, f.URL)
 
 		assertAncestorUIDs(t, folderStore, f, []string{folder.GeneralFolderUID, parent.UID})
 		assertChildrenUIDs(t, folderStore, parent, []string{f.UID})
@@ -133,6 +144,7 @@ func TestIntegrationCreate(t *testing.T) {
 		assert.Equal(t, folderTitle, ff.Title)
 		assert.Equal(t, folderDsc, ff.Description)
 		assert.Equal(t, parentUID, ff.ParentUID)
+		assert.NotEmpty(t, ff.URL)
 	})
 }
 
@@ -141,10 +153,10 @@ func TestIntegrationDelete(t *testing.T) {
 		t.Skip("skipping integration test")
 	}
 
-	db := sqlstore.InitTestDB(t)
-	folderStore := ProvideStore(db, db.Cfg, featuremgmt.WithFeatures(featuremgmt.FlagNestedFolders))
+	db, cfg := sqlstore.InitTestDB(t)
+	folderStore := ProvideStore(db)
 
-	orgID := CreateOrg(t, db)
+	orgID := CreateOrg(t, db, cfg)
 
 	/*
 		t.Run("attempt to delete unknown folder should fail", func(t *testing.T) {
@@ -158,7 +170,7 @@ func TestIntegrationDelete(t *testing.T) {
 
 	t.Cleanup(func() {
 		for _, uid := range ancestorUIDs[1:] {
-			err := folderStore.Delete(context.Background(), uid, orgID)
+			err := folderStore.Delete(context.Background(), []string{uid}, orgID)
 			require.NoError(t, err)
 		}
 	})
@@ -171,7 +183,7 @@ func TestIntegrationDelete(t *testing.T) {
 	*/
 
 	t.Run("deleting a leaf folder should succeed", func(t *testing.T) {
-		err := folderStore.Delete(context.Background(), ancestorUIDs[len(ancestorUIDs)-1], orgID)
+		err := folderStore.Delete(context.Background(), []string{ancestorUIDs[len(ancestorUIDs)-1]}, orgID)
 		require.NoError(t, err)
 
 		children, err := folderStore.GetChildren(context.Background(), folder.GetChildrenQuery{
@@ -188,10 +200,10 @@ func TestIntegrationUpdate(t *testing.T) {
 		t.Skip("skipping integration test")
 	}
 
-	db := sqlstore.InitTestDB(t)
-	folderStore := ProvideStore(db, db.Cfg, featuremgmt.WithFeatures(featuremgmt.FlagNestedFolders))
+	db, cfg := sqlstore.InitTestDB(t)
+	folderStore := ProvideStore(db)
 
-	orgID := CreateOrg(t, db)
+	orgID := CreateOrg(t, db, cfg)
 
 	// create parent folder
 	parent, err := folderStore.Create(context.Background(), folder.CreateFolderCommand{
@@ -214,7 +226,7 @@ func TestIntegrationUpdate(t *testing.T) {
 	require.NoError(t, err)
 	require.Equal(t, f.ParentUID, parent.UID)
 	t.Cleanup(func() {
-		err := folderStore.Delete(context.Background(), f.UID, orgID)
+		err := folderStore.Delete(context.Background(), []string{f.UID}, orgID)
 		require.NoError(t, err)
 	})
 
@@ -263,6 +275,7 @@ func TestIntegrationUpdate(t *testing.T) {
 		assert.Equal(t, newTitle, updated.Title)
 		assert.Equal(t, newDesc, updated.Description)
 		assert.Equal(t, parent.UID, updated.ParentUID)
+		assert.NotEmpty(t, updated.URL)
 		// assert.GreaterOrEqual(t, updated.Updated.UnixNano(), existingUpdated.UnixNano())
 
 		updated, err = folderStore.Get(context.Background(), folder.GetFolderQuery{
@@ -274,33 +287,11 @@ func TestIntegrationUpdate(t *testing.T) {
 		assert.Equal(t, newDesc, updated.Description)
 		// parent should not change
 		assert.Equal(t, parent.UID, updated.ParentUID)
+		assert.NotEmpty(t, updated.URL)
 
 		f = updated
 	})
 
-	t.Run("updating folder UID should succeed", func(t *testing.T) {
-		newUID := "new"
-		existingTitle := f.Title
-		existingDesc := f.Description
-		updated, err := folderStore.Update(context.Background(), folder.UpdateFolderCommand{
-			UID:    f.UID,
-			OrgID:  f.OrgID,
-			NewUID: &newUID,
-		})
-		require.NoError(t, err)
-
-		assert.Equal(t, newUID, updated.UID)
-
-		updated, err = folderStore.Get(context.Background(), folder.GetFolderQuery{
-			UID:   &updated.UID,
-			OrgID: orgID,
-		})
-		require.NoError(t, err)
-		assert.Equal(t, newUID, updated.UID)
-		assert.Equal(t, existingTitle, updated.Title)
-		assert.Equal(t, existingDesc, updated.Description)
-	})
-
 	t.Run("updating folder parent UID", func(t *testing.T) {
 		testCases := []struct {
 			desc                  string
@@ -373,6 +364,7 @@ func TestIntegrationUpdate(t *testing.T) {
 				assert.Equal(t, existingTitle, updated.Title)
 				assert.Equal(t, existingDesc, updated.Description)
 				assert.Equal(t, existingUID, updated.UID)
+				assert.NotEmpty(t, updated.URL)
 			})
 		}
 	})
@@ -383,10 +375,10 @@ func TestIntegrationGet(t *testing.T) {
 		t.Skip("skipping integration test")
 	}
 
-	db := sqlstore.InitTestDB(t)
-	folderStore := ProvideStore(db, db.Cfg, featuremgmt.WithFeatures(featuremgmt.FlagNestedFolders))
+	db, cfg := sqlstore.InitTestDB(t)
+	folderStore := ProvideStore(db)
 
-	orgID := CreateOrg(t, db)
+	orgID := CreateOrg(t, db, cfg)
 
 	// create folder
 	uid1 := util.GenerateShortUID()
@@ -397,11 +389,14 @@ func TestIntegrationGet(t *testing.T) {
 		UID:         uid1,
 	})
 	require.NoError(t, err)
-
-	t.Cleanup(func() {
-		err := folderStore.Delete(context.Background(), f.UID, orgID)
-		require.NoError(t, err)
+	subfolderWithSameName, err := folderStore.Create(context.Background(), folder.CreateFolderCommand{
+		Title:       folderTitle,
+		Description: folderDsc,
+		OrgID:       orgID,
+		UID:         util.GenerateShortUID(),
+		ParentUID:   f.UID,
 	})
+	require.NoError(t, err)
 
 	t.Run("should gently fail in case of bad request", func(t *testing.T) {
 		_, err = folderStore.Get(context.Background(), folder.GetFolderQuery{})
@@ -414,7 +409,6 @@ func TestIntegrationGet(t *testing.T) {
 			OrgID: orgID,
 		})
 		require.NoError(t, err)
-		assert.Equal(t, f.ID, ff.ID)
 		assert.Equal(t, f.UID, ff.UID)
 		assert.Equal(t, f.OrgID, ff.OrgID)
 		assert.Equal(t, f.Title, ff.Title)
@@ -422,6 +416,7 @@ func TestIntegrationGet(t *testing.T) {
 		//assert.Equal(t, folder.GeneralFolderUID, ff.ParentUID)
 		assert.NotEmpty(t, ff.Created)
 		assert.NotEmpty(t, ff.Updated)
+		assert.NotEmpty(t, ff.URL)
 	})
 
 	t.Run("get folder by title should succeed", func(t *testing.T) {
@@ -430,7 +425,6 @@ func TestIntegrationGet(t *testing.T) {
 			OrgID: orgID,
 		})
 		require.NoError(t, err)
-		assert.Equal(t, f.ID, ff.ID)
 		assert.Equal(t, f.UID, ff.UID)
 		assert.Equal(t, f.OrgID, ff.OrgID)
 		assert.Equal(t, f.Title, ff.Title)
@@ -438,14 +432,32 @@ func TestIntegrationGet(t *testing.T) {
 		//assert.Equal(t, folder.GeneralFolderUID, ff.ParentUID)
 		assert.NotEmpty(t, ff.Created)
 		assert.NotEmpty(t, ff.Updated)
+		assert.NotEmpty(t, ff.URL)
+	})
+
+	t.Run("get folder by title and parent UID should succeed", func(t *testing.T) {
+		ff, err := folderStore.Get(context.Background(), folder.GetFolderQuery{
+			Title:     &f.Title,
+			OrgID:     orgID,
+			ParentUID: &uid1,
+		})
+		require.NoError(t, err)
+		assert.Equal(t, subfolderWithSameName.UID, ff.UID)
+		assert.Equal(t, subfolderWithSameName.OrgID, ff.OrgID)
+		assert.Equal(t, subfolderWithSameName.Title, ff.Title)
+		assert.Equal(t, subfolderWithSameName.Description, ff.Description)
+		assert.Equal(t, subfolderWithSameName.ParentUID, ff.ParentUID)
+		assert.NotEmpty(t, ff.Created)
+		assert.NotEmpty(t, ff.Updated)
+		assert.NotEmpty(t, ff.URL)
 	})
 
 	t.Run("get folder by title should succeed", func(t *testing.T) {
 		ff, err := folderStore.Get(context.Background(), folder.GetFolderQuery{
-			ID: &f.ID,
+			UID:   &f.UID,
+			OrgID: orgID,
 		})
 		require.NoError(t, err)
-		assert.Equal(t, f.ID, ff.ID)
 		assert.Equal(t, f.UID, ff.UID)
 		assert.Equal(t, f.OrgID, ff.OrgID)
 		assert.Equal(t, f.Title, ff.Title)
@@ -453,6 +465,25 @@ func TestIntegrationGet(t *testing.T) {
 		//assert.Equal(t, folder.GeneralFolderUID, ff.ParentUID)
 		assert.NotEmpty(t, ff.Created)
 		assert.NotEmpty(t, ff.Updated)
+		assert.NotEmpty(t, ff.URL)
+	})
+
+	t.Run("get folder with fullpath should set fullpath as expected", func(t *testing.T) {
+		ff, err := folderStore.Get(context.Background(), folder.GetFolderQuery{
+			UID:          &subfolderWithSameName.UID,
+			OrgID:        orgID,
+			WithFullpath: true,
+		})
+		require.NoError(t, err)
+		assert.Equal(t, subfolderWithSameName.UID, ff.UID)
+		assert.Equal(t, subfolderWithSameName.OrgID, ff.OrgID)
+		assert.Equal(t, subfolderWithSameName.Title, ff.Title)
+		assert.Equal(t, subfolderWithSameName.Description, ff.Description)
+		assert.Equal(t, path.Join(f.Title, subfolderWithSameName.Title), ff.Fullpath)
+		assert.Equal(t, f.UID, ff.ParentUID)
+		assert.NotEmpty(t, ff.Created)
+		assert.NotEmpty(t, ff.Updated)
+		assert.NotEmpty(t, ff.URL)
 	})
 }
 
@@ -461,10 +492,10 @@ func TestIntegrationGetParents(t *testing.T) {
 		t.Skip("skipping integration test")
 	}
 
-	db := sqlstore.InitTestDB(t)
-	folderStore := ProvideStore(db, db.Cfg, featuremgmt.WithFeatures(featuremgmt.FlagNestedFolders))
+	db, cfg := sqlstore.InitTestDB(t)
+	folderStore := ProvideStore(db)
 
-	orgID := CreateOrg(t, db)
+	orgID := CreateOrg(t, db, cfg)
 
 	// create folder
 	uid1 := util.GenerateShortUID()
@@ -477,7 +508,7 @@ func TestIntegrationGetParents(t *testing.T) {
 	require.NoError(t, err)
 
 	t.Cleanup(func() {
-		err := folderStore.Delete(context.Background(), f.UID, orgID)
+		err := folderStore.Delete(context.Background(), []string{f.UID}, orgID)
 		require.NoError(t, err)
 	})
 
@@ -517,6 +548,7 @@ func TestIntegrationGetParents(t *testing.T) {
 		require.NoError(t, err)
 		parentUIDs := make([]string, 0)
 		for _, p := range parents {
+			assert.NotEmpty(t, p.URL)
 			parentUIDs = append(parentUIDs, p.UID)
 		}
 		require.Equal(t, []string{uid1}, parentUIDs)
@@ -528,10 +560,10 @@ func TestIntegrationGetChildren(t *testing.T) {
 		t.Skip("skipping integration test")
 	}
 
-	db := sqlstore.InitTestDB(t)
-	folderStore := ProvideStore(db, db.Cfg, featuremgmt.WithFeatures(featuremgmt.FlagNestedFolders))
+	db, cfg := sqlstore.InitTestDB(t)
+	folderStore := ProvideStore(db)
 
-	orgID := CreateOrg(t, db)
+	orgID := CreateOrg(t, db, cfg)
 
 	// create folder
 	uid1 := util.GenerateShortUID()
@@ -544,10 +576,11 @@ func TestIntegrationGetChildren(t *testing.T) {
 	require.NoError(t, err)
 
 	treeLeaves := CreateLeaves(t, folderStore, parent, 8)
+	sort.Strings(treeLeaves)
 
 	t.Cleanup(func() {
 		for _, uid := range treeLeaves {
-			err := folderStore.Delete(context.Background(), uid, orgID)
+			err := folderStore.Delete(context.Background(), []string{uid}, orgID)
 			require.NoError(t, err)
 		}
 	})
@@ -568,6 +601,7 @@ func TestIntegrationGetChildren(t *testing.T) {
 
 		childrenUIDs := make([]string, 0, len(children))
 		for _, c := range children {
+			assert.NotEmpty(t, c.URL)
 			childrenUIDs = append(childrenUIDs, c.UID)
 		}
 
@@ -584,6 +618,7 @@ func TestIntegrationGetChildren(t *testing.T) {
 
 		childrenUIDs := make([]string, 0, len(children))
 		for _, c := range children {
+			assert.NotEmpty(t, c.URL)
 			childrenUIDs = append(childrenUIDs, c.UID)
 		}
 		assert.Equal(t, []string{parent.UID}, childrenUIDs)
@@ -616,6 +651,7 @@ func TestIntegrationGetChildren(t *testing.T) {
 
 		childrenUIDs = make([]string, 0, len(children))
 		for _, c := range children {
+			assert.NotEmpty(t, c.URL)
 			childrenUIDs = append(childrenUIDs, c.UID)
 		}
 
@@ -633,6 +669,7 @@ func TestIntegrationGetChildren(t *testing.T) {
 
 		childrenUIDs = make([]string, 0, len(children))
 		for _, c := range children {
+			assert.NotEmpty(t, c.URL)
 			childrenUIDs = append(childrenUIDs, c.UID)
 		}
 
@@ -640,6 +677,26 @@ func TestIntegrationGetChildren(t *testing.T) {
 			t.Errorf("Result mismatch (-want +got):\n%s", diff)
 		}
 
+		// fetch folder with specific UIDs and pagination
+		children, err = folderStore.GetChildren(context.Background(), folder.GetChildrenQuery{
+			UID:        parent.UID,
+			OrgID:      orgID,
+			Limit:      2,
+			Page:       1,
+			FolderUIDs: treeLeaves[3:4],
+		})
+		require.NoError(t, err)
+
+		childrenUIDs = make([]string, 0, len(children))
+		for _, c := range children {
+			assert.NotEmpty(t, c.URL)
+			childrenUIDs = append(childrenUIDs, c.UID)
+		}
+
+		if diff := cmp.Diff(treeLeaves[3:4], childrenUIDs); diff != "" {
+			t.Errorf("Result mismatch (-want +got):\n%s", diff)
+		}
+
 		// no page is set
 		children, err = folderStore.GetChildren(context.Background(), folder.GetChildrenQuery{
 			UID:   parent.UID,
@@ -650,6 +707,7 @@ func TestIntegrationGetChildren(t *testing.T) {
 
 		childrenUIDs = make([]string, 0, len(children))
 		for _, c := range children {
+			assert.NotEmpty(t, c.URL)
 			childrenUIDs = append(childrenUIDs, c.UID)
 		}
 
@@ -667,6 +725,7 @@ func TestIntegrationGetChildren(t *testing.T) {
 
 		childrenUIDs = make([]string, 0, len(children))
 		for _, c := range children {
+			assert.NotEmpty(t, c.URL)
 			childrenUIDs = append(childrenUIDs, c.UID)
 		}
 
@@ -674,6 +733,68 @@ func TestIntegrationGetChildren(t *testing.T) {
 			t.Errorf("Result mismatch (-want +got):\n%s", diff)
 		}
 	})
+
+	t.Run("should hide k6-app folder for users but not for service accounts", func(t *testing.T) {
+		_, err = folderStore.Create(context.Background(), folder.CreateFolderCommand{
+			Title: "k6-app-folder",
+			OrgID: orgID,
+			UID:   accesscontrol.K6FolderUID,
+		})
+		require.NoError(t, err)
+
+		children, err := folderStore.GetChildren(context.Background(), folder.GetChildrenQuery{
+			OrgID:        orgID,
+			SignedInUser: usr,
+		})
+		require.NoError(t, err)
+		require.Equal(t, 1, len(children))
+		assert.Equal(t, parent.UID, children[0].UID)
+
+		// Service account should be able to list k6 folder
+		children, err = folderStore.GetChildren(context.Background(), folder.GetChildrenQuery{
+			OrgID:        orgID,
+			SignedInUser: &user.SignedInUser{UserID: 2, OrgID: orgID, IsServiceAccount: true},
+		})
+		require.NoError(t, err)
+		require.Equal(t, 2, len(children))
+		childrenUIDs := make([]string, 0, len(children))
+		for _, child := range children {
+			childrenUIDs = append(childrenUIDs, child.UID)
+		}
+		assert.EqualValues(t, []string{parent.UID, accesscontrol.K6FolderUID}, childrenUIDs)
+	})
+
+	t.Run("pagination works if k6-app folder is hidden", func(t *testing.T) {
+		for i := 0; i < 4; i++ {
+			_, err = folderStore.Create(context.Background(), folder.CreateFolderCommand{
+				Title: fmt.Sprintf("root-%d", i),
+				OrgID: orgID,
+				UID:   fmt.Sprintf("root-%d", i),
+			})
+			require.NoError(t, err)
+		}
+
+		// Should skip k6-app folder but get parent folder and two more folders
+		children, err := folderStore.GetChildren(context.Background(), folder.GetChildrenQuery{
+			OrgID:        orgID,
+			SignedInUser: usr,
+			Limit:        3,
+		})
+		require.NoError(t, err)
+		require.Equal(t, 3, len(children))
+		assert.EqualValues(t, []string{parent.UID, "root-0", "root-1"}, []string{children[0].UID, children[1].UID, children[2].UID})
+
+		// Should get the two remaining folders
+		children, err = folderStore.GetChildren(context.Background(), folder.GetChildrenQuery{
+			OrgID:        orgID,
+			SignedInUser: usr,
+			Page:         2,
+			Limit:        3,
+		})
+		require.NoError(t, err)
+		require.Equal(t, 2, len(children))
+		assert.EqualValues(t, []string{"root-2", "root-3"}, []string{children[0].UID, children[1].UID})
+	})
 }
 
 func TestIntegrationGetHeight(t *testing.T) {
@@ -681,10 +802,10 @@ func TestIntegrationGetHeight(t *testing.T) {
 		t.Skip("skipping integration test")
 	}
 
-	db := sqlstore.InitTestDB(t)
-	folderStore := ProvideStore(db, db.Cfg, featuremgmt.WithFeatures(featuremgmt.FlagNestedFolders))
+	db, cfg := sqlstore.InitTestDB(t)
+	folderStore := ProvideStore(db)
 
-	orgID := CreateOrg(t, db)
+	orgID := CreateOrg(t, db, cfg)
 
 	// create folder
 	uid1 := util.GenerateShortUID()
@@ -708,10 +829,133 @@ func TestIntegrationGetHeight(t *testing.T) {
 	})
 }
 
-func CreateOrg(t *testing.T, db *sqlstore.SQLStore) int64 {
+func TestIntegrationGetFolders(t *testing.T) {
+	if testing.Short() {
+		t.Skip("skipping integration test")
+	}
+
+	foldersNum := 10
+	db, cfg := sqlstore.InitTestDB(t)
+	folderStore := ProvideStore(db)
+
+	orgID := CreateOrg(t, db, cfg)
+
+	// create folders
+	uids := make([]string, 0)
+	folders := make([]*folder.Folder, 0)
+	for i := 0; i < foldersNum; i++ {
+		uid := util.GenerateShortUID()
+		f, err := folderStore.Create(context.Background(), folder.CreateFolderCommand{
+			Title:       folderTitle,
+			Description: folderDsc,
+			OrgID:       orgID,
+			UID:         uid,
+		})
+		require.NoError(t, err)
+
+		uids = append(uids, uid)
+		folders = append(folders, f)
+	}
+
+	t.Cleanup(func() {
+		for _, uid := range uids {
+			err := folderStore.Delete(context.Background(), []string{uid}, orgID)
+			require.NoError(t, err)
+		}
+	})
+
+	t.Run("get folders by UIDs should succeed", func(t *testing.T) {
+		actualFolders, err := folderStore.GetFolders(context.Background(), folder.NewGetFoldersQuery(folder.GetFoldersQuery{OrgID: orgID, UIDs: uids[1:]}))
+		require.NoError(t, err)
+		assert.Equal(t, len(uids[1:]), len(actualFolders))
+		for _, f := range folders[1:] {
+			folderInResponseIdx := slices.IndexFunc(actualFolders, func(rf *folder.Folder) bool {
+				return rf.UID == f.UID
+			})
+			assert.NotEqual(t, -1, folderInResponseIdx)
+			actualFolder := actualFolders[folderInResponseIdx]
+			assert.Equal(t, f.UID, actualFolder.UID)
+			assert.Equal(t, f.OrgID, actualFolder.OrgID)
+			assert.Equal(t, f.Title, actualFolder.Title)
+			assert.Equal(t, f.Description, actualFolder.Description)
+			assert.NotEmpty(t, actualFolder.Created)
+			assert.NotEmpty(t, actualFolder.Updated)
+			assert.NotEmpty(t, actualFolder.URL)
+		}
+	})
+
+	t.Run("get folders by UIDs batching should work as expected", func(t *testing.T) {
+		q := folder.NewGetFoldersQuery(folder.GetFoldersQuery{OrgID: orgID, UIDs: uids[1:], BatchSize: 3})
+		actualFolders, err := folderStore.GetFolders(context.Background(), q)
+		require.NoError(t, err)
+		assert.Equal(t, len(uids[1:]), len(actualFolders))
+		for _, f := range folders[1:] {
+			folderInResponseIdx := slices.IndexFunc(actualFolders, func(rf *folder.Folder) bool {
+				return rf.UID == f.UID
+			})
+			assert.NotEqual(t, -1, folderInResponseIdx)
+			actualFolder := actualFolders[folderInResponseIdx]
+			assert.Equal(t, f.UID, actualFolder.UID)
+			assert.Equal(t, f.OrgID, actualFolder.OrgID)
+			assert.Equal(t, f.Title, actualFolder.Title)
+			assert.Equal(t, f.Description, actualFolder.Description)
+			assert.NotEmpty(t, actualFolder.Created)
+			assert.NotEmpty(t, actualFolder.Updated)
+			assert.NotEmpty(t, actualFolder.URL)
+		}
+	})
+
+	t.Run("get folders by UIDs with fullpath should succeed", func(t *testing.T) {
+		q := folder.NewGetFoldersQuery(folder.GetFoldersQuery{OrgID: orgID, UIDs: uids[1:], WithFullpath: true})
+		q.BatchSize = 3
+		actualFolders, err := folderStore.GetFolders(context.Background(), q)
+		require.NoError(t, err)
+		assert.Equal(t, len(uids[1:]), len(actualFolders))
+		for _, f := range folders[1:] {
+			folderInResponseIdx := slices.IndexFunc(actualFolders, func(rf *folder.Folder) bool {
+				return rf.UID == f.UID
+			})
+			assert.NotEqual(t, -1, folderInResponseIdx)
+			actualFolder := actualFolders[folderInResponseIdx]
+			assert.Equal(t, f.UID, actualFolder.UID)
+			assert.Equal(t, f.OrgID, actualFolder.OrgID)
+			assert.Equal(t, f.Title, actualFolder.Title)
+			assert.Equal(t, f.Description, actualFolder.Description)
+			assert.NotEmpty(t, actualFolder.Created)
+			assert.NotEmpty(t, actualFolder.Updated)
+			assert.NotEmpty(t, actualFolder.URL)
+			assert.NotEmpty(t, actualFolder.Fullpath)
+		}
+	})
+
+	t.Run("get folders by UIDs and ancestor UIDs should work as expected", func(t *testing.T) {
+		q := folder.NewGetFoldersQuery(folder.GetFoldersQuery{OrgID: orgID, UIDs: uids[1:], BatchSize: 3})
+		q.AncestorUIDs = make([]string, 0, int(q.BatchSize)+1)
+		for i := 0; i < int(q.BatchSize); i++ {
+			q.AncestorUIDs = append(q.AncestorUIDs, uuid.New().String())
+		}
+		q.AncestorUIDs = append(q.AncestorUIDs, folders[len(folders)-1].UID)
+
+		actualFolders, err := folderStore.GetFolders(context.Background(), q)
+		require.NoError(t, err)
+		assert.Equal(t, 1, len(actualFolders))
+
+		f := folders[len(folders)-1]
+		actualFolder := actualFolders[0]
+		assert.Equal(t, f.UID, actualFolder.UID)
+		assert.Equal(t, f.OrgID, actualFolder.OrgID)
+		assert.Equal(t, f.Title, actualFolder.Title)
+		assert.Equal(t, f.Description, actualFolder.Description)
+		assert.NotEmpty(t, actualFolder.Created)
+		assert.NotEmpty(t, actualFolder.Updated)
+		assert.NotEmpty(t, actualFolder.URL)
+	})
+}
+
+func CreateOrg(t *testing.T, db db.DB, cfg *setting.Cfg) int64 {
 	t.Helper()
 
-	orgService, err := orgimpl.ProvideService(db, db.Cfg, quotatest.New(false, nil))
+	orgService, err := orgimpl.ProvideService(db, cfg, quotatest.New(false, nil))
 	require.NoError(t, err)
 	orgID, err := orgService.GetOrCreate(context.Background(), "test-org")
 	require.NoError(t, err)
@@ -723,7 +967,7 @@ func CreateOrg(t *testing.T, db *sqlstore.SQLStore) int64 {
 	return orgID
 }
 
-func CreateSubtree(t *testing.T, store *sqlStore, orgID int64, parentUID string, depth int, prefix string) []string {
+func CreateSubtree(t *testing.T, store *FolderStoreImpl, orgID int64, parentUID string, depth int, prefix string) []string {
 	t.Helper()
 
 	ancestorUIDs := []string{}
@@ -741,7 +985,6 @@ func CreateSubtree(t *testing.T, store *sqlStore, orgID int64, parentUID string,
 		f, err := store.Create(context.Background(), cmd)
 		require.NoError(t, err)
 		require.Equal(t, title, f.Title)
-		require.NotEmpty(t, f.ID)
 		require.NotEmpty(t, f.UID)
 
 		parents, err := store.GetParents(context.Background(), folder.GetParentsQuery{
@@ -763,14 +1006,15 @@ func CreateSubtree(t *testing.T, store *sqlStore, orgID int64, parentUID string,
 	return ancestorUIDs
 }
 
-func CreateLeaves(t *testing.T, store *sqlStore, parent *folder.Folder, num int) []string {
+func CreateLeaves(t *testing.T, store *FolderStoreImpl, parent *folder.Folder, num int) []string {
 	t.Helper()
 
 	leaves := make([]string, 0)
 	for i := 0; i < num; i++ {
+		uid := util.GenerateShortUID()
 		f, err := store.Create(context.Background(), folder.CreateFolderCommand{
-			Title:     fmt.Sprintf("folder-%d", i),
-			UID:       util.GenerateShortUID(),
+			Title:     fmt.Sprintf("folder-%s", uid),
+			UID:       uid,
 			OrgID:     parent.OrgID,
 			ParentUID: parent.UID,
 		})
@@ -780,7 +1024,7 @@ func CreateLeaves(t *testing.T, store *sqlStore, parent *folder.Folder, num int)
 	return leaves
 }
 
-func assertAncestorUIDs(t *testing.T, store *sqlStore, f *folder.Folder, expected []string) {
+func assertAncestorUIDs(t *testing.T, store *FolderStoreImpl, f *folder.Folder, expected []string) {
 	t.Helper()
 
 	ancestors, err := store.GetParents(context.Background(), folder.GetParentsQuery{
@@ -795,7 +1039,7 @@ func assertAncestorUIDs(t *testing.T, store *sqlStore, f *folder.Folder, expecte
 	assert.Equal(t, expected, actualAncestorsUIDs)
 }
 
-func assertChildrenUIDs(t *testing.T, store *sqlStore, f *folder.Folder, expected []string) {
+func assertChildrenUIDs(t *testing.T, store *FolderStoreImpl, f *folder.Folder, expected []string) {
 	t.Helper()
 
 	ancestors, err := store.GetChildren(context.Background(), folder.GetChildrenQuery{
