@@ -9,28 +9,36 @@ import (
 	"github.com/stretchr/testify/mock"
 	"github.com/stretchr/testify/require"
 
-	"github.com/grafana/grafana/pkg/api/routing"
-	"github.com/grafana/grafana/pkg/infra/db"
 	"github.com/grafana/grafana/pkg/services/accesscontrol"
-	accesscontrolmock "github.com/grafana/grafana/pkg/services/accesscontrol/mock"
-	"github.com/grafana/grafana/pkg/services/accesscontrol/ossaccesscontrol"
+	"github.com/grafana/grafana/pkg/services/accesscontrol/acimpl"
+	"github.com/grafana/grafana/pkg/services/authz/zanzana"
 	"github.com/grafana/grafana/pkg/services/dashboards"
-	dashdb "github.com/grafana/grafana/pkg/services/dashboards/database"
 	"github.com/grafana/grafana/pkg/services/featuremgmt"
+	"github.com/grafana/grafana/pkg/services/folder"
 	"github.com/grafana/grafana/pkg/services/folder/foldertest"
 	"github.com/grafana/grafana/pkg/services/licensing/licensingtest"
-	"github.com/grafana/grafana/pkg/services/quota/quotatest"
-	"github.com/grafana/grafana/pkg/services/supportbundles/supportbundlestest"
-	"github.com/grafana/grafana/pkg/services/tag/tagimpl"
-	"github.com/grafana/grafana/pkg/services/team/teamimpl"
 	"github.com/grafana/grafana/pkg/services/user"
-	"github.com/grafana/grafana/pkg/services/user/userimpl"
 	"github.com/grafana/grafana/pkg/setting"
 )
 
+const (
+	orgID            = 1
+	dashUID          = "1"
+	folderID         = 42
+	folderUID        = "42"
+	invalidFolderUID = "142"
+)
+
+var (
+	folderUIDScope        = fmt.Sprintf("folders:uid:%s", folderUID)
+	invalidFolderUIDScope = fmt.Sprintf("folders:uid:%s", invalidFolderUID)
+	dashboard             = &dashboards.Dashboard{OrgID: orgID, UID: dashUID, IsFolder: false, FolderUID: folderUID}
+	fldr                  = &dashboards.Dashboard{OrgID: orgID, UID: folderUID, IsFolder: true}
+)
+
 type accessControlGuardianTestCase struct {
 	desc           string
-	dashUID        string
+	dashboard      *dashboards.Dashboard
 	permissions    []accesscontrol.Permission
 	viewersCanEdit bool
 	expected       bool
@@ -39,8 +47,8 @@ type accessControlGuardianTestCase struct {
 func TestAccessControlDashboardGuardian_CanSave(t *testing.T) {
 	tests := []accessControlGuardianTestCase{
 		{
-			desc:    "should be able to save with dashboard wildcard scope",
-			dashUID: "1",
+			desc:      "should be able to save dashboard with dashboard wildcard scope",
+			dashboard: dashboard,
 			permissions: []accesscontrol.Permission{
 				{
 					Action: dashboards.ActionDashboardsWrite,
@@ -50,8 +58,8 @@ func TestAccessControlDashboardGuardian_CanSave(t *testing.T) {
 			expected: true,
 		},
 		{
-			desc:    "should be able to save with folder wildcard scope",
-			dashUID: "1",
+			desc:      "should be able to save dashboard with folder wildcard scope",
+			dashboard: dashboard,
 			permissions: []accesscontrol.Permission{
 				{
 					Action: dashboards.ActionDashboardsWrite,
@@ -61,8 +69,8 @@ func TestAccessControlDashboardGuardian_CanSave(t *testing.T) {
 			expected: true,
 		},
 		{
-			desc:    "should be able to save with dashboard scope",
-			dashUID: "1",
+			desc:      "should be able to save dashboard with dashboard scope",
+			dashboard: dashboard,
 			permissions: []accesscontrol.Permission{
 				{
 					Action: dashboards.ActionDashboardsWrite,
@@ -72,8 +80,8 @@ func TestAccessControlDashboardGuardian_CanSave(t *testing.T) {
 			expected: true,
 		},
 		{
-			desc:    "should be able to save with folder scope",
-			dashUID: "1",
+			desc:      "should be able to save dashboard under root with general folder scope",
+			dashboard: &dashboards.Dashboard{OrgID: orgID, UID: dashUID, IsFolder: false},
 			permissions: []accesscontrol.Permission{
 				{
 					Action: dashboards.ActionDashboardsWrite,
@@ -83,8 +91,8 @@ func TestAccessControlDashboardGuardian_CanSave(t *testing.T) {
 			expected: true,
 		},
 		{
-			desc:    "should not be able to save with incorrect dashboard scope",
-			dashUID: "1",
+			desc:      "should not be able to save dashboard with incorrect dashboard scope",
+			dashboard: dashboard,
 			permissions: []accesscontrol.Permission{
 				{
 					Action: dashboards.ActionDashboardsWrite,
@@ -94,12 +102,78 @@ func TestAccessControlDashboardGuardian_CanSave(t *testing.T) {
 			expected: false,
 		},
 		{
-			desc:    "should not be able to save with incorrect folder scope",
-			dashUID: "1",
+			desc:      "should not be able to save dashboard with incorrect folder scope",
+			dashboard: dashboard,
 			permissions: []accesscontrol.Permission{
 				{
 					Action: dashboards.ActionDashboardsWrite,
-					Scope:  "folders:uid:100",
+					Scope:  invalidFolderUIDScope,
+				},
+			},
+			expected: false,
+		},
+		{
+			desc:      "should not be able to save folder with folder write and dashboard wildcard scope",
+			dashboard: fldr,
+			permissions: []accesscontrol.Permission{
+				{
+					Action: dashboards.ActionFoldersWrite,
+					Scope:  "dashboards:*",
+				},
+			},
+			expected: false,
+		},
+		{
+			desc:      "should be able to save folder with folder write and folder wildcard scope",
+			dashboard: fldr,
+			permissions: []accesscontrol.Permission{
+				{
+					Action: dashboards.ActionFoldersWrite,
+					Scope:  "folders:*",
+				},
+			},
+			expected: true,
+		},
+		{
+			desc:      "should not be able to save folder with folder write and dashboard scope",
+			dashboard: fldr,
+			permissions: []accesscontrol.Permission{
+				{
+					Action: dashboards.ActionFoldersWrite,
+					Scope:  "dashboards:uid:1",
+				},
+			},
+			expected: false,
+		},
+		{
+			desc:      "should be able to save folder with folder write and folder scope",
+			dashboard: fldr,
+			permissions: []accesscontrol.Permission{
+				{
+					Action: dashboards.ActionFoldersWrite,
+					Scope:  folderUIDScope,
+				},
+			},
+			expected: true,
+		},
+		{
+			desc:      "should not be able to save folder with folder write and incorrect dashboard scope",
+			dashboard: fldr,
+			permissions: []accesscontrol.Permission{
+				{
+					Action: dashboards.ActionFoldersWrite,
+					Scope:  "dashboards:uid:10",
+				},
+			},
+			expected: false,
+		},
+		{
+			desc:      "should not be able to save folder with folder write and incorrect folder scope",
+			dashboard: fldr,
+			permissions: []accesscontrol.Permission{
+				{
+					Action: dashboards.ActionFoldersWrite,
+					Scope:  invalidFolderUID,
 				},
 			},
 			expected: false,
@@ -108,7 +182,7 @@ func TestAccessControlDashboardGuardian_CanSave(t *testing.T) {
 
 	for _, tt := range tests {
 		t.Run(tt.desc, func(t *testing.T) {
-			guardian, _ := setupAccessControlGuardianTest(t, tt.dashUID, tt.permissions, nil, testDashSvc(t))
+			guardian := setupAccessControlGuardianTest(t, tt.dashboard, tt.permissions, nil)
 			can, err := guardian.CanSave()
 			require.NoError(t, err)
 			assert.Equal(t, tt.expected, can)
@@ -119,8 +193,8 @@ func TestAccessControlDashboardGuardian_CanSave(t *testing.T) {
 func TestAccessControlDashboardGuardian_CanEdit(t *testing.T) {
 	tests := []accessControlGuardianTestCase{
 		{
-			desc:    "should be able to edit with dashboard wildcard scope",
-			dashUID: "1",
+			desc:      "should be able to edit dashboard with dashboard wildcard scope",
+			dashboard: dashboard,
 			permissions: []accesscontrol.Permission{
 				{
 					Action: dashboards.ActionDashboardsWrite,
@@ -130,8 +204,8 @@ func TestAccessControlDashboardGuardian_CanEdit(t *testing.T) {
 			expected: true,
 		},
 		{
-			desc:    "should be able to edit with folder wildcard scope",
-			dashUID: "1",
+			desc:      "should be able to edit dashboard with folder wildcard scope",
+			dashboard: dashboard,
 			permissions: []accesscontrol.Permission{
 				{
 					Action: dashboards.ActionDashboardsWrite,
@@ -141,8 +215,8 @@ func TestAccessControlDashboardGuardian_CanEdit(t *testing.T) {
 			expected: true,
 		},
 		{
-			desc:    "should be able to edit with dashboard scope",
-			dashUID: "1",
+			desc:      "should be able to edit dashboard with dashboard scope",
+			dashboard: dashboard,
 			permissions: []accesscontrol.Permission{
 				{
 					Action: dashboards.ActionDashboardsWrite,
@@ -152,8 +226,8 @@ func TestAccessControlDashboardGuardian_CanEdit(t *testing.T) {
 			expected: true,
 		},
 		{
-			desc:    "should be able to edit with folder scope",
-			dashUID: "1",
+			desc:      "should be able to edit dashboard under root with general folder scope",
+			dashboard: &dashboards.Dashboard{OrgID: orgID, UID: dashUID, IsFolder: false},
 			permissions: []accesscontrol.Permission{
 				{
 					Action: dashboards.ActionDashboardsWrite,
@@ -163,8 +237,8 @@ func TestAccessControlDashboardGuardian_CanEdit(t *testing.T) {
 			expected: true,
 		},
 		{
-			desc:    "should not be able to edit with incorrect dashboard scope",
-			dashUID: "1",
+			desc:      "should not be able to edit dashboard with incorrect dashboard scope",
+			dashboard: dashboard,
 			permissions: []accesscontrol.Permission{
 				{
 					Action: dashboards.ActionDashboardsWrite,
@@ -174,19 +248,19 @@ func TestAccessControlDashboardGuardian_CanEdit(t *testing.T) {
 			expected: false,
 		},
 		{
-			desc:    "should not be able to edit with incorrect folder scope",
-			dashUID: "1",
+			desc:      "should not be able to edit dashboard with incorrect folder scope",
+			dashboard: dashboard,
 			permissions: []accesscontrol.Permission{
 				{
 					Action: dashboards.ActionDashboardsWrite,
-					Scope:  "folders:uid:10",
+					Scope:  invalidFolderUIDScope,
 				},
 			},
 			expected: false,
 		},
 		{
-			desc:    "should be able to edit with read action when viewer_can_edit is true",
-			dashUID: "1",
+			desc:      "should be able to edit dashboard with read action when viewer_can_edit is true",
+			dashboard: dashboard,
 			permissions: []accesscontrol.Permission{
 				{
 					Action: dashboards.ActionDashboardsRead,
@@ -196,14 +270,80 @@ func TestAccessControlDashboardGuardian_CanEdit(t *testing.T) {
 			viewersCanEdit: true,
 			expected:       true,
 		},
+		{
+			desc:      "should not be able to edit folder with folder write and dashboard wildcard scope",
+			dashboard: fldr,
+			permissions: []accesscontrol.Permission{
+				{
+					Action: dashboards.ActionFoldersWrite,
+					Scope:  "dashboards:*",
+				},
+			},
+			expected: false,
+		},
+		{
+			desc:      "should be able to edit folder with folder write and folder wildcard scope",
+			dashboard: fldr,
+			permissions: []accesscontrol.Permission{
+				{
+					Action: dashboards.ActionFoldersWrite,
+					Scope:  "folders:*",
+				},
+			},
+			expected: true,
+		},
+		{
+			desc:      "should not be able to edit folder with folder write and dashboard scope",
+			dashboard: fldr,
+			permissions: []accesscontrol.Permission{
+				{
+					Action: dashboards.ActionFoldersWrite,
+					Scope:  "dashboards:uid:1",
+				},
+			},
+			expected: false,
+		},
+		{
+			desc:      "should be able to edit folder with folder write and folder scope",
+			dashboard: fldr,
+			permissions: []accesscontrol.Permission{
+				{
+					Action: dashboards.ActionFoldersWrite,
+					Scope:  folderUIDScope,
+				},
+			},
+			expected: true,
+		},
+		{
+			desc:      "should not be able to edit folder with folder write and incorrect folder scope",
+			dashboard: fldr,
+			permissions: []accesscontrol.Permission{
+				{
+					Action: dashboards.ActionFoldersWrite,
+					Scope:  invalidFolderUIDScope,
+				},
+			},
+			expected: false,
+		},
+		{
+			desc:      "should be able to edit folder with folder read action when viewer_can_edit is true",
+			dashboard: fldr,
+			permissions: []accesscontrol.Permission{
+				{
+					Action: dashboards.ActionFoldersRead,
+					Scope:  folderUIDScope,
+				},
+			},
+			viewersCanEdit: true,
+			expected:       true,
+		},
 	}
 
 	for _, tt := range tests {
 		t.Run(tt.desc, func(t *testing.T) {
 			cfg := setting.NewCfg()
 			cfg.ViewersCanEdit = tt.viewersCanEdit
-			dashSvc := testDashSvc(t)
-			guardian, _ := setupAccessControlGuardianTest(t, tt.dashUID, tt.permissions, cfg, dashSvc)
+			guardian := setupAccessControlGuardianTest(t, tt.dashboard, tt.permissions, cfg)
 
 			can, err := guardian.CanEdit()
 			require.NoError(t, err)
@@ -211,11 +351,12 @@ func TestAccessControlDashboardGuardian_CanEdit(t *testing.T) {
 		})
 	}
 }
+
 func TestAccessControlDashboardGuardian_CanView(t *testing.T) {
 	tests := []accessControlGuardianTestCase{
 		{
-			desc:    "should be able to view with dashboard wildcard scope",
-			dashUID: "1",
+			desc:      "should be able to view dashboard with dashboard wildcard scope",
+			dashboard: dashboard,
 			permissions: []accesscontrol.Permission{
 				{
 					Action: dashboards.ActionDashboardsRead,
@@ -225,8 +366,8 @@ func TestAccessControlDashboardGuardian_CanView(t *testing.T) {
 			expected: true,
 		},
 		{
-			desc:    "should be able to view with folder wildcard scope",
-			dashUID: "1",
+			desc:      "should be able to view dashboard with folder wildcard scope",
+			dashboard: dashboard,
 			permissions: []accesscontrol.Permission{
 				{
 					Action: dashboards.ActionDashboardsRead,
@@ -236,8 +377,8 @@ func TestAccessControlDashboardGuardian_CanView(t *testing.T) {
 			expected: true,
 		},
 		{
-			desc:    "should be able to view with dashboard scope",
-			dashUID: "1",
+			desc:      "should be able to view dashboard with dashboard scope",
+			dashboard: dashboard,
 			permissions: []accesscontrol.Permission{
 				{
 					Action: dashboards.ActionDashboardsRead,
@@ -247,8 +388,8 @@ func TestAccessControlDashboardGuardian_CanView(t *testing.T) {
 			expected: true,
 		},
 		{
-			desc:    "should be able to view with folder scope",
-			dashUID: "1",
+			desc:      "should be able to view dashboard under root with general folder scope",
+			dashboard: &dashboards.Dashboard{OrgID: orgID, UID: dashUID, IsFolder: false},
 			permissions: []accesscontrol.Permission{
 				{
 					Action: dashboards.ActionDashboardsRead,
@@ -258,8 +399,8 @@ func TestAccessControlDashboardGuardian_CanView(t *testing.T) {
 			expected: true,
 		},
 		{
-			desc:    "should not be able to view with incorrect dashboard scope",
-			dashUID: "1",
+			desc:      "should not be able to view dashboard with incorrect dashboard scope",
+			dashboard: dashboard,
 			permissions: []accesscontrol.Permission{
 				{
 					Action: dashboards.ActionDashboardsRead,
@@ -269,12 +410,78 @@ func TestAccessControlDashboardGuardian_CanView(t *testing.T) {
 			expected: false,
 		},
 		{
-			desc:    "should not be able to view with incorrect folder scope",
-			dashUID: "1",
+			desc:      "should not be able to view dashboard with incorrect folder scope",
+			dashboard: dashboard,
 			permissions: []accesscontrol.Permission{
 				{
 					Action: dashboards.ActionDashboardsRead,
-					Scope:  "folders:uid:10",
+					Scope:  invalidFolderUIDScope,
+				},
+			},
+			expected: false,
+		},
+		{
+			desc:      "should not be able to view folder with folders read and dashboard wildcard scope",
+			dashboard: fldr,
+			permissions: []accesscontrol.Permission{
+				{
+					Action: dashboards.ActionFoldersRead,
+					Scope:  "dashboards:*",
+				},
+			},
+			expected: false,
+		},
+		{
+			desc:      "should be able to view folder with folders read and folder wildcard scope",
+			dashboard: fldr,
+			permissions: []accesscontrol.Permission{
+				{
+					Action: dashboards.ActionFoldersRead,
+					Scope:  "folders:*",
+				},
+			},
+			expected: true,
+		},
+		{
+			desc:      "should not be able to folder view with folders read and dashboard scope",
+			dashboard: fldr,
+			permissions: []accesscontrol.Permission{
+				{
+					Action: dashboards.ActionFoldersRead,
+					Scope:  "dashboards:uid:1",
+				},
+			},
+			expected: false,
+		},
+		{
+			desc:      "should be able to view folder with folders read and folder scope",
+			dashboard: fldr,
+			permissions: []accesscontrol.Permission{
+				{
+					Action: dashboards.ActionFoldersRead,
+					Scope:  folderUIDScope,
+				},
+			},
+			expected: true,
+		},
+		{
+			desc:      "should not be able to view folder with folders read incorrect dashboard scope",
+			dashboard: fldr,
+			permissions: []accesscontrol.Permission{
+				{
+					Action: dashboards.ActionFoldersRead,
+					Scope:  "dashboards:uid:10",
+				},
+			},
+			expected: false,
+		},
+		{
+			desc:      "should not be able to view folder with folders read and incorrect folder scope",
+			dashboard: fldr,
+			permissions: []accesscontrol.Permission{
+				{
+					Action: dashboards.ActionFoldersRead,
+					Scope:  invalidFolderUIDScope,
 				},
 			},
 			expected: false,
@@ -283,7 +490,7 @@ func TestAccessControlDashboardGuardian_CanView(t *testing.T) {
 
 	for _, tt := range tests {
 		t.Run(tt.desc, func(t *testing.T) {
-			guardian, _ := setupAccessControlGuardianTest(t, tt.dashUID, tt.permissions, nil, testDashSvc(t))
+			guardian := setupAccessControlGuardianTest(t, tt.dashboard, tt.permissions, nil)
 
 			can, err := guardian.CanView()
 			require.NoError(t, err)
@@ -291,11 +498,12 @@ func TestAccessControlDashboardGuardian_CanView(t *testing.T) {
 		})
 	}
 }
+
 func TestAccessControlDashboardGuardian_CanAdmin(t *testing.T) {
 	tests := []accessControlGuardianTestCase{
 		{
-			desc:    "should be able to admin with dashboard wildcard scope",
-			dashUID: "1",
+			desc:      "should be able to admin dashboard with dashboard wildcard scope",
+			dashboard: dashboard,
 			permissions: []accesscontrol.Permission{
 				{
 					Action: dashboards.ActionDashboardsPermissionsRead,
@@ -309,8 +517,8 @@ func TestAccessControlDashboardGuardian_CanAdmin(t *testing.T) {
 			expected: true,
 		},
 		{
-			desc:    "should be able to admin with folder wildcard scope",
-			dashUID: "1",
+			desc:      "should be able to admin dashboard with folder wildcard scope",
+			dashboard: dashboard,
 			permissions: []accesscontrol.Permission{
 				{
 					Action: dashboards.ActionDashboardsPermissionsRead,
@@ -324,8 +532,8 @@ func TestAccessControlDashboardGuardian_CanAdmin(t *testing.T) {
 			expected: true,
 		},
 		{
-			desc:    "should be able to admin with dashboard scope",
-			dashUID: "1",
+			desc:      "should be able to admin dashboard with dashboard scope",
+			dashboard: dashboard,
 			permissions: []accesscontrol.Permission{
 				{
 					Action: dashboards.ActionDashboardsPermissionsRead,
@@ -339,8 +547,8 @@ func TestAccessControlDashboardGuardian_CanAdmin(t *testing.T) {
 			expected: true,
 		},
 		{
-			desc:    "should be able to admin with folder scope",
-			dashUID: "1",
+			desc:      "should be able to admin dashboard under root with general folder scope",
+			dashboard: &dashboards.Dashboard{OrgID: orgID, UID: dashUID, IsFolder: false},
 			permissions: []accesscontrol.Permission{
 				{
 					Action: dashboards.ActionDashboardsPermissionsRead,
@@ -354,8 +562,8 @@ func TestAccessControlDashboardGuardian_CanAdmin(t *testing.T) {
 			expected: true,
 		},
 		{
-			desc:    "should not be able to admin with incorrect dashboard scope",
-			dashUID: "1",
+			desc:      "should not be able to admin dashboard with incorrect dashboard scope",
+			dashboard: dashboard,
 			permissions: []accesscontrol.Permission{
 				{
 					Action: dashboards.ActionDashboardsPermissionsRead,
@@ -369,16 +577,150 @@ func TestAccessControlDashboardGuardian_CanAdmin(t *testing.T) {
 			expected: false,
 		},
 		{
-			desc:    "should not be able to admin with incorrect folder scope",
-			dashUID: "1",
+			desc:      "should not be able to admin dashboard with incorrect folder scope",
+			dashboard: dashboard,
 			permissions: []accesscontrol.Permission{
 				{
 					Action: dashboards.ActionDashboardsPermissionsRead,
-					Scope:  "folders:uid:10",
+					Scope:  invalidFolderUIDScope,
 				},
 				{
 					Action: dashboards.ActionDashboardsPermissionsWrite,
-					Scope:  "folders:uid:10",
+					Scope:  invalidFolderUIDScope,
+				},
+			},
+			expected: false,
+		},
+		{
+			desc:      "should not be able to admin folder with folder read and write and dashboard wildcard scope",
+			dashboard: fldr,
+			permissions: []accesscontrol.Permission{
+				{
+					Action: dashboards.ActionFoldersPermissionsRead,
+					Scope:  "dashboards:*",
+				},
+				{
+					Action: dashboards.ActionFoldersPermissionsWrite,
+					Scope:  "dashboards:*",
+				},
+			},
+			expected: false,
+		},
+		{
+			desc:      "should be able to admin folder with folder read and write and wildcard scope",
+			dashboard: fldr,
+			permissions: []accesscontrol.Permission{
+				{
+					Action: dashboards.ActionFoldersPermissionsRead,
+					Scope:  "folders:*",
+				},
+				{
+					Action: dashboards.ActionFoldersPermissionsWrite,
+					Scope:  "folders:*",
+				},
+			},
+			expected: true,
+		},
+		{
+			desc:      "should not be able to admin folder with folder read and wildcard scope",
+			dashboard: fldr,
+			permissions: []accesscontrol.Permission{
+				{
+					Action: dashboards.ActionFoldersPermissionsRead,
+					Scope:  "folders:*",
+				},
+			},
+			expected: false,
+		},
+		{
+			desc:      "should not be able to admin folder with folder write and wildcard scope",
+			dashboard: fldr,
+			permissions: []accesscontrol.Permission{
+				{
+					Action: dashboards.ActionFoldersPermissionsWrite,
+					Scope:  "folders:*",
+				},
+			},
+			expected: false,
+		},
+		{
+			desc:      "should not be able to admin folder with folder read and write and dashboard scope",
+			dashboard: fldr,
+			permissions: []accesscontrol.Permission{
+				{
+					Action: dashboards.ActionFoldersPermissionsRead,
+					Scope:  "dashboards:uid:1",
+				},
+				{
+					Action: dashboards.ActionFoldersPermissionsWrite,
+					Scope:  "dashboards:uid:1",
+				},
+			},
+			expected: false,
+		},
+		{
+			desc:      "should be able to admin folder with folder read and write and folder scope",
+			dashboard: fldr,
+			permissions: []accesscontrol.Permission{
+				{
+					Action: dashboards.ActionFoldersPermissionsRead,
+					Scope:  folderUIDScope,
+				},
+				{
+					Action: dashboards.ActionFoldersPermissionsWrite,
+					Scope:  folderUIDScope,
+				},
+			},
+			expected: true,
+		},
+		{
+			desc:      "should not be able to admin folder with folder read and folder scope",
+			dashboard: fldr,
+			permissions: []accesscontrol.Permission{
+				{
+					Action: dashboards.ActionFoldersPermissionsRead,
+					Scope:  folderUIDScope,
+				},
+			},
+			expected: false,
+		},
+		{
+			desc:      "should not be able to admin folder with folder write and folder scope",
+			dashboard: fldr,
+			permissions: []accesscontrol.Permission{
+				{
+					Action: dashboards.ActionFoldersPermissionsWrite,
+					Scope:  folderUIDScope,
+				},
+			},
+			expected: false,
+		},
+		{
+			desc:      "should not be able to admin folder with folder read and write and incorrect dashboard scope",
+			dashboard: fldr,
+			permissions: []accesscontrol.Permission{
+				{
+					Action: dashboards.ActionFoldersPermissionsRead,
+					Scope:  "dashboards:uid:10",
+				},
+				{
+					Action: dashboards.ActionFoldersPermissionsWrite,
+					Scope:  "dashboards:uid:10",
+				},
+			},
+			expected: false,
+		},
+		{
+			desc:      "should not be able to admin folder with folder read and write and incorrect folder scope",
+			dashboard: fldr,
+			permissions: []accesscontrol.Permission{
+				{
+					Action: dashboards.ActionFoldersPermissionsRead,
+					Scope:  invalidFolderUIDScope,
+				},
+				{
+					Action: dashboards.ActionFoldersPermissionsWrite,
+					Scope:  invalidFolderUIDScope,
 				},
 			},
 			expected: false,
@@ -387,7 +729,7 @@ func TestAccessControlDashboardGuardian_CanAdmin(t *testing.T) {
 
 	for _, tt := range tests {
 		t.Run(tt.desc, func(t *testing.T) {
-			guardian, _ := setupAccessControlGuardianTest(t, tt.dashUID, tt.permissions, nil, testDashSvc(t))
+			guardian := setupAccessControlGuardianTest(t, tt.dashboard, tt.permissions, nil)
 
 			can, err := guardian.CanAdmin()
 			require.NoError(t, err)
@@ -395,11 +737,12 @@ func TestAccessControlDashboardGuardian_CanAdmin(t *testing.T) {
 		})
 	}
 }
+
 func TestAccessControlDashboardGuardian_CanDelete(t *testing.T) {
 	tests := []accessControlGuardianTestCase{
 		{
-			desc:    "should be able to delete with dashboard wildcard scope",
-			dashUID: "1",
+			desc:      "should be able to delete dashboard with dashboard wildcard scope",
+			dashboard: dashboard,
 			permissions: []accesscontrol.Permission{
 				{
 					Action: dashboards.ActionDashboardsDelete,
@@ -409,8 +752,8 @@ func TestAccessControlDashboardGuardian_CanDelete(t *testing.T) {
 			expected: true,
 		},
 		{
-			desc:    "should be able to delete with folder wildcard scope",
-			dashUID: "1",
+			desc:      "should be able to delete dashboard with folder wildcard scope",
+			dashboard: dashboard,
 			permissions: []accesscontrol.Permission{
 				{
 					Action: dashboards.ActionDashboardsDelete,
@@ -420,8 +763,8 @@ func TestAccessControlDashboardGuardian_CanDelete(t *testing.T) {
 			expected: true,
 		},
 		{
-			desc:    "should be able to delete with dashboard scope",
-			dashUID: "1",
+			desc:      "should be able to delete dashboard with dashboard scope",
+			dashboard: dashboard,
 			permissions: []accesscontrol.Permission{
 				{
 					Action: dashboards.ActionDashboardsDelete,
@@ -431,8 +774,8 @@ func TestAccessControlDashboardGuardian_CanDelete(t *testing.T) {
 			expected: true,
 		},
 		{
-			desc:    "should be able to delete with folder scope",
-			dashUID: "1",
+			desc:      "should be able to delete dashboard under root with general folder scope",
+			dashboard: &dashboards.Dashboard{OrgID: orgID, UID: dashUID, IsFolder: false},
 			permissions: []accesscontrol.Permission{
 				{
 					Action: dashboards.ActionDashboardsDelete,
@@ -442,8 +785,8 @@ func TestAccessControlDashboardGuardian_CanDelete(t *testing.T) {
 			expected: true,
 		},
 		{
-			desc:    "should not be able to delete with incorrect dashboard scope",
-			dashUID: "1",
+			desc:      "should not be able to delete dashboard with incorrect dashboard scope",
+			dashboard: dashboard,
 			permissions: []accesscontrol.Permission{
 				{
 					Action: dashboards.ActionDashboardsDelete,
@@ -453,12 +796,78 @@ func TestAccessControlDashboardGuardian_CanDelete(t *testing.T) {
 			expected: false,
 		},
 		{
-			desc:    "should not be able to delete with incorrect folder scope",
-			dashUID: "1",
+			desc:      "should not be able to delete dashboard with incorrect folder scope",
+			dashboard: dashboard,
 			permissions: []accesscontrol.Permission{
 				{
 					Action: dashboards.ActionDashboardsDelete,
-					Scope:  "folders:uid:10",
+					Scope:  invalidFolderUIDScope,
+				},
+			},
+			expected: false,
+		},
+		{
+			desc:      "should not be able to delete folder with folder delete and dashboard wildcard scope",
+			dashboard: fldr,
+			permissions: []accesscontrol.Permission{
+				{
+					Action: dashboards.ActionFoldersDelete,
+					Scope:  "dashboards:*",
+				},
+			},
+			expected: false,
+		},
+		{
+			desc:      "should be able to delete folder with folder deletea and folder wildcard scope",
+			dashboard: fldr,
+			permissions: []accesscontrol.Permission{
+				{
+					Action: dashboards.ActionFoldersDelete,
+					Scope:  "folders:*",
+				},
+			},
+			expected: true,
+		},
+		{
+			desc:      "should not be able to delete folder with folder delete and dashboard scope",
+			dashboard: fldr,
+			permissions: []accesscontrol.Permission{
+				{
+					Action: dashboards.ActionFoldersDelete,
+					Scope:  "dashboards:uid:1",
+				},
+			},
+			expected: false,
+		},
+		{
+			desc:      "should be able to delete folder with folder delete and folder scope",
+			dashboard: fldr,
+			permissions: []accesscontrol.Permission{
+				{
+					Action: dashboards.ActionFoldersDelete,
+					Scope:  folderUIDScope,
+				},
+			},
+			expected: true,
+		},
+		{
+			desc:      "should not be able to delete folder with folder delete and incorrect dashboard scope",
+			dashboard: fldr,
+			permissions: []accesscontrol.Permission{
+				{
+					Action: dashboards.ActionFoldersDelete,
+					Scope:  "dashboards:uid:10",
+				},
+			},
+			expected: false,
+		},
+		{
+			desc:      "should not be able to delete folder with folder delete and incorrect folder scope",
+			dashboard: fldr,
+			permissions: []accesscontrol.Permission{
+				{
+					Action: dashboards.ActionFoldersDelete,
+					Scope:  invalidFolderUIDScope,
 				},
 			},
 			expected: false,
@@ -467,7 +876,7 @@ func TestAccessControlDashboardGuardian_CanDelete(t *testing.T) {
 
 	for _, tt := range tests {
 		t.Run(tt.desc, func(t *testing.T) {
-			guardian, _ := setupAccessControlGuardianTest(t, tt.dashUID, tt.permissions, nil, testDashSvc(t))
+			guardian := setupAccessControlGuardianTest(t, tt.dashboard, tt.permissions, nil)
 
 			can, err := guardian.CanDelete()
 			require.NoError(t, err)
@@ -531,7 +940,7 @@ func TestAccessControlDashboardGuardian_CanCreate(t *testing.T) {
 
 	for _, tt := range tests {
 		t.Run(tt.desc, func(t *testing.T) {
-			guardian, _ := setupAccessControlGuardianTest(t, "0", tt.permissions, nil, nil)
+			guardian := setupAccessControlGuardianTest(t, &dashboards.Dashboard{OrgID: orgID, UID: "0", IsFolder: tt.isFolder}, tt.permissions, nil)
 
 			can, err := guardian.CanCreate(tt.folderID, tt.isFolder)
 			require.NoError(t, err)
@@ -540,114 +949,36 @@ func TestAccessControlDashboardGuardian_CanCreate(t *testing.T) {
 	}
 }
 
-type accessControlGuardianGetHiddenACLTestCase struct {
-	desc        string
-	permissions []accesscontrol.ResourcePermission
-	hiddenUsers map[string]struct{}
-}
-
-func TestAccessControlDashboardGuardian_GetHiddenACL(t *testing.T) {
-	tests := []accessControlGuardianGetHiddenACLTestCase{
-		{
-			desc: "should only return permissions containing hidden users",
-			permissions: []accesscontrol.ResourcePermission{
-				{RoleName: "managed:users:1:permissions", UserId: 1, UserLogin: "user1", IsManaged: true},
-				{RoleName: "managed:teams:1:permissions", TeamId: 1, Team: "team1", IsManaged: true},
-				{RoleName: "managed:users:2:permissions", UserId: 2, UserLogin: "user2", IsManaged: true},
-				{RoleName: "managed:users:3:permissions", UserId: 3, UserLogin: "user3", IsManaged: true},
-				{RoleName: "managed:users:4:permissions", UserId: 4, UserLogin: "user4", IsManaged: true},
-			},
-			hiddenUsers: map[string]struct{}{"user2": {}, "user3": {}},
-		},
-	}
-
-	for _, tt := range tests {
-		t.Run(tt.desc, func(t *testing.T) {
-			guardian, _ := setupAccessControlGuardianTest(t, "1", nil, nil, testDashSvc(t))
-
-			mocked := accesscontrolmock.NewMockedPermissionsService()
-			guardian.dashboardPermissionsService = mocked
-			mocked.On("MapActions", mock.Anything).Return("View")
-			mocked.On("GetPermissions", mock.Anything, mock.Anything, mock.Anything).Return(tt.permissions, nil)
-			cfg := setting.NewCfg()
-			cfg.HiddenUsers = tt.hiddenUsers
-			permissions, err := guardian.GetHiddenACL(cfg)
-			require.NoError(t, err)
-			var hiddenUserNames []string
-			for name := range tt.hiddenUsers {
-				hiddenUserNames = append(hiddenUserNames, name)
-			}
-			assert.Len(t, permissions, len(hiddenUserNames))
-			for _, p := range permissions {
-				assert.Contains(t, hiddenUserNames, fmt.Sprintf("user%d", p.UserID))
-			}
-		})
-	}
-}
-
-func setupAccessControlGuardianTest(t *testing.T, uid string,
-	permissions []accesscontrol.Permission,
-	cfg *setting.Cfg,
-	dashboardSvc dashboards.DashboardService) (*AccessControlDashboardGuardian, *dashboards.Dashboard) {
+func setupAccessControlGuardianTest(
+	t *testing.T, d *dashboards.Dashboard,
+	permissions []accesscontrol.Permission, cfg *setting.Cfg,
+) DashboardGuardian {
 	t.Helper()
-	store := db.InitTestDB(t)
 
-	toSave := dashboards.NewDashboard(uid)
-	toSave.SetUID(uid)
+	fakeDashboardService := dashboards.NewFakeDashboardService(t)
+	fakeDashboardService.On("GetDashboard", mock.Anything, mock.AnythingOfType("*dashboards.GetDashboardQuery")).Maybe().Return(d, nil)
 
-	// seed dashboard
-	quotaService := quotatest.New(false, nil)
-	dashStore, err := dashdb.ProvideDashboardStore(store, store.Cfg, featuremgmt.WithFeatures(), tagimpl.ProvideService(store, store.Cfg), quotaService)
-	require.NoError(t, err)
-	dash, err := dashStore.SaveDashboard(context.Background(), dashboards.SaveDashboardCommand{
-		Dashboard: toSave.Data,
-		UserID:    1,
-		OrgID:     1,
-	})
-	require.NoError(t, err)
-	if dashboardSvc == nil {
-		fakeDashboardService := dashboards.NewFakeDashboardService(t)
-		qResult := &dashboards.Dashboard{}
-		fakeDashboardService.On("GetDashboard", mock.Anything, mock.AnythingOfType("*dashboards.GetDashboardQuery")).Run(func(args mock.Arguments) {
-			q := args.Get(1).(*dashboards.GetDashboardQuery)
-			qResult = &dashboards.Dashboard{
-				ID:    q.ID,
-				UID:   q.UID,
-				OrgID: q.OrgID,
-			}
-		}).Return(qResult, nil)
-		dashboardSvc = fakeDashboardService
-	}
+	ac := acimpl.ProvideAccessControl(featuremgmt.WithFeatures(), zanzana.NewNoopClient())
+
+	fStore := folder.NewFakeStore()
+	folderStore := foldertest.NewFakeFolderStore(t)
+
+	ac.RegisterScopeAttributeResolver(dashboards.NewDashboardUIDScopeResolver(folderStore, fakeDashboardService, fStore))
+	ac.RegisterScopeAttributeResolver(dashboards.NewFolderUIDScopeResolver(fStore))
+	ac.RegisterScopeAttributeResolver(dashboards.NewFolderIDScopeResolver(folderStore, fStore))
 
-	ac := accesscontrolmock.New().WithPermissions(permissions)
-	// TODO replace with actual folder store implementation after resolving import cycles
-	ac.RegisterScopeAttributeResolver(dashboards.NewDashboardUIDScopeResolver(foldertest.NewFakeFolderStore(t), dashboardSvc, foldertest.NewFakeService()))
 	license := licensingtest.NewFakeLicensing()
 	license.On("FeatureEnabled", "accesscontrol.enforcement").Return(true).Maybe()
-	teamSvc := teamimpl.ProvideService(store, store.Cfg)
-	userSvc, err := userimpl.ProvideService(store, nil, store.Cfg, nil, nil, quotatest.New(false, nil), supportbundlestest.NewFakeBundleService())
-	require.NoError(t, err)
 
-	folderPermissions, err := ossaccesscontrol.ProvideFolderPermissions(
-		cfg, routing.NewRouteRegister(), store, ac, license, &dashboards.FakeDashboardStore{}, foldertest.NewFakeService(), ac, teamSvc, userSvc)
-	require.NoError(t, err)
-	dashboardPermissions, err := ossaccesscontrol.ProvideDashboardPermissions(
-		cfg, routing.NewRouteRegister(), store, ac, license, &dashboards.FakeDashboardStore{}, foldertest.NewFakeService(), ac, teamSvc, userSvc)
-	require.NoError(t, err)
+	userPermissions := map[int64]map[string][]string{}
+	for _, p := range permissions {
+		if _, ok := userPermissions[orgID]; !ok {
+			userPermissions[orgID] = map[string][]string{}
+		}
+		userPermissions[orgID][p.Action] = append(userPermissions[orgID][p.Action], p.Scope)
+	}
 
-	g, err := NewAccessControlDashboardGuardian(context.Background(), cfg, dash.ID, &user.SignedInUser{OrgID: 1}, store, ac, folderPermissions, dashboardPermissions, dashboardSvc)
+	g, err := NewAccessControlDashboardGuardianByDashboard(context.Background(), cfg, d, &user.SignedInUser{OrgID: orgID, Permissions: userPermissions}, ac, fakeDashboardService)
 	require.NoError(t, err)
-	g.dashboard = dash
-	return g, dash
-}
-
-func testDashSvc(t *testing.T) dashboards.DashboardService {
-	dashSvc := dashboards.NewFakeDashboardService(t)
-	d := &dashboards.Dashboard{}
-	dashSvc.On("GetDashboard", mock.Anything, mock.AnythingOfType("*dashboards.GetDashboardQuery")).Run(func(args mock.Arguments) {
-		d := dashboards.NewDashboard("mocked")
-		d.ID = 1
-		d.UID = "1"
-	}).Return(d, nil)
-	return dashSvc
+	return g
 }
