@@ -1,6 +1,7 @@
 package api
 
 import (
+	"context"
 	"encoding/json"
 	"errors"
 	"io"
@@ -15,11 +16,14 @@ import (
 	"github.com/grafana/grafana/pkg/services/accesscontrol"
 	"github.com/grafana/grafana/pkg/services/accesscontrol/acimpl"
 	"github.com/grafana/grafana/pkg/services/auth/authtest"
+	"github.com/grafana/grafana/pkg/services/authn/authntest"
+	"github.com/grafana/grafana/pkg/services/authz/zanzana"
+	"github.com/grafana/grafana/pkg/services/featuremgmt"
 	"github.com/grafana/grafana/pkg/services/ldap"
 	"github.com/grafana/grafana/pkg/services/ldap/multildap"
 	"github.com/grafana/grafana/pkg/services/ldap/service"
 	"github.com/grafana/grafana/pkg/services/login"
-	"github.com/grafana/grafana/pkg/services/login/logintest"
+	"github.com/grafana/grafana/pkg/services/login/authinfotest"
 	"github.com/grafana/grafana/pkg/services/org"
 	"github.com/grafana/grafana/pkg/services/org/orgtest"
 	"github.com/grafana/grafana/pkg/services/supportbundles/supportbundlestest"
@@ -36,8 +40,10 @@ type LDAPMock struct {
 	UserSearchError  error
 }
 
-var pingResult []*multildap.ServerStatus
-var pingError error
+var (
+	pingResult []*multildap.ServerStatus
+	pingError  error
+)
 
 func (m *LDAPMock) Ping() ([]*multildap.ServerStatus, error) {
 	return pingResult, pingError
@@ -64,11 +70,11 @@ func setupAPITest(t *testing.T, opts ...func(a *Service)) (*Service, *webtest.Se
 
 	a := ProvideService(cfg,
 		router,
-		acimpl.ProvideAccessControl(cfg),
+		acimpl.ProvideAccessControl(featuremgmt.WithFeatures(), zanzana.NewNoopClient()),
 		usertest.NewUserServiceFake(),
-		&logintest.AuthInfoServiceFake{},
+		&authinfotest.FakeService{},
 		ldap.ProvideGroupsService(),
-		&logintest.LoginServiceFake{},
+		&authntest.FakeService{},
 		&orgtest.FakeOrgService{},
 		service.NewLDAPFakeService(),
 		authtest.NewFakeUserAuthTokenService(),
@@ -93,7 +99,7 @@ func TestGetUserFromLDAPAPIEndpoint_UserNotFound(t *testing.T) {
 			ExpectedClient: &LDAPMock{
 				UserSearchResult: nil,
 			},
-			ExpectedConfig: &ldap.Config{},
+			ExpectedConfig: &ldap.ServersConfig{},
 		}
 	})
 
@@ -101,7 +107,8 @@ func TestGetUserFromLDAPAPIEndpoint_UserNotFound(t *testing.T) {
 	webtest.RequestWithSignedInUser(req, &user.SignedInUser{
 		OrgID: 1,
 		Permissions: map[int64]map[string][]string{
-			1: {"ldap.user:read": {"*"}}},
+			1: {"ldap.user:read": {"*"}},
+		},
 	})
 
 	res, err := server.Send(req)
@@ -158,7 +165,7 @@ func TestGetUserFromLDAPAPIEndpoint_OrgNotfound(t *testing.T) {
 				UserSearchResult: userSearchResult,
 				UserSearchConfig: userSearchConfig,
 			},
-			ExpectedConfig: &ldap.Config{},
+			ExpectedConfig: &ldap.ServersConfig{},
 		}
 	})
 
@@ -166,7 +173,8 @@ func TestGetUserFromLDAPAPIEndpoint_OrgNotfound(t *testing.T) {
 	webtest.RequestWithSignedInUser(req, &user.SignedInUser{
 		OrgID: 1,
 		Permissions: map[int64]map[string][]string{
-			1: {"ldap.user:read": {"*"}}},
+			1: {"ldap.user:read": {"*"}},
+		},
 	})
 
 	res, err := server.Send(req)
@@ -179,7 +187,6 @@ func TestGetUserFromLDAPAPIEndpoint_OrgNotfound(t *testing.T) {
 	var resMap map[string]interface{}
 	err = json.Unmarshal(bodyBytes, &resMap)
 	assert.NoError(t, err)
-	assert.Equal(t, "unable to find organization with ID '2'", resMap["error"])
 	assert.Equal(t, "An organization was not found - Please verify your LDAP configuration", resMap["message"])
 }
 
@@ -228,7 +235,7 @@ func TestGetUserFromLDAPAPIEndpoint(t *testing.T) {
 				UserSearchResult: userSearchResult,
 				UserSearchConfig: userSearchConfig,
 			},
-			ExpectedConfig: &ldap.Config{},
+			ExpectedConfig: &ldap.ServersConfig{},
 		}
 	})
 
@@ -236,7 +243,8 @@ func TestGetUserFromLDAPAPIEndpoint(t *testing.T) {
 	webtest.RequestWithSignedInUser(req, &user.SignedInUser{
 		OrgID: 1,
 		Permissions: map[int64]map[string][]string{
-			1: {"ldap.user:read": {"*"}}},
+			1: {"ldap.user:read": {"*"}},
+		},
 	})
 
 	res, err := server.Send(req)
@@ -313,7 +321,7 @@ func TestGetUserFromLDAPAPIEndpoint_WithTeamHandler(t *testing.T) {
 				UserSearchResult: userSearchResult,
 				UserSearchConfig: userSearchConfig,
 			},
-			ExpectedConfig: &ldap.Config{},
+			ExpectedConfig: &ldap.ServersConfig{},
 		}
 	})
 
@@ -321,7 +329,8 @@ func TestGetUserFromLDAPAPIEndpoint_WithTeamHandler(t *testing.T) {
 	webtest.RequestWithSignedInUser(req, &user.SignedInUser{
 		OrgID: 1,
 		Permissions: map[int64]map[string][]string{
-			1: {"ldap.user:read": {"*"}}},
+			1: {"ldap.user:read": {"*"}},
+		},
 	})
 
 	res, err := server.Send(req)
@@ -367,7 +376,7 @@ func TestGetLDAPStatusAPIEndpoint(t *testing.T) {
 	_, server := setupAPITest(t, func(a *Service) {
 		a.ldapService = &service.LDAPFakeService{
 			ExpectedClient: &LDAPMock{},
-			ExpectedConfig: &ldap.Config{},
+			ExpectedConfig: &ldap.ServersConfig{},
 		}
 	})
 
@@ -375,7 +384,8 @@ func TestGetLDAPStatusAPIEndpoint(t *testing.T) {
 	webtest.RequestWithSignedInUser(req, &user.SignedInUser{
 		OrgID: 1,
 		Permissions: map[int64]map[string][]string{
-			1: {"ldap.status:read": {}}},
+			1: {"ldap.status:read": {}},
+		},
 	})
 
 	res, err := server.Send(req)
@@ -406,7 +416,7 @@ func TestPostSyncUserWithLDAPAPIEndpoint_Success(t *testing.T) {
 			ExpectedClient: &LDAPMock{UserSearchResult: &login.ExternalUserInfo{
 				Login: "ldap-daniel",
 			}},
-			ExpectedConfig: &ldap.Config{},
+			ExpectedConfig: &ldap.ServersConfig{},
 		}
 	})
 
@@ -414,7 +424,8 @@ func TestPostSyncUserWithLDAPAPIEndpoint_Success(t *testing.T) {
 	webtest.RequestWithSignedInUser(req, &user.SignedInUser{
 		OrgID: 1,
 		Permissions: map[int64]map[string][]string{
-			1: {"ldap.user:sync": {}}},
+			1: {"ldap.user:sync": {}},
+		},
 	})
 
 	res, err := server.Send(req)
@@ -441,7 +452,7 @@ func TestPostSyncUserWithLDAPAPIEndpoint_WhenUserNotFound(t *testing.T) {
 		a.userService = userServiceMock
 		a.ldapService = &service.LDAPFakeService{
 			ExpectedClient: &LDAPMock{},
-			ExpectedConfig: &ldap.Config{},
+			ExpectedConfig: &ldap.ServersConfig{},
 		}
 	})
 
@@ -449,7 +460,8 @@ func TestPostSyncUserWithLDAPAPIEndpoint_WhenUserNotFound(t *testing.T) {
 	webtest.RequestWithSignedInUser(req, &user.SignedInUser{
 		OrgID: 1,
 		Permissions: map[int64]map[string][]string{
-			1: {"ldap.user:sync": {}}},
+			1: {"ldap.user:sync": {}},
+		},
 	})
 
 	res, err := server.Send(req)
@@ -474,10 +486,10 @@ func TestPostSyncUserWithLDAPAPIEndpoint_WhenGrafanaAdmin(t *testing.T) {
 
 	_, server := setupAPITest(t, func(a *Service) {
 		a.userService = userServiceMock
-		a.cfg.AdminUser = "ldap-daniel"
+		a.adminUser = "ldap-daniel"
 		a.ldapService = &service.LDAPFakeService{
 			ExpectedClient: &LDAPMock{UserSearchError: multildap.ErrDidNotFindUser},
-			ExpectedConfig: &ldap.Config{},
+			ExpectedConfig: &ldap.ServersConfig{},
 		}
 	})
 
@@ -485,7 +497,8 @@ func TestPostSyncUserWithLDAPAPIEndpoint_WhenGrafanaAdmin(t *testing.T) {
 	webtest.RequestWithSignedInUser(req, &user.SignedInUser{
 		OrgID: 1,
 		Permissions: map[int64]map[string][]string{
-			1: {"ldap.user:sync": {}}},
+			1: {"ldap.user:sync": {}},
+		},
 	})
 
 	res, err := server.Send(req)
@@ -498,7 +511,6 @@ func TestPostSyncUserWithLDAPAPIEndpoint_WhenGrafanaAdmin(t *testing.T) {
 	var resMap map[string]interface{}
 	err = json.Unmarshal(bodyBytes, &resMap)
 	assert.NoError(t, err)
-	assert.Equal(t, "did not find a user", resMap["error"])
 	assert.Equal(t, "Refusing to sync grafana super admin \"ldap-daniel\" - it would be disabled", resMap["message"])
 }
 
@@ -508,10 +520,10 @@ func TestPostSyncUserWithLDAPAPIEndpoint_WhenUserNotInLDAP(t *testing.T) {
 
 	_, server := setupAPITest(t, func(a *Service) {
 		a.userService = userServiceMock
-		a.authInfoService = &logintest.AuthInfoServiceFake{ExpectedExternalUser: &login.ExternalUserInfo{IsDisabled: true, UserId: 34}}
+		a.authInfoService = &authinfotest.FakeService{ExpectedExternalUser: &login.ExternalUserInfo{IsDisabled: true, UserId: 34}}
 		a.ldapService = &service.LDAPFakeService{
 			ExpectedClient: &LDAPMock{UserSearchError: multildap.ErrDidNotFindUser},
-			ExpectedConfig: &ldap.Config{},
+			ExpectedConfig: &ldap.ServersConfig{},
 		}
 	})
 
@@ -519,7 +531,8 @@ func TestPostSyncUserWithLDAPAPIEndpoint_WhenUserNotInLDAP(t *testing.T) {
 	webtest.RequestWithSignedInUser(req, &user.SignedInUser{
 		OrgID: 1,
 		Permissions: map[int64]map[string][]string{
-			1: {"ldap.user:sync": {}}},
+			1: {"ldap.user:sync": {}},
+		},
 	})
 
 	res, err := server.Send(req)
@@ -641,12 +654,12 @@ search_base_dns = ["dc=grafana,dc=org"]`)
 		t.Run(tt.desc, func(t *testing.T) {
 			_, server := setupAPITest(t, func(a *Service) {
 				a.userService = &usertest.FakeUserService{ExpectedUser: &user.User{Login: "ldap-daniel", ID: 1}}
-				a.cfg.LDAPConfigFilePath = ldapConfigFile
+				a.cfg.ConfigFilePath = ldapConfigFile
 				a.ldapService = &service.LDAPFakeService{
 					ExpectedClient: &LDAPMock{UserSearchResult: &login.ExternalUserInfo{
 						Login: "ldap-daniel",
 					}},
-					ExpectedConfig: &ldap.Config{},
+					ExpectedConfig: &ldap.ServersConfig{},
 				}
 			})
 			// Add minimal setup to pass handler
@@ -663,5 +676,5 @@ search_base_dns = ["dc=grafana,dc=org"]`)
 }
 
 func userWithPermissions(orgID int64, permissions []accesscontrol.Permission) *user.SignedInUser {
-	return &user.SignedInUser{OrgID: orgID, OrgRole: org.RoleViewer, Permissions: map[int64]map[string][]string{orgID: accesscontrol.GroupScopesByAction(permissions)}}
+	return &user.SignedInUser{OrgID: orgID, OrgRole: org.RoleViewer, Permissions: map[int64]map[string][]string{orgID: accesscontrol.GroupScopesByActionContext(context.Background(), permissions)}}
 }
