@@ -4,19 +4,68 @@ import (
 	"encoding/json"
 	"fmt"
 	"math/rand"
+	"reflect"
 	"sort"
 	"strings"
 	"testing"
 	"time"
 
 	"github.com/google/go-cmp/cmp"
+	"github.com/google/go-cmp/cmp/cmpopts"
+	"github.com/prometheus/common/model"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 	"gopkg.in/yaml.v3"
 
 	"github.com/grafana/grafana/pkg/util"
+	"github.com/grafana/grafana/pkg/util/cmputil"
 )
 
+func TestSortAlertRulesByGroupKeyAndIndex(t *testing.T) {
+	tc := []struct {
+		name     string
+		input    []*AlertRule
+		expected []*AlertRule
+	}{{
+		name: "alert rules are ordered by organization",
+		input: []*AlertRule{
+			{OrgID: 2, NamespaceUID: "test2"},
+			{OrgID: 1, NamespaceUID: "test1"},
+		},
+		expected: []*AlertRule{
+			{OrgID: 1, NamespaceUID: "test1"},
+			{OrgID: 2, NamespaceUID: "test2"},
+		},
+	}, {
+		name: "alert rules in same organization are ordered by namespace",
+		input: []*AlertRule{
+			{OrgID: 1, NamespaceUID: "test2"},
+			{OrgID: 1, NamespaceUID: "test1"},
+		},
+		expected: []*AlertRule{
+			{OrgID: 1, NamespaceUID: "test1"},
+			{OrgID: 1, NamespaceUID: "test2"},
+		},
+	}, {
+		name: "alert rules with same group key are ordered by index",
+		input: []*AlertRule{
+			{OrgID: 1, NamespaceUID: "test", RuleGroupIndex: 2},
+			{OrgID: 1, NamespaceUID: "test", RuleGroupIndex: 1},
+		},
+		expected: []*AlertRule{
+			{OrgID: 1, NamespaceUID: "test", RuleGroupIndex: 1},
+			{OrgID: 1, NamespaceUID: "test", RuleGroupIndex: 2},
+		},
+	}}
+
+	for _, tt := range tc {
+		t.Run(tt.name, func(t *testing.T) {
+			AlertRulesBy(AlertRulesByGroupKeyAndIndex).Sort(tt.input)
+			assert.EqualValues(t, tt.expected, tt.input)
+		})
+	}
+}
+
 func TestNoDataStateFromString(t *testing.T) {
 	allKnownNoDataStates := [...]NoDataState{
 		Alerting,
@@ -88,6 +137,7 @@ func TestSetDashboardAndPanelFromAnnotations(t *testing.T) {
 		name                 string
 		annotations          map[string]string
 		expectedError        error
+		expectedErrContains  string
 		expectedDashboardUID string
 		expectedPanelID      int64
 	}{
@@ -99,41 +149,42 @@ func TestSetDashboardAndPanelFromAnnotations(t *testing.T) {
 			expectedPanelID:      -1,
 		},
 		{
-			name:        "dashboardUID is not present",
-			annotations: map[string]string{PanelIDAnnotation: "1234567890"},
-			expectedError: fmt.Errorf("both annotations %s and %s must be specified",
-				DashboardUIDAnnotation, PanelIDAnnotation),
+			name:                 "dashboardUID is not present",
+			annotations:          map[string]string{PanelIDAnnotation: "1234567890"},
+			expectedError:        ErrAlertRuleFailedValidation,
+			expectedErrContains:  fmt.Sprintf("%s and %s", DashboardUIDAnnotation, PanelIDAnnotation),
 			expectedDashboardUID: "",
 			expectedPanelID:      -1,
 		},
 		{
-			name:        "dashboardUID is present but empty",
-			annotations: map[string]string{DashboardUIDAnnotation: "", PanelIDAnnotation: "1234567890"},
-			expectedError: fmt.Errorf("both annotations %s and %s must be specified",
-				DashboardUIDAnnotation, PanelIDAnnotation),
+			name:                 "dashboardUID is present but empty",
+			annotations:          map[string]string{DashboardUIDAnnotation: "", PanelIDAnnotation: "1234567890"},
+			expectedError:        ErrAlertRuleFailedValidation,
+			expectedErrContains:  fmt.Sprintf("%s and %s", DashboardUIDAnnotation, PanelIDAnnotation),
 			expectedDashboardUID: "",
 			expectedPanelID:      -1,
 		},
 		{
-			name:        "panelID is not present",
-			annotations: map[string]string{DashboardUIDAnnotation: "cKy7f6Hk"},
-			expectedError: fmt.Errorf("both annotations %s and %s must be specified",
-				DashboardUIDAnnotation, PanelIDAnnotation),
+			name:                 "panelID is not present",
+			annotations:          map[string]string{DashboardUIDAnnotation: "cKy7f6Hk"},
+			expectedError:        ErrAlertRuleFailedValidation,
+			expectedErrContains:  fmt.Sprintf("%s and %s", DashboardUIDAnnotation, PanelIDAnnotation),
 			expectedDashboardUID: "",
 			expectedPanelID:      -1,
 		},
 		{
-			name:        "panelID is present but empty",
-			annotations: map[string]string{DashboardUIDAnnotation: "cKy7f6Hk", PanelIDAnnotation: ""},
-			expectedError: fmt.Errorf("both annotations %s and %s must be specified",
-				DashboardUIDAnnotation, PanelIDAnnotation),
+			name:                 "panelID is present but empty",
+			annotations:          map[string]string{DashboardUIDAnnotation: "cKy7f6Hk", PanelIDAnnotation: ""},
+			expectedError:        ErrAlertRuleFailedValidation,
+			expectedErrContains:  fmt.Sprintf("%s and %s", DashboardUIDAnnotation, PanelIDAnnotation),
 			expectedDashboardUID: "",
 			expectedPanelID:      -1,
 		},
 		{
 			name:                 "dashboardUID and panelID are present but panelID is not a correct int64",
 			annotations:          map[string]string{DashboardUIDAnnotation: "cKy7f6Hk", PanelIDAnnotation: "fgh"},
-			expectedError:        fmt.Errorf("annotation %s must be a valid integer Panel ID", PanelIDAnnotation),
+			expectedError:        ErrAlertRuleFailedValidation,
+			expectedErrContains:  PanelIDAnnotation,
 			expectedDashboardUID: "",
 			expectedPanelID:      -1,
 		},
@@ -148,14 +199,16 @@ func TestSetDashboardAndPanelFromAnnotations(t *testing.T) {
 
 	for _, tc := range testCases {
 		t.Run(tc.name, func(t *testing.T) {
-			rule := AlertRuleGen(func(rule *AlertRule) {
-				rule.Annotations = tc.annotations
-				rule.DashboardUID = nil
-				rule.PanelID = nil
-			})()
+			rule := RuleGen.With(
+				RuleMuts.WithDashboardAndPanel(nil, nil),
+				RuleMuts.WithAnnotations(tc.annotations),
+			).Generate()
 			err := rule.SetDashboardAndPanelFromAnnotations()
 
-			require.Equal(t, tc.expectedError, err)
+			require.ErrorIs(t, err, tc.expectedError)
+			if tc.expectedErrContains != "" {
+				require.ErrorContains(t, err, tc.expectedErrContains)
+			}
 			require.Equal(t, tc.expectedDashboardUID, rule.GetDashboardUID())
 			require.Equal(t, tc.expectedPanelID, rule.GetPanelID())
 		})
@@ -205,16 +258,26 @@ func TestPatchPartialAlertRule(t *testing.T) {
 					r.IsPaused = true
 				},
 			},
+			{
+				name: "No metadata",
+				mutator: func(r *AlertRuleWithOptionals) {
+					r.Metadata = AlertRuleMetadata{}
+					r.HasMetadata = false
+				},
+			},
 		}
 
+		gen := RuleGen.With(
+			RuleMuts.WithFor(time.Duration(rand.Int63n(1000)+1)),
+			RuleMuts.WithEditorSettingsSimplifiedQueryAndExpressionsSection(true),
+		)
+
 		for _, testCase := range testCases {
 			t.Run(testCase.name, func(t *testing.T) {
 				var existing *AlertRuleWithOptionals
-				for {
-					rule := AlertRuleGen(func(rule *AlertRule) {
-						rule.For = time.Duration(rand.Int63n(1000) + 1)
-					})()
-					existing = &AlertRuleWithOptionals{AlertRule: *rule}
+				for i := 0; i < 10; i++ {
+					rule := gen.Generate()
+					existing = &AlertRuleWithOptionals{AlertRule: rule}
 					cloned := *existing
 					testCase.mutator(&cloned)
 					if !cmp.Equal(existing, cloned, cmp.FilterPath(func(path cmp.Path) bool {
@@ -294,15 +357,20 @@ func TestPatchPartialAlertRule(t *testing.T) {
 			},
 		}
 
+		gen := RuleGen.With(
+			RuleMuts.WithUniqueID(),
+			RuleMuts.WithFor(time.Duration(rand.Int63n(1000)+1)),
+		)
+
 		for _, testCase := range testCases {
 			t.Run(testCase.name, func(t *testing.T) {
 				var existing *AlertRule
 				for {
-					existing = AlertRuleGen()()
-					cloned := *existing
+					existing = gen.GenerateRef()
+					cloned := CopyRule(existing)
 					// make sure the generated rule does not match the mutated one
-					testCase.mutator(&cloned)
-					if !cmp.Equal(*existing, cloned, cmp.FilterPath(func(path cmp.Path) bool {
+					testCase.mutator(cloned)
+					if !cmp.Equal(existing, cloned, cmp.FilterPath(func(path cmp.Path) bool {
 						return path.String() == "Data.modelProps"
 					}, cmp.Ignore())) {
 						break
@@ -319,14 +387,14 @@ func TestPatchPartialAlertRule(t *testing.T) {
 
 func TestDiff(t *testing.T) {
 	t.Run("should return nil if there is no diff", func(t *testing.T) {
-		rule1 := AlertRuleGen()()
+		rule1 := RuleGen.GenerateRef()
 		rule2 := CopyRule(rule1)
 		result := rule1.Diff(rule2)
 		require.Emptyf(t, result, "expected diff to be empty. rule1: %#v, rule2: %#v\ndiff: %s", rule1, rule2, result)
 	})
 
 	t.Run("should respect fields to ignore", func(t *testing.T) {
-		rule1 := AlertRuleGen()()
+		rule1 := RuleGen.GenerateRef()
 		rule2 := CopyRule(rule1)
 		rule2.ID = rule1.ID/2 + 1
 		rule2.Version = rule1.Version/2 + 1
@@ -336,10 +404,10 @@ func TestDiff(t *testing.T) {
 	})
 
 	t.Run("should find diff in simple fields", func(t *testing.T) {
-		rule1 := AlertRuleGen()()
-		rule2 := AlertRuleGen()()
+		rule1 := RuleGen.GenerateRef()
+		rule2 := RuleGen.GenerateRef()
 
-		diffs := rule1.Diff(rule2, "Data", "Annotations", "Labels") // these fields will be tested separately
+		diffs := rule1.Diff(rule2, "Data", "Annotations", "Labels", "NotificationSettings") // these fields will be tested separately
 
 		difCnt := 0
 		if rule1.ID != rule2.ID {
@@ -450,6 +518,13 @@ func TestDiff(t *testing.T) {
 			assert.Equal(t, rule2.RuleGroupIndex, diff[0].Right.Interface())
 			difCnt++
 		}
+		if rule1.Record != rule2.Record {
+			diff := diffs.GetDiffsForField("Record")
+			assert.Len(t, diff, 1)
+			assert.Equal(t, rule1.Record, diff[0].Left.String())
+			assert.Equal(t, rule2.Record, diff[0].Right.String())
+			difCnt++
+		}
 
 		require.Lenf(t, diffs, difCnt, "Got some unexpected diffs. Either add to ignore or add assert to it")
 
@@ -459,7 +534,7 @@ func TestDiff(t *testing.T) {
 	})
 
 	t.Run("should not see difference between nil and empty Annotations", func(t *testing.T) {
-		rule1 := AlertRuleGen()()
+		rule1 := RuleGen.GenerateRef()
 		rule1.Annotations = make(map[string]string)
 		rule2 := CopyRule(rule1)
 		rule2.Annotations = nil
@@ -469,7 +544,7 @@ func TestDiff(t *testing.T) {
 	})
 
 	t.Run("should detect changes in Annotations", func(t *testing.T) {
-		rule1 := AlertRuleGen()()
+		rule1 := RuleGen.GenerateRef()
 		rule2 := CopyRule(rule1)
 
 		rule1.Annotations = map[string]string{
@@ -506,7 +581,7 @@ func TestDiff(t *testing.T) {
 	})
 
 	t.Run("should not see difference between nil and empty Labels", func(t *testing.T) {
-		rule1 := AlertRuleGen()()
+		rule1 := RuleGen.GenerateRef()
 		rule1.Annotations = make(map[string]string)
 		rule2 := CopyRule(rule1)
 		rule2.Annotations = nil
@@ -516,7 +591,7 @@ func TestDiff(t *testing.T) {
 	})
 
 	t.Run("should detect changes in Labels", func(t *testing.T) {
-		rule1 := AlertRuleGen()()
+		rule1 := RuleGen.GenerateRef()
 		rule2 := CopyRule(rule1)
 
 		rule1.Labels = map[string]string{
@@ -553,7 +628,7 @@ func TestDiff(t *testing.T) {
 	})
 
 	t.Run("should detect changes in Data", func(t *testing.T) {
-		rule1 := AlertRuleGen()()
+		rule1 := RuleGen.GenerateRef()
 		rule2 := CopyRule(rule1)
 
 		query1 := AlertQuery{
@@ -565,7 +640,7 @@ func TestDiff(t *testing.T) {
 			},
 			DatasourceUID: util.GenerateShortUID(),
 			Model:         json.RawMessage(`{ "test": "data"}`),
-			modelProps: map[string]interface{}{
+			modelProps: map[string]any{
 				"test": 1,
 			},
 		}
@@ -574,7 +649,7 @@ func TestDiff(t *testing.T) {
 
 		t.Run("should ignore modelProps", func(t *testing.T) {
 			query2 := query1
-			query2.modelProps = map[string]interface{}{
+			query2.modelProps = map[string]any{
 				"some": "other value",
 			}
 			rule2.Data = []AlertQuery{query2}
@@ -607,6 +682,21 @@ func TestDiff(t *testing.T) {
 			}
 		})
 
+		t.Run("should correctly detect no change with '<' and '>' in query", func(t *testing.T) {
+			old := query1
+			newQuery := query1
+			old.Model = json.RawMessage(`{"field1": "$A \u003c 1"}`)
+			newQuery.Model = json.RawMessage(`{"field1": "$A < 1"}`)
+			rule1.Data = []AlertQuery{old}
+			rule2.Data = []AlertQuery{newQuery}
+
+			diff := rule1.Diff(rule2)
+			assert.Nil(t, diff)
+
+			// reset rule1
+			rule1.Data = []AlertQuery{query1}
+		})
+
 		t.Run("should detect new changes in array if too many fields changed", func(t *testing.T) {
 			query2 := query1
 			query2.QueryType = "test"
@@ -633,6 +723,117 @@ func TestDiff(t *testing.T) {
 			}
 		})
 	})
+
+	t.Run("should detect changes in NotificationSettings", func(t *testing.T) {
+		rule1 := RuleGen.GenerateRef()
+
+		baseSettings := NotificationSettingsGen(NSMuts.WithGroupBy("test1", "test2"))()
+		rule1.NotificationSettings = []NotificationSettings{baseSettings}
+
+		addTime := func(d *model.Duration, duration time.Duration) *time.Duration {
+			dur := time.Duration(*d)
+			dur += duration
+			return &dur
+		}
+
+		testCases := []struct {
+			name                 string
+			notificationSettings NotificationSettings
+			diffs                cmputil.DiffReport
+		}{
+			{
+				name:                 "should detect changes in Receiver",
+				notificationSettings: CopyNotificationSettings(baseSettings, NSMuts.WithReceiver(baseSettings.Receiver+"-modified")),
+				diffs: []cmputil.Diff{
+					{
+						Path:  "NotificationSettings[0].Receiver",
+						Left:  reflect.ValueOf(baseSettings.Receiver),
+						Right: reflect.ValueOf(baseSettings.Receiver + "-modified"),
+					},
+				},
+			},
+			{
+				name:                 "should detect changes in GroupWait",
+				notificationSettings: CopyNotificationSettings(baseSettings, NSMuts.WithGroupWait(addTime(baseSettings.GroupWait, 1*time.Second))),
+				diffs: []cmputil.Diff{
+					{
+						Path:  "NotificationSettings[0].GroupWait",
+						Left:  reflect.ValueOf(*baseSettings.GroupWait),
+						Right: reflect.ValueOf(model.Duration(*addTime(baseSettings.GroupWait, 1*time.Second))),
+					},
+				},
+			},
+			{
+				name:                 "should detect changes in GroupInterval",
+				notificationSettings: CopyNotificationSettings(baseSettings, NSMuts.WithGroupInterval(addTime(baseSettings.GroupInterval, 1*time.Second))),
+				diffs: []cmputil.Diff{
+					{
+						Path:  "NotificationSettings[0].GroupInterval",
+						Left:  reflect.ValueOf(*baseSettings.GroupInterval),
+						Right: reflect.ValueOf(model.Duration(*addTime(baseSettings.GroupInterval, 1*time.Second))),
+					},
+				},
+			},
+			{
+				name:                 "should detect changes in RepeatInterval",
+				notificationSettings: CopyNotificationSettings(baseSettings, NSMuts.WithRepeatInterval(addTime(baseSettings.RepeatInterval, 1*time.Second))),
+				diffs: []cmputil.Diff{
+					{
+						Path:  "NotificationSettings[0].RepeatInterval",
+						Left:  reflect.ValueOf(*baseSettings.RepeatInterval),
+						Right: reflect.ValueOf(model.Duration(*addTime(baseSettings.RepeatInterval, 1*time.Second))),
+					},
+				},
+			},
+			{
+				name:                 "should detect changes in GroupBy",
+				notificationSettings: CopyNotificationSettings(baseSettings, NSMuts.WithGroupBy(baseSettings.GroupBy[0]+"-modified", baseSettings.GroupBy[1]+"-modified")),
+				diffs: []cmputil.Diff{
+					{
+						Path:  "NotificationSettings[0].GroupBy[0]",
+						Left:  reflect.ValueOf(baseSettings.GroupBy[0]),
+						Right: reflect.ValueOf(baseSettings.GroupBy[0] + "-modified"),
+					},
+					{
+						Path:  "NotificationSettings[0].GroupBy[1]",
+						Left:  reflect.ValueOf(baseSettings.GroupBy[1]),
+						Right: reflect.ValueOf(baseSettings.GroupBy[1] + "-modified"),
+					},
+				},
+			},
+			{
+				name:                 "should detect changes in MuteTimeIntervals",
+				notificationSettings: CopyNotificationSettings(baseSettings, NSMuts.WithMuteTimeIntervals(baseSettings.MuteTimeIntervals[0]+"-modified", baseSettings.MuteTimeIntervals[1]+"-modified")),
+				diffs: []cmputil.Diff{
+					{
+						Path:  "NotificationSettings[0].MuteTimeIntervals[0]",
+						Left:  reflect.ValueOf(baseSettings.MuteTimeIntervals[0]),
+						Right: reflect.ValueOf(baseSettings.MuteTimeIntervals[0] + "-modified"),
+					},
+					{
+						Path:  "NotificationSettings[0].MuteTimeIntervals[1]",
+						Left:  reflect.ValueOf(baseSettings.MuteTimeIntervals[1]),
+						Right: reflect.ValueOf(baseSettings.MuteTimeIntervals[1] + "-modified"),
+					},
+				},
+			},
+		}
+
+		for _, tt := range testCases {
+			t.Run(tt.name, func(t *testing.T) {
+				rule2 := CopyRule(rule1)
+				rule2.NotificationSettings = []NotificationSettings{tt.notificationSettings}
+				diffs := rule1.Diff(rule2)
+
+				cOpt := []cmp.Option{
+					cmpopts.IgnoreUnexported(cmputil.Diff{}),
+				}
+				if !cmp.Equal(diffs, tt.diffs, cOpt...) {
+					t.Errorf("Unexpected Diffs: %v", cmp.Diff(diffs, tt.diffs, cOpt...))
+				}
+			})
+		}
+	})
 }
 
 func TestSortByGroupIndex(t *testing.T) {
@@ -649,7 +850,9 @@ func TestSortByGroupIndex(t *testing.T) {
 	}
 
 	t.Run("should sort rules by GroupIndex", func(t *testing.T) {
-		rules := GenerateAlertRules(rand.Intn(15)+5, AlertRuleGen(WithUniqueGroupIndex()))
+		rules := RuleGen.With(
+			RuleMuts.WithUniqueGroupIndex(),
+		).GenerateManyRef(5, 20)
 		ensureNotSorted(t, rules, func(i, j int) bool {
 			return rules[i].RuleGroupIndex < rules[j].RuleGroupIndex
 		})
@@ -660,7 +863,10 @@ func TestSortByGroupIndex(t *testing.T) {
 	})
 
 	t.Run("should sort by ID if same GroupIndex", func(t *testing.T) {
-		rules := GenerateAlertRules(rand.Intn(15)+5, AlertRuleGen(WithUniqueID(), WithGroupIndex(rand.Int())))
+		rules := RuleGen.With(
+			RuleMuts.WithUniqueID(),
+			RuleMuts.WithGroupIndex(rand.Int()),
+		).GenerateManyRef(5, 20)
 		ensureNotSorted(t, rules, func(i, j int) bool {
 			return rules[i].ID < rules[j].ID
 		})
