@@ -19,12 +19,13 @@ import (
 	"github.com/grafana/grafana/pkg/infra/log/logtest"
 	"github.com/grafana/grafana/pkg/services/datasources"
 	fake_ds "github.com/grafana/grafana/pkg/services/datasources/fakes"
+	"github.com/grafana/grafana/pkg/services/featuremgmt"
 	"github.com/grafana/grafana/pkg/services/ngalert/api/tooling/definitions"
 	"github.com/grafana/grafana/pkg/services/ngalert/metrics"
 	"github.com/grafana/grafana/pkg/services/ngalert/models"
 	"github.com/grafana/grafana/pkg/services/ngalert/notifier"
-	"github.com/grafana/grafana/pkg/services/ngalert/provisioning"
 	"github.com/grafana/grafana/pkg/services/ngalert/store"
+	"github.com/grafana/grafana/pkg/services/ngalert/tests/fakes"
 	fake_secrets "github.com/grafana/grafana/pkg/services/secrets/fakes"
 	secretsManager "github.com/grafana/grafana/pkg/services/secrets/manager"
 	"github.com/grafana/grafana/pkg/setting"
@@ -56,20 +57,20 @@ func TestIntegrationSendingToExternalAlertmanager(t *testing.T) {
 		URL:   fakeAM.Server.URL,
 		OrgID: ruleKey.OrgID,
 		Type:  datasources.DS_ALERTMANAGER,
-		JsonData: simplejson.NewFromAny(map[string]interface{}{
+		JsonData: simplejson.NewFromAny(map[string]any{
 			"handleGrafanaManagedAlerts": true,
 			"implementation":             "prometheus",
 		}),
 	}
 	alertsRouter := NewAlertsRouter(moa, fakeAdminConfigStore, mockedClock, appUrl, map[int64]struct{}{}, 10*time.Minute,
-		&fake_ds.FakeDataSourceService{DataSources: []*datasources.DataSource{&ds1}}, fake_secrets.NewFakeSecretsService())
+		&fake_ds.FakeDataSourceService{DataSources: []*datasources.DataSource{&ds1}}, fake_secrets.NewFakeSecretsService(), featuremgmt.WithFeatures())
 
 	mockedGetAdminConfigurations.Return([]*models.AdminConfiguration{
 		{OrgID: ruleKey.OrgID, SendAlertsTo: models.AllAlertmanagers},
 	}, nil)
 	// Make sure we sync the configuration at least once before the evaluation happens to guarantee the sender is running
 	// when the first alert triggers.
-	require.NoError(t, alertsRouter.SyncAndApplyConfigFromDatabase())
+	require.NoError(t, alertsRouter.SyncAndApplyConfigFromDatabase(context.Background()))
 	require.Equal(t, 1, len(alertsRouter.externalAlertmanagers))
 	require.Equal(t, 1, len(alertsRouter.externalAlertmanagersCfgHash))
 
@@ -84,7 +85,7 @@ func TestIntegrationSendingToExternalAlertmanager(t *testing.T) {
 		alerts.PostableAlerts = append(alerts.PostableAlerts, alert)
 	}
 
-	alertsRouter.Send(ruleKey, alerts)
+	alertsRouter.Send(context.Background(), ruleKey, alerts)
 
 	// Eventually, our Alertmanager should have received at least one alert.
 	assertAlertsDelivered(t, fakeAM, expected)
@@ -92,7 +93,7 @@ func TestIntegrationSendingToExternalAlertmanager(t *testing.T) {
 	// Now, let's remove the Alertmanager from the admin configuration.
 	mockedGetAdminConfigurations.Return(nil, nil)
 	// Again, make sure we sync and verify the externalAlertmanagers.
-	require.NoError(t, alertsRouter.SyncAndApplyConfigFromDatabase())
+	require.NoError(t, alertsRouter.SyncAndApplyConfigFromDatabase(context.Background()))
 	require.Equal(t, 0, len(alertsRouter.externalAlertmanagers))
 	require.Equal(t, 0, len(alertsRouter.externalAlertmanagersCfgHash))
 
@@ -126,14 +127,14 @@ func TestIntegrationSendingToExternalAlertmanager_WithMultipleOrgs(t *testing.T)
 		URL:   fakeAM.Server.URL,
 		OrgID: ruleKey1.OrgID,
 		Type:  datasources.DS_ALERTMANAGER,
-		JsonData: simplejson.NewFromAny(map[string]interface{}{
+		JsonData: simplejson.NewFromAny(map[string]any{
 			"handleGrafanaManagedAlerts": true,
 			"implementation":             "prometheus",
 		}),
 	}
 	fakeDs := &fake_ds.FakeDataSourceService{DataSources: []*datasources.DataSource{&ds1}}
 	alertsRouter := NewAlertsRouter(moa, fakeAdminConfigStore, mockedClock, appUrl, map[int64]struct{}{}, 10*time.Minute,
-		fakeDs, fake_secrets.NewFakeSecretsService())
+		fakeDs, fake_secrets.NewFakeSecretsService(), featuremgmt.WithFeatures())
 
 	mockedGetAdminConfigurations.Return([]*models.AdminConfiguration{
 		{OrgID: ruleKey1.OrgID, SendAlertsTo: models.AllAlertmanagers},
@@ -141,7 +142,7 @@ func TestIntegrationSendingToExternalAlertmanager_WithMultipleOrgs(t *testing.T)
 
 	// Make sure we sync the configuration at least once before the evaluation happens to guarantee the sender is running
 	// when the first alert triggers.
-	require.NoError(t, alertsRouter.SyncAndApplyConfigFromDatabase())
+	require.NoError(t, alertsRouter.SyncAndApplyConfigFromDatabase(context.Background()))
 	require.Equal(t, 1, len(alertsRouter.externalAlertmanagers))
 	require.Equal(t, 1, len(alertsRouter.externalAlertmanagersCfgHash))
 
@@ -153,7 +154,7 @@ func TestIntegrationSendingToExternalAlertmanager_WithMultipleOrgs(t *testing.T)
 		URL:   fakeAM.Server.URL,
 		OrgID: ruleKey2.OrgID,
 		Type:  datasources.DS_ALERTMANAGER,
-		JsonData: simplejson.NewFromAny(map[string]interface{}{
+		JsonData: simplejson.NewFromAny(map[string]any{
 			"handleGrafanaManagedAlerts": true,
 			"implementation":             "prometheus",
 		}),
@@ -166,7 +167,7 @@ func TestIntegrationSendingToExternalAlertmanager_WithMultipleOrgs(t *testing.T)
 	}, nil)
 
 	// If we sync again, new externalAlertmanagers must have spawned.
-	require.NoError(t, alertsRouter.SyncAndApplyConfigFromDatabase())
+	require.NoError(t, alertsRouter.SyncAndApplyConfigFromDatabase(context.Background()))
 	require.Equal(t, 2, len(alertsRouter.externalAlertmanagers))
 	require.Equal(t, 2, len(alertsRouter.externalAlertmanagersCfgHash))
 
@@ -188,8 +189,8 @@ func TestIntegrationSendingToExternalAlertmanager_WithMultipleOrgs(t *testing.T)
 		alerts2.PostableAlerts = append(alerts2.PostableAlerts, alert)
 	}
 
-	alertsRouter.Send(ruleKey1, alerts1)
-	alertsRouter.Send(ruleKey2, alerts2)
+	alertsRouter.Send(context.Background(), ruleKey1, alerts1)
+	alertsRouter.Send(context.Background(), ruleKey2, alerts2)
 
 	assertAlertsDelivered(t, fakeAM, expected)
 
@@ -199,7 +200,7 @@ func TestIntegrationSendingToExternalAlertmanager_WithMultipleOrgs(t *testing.T)
 		URL:   fakeAM2.Server.URL,
 		OrgID: ruleKey2.OrgID,
 		Type:  datasources.DS_ALERTMANAGER,
-		JsonData: simplejson.NewFromAny(map[string]interface{}{
+		JsonData: simplejson.NewFromAny(map[string]any{
 			"handleGrafanaManagedAlerts": true,
 			"implementation":             "prometheus",
 		}),
@@ -215,7 +216,7 @@ func TestIntegrationSendingToExternalAlertmanager_WithMultipleOrgs(t *testing.T)
 	currentHash := alertsRouter.externalAlertmanagersCfgHash[ruleKey2.OrgID]
 
 	// Now, sync again.
-	require.NoError(t, alertsRouter.SyncAndApplyConfigFromDatabase())
+	require.NoError(t, alertsRouter.SyncAndApplyConfigFromDatabase(context.Background()))
 
 	// The hash for org two should not be the same and we should still have two externalAlertmanagers.
 	require.NotEqual(t, alertsRouter.externalAlertmanagersCfgHash[ruleKey2.OrgID], currentHash)
@@ -235,7 +236,7 @@ func TestIntegrationSendingToExternalAlertmanager_WithMultipleOrgs(t *testing.T)
 	currentHash = alertsRouter.externalAlertmanagersCfgHash[ruleKey1.OrgID]
 
 	// Now, sync again.
-	require.NoError(t, alertsRouter.SyncAndApplyConfigFromDatabase())
+	require.NoError(t, alertsRouter.SyncAndApplyConfigFromDatabase(context.Background()))
 
 	// The old configuration should not be running.
 	require.NotEqual(t, alertsRouter.externalAlertmanagersCfgHash[ruleKey1.OrgID], currentHash)
@@ -248,13 +249,13 @@ func TestIntegrationSendingToExternalAlertmanager_WithMultipleOrgs(t *testing.T)
 		{OrgID: ruleKey2.OrgID},
 	}, nil)
 
-	require.NoError(t, alertsRouter.SyncAndApplyConfigFromDatabase())
+	require.NoError(t, alertsRouter.SyncAndApplyConfigFromDatabase(context.Background()))
 	require.NotEqual(t, alertsRouter.externalAlertmanagersCfgHash[ruleKey1.OrgID], currentHash)
 
 	// Finally, remove everything.
 	mockedGetAdminConfigurations.Return([]*models.AdminConfiguration{}, nil)
 
-	require.NoError(t, alertsRouter.SyncAndApplyConfigFromDatabase())
+	require.NoError(t, alertsRouter.SyncAndApplyConfigFromDatabase(context.Background()))
 
 	require.Equal(t, 0, len(alertsRouter.externalAlertmanagers))
 	require.Equal(t, 0, len(alertsRouter.externalAlertmanagersCfgHash))
@@ -286,20 +287,20 @@ func TestChangingAlertmanagersChoice(t *testing.T) {
 		URL:   fakeAM.Server.URL,
 		OrgID: ruleKey.OrgID,
 		Type:  datasources.DS_ALERTMANAGER,
-		JsonData: simplejson.NewFromAny(map[string]interface{}{
+		JsonData: simplejson.NewFromAny(map[string]any{
 			"handleGrafanaManagedAlerts": true,
 			"implementation":             "prometheus",
 		}),
 	}
 	alertsRouter := NewAlertsRouter(moa, fakeAdminConfigStore, mockedClock, appUrl, map[int64]struct{}{},
-		10*time.Minute, &fake_ds.FakeDataSourceService{DataSources: []*datasources.DataSource{&ds}}, fake_secrets.NewFakeSecretsService())
+		10*time.Minute, &fake_ds.FakeDataSourceService{DataSources: []*datasources.DataSource{&ds}}, fake_secrets.NewFakeSecretsService(), featuremgmt.WithFeatures())
 
 	mockedGetAdminConfigurations.Return([]*models.AdminConfiguration{
 		{OrgID: ruleKey.OrgID, SendAlertsTo: models.AllAlertmanagers},
 	}, nil)
 	// Make sure we sync the configuration at least once before the evaluation happens to guarantee the sender is running
 	// when the first alert triggers.
-	require.NoError(t, alertsRouter.SyncAndApplyConfigFromDatabase())
+	require.NoError(t, alertsRouter.SyncAndApplyConfigFromDatabase(context.Background()))
 	require.Equal(t, 1, len(alertsRouter.externalAlertmanagers))
 	require.Equal(t, 1, len(alertsRouter.externalAlertmanagersCfgHash))
 	require.Equal(t, models.AllAlertmanagers, alertsRouter.sendAlertsTo[ruleKey.OrgID])
@@ -314,7 +315,7 @@ func TestChangingAlertmanagersChoice(t *testing.T) {
 		expected = append(expected, &alert)
 		alerts.PostableAlerts = append(alerts.PostableAlerts, alert)
 	}
-	alertsRouter.Send(ruleKey, alerts)
+	alertsRouter.Send(context.Background(), ruleKey, alerts)
 
 	// Eventually, our Alertmanager should have received at least one alert.
 	assertAlertsDelivered(t, fakeAM, expected)
@@ -324,7 +325,7 @@ func TestChangingAlertmanagersChoice(t *testing.T) {
 		{OrgID: ruleKey.OrgID, SendAlertsTo: models.ExternalAlertmanagers},
 	}, nil)
 	// Again, make sure we sync and verify the externalAlertmanagers.
-	require.NoError(t, alertsRouter.SyncAndApplyConfigFromDatabase())
+	require.NoError(t, alertsRouter.SyncAndApplyConfigFromDatabase(context.Background()))
 	require.Equal(t, 1, len(alertsRouter.externalAlertmanagers))
 	require.Equal(t, 1, len(alertsRouter.externalAlertmanagersCfgHash))
 
@@ -338,7 +339,7 @@ func TestChangingAlertmanagersChoice(t *testing.T) {
 
 	// Again, make sure we sync and verify the externalAlertmanagers.
 	// externalAlertmanagers should be running even though alerts are being handled externally.
-	require.NoError(t, alertsRouter.SyncAndApplyConfigFromDatabase())
+	require.NoError(t, alertsRouter.SyncAndApplyConfigFromDatabase(context.Background()))
 	require.Equal(t, 1, len(alertsRouter.externalAlertmanagers))
 	require.Equal(t, 1, len(alertsRouter.externalAlertmanagersCfgHash))
 
@@ -346,11 +347,91 @@ func TestChangingAlertmanagersChoice(t *testing.T) {
 	assertAlertmanagersStatusForOrg(t, alertsRouter, ruleKey.OrgID, 1, 0)
 	require.Equal(t, models.InternalAlertmanager, alertsRouter.sendAlertsTo[ruleKey.OrgID])
 
-	alertsRouter.Send(ruleKey, alerts)
+	alertsRouter.Send(context.Background(), ruleKey, alerts)
 
 	am, err := moa.AlertmanagerFor(ruleKey.OrgID)
 	require.NoError(t, err)
-	actualAlerts, err := am.GetAlerts(true, true, true, nil, "")
+	actualAlerts, err := am.GetAlerts(context.Background(), true, true, true, nil, "")
+	require.NoError(t, err)
+	require.Len(t, actualAlerts, len(expected))
+}
+
+func TestAlertmanagersChoiceWithDisableExternalFeatureToggle(t *testing.T) {
+	ruleKey := models.GenerateRuleKey(1)
+
+	fakeAM := NewFakeExternalAlertmanager(t)
+	defer fakeAM.Close()
+
+	fakeAdminConfigStore := &store.AdminConfigurationStoreMock{}
+	mockedGetAdminConfigurations := fakeAdminConfigStore.EXPECT().GetAdminConfigurations()
+
+	mockedClock := clock.NewMock()
+	mockedClock.Set(time.Now())
+
+	moa := createMultiOrgAlertmanager(t, []int64{1})
+
+	appUrl := &url.URL{
+		Scheme: "http",
+		Host:   "localhost",
+	}
+
+	ds := datasources.DataSource{
+		URL:   fakeAM.Server.URL,
+		OrgID: ruleKey.OrgID,
+		Type:  datasources.DS_ALERTMANAGER,
+		JsonData: simplejson.NewFromAny(map[string]any{
+			"handleGrafanaManagedAlerts": true,
+			"implementation":             "prometheus",
+		}),
+	}
+
+	var expected []*models2.PostableAlert
+	alerts := definitions.PostableAlerts{}
+	for i := 0; i < rand.Intn(5)+1; i++ {
+		alert := generatePostableAlert(t, mockedClock)
+		expected = append(expected, &alert)
+		alerts.PostableAlerts = append(alerts.PostableAlerts, alert)
+	}
+
+	alertsRouter := NewAlertsRouter(moa, fakeAdminConfigStore, mockedClock, appUrl, map[int64]struct{}{},
+		10*time.Minute, &fake_ds.FakeDataSourceService{DataSources: []*datasources.DataSource{&ds}},
+		fake_secrets.NewFakeSecretsService(), featuremgmt.WithFeatures(featuremgmt.FlagAlertingDisableSendAlertsExternal))
+
+	// Test that we only send to the internal Alertmanager even though the configuration specifies AllAlertmanagers.
+
+	mockedGetAdminConfigurations.Return([]*models.AdminConfiguration{
+		{OrgID: ruleKey.OrgID, SendAlertsTo: models.AllAlertmanagers},
+	}, nil)
+
+	require.NoError(t, alertsRouter.SyncAndApplyConfigFromDatabase(context.Background()))
+	require.Equal(t, 0, len(alertsRouter.externalAlertmanagers))
+	require.Equal(t, 0, len(alertsRouter.externalAlertmanagersCfgHash))
+	require.Equal(t, models.InternalAlertmanager, alertsRouter.sendAlertsTo[ruleKey.OrgID])
+
+	alertsRouter.Send(context.Background(), ruleKey, alerts)
+
+	am, err := moa.AlertmanagerFor(ruleKey.OrgID)
+	require.NoError(t, err)
+	actualAlerts, err := am.GetAlerts(context.Background(), true, true, true, nil, "")
+	require.NoError(t, err)
+	require.Len(t, actualAlerts, len(expected))
+
+	// Test that we still only send to the internal alertmanager even though the configuration specifies ExternalAlertmanagers.
+
+	mockedGetAdminConfigurations.Return([]*models.AdminConfiguration{
+		{OrgID: ruleKey.OrgID, SendAlertsTo: models.ExternalAlertmanagers},
+	}, nil)
+
+	require.NoError(t, alertsRouter.SyncAndApplyConfigFromDatabase(context.Background()))
+	require.Equal(t, 0, len(alertsRouter.externalAlertmanagers))
+	require.Equal(t, 0, len(alertsRouter.externalAlertmanagersCfgHash))
+	require.Equal(t, models.InternalAlertmanager, alertsRouter.sendAlertsTo[ruleKey.OrgID])
+
+	alertsRouter.Send(context.Background(), ruleKey, alerts)
+
+	am, err = moa.AlertmanagerFor(ruleKey.OrgID)
+	require.NoError(t, err)
+	actualAlerts, err = am.GetAlerts(context.Background(), true, true, true, nil, "")
 	require.NoError(t, err)
 	require.Len(t, actualAlerts, len(expected))
 }
@@ -405,12 +486,25 @@ func createMultiOrgAlertmanager(t *testing.T, orgs []int64) *notifier.MultiOrgAl
 	}
 
 	cfgStore := notifier.NewFakeConfigStore(t, make(map[int64]*models.AlertConfiguration))
-	kvStore := notifier.NewFakeKVStore(t)
+	kvStore := fakes.NewFakeKVStore(t)
 	registry := prometheus.NewPedanticRegistry()
 	m := metrics.NewNGAlert(registry)
 	secretsService := secretsManager.SetupTestService(t, fake_secrets.NewFakeSecretsStore())
 	decryptFn := secretsService.GetDecryptedValue
-	moa, err := notifier.NewMultiOrgAlertmanager(cfg, cfgStore, &orgStore, kvStore, provisioning.NewFakeProvisioningStore(), decryptFn, m.GetMultiOrgAlertmanagerMetrics(), nil, log.New("testlogger"), secretsService)
+	moa, err := notifier.NewMultiOrgAlertmanager(
+		cfg,
+		cfgStore,
+		orgStore,
+		kvStore,
+		fakes.NewFakeProvisioningStore(),
+		decryptFn,
+		m.GetMultiOrgAlertmanagerMetrics(),
+		nil,
+		fakes.NewFakeReceiverPermissionsService(),
+		log.New("testlogger"),
+		secretsService,
+		featuremgmt.WithFeatures(),
+	)
 	require.NoError(t, err)
 	require.NoError(t, moa.LoadAndSyncAlertmanagersForOrgs(context.Background()))
 	require.Eventually(t, func() bool {
@@ -460,6 +554,18 @@ func TestBuildExternalURL(t *testing.T) {
 			},
 			expectedURL: "https://johndoe:123@localhost:9000",
 		},
+		{
+			name: "datasource with auth that needs escaping",
+			ds: &datasources.DataSource{
+				URL:           "https://localhost:9000",
+				BasicAuth:     true,
+				BasicAuthUser: "johndoe",
+				SecureJsonData: map[string][]byte{
+					"basicAuthPassword": []byte("123#!"),
+				},
+			},
+			expectedURL: "https://johndoe:123%23%21@localhost:9000",
+		},
 		{
 			name: "datasource with auth and path",
 			ds: &datasources.DataSource{
@@ -527,6 +633,42 @@ func TestBuildExternalURL(t *testing.T) {
 			},
 			expectedURL: "https://localhost:9000/path/to/am",
 		},
+		{
+			name: "do not add /alertmanager to path when last segment already contains it",
+			ds: &datasources.DataSource{
+				URL: "https://localhost:9000/path/to/alertmanager",
+				JsonData: func() *simplejson.Json {
+					r := simplejson.New()
+					r.Set("implementation", "mimir")
+					return r
+				}(),
+			},
+			expectedURL: "https://localhost:9000/path/to/alertmanager",
+		},
+		{
+			name: "add /alertmanager to path when last segment does not exactly match",
+			ds: &datasources.DataSource{
+				URL: "https://localhost:9000/path/to/alertmanagerasdf",
+				JsonData: func() *simplejson.Json {
+					r := simplejson.New()
+					r.Set("implementation", "mimir")
+					return r
+				}(),
+			},
+			expectedURL: "https://localhost:9000/path/to/alertmanagerasdf/alertmanager",
+		},
+		{
+			name: "add /alertmanager to path when exists but is not last segment",
+			ds: &datasources.DataSource{
+				URL: "https://localhost:9000/alertmanager/path/to/am",
+				JsonData: func() *simplejson.Json {
+					r := simplejson.New()
+					r.Set("implementation", "mimir")
+					return r
+				}(),
+			},
+			expectedURL: "https://localhost:9000/alertmanager/path/to/am/alertmanager",
+		},
 	}
 	for _, test := range tests {
 		t.Run(test.name, func(t *testing.T) {
@@ -566,7 +708,7 @@ func TestAlertManagers_buildRedactedAMs(t *testing.T) {
 		amUrls   []string
 		errCalls int
 		errLog   string
-		errCtx   []interface{}
+		errCtx   []any
 		expected []string
 	}{
 		{
@@ -583,13 +725,20 @@ func TestAlertManagers_buildRedactedAMs(t *testing.T) {
 			amUrls:   []string{"1234://user:password@localhost:9094"},
 			errCalls: 1,
 			errLog:   "Failed to parse alertmanager string",
-			expected: nil,
+			expected: []string{},
 		},
 	}
 
 	for _, tt := range tc {
 		t.Run(tt.name, func(t *testing.T) {
-			require.Equal(t, tt.expected, buildRedactedAMs(&fakeLogger, tt.amUrls, tt.orgId))
+			var cfgs []ExternalAMcfg
+			for _, url := range tt.amUrls {
+				cfgs = append(cfgs, ExternalAMcfg{
+					URL: url,
+				})
+			}
+
+			require.Equal(t, tt.expected, buildRedactedAMs(&fakeLogger, cfgs, tt.orgId))
 			require.Equal(t, tt.errCalls, fakeLogger.ErrorLogs.Calls)
 			require.Equal(t, tt.errLog, fakeLogger.ErrorLogs.Message)
 		})
