@@ -8,6 +8,7 @@ import (
 	"github.com/stretchr/testify/require"
 
 	"github.com/grafana/grafana/pkg/components/satokengen"
+	"github.com/grafana/grafana/pkg/services/org"
 	"github.com/grafana/grafana/pkg/services/serviceaccounts"
 	"github.com/grafana/grafana/pkg/services/serviceaccounts/tests"
 )
@@ -15,9 +16,9 @@ import (
 func TestIntegrationStore_UsageStats(t *testing.T) {
 	saToCreate := tests.TestUser{Login: "servicetestwithTeam@admin", IsServiceAccount: true}
 	db, store := setupTestDatabase(t)
-	sa := tests.SetupUserServiceAccount(t, db, saToCreate)
+	sa := tests.SetupUserServiceAccount(t, db, store.cfg, saToCreate)
 
-	db.Cfg.SATokenExpirationDayLimit = 4
+	store.cfg.SATokenExpirationDayLimit = 4
 
 	keyName := t.Name()
 	key, err := satokengen.New(keyName)
@@ -33,10 +34,18 @@ func TestIntegrationStore_UsageStats(t *testing.T) {
 	_, err = store.AddServiceAccountToken(context.Background(), sa.ID, &cmd)
 	require.NoError(t, err)
 
+	role := org.RoleNone
+	form := serviceaccounts.UpdateServiceAccountForm{
+		Role: &role,
+	}
+	_, err = store.UpdateServiceAccount(context.Background(), sa.OrgID, sa.ID, &form)
+	require.NoError(t, err)
+
 	stats, err := store.GetUsageMetrics(context.Background())
 	require.NoError(t, err)
 
 	assert.Equal(t, int64(1), stats.ServiceAccounts)
+	assert.Equal(t, int64(1), stats.ServiceAccountsWithNoRole)
 	assert.Equal(t, int64(1), stats.Tokens)
 	assert.Equal(t, true, stats.ForcedExpiryEnabled)
 }
