@@ -8,14 +8,15 @@ import (
 	"time"
 
 	"github.com/grafana/grafana-plugin-sdk-go/backend"
+	"github.com/grafana/grafana-plugin-sdk-go/backend/log"
+	"github.com/grafana/grafana/pkg/tsdb/cloudwatch/kinds/dataquery"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 
-	"github.com/grafana/grafana/pkg/infra/log/logtest"
 	"github.com/grafana/grafana/pkg/tsdb/cloudwatch/utils"
 )
 
-var logger = &logtest.Fake{}
+var logger = log.NewNullLogger()
 
 func TestCloudWatchQuery(t *testing.T) {
 	t.Run("Deeplink", func(t *testing.T) {
@@ -37,12 +38,12 @@ func TestCloudWatchQuery(t *testing.T) {
 				MetricEditorMode: MetricEditorModeBuilder,
 			}
 
-			deepLink, err := query.BuildDeepLink(startTime, endTime, false)
+			deepLink, err := query.BuildDeepLink(startTime, endTime)
 			require.NoError(t, err)
 			assert.Empty(t, deepLink)
 		})
 
-		t.Run("does not include label in case dynamic label is diabled", func(t *testing.T) {
+		t.Run("includes label and it's a metric stat query", func(t *testing.T) {
 			startTime := time.Now()
 			endTime := startTime.Add(2 * time.Hour)
 			query := &CloudWatchQuery{
@@ -61,36 +62,12 @@ func TestCloudWatchQuery(t *testing.T) {
 				MetricEditorMode: MetricEditorModeBuilder,
 			}
 
-			deepLink, err := query.BuildDeepLink(startTime, endTime, false)
+			deepLink, err := query.BuildDeepLink(startTime, endTime)
 			require.NoError(t, err)
-			assert.NotContains(t, deepLink, "label")
+			assert.Contains(t, deepLink, "label")
 		})
 
-		t.Run("includes label in case dynamic label is enabled and it's a metric stat query", func(t *testing.T) {
-			startTime := time.Now()
-			endTime := startTime.Add(2 * time.Hour)
-			query := &CloudWatchQuery{
-				RefId:      "A",
-				Region:     "us-east-1",
-				Expression: "",
-				Statistic:  "Average",
-				Period:     300,
-				Id:         "id1",
-				MatchExact: true,
-				Label:      "${PROP('Namespace')}",
-				Dimensions: map[string][]string{
-					"InstanceId": {"i-12345678"},
-				},
-				MetricQueryType:  MetricQueryTypeSearch,
-				MetricEditorMode: MetricEditorModeBuilder,
-			}
-
-			deepLink, err := query.BuildDeepLink(startTime, endTime, false)
-			require.NoError(t, err)
-			assert.NotContains(t, deepLink, "label")
-		})
-
-		t.Run("includes label in case dynamic label is enabled and it's a math expression query", func(t *testing.T) {
+		t.Run("includes label and it's a math expression query", func(t *testing.T) {
 			startTime := time.Now()
 			endTime := startTime.Add(2 * time.Hour)
 			query := &CloudWatchQuery{
@@ -106,9 +83,9 @@ func TestCloudWatchQuery(t *testing.T) {
 				MetricEditorMode: MetricEditorModeRaw,
 			}
 
-			deepLink, err := query.BuildDeepLink(startTime, endTime, false)
+			deepLink, err := query.BuildDeepLink(startTime, endTime)
 			require.NoError(t, err)
-			assert.NotContains(t, deepLink, "label")
+			assert.Contains(t, deepLink, "label")
 		})
 
 		t.Run("includes account id in case its a metric stat query and an account id is set", func(t *testing.T) {
@@ -131,7 +108,7 @@ func TestCloudWatchQuery(t *testing.T) {
 				MetricEditorMode: MetricEditorModeBuilder,
 			}
 
-			deepLink, err := query.BuildDeepLink(startTime, endTime, false)
+			deepLink, err := query.BuildDeepLink(startTime, endTime)
 			require.NoError(t, err)
 			assert.Contains(t, deepLink, "accountId%22%3A%22123456789")
 		})
@@ -153,7 +130,7 @@ func TestCloudWatchQuery(t *testing.T) {
 				MetricEditorMode: MetricEditorModeRaw,
 			}
 
-			deepLink, err := query.BuildDeepLink(startTime, endTime, false)
+			deepLink, err := query.BuildDeepLink(startTime, endTime)
 			require.NoError(t, err)
 			assert.NotContains(t, deepLink, "accountId%22%3A%22123456789")
 		})
@@ -287,16 +264,6 @@ func TestCloudWatchQuery(t *testing.T) {
 	})
 }
 
-func TestQueryJSON(t *testing.T) {
-	jsonString := []byte(`{
-		"type": "timeSeriesQuery"
-	}`)
-	var res metricsDataQuery
-	err := json.Unmarshal(jsonString, &res)
-	require.NoError(t, err)
-	assert.Equal(t, "timeSeriesQuery", res.QueryType)
-}
-
 func TestRequestParser(t *testing.T) {
 	t.Run("legacy statistics field is migrated: migrates first stat only", func(t *testing.T) {
 		oldQuery := []backend.DataQuery{
@@ -319,7 +286,38 @@ func TestRequestParser(t *testing.T) {
 			},
 		}
 
-		migratedQueries, err := ParseMetricDataQueries(oldQuery, time.Now(), time.Now(), "us-east-2", logger, false, false)
+		migratedQueries, err := ParseMetricDataQueries(oldQuery, time.Now(), time.Now(), "us-east-2", logger, false)
+		assert.NoError(t, err)
+		require.Len(t, migratedQueries, 1)
+		require.NotNil(t, migratedQueries[0])
+
+		migratedQuery := migratedQueries[0]
+		assert.Equal(t, "A", migratedQuery.RefId)
+		assert.Equal(t, "Average", migratedQuery.Statistic)
+	})
+
+	t.Run("legacy statistics field is migrated: if no stat, uses Average", func(t *testing.T) {
+		oldQuery := []backend.DataQuery{
+			{
+				MaxDataPoints: 0,
+				QueryType:     "timeSeriesQuery",
+				Interval:      0,
+				RefID:         "A",
+				JSON: json.RawMessage(`{
+				   "region":"us-east-1",
+				   "namespace":"ec2",
+				   "metricName":"CPUUtilization",
+				   "dimensions":{
+						"InstanceId": ["test"]
+					},
+				   "statistics":[],
+				   "period":"600",
+				   "hide":false
+				}`),
+			},
+		}
+
+		migratedQueries, err := ParseMetricDataQueries(oldQuery, time.Now(), time.Now(), "us-east-2", logger, false)
 		assert.NoError(t, err)
 		require.Len(t, migratedQueries, 1)
 		require.NotNil(t, migratedQueries[0])
@@ -350,7 +348,7 @@ func TestRequestParser(t *testing.T) {
 			},
 		}
 
-		results, err := ParseMetricDataQueries(query, time.Now().Add(-2*time.Hour), time.Now().Add(-time.Hour), "us-east-2", logger, false, false)
+		results, err := ParseMetricDataQueries(query, time.Now().Add(-2*time.Hour), time.Now().Add(-time.Hour), "us-east-2", logger, false)
 		require.NoError(t, err)
 		require.Len(t, results, 1)
 		res := results[0]
@@ -393,7 +391,7 @@ func TestRequestParser(t *testing.T) {
 			},
 		}
 
-		results, err := ParseMetricDataQueries(query, time.Now().Add(-2*time.Hour), time.Now().Add(-time.Hour), "us-east-2", logger, false, false)
+		results, err := ParseMetricDataQueries(query, time.Now().Add(-2*time.Hour), time.Now().Add(-time.Hour), "us-east-2", logger, false)
 		assert.NoError(t, err)
 		require.Len(t, results, 1)
 		res := results[0]
@@ -426,7 +424,7 @@ func TestRequestParser(t *testing.T) {
 			},
 		}
 
-		_, err := ParseMetricDataQueries(query, time.Now().Add(-2*time.Hour), time.Now().Add(-time.Hour), "us-east-2", logger, false, false)
+		_, err := ParseMetricDataQueries(query, time.Now().Add(-2*time.Hour), time.Now().Add(-time.Hour), "us-east-2", logger, false)
 		require.Error(t, err)
 
 		assert.Equal(t, `error parsing query "", failed to parse dimensions: unknown type as dimension value`, err.Error())
@@ -455,7 +453,7 @@ func Test_ParseMetricDataQueries_periods(t *testing.T) {
 			},
 		}
 
-		res, err := ParseMetricDataQueries(query, time.Now().Add(-2*time.Hour), time.Now().Add(-time.Hour), "us-east-2", logger, false, false)
+		res, err := ParseMetricDataQueries(query, time.Now().Add(-2*time.Hour), time.Now().Add(-time.Hour), "us-east-2", logger, false)
 		assert.NoError(t, err)
 		require.Len(t, res, 1)
 		require.NotNil(t, res[0])
@@ -487,7 +485,7 @@ func Test_ParseMetricDataQueries_periods(t *testing.T) {
 			to := time.Now()
 			from := to.Local().Add(time.Minute * time.Duration(5))
 
-			res, err := ParseMetricDataQueries(query, from, to, "us-east-2", logger, false, false)
+			res, err := ParseMetricDataQueries(query, from, to, "us-east-2", logger, false)
 			require.NoError(t, err)
 			require.Len(t, res, 1)
 			assert.Equal(t, 60, res[0].Period)
@@ -497,7 +495,7 @@ func Test_ParseMetricDataQueries_periods(t *testing.T) {
 			to := time.Now()
 			from := to.AddDate(0, 0, -1)
 
-			res, err := ParseMetricDataQueries(query, from, to, "us-east-2", logger, false, false)
+			res, err := ParseMetricDataQueries(query, from, to, "us-east-2", logger, false)
 			require.NoError(t, err)
 			require.Len(t, res, 1)
 			assert.Equal(t, 60, res[0].Period)
@@ -506,7 +504,7 @@ func Test_ParseMetricDataQueries_periods(t *testing.T) {
 		t.Run("Time range is 2 days", func(t *testing.T) {
 			to := time.Now()
 			from := to.AddDate(0, 0, -2)
-			res, err := ParseMetricDataQueries(query, from, to, "us-east-2", logger, false, false)
+			res, err := ParseMetricDataQueries(query, from, to, "us-east-2", logger, false)
 			require.NoError(t, err)
 			require.Len(t, res, 1)
 			assert.Equal(t, 300, res[0].Period)
@@ -516,7 +514,7 @@ func Test_ParseMetricDataQueries_periods(t *testing.T) {
 			to := time.Now()
 			from := to.AddDate(0, 0, -7)
 
-			res, err := ParseMetricDataQueries(query, from, to, "us-east-2", logger, false, false)
+			res, err := ParseMetricDataQueries(query, from, to, "us-east-2", logger, false)
 			require.NoError(t, err)
 			require.Len(t, res, 1)
 			assert.Equal(t, 900, res[0].Period)
@@ -526,7 +524,7 @@ func Test_ParseMetricDataQueries_periods(t *testing.T) {
 			to := time.Now()
 			from := to.AddDate(0, 0, -30)
 
-			res, err := ParseMetricDataQueries(query, from, to, "us-east-2", logger, false, false)
+			res, err := ParseMetricDataQueries(query, from, to, "us-east-2", logger, false)
 			require.NoError(t, err)
 			require.Len(t, res, 1)
 			assert.Equal(t, 3600, res[0].Period)
@@ -536,7 +534,7 @@ func Test_ParseMetricDataQueries_periods(t *testing.T) {
 			to := time.Now()
 			from := to.AddDate(0, 0, -90)
 
-			res, err := ParseMetricDataQueries(query, from, to, "us-east-2", logger, false, false)
+			res, err := ParseMetricDataQueries(query, from, to, "us-east-2", logger, false)
 			require.NoError(t, err)
 			require.Len(t, res, 1)
 			assert.Equal(t, 21600, res[0].Period)
@@ -546,7 +544,7 @@ func Test_ParseMetricDataQueries_periods(t *testing.T) {
 			to := time.Now()
 			from := to.AddDate(-1, 0, 0)
 
-			res, err := ParseMetricDataQueries(query, from, to, "us-east-2", logger, false, false)
+			res, err := ParseMetricDataQueries(query, from, to, "us-east-2", logger, false)
 			require.Nil(t, err)
 			require.Len(t, res, 1)
 			assert.Equal(t, 21600, res[0].Period)
@@ -556,7 +554,7 @@ func Test_ParseMetricDataQueries_periods(t *testing.T) {
 			to := time.Now()
 			from := to.AddDate(-2, 0, 0)
 
-			res, err := ParseMetricDataQueries(query, from, to, "us-east-2", logger, false, false)
+			res, err := ParseMetricDataQueries(query, from, to, "us-east-2", logger, false)
 			require.NoError(t, err)
 			require.Len(t, res, 1)
 			assert.Equal(t, 86400, res[0].Period)
@@ -565,7 +563,7 @@ func Test_ParseMetricDataQueries_periods(t *testing.T) {
 		t.Run("Time range is 2 days, but 16 days ago", func(t *testing.T) {
 			to := time.Now().AddDate(0, 0, -14)
 			from := to.AddDate(0, 0, -2)
-			res, err := ParseMetricDataQueries(query, from, to, "us-east-2", logger, false, false)
+			res, err := ParseMetricDataQueries(query, from, to, "us-east-2", logger, false)
 			require.NoError(t, err)
 			require.Len(t, res, 1)
 			assert.Equal(t, 300, res[0].Period)
@@ -574,7 +572,7 @@ func Test_ParseMetricDataQueries_periods(t *testing.T) {
 		t.Run("Time range is 2 days, but 90 days ago", func(t *testing.T) {
 			to := time.Now().AddDate(0, 0, -88)
 			from := to.AddDate(0, 0, -2)
-			res, err := ParseMetricDataQueries(query, from, to, "us-east-2", logger, false, false)
+			res, err := ParseMetricDataQueries(query, from, to, "us-east-2", logger, false)
 			require.NoError(t, err)
 			require.Len(t, res, 1)
 			assert.Equal(t, 3600, res[0].Period)
@@ -583,7 +581,7 @@ func Test_ParseMetricDataQueries_periods(t *testing.T) {
 		t.Run("Time range is 2 days, but 456 days ago", func(t *testing.T) {
 			to := time.Now().AddDate(0, 0, -454)
 			from := to.AddDate(0, 0, -2)
-			res, err := ParseMetricDataQueries(query, from, to, "us-east-2", logger, false, false)
+			res, err := ParseMetricDataQueries(query, from, to, "us-east-2", logger, false)
 			require.NoError(t, err)
 			require.Len(t, res, 1)
 			assert.Equal(t, 21600, res[0].Period)
@@ -598,7 +596,7 @@ func Test_ParseMetricDataQueries_periods(t *testing.T) {
 				}`),
 			},
 		}
-		_, err := ParseMetricDataQueries(query, time.Now().Add(-2*time.Hour), time.Now().Add(-time.Hour), "us-east-2", logger, false, false)
+		_, err := ParseMetricDataQueries(query, time.Now().Add(-2*time.Hour), time.Now().Add(-time.Hour), "us-east-2", logger, false)
 		require.Error(t, err)
 		assert.Equal(t, `error parsing query "", failed to parse period as duration: time: invalid duration "invalid"`, err.Error())
 	})
@@ -613,7 +611,7 @@ func Test_ParseMetricDataQueries_periods(t *testing.T) {
 			},
 		}
 
-		res, err := ParseMetricDataQueries(query, time.Now().Add(-2*time.Hour), time.Now().Add(-time.Hour), "us-east-2", logger, false, false)
+		res, err := ParseMetricDataQueries(query, time.Now().Add(-2*time.Hour), time.Now().Add(-time.Hour), "us-east-2", logger, false)
 		assert.NoError(t, err)
 
 		require.Len(t, res, 1)
@@ -622,11 +620,11 @@ func Test_ParseMetricDataQueries_periods(t *testing.T) {
 }
 
 func Test_ParseMetricDataQueries_query_type_and_metric_editor_mode_and_GMD_query_api_mode(t *testing.T) {
-	const dummyTestEditorMode MetricEditorMode = 99
+	const dummyTestEditorMode dataquery.MetricEditorMode = 99
 	testCases := map[string]struct {
 		extraDataQueryJson       string
-		expectedMetricQueryType  MetricQueryType
-		expectedMetricEditorMode MetricEditorMode
+		expectedMetricQueryType  dataquery.MetricQueryType
+		expectedMetricEditorMode dataquery.MetricEditorMode
 		expectedGMDApiMode       GMDApiMode
 	}{
 		"no metric query type, no metric editor mode, no expression": {
@@ -700,7 +698,7 @@ func Test_ParseMetricDataQueries_query_type_and_metric_editor_mode_and_GMD_query
 					),
 				},
 			}
-			res, err := ParseMetricDataQueries(query, time.Now(), time.Now(), "us-east-2", logger, false, false)
+			res, err := ParseMetricDataQueries(query, time.Now(), time.Now(), "us-east-2", logger, false)
 			require.NoError(t, err)
 			require.Len(t, res, 1)
 			require.NotNil(t, res[0])
@@ -726,7 +724,7 @@ func Test_ParseMetricDataQueries_hide_and_ReturnData(t *testing.T) {
 				}`),
 			},
 		}
-		res, err := ParseMetricDataQueries(query, time.Now().Add(-2*time.Hour), time.Now().Add(-time.Hour), "us-east-2", logger, false, false)
+		res, err := ParseMetricDataQueries(query, time.Now().Add(-2*time.Hour), time.Now().Add(-time.Hour), "us-east-2", logger, false)
 		require.NoError(t, err)
 		require.Len(t, res, 1)
 		require.NotNil(t, res[0])
@@ -747,7 +745,7 @@ func Test_ParseMetricDataQueries_hide_and_ReturnData(t *testing.T) {
 				}`),
 			},
 		}
-		res, err := ParseMetricDataQueries(query, time.Now().Add(-2*time.Hour), time.Now().Add(-time.Hour), "us-east-2", logger, false, false)
+		res, err := ParseMetricDataQueries(query, time.Now().Add(-2*time.Hour), time.Now().Add(-time.Hour), "us-east-2", logger, false)
 		require.NoError(t, err)
 		require.Len(t, res, 1)
 		require.NotNil(t, res[0])
@@ -768,7 +766,7 @@ func Test_ParseMetricDataQueries_hide_and_ReturnData(t *testing.T) {
 				}`),
 			},
 		}
-		res, err := ParseMetricDataQueries(query, time.Now().Add(-2*time.Hour), time.Now().Add(-time.Hour), "us-east-2", logger, false, false)
+		res, err := ParseMetricDataQueries(query, time.Now().Add(-2*time.Hour), time.Now().Add(-time.Hour), "us-east-2", logger, false)
 		require.NoError(t, err)
 		require.Len(t, res, 1)
 		require.NotNil(t, res[0])
@@ -787,7 +785,7 @@ func Test_ParseMetricDataQueries_hide_and_ReturnData(t *testing.T) {
 				}`),
 			},
 		}
-		res, err := ParseMetricDataQueries(query, time.Now().Add(-2*time.Hour), time.Now().Add(-time.Hour), "us-east-2", logger, false, false)
+		res, err := ParseMetricDataQueries(query, time.Now().Add(-2*time.Hour), time.Now().Add(-time.Hour), "us-east-2", logger, false)
 		require.NoError(t, err)
 		require.Len(t, res, 1)
 		require.NotNil(t, res[0])
@@ -808,7 +806,7 @@ func Test_ParseMetricDataQueries_hide_and_ReturnData(t *testing.T) {
 				}`),
 			},
 		}
-		res, err := ParseMetricDataQueries(query, time.Now().Add(-2*time.Hour), time.Now().Add(-time.Hour), "us-east-2", logger, false, false)
+		res, err := ParseMetricDataQueries(query, time.Now().Add(-2*time.Hour), time.Now().Add(-time.Hour), "us-east-2", logger, false)
 		require.NoError(t, err)
 		require.Len(t, res, 1)
 		require.NotNil(t, res[0])
@@ -829,7 +827,7 @@ func Test_ParseMetricDataQueries_hide_and_ReturnData(t *testing.T) {
 				}`),
 			},
 		}
-		res, err := ParseMetricDataQueries(query, time.Now().Add(-2*time.Hour), time.Now().Add(-time.Hour), "us-east-2", logger, false, false)
+		res, err := ParseMetricDataQueries(query, time.Now().Add(-2*time.Hour), time.Now().Add(-time.Hour), "us-east-2", logger, false)
 		require.NoError(t, err)
 		require.Len(t, res, 1)
 		require.NotNil(t, res[0])
@@ -852,7 +850,7 @@ func Test_ParseMetricDataQueries_ID(t *testing.T) {
 				}`),
 			},
 		}
-		res, err := ParseMetricDataQueries(query, time.Now().Add(-2*time.Hour), time.Now().Add(-time.Hour), "us-east-2", logger, false, false)
+		res, err := ParseMetricDataQueries(query, time.Now().Add(-2*time.Hour), time.Now().Add(-time.Hour), "us-east-2", logger, false)
 		require.NoError(t, err)
 		require.Len(t, res, 1)
 		require.NotNil(t, res[0])
@@ -873,7 +871,7 @@ func Test_ParseMetricDataQueries_ID(t *testing.T) {
 				}`),
 			},
 		}
-		res, err := ParseMetricDataQueries(query, time.Now().Add(-2*time.Hour), time.Now().Add(-time.Hour), "us-east-2", logger, false, false)
+		res, err := ParseMetricDataQueries(query, time.Now().Add(-2*time.Hour), time.Now().Add(-time.Hour), "us-east-2", logger, false)
 		require.NoError(t, err)
 		require.Len(t, res, 1)
 		require.NotNil(t, res[0])
@@ -890,7 +888,6 @@ func Test_ParseMetricDataQueries_sets_label_when_label_is_present_in_json_query(
 				   "region":"us-east-1",
 				   "namespace":"ec2",
 				   "metricName":"CPUUtilization",
-				   "alias":"some alias",
 				   "label":"some label",
 				   "dimensions":{"InstanceId":["test"]},
 				   "statistic":"Average",
@@ -900,11 +897,10 @@ func Test_ParseMetricDataQueries_sets_label_when_label_is_present_in_json_query(
 		},
 	}
 
-	res, err := ParseMetricDataQueries(query, time.Now(), time.Now(), "us-east-2", logger, true, false)
+	res, err := ParseMetricDataQueries(query, time.Now(), time.Now(), "us-east-2", logger, false)
 	assert.NoError(t, err)
 	require.Len(t, res, 1)
 	require.NotNil(t, res[0])
-	assert.Equal(t, "some alias", res[0].Alias) // untouched
 	assert.Equal(t, "some label", res[0].Label)
 }
 
@@ -930,24 +926,26 @@ func Test_migrateAliasToDynamicLabel_single_query_preserves_old_alias_and_create
 			false := false
 
 			queryToMigrate := metricsDataQuery{
-				Region:     "us-east-1",
-				Namespace:  "ec2",
-				MetricName: "CPUUtilization",
-				Alias:      tc.inputAlias,
-				Dimensions: map[string]interface{}{
-					"InstanceId": []interface{}{"test"},
+				CloudWatchMetricsQuery: dataquery.CloudWatchMetricsQuery{
+					Region:     utils.Pointer("us-east-1"),
+					Namespace:  utils.Pointer("ec2"),
+					MetricName: utils.Pointer("CPUUtilization"),
+					Alias:      utils.Pointer(tc.inputAlias),
+					Dimensions: &dataquery.Dimensions{
+						"InstanceId": []any{"test"},
+					},
+					Statistic: &average,
+					Period:    utils.Pointer("600"),
+					Hide:      &false,
 				},
-				Statistic: &average,
-				Period:    "600",
-				Hide:      &false,
 			}
 
-			assert.Equal(t, tc.expectedLabel, getLabel(queryToMigrate, true))
+			assert.Equal(t, tc.expectedLabel, getLabel(queryToMigrate))
 		})
 	}
 }
 func Test_ParseMetricDataQueries_migrate_alias_to_label(t *testing.T) {
-	t.Run("migrates alias to label when label does not already exist and feature toggle enabled", func(t *testing.T) {
+	t.Run("migrates alias to label when label does not already exist", func(t *testing.T) {
 		query := []backend.DataQuery{
 			{
 				JSON: []byte(`{
@@ -964,13 +962,12 @@ func Test_ParseMetricDataQueries_migrate_alias_to_label(t *testing.T) {
 			},
 		}
 
-		res, err := ParseMetricDataQueries(query, time.Now(), time.Now(), "us-east-2", logger, true, false)
+		res, err := ParseMetricDataQueries(query, time.Now(), time.Now(), "us-east-2", logger, false)
 		assert.NoError(t, err)
 
 		require.Len(t, res, 1)
 		require.NotNil(t, res[0])
 
-		assert.Equal(t, "{{period}} {{any_other_word}}", res[0].Alias)
 		assert.Equal(t, "${PROP('Period')} ${PROP('Dim.any_other_word')}", res[0].Label)
 		assert.Equal(t, map[string][]string{"InstanceId": {"test"}}, res[0].Dimensions)
 		assert.Equal(t, true, res[0].ReturnData)
@@ -1011,7 +1008,7 @@ func Test_ParseMetricDataQueries_migrate_alias_to_label(t *testing.T) {
 			},
 		}
 
-		res, err := ParseMetricDataQueries(query, time.Now(), time.Now(), "us-east-2", logger, true, false)
+		res, err := ParseMetricDataQueries(query, time.Now(), time.Now(), "us-east-2", logger, false)
 		assert.NoError(t, err)
 		require.Len(t, res, 2)
 
@@ -1020,7 +1017,6 @@ func Test_ParseMetricDataQueries_migrate_alias_to_label(t *testing.T) {
 		})
 
 		require.NotNil(t, res[0])
-		assert.Equal(t, "{{period}} {{any_other_word}}", res[0].Alias)
 		assert.Equal(t, "${PROP('Period')} ${PROP('Dim.any_other_word')}", res[0].Label)
 		assert.Equal(t, map[string][]string{"InstanceId": {"test"}}, res[0].Dimensions)
 		assert.Equal(t, true, res[0].ReturnData)
@@ -1031,7 +1027,6 @@ func Test_ParseMetricDataQueries_migrate_alias_to_label(t *testing.T) {
 		assert.Equal(t, "Average", res[0].Statistic)
 
 		require.NotNil(t, res[1])
-		assert.Equal(t, "{{  label }}", res[1].Alias)
 		assert.Equal(t, "${LABEL}", res[1].Label)
 		assert.Equal(t, map[string][]string{"InstanceId": {"test"}}, res[1].Dimensions)
 		assert.Equal(t, true, res[1].ReturnData)
@@ -1048,19 +1043,11 @@ func Test_ParseMetricDataQueries_migrate_alias_to_label(t *testing.T) {
 			dynamicLabelsFeatureToggleEnabled bool
 			expectedLabel                     string
 		}{
-			"when label already exists, feature toggle enabled": {
+			"when label already exists": {
 				labelJson:                         `"label":"some label",`,
 				dynamicLabelsFeatureToggleEnabled: true,
-				expectedLabel:                     "some label"},
-			"when label does not exist, feature toggle is disabled": {
-				labelJson:                         "",
-				dynamicLabelsFeatureToggleEnabled: false,
-				expectedLabel:                     "",
+				expectedLabel:                     "some label",
 			},
-			"when label already exists, feature toggle is disabled": {
-				labelJson:                         `"label":"some label",`,
-				dynamicLabelsFeatureToggleEnabled: false,
-				expectedLabel:                     "some label"},
 		}
 		for name, tc := range testCases {
 			t.Run(name, func(t *testing.T) {
@@ -1080,13 +1067,12 @@ func Test_ParseMetricDataQueries_migrate_alias_to_label(t *testing.T) {
 				}`, tc.labelJson)),
 					},
 				}
-				res, err := ParseMetricDataQueries(query, time.Now(), time.Now(), "us-east-2", logger, tc.dynamicLabelsFeatureToggleEnabled, false)
+				res, err := ParseMetricDataQueries(query, time.Now(), time.Now(), "us-east-2", logger, false)
 				assert.NoError(t, err)
 
 				require.Len(t, res, 1)
 				require.NotNil(t, res[0])
 
-				assert.Equal(t, "{{period}} {{any_other_word}}", res[0].Alias)
 				assert.Equal(t, tc.expectedLabel, res[0].Label)
 				assert.Equal(t, map[string][]string{"InstanceId": {"test"}}, res[0].Dimensions)
 				assert.Equal(t, true, res[0].ReturnData)
@@ -1107,7 +1093,7 @@ func Test_ParseMetricDataQueries_statistics_and_query_type_validation_and_MatchE
 				{
 					JSON: []byte("{}"),
 				},
-			}, time.Now(), time.Now(), "us-east-2", logger, false, false)
+			}, time.Now(), time.Now(), "us-east-2", logger, false)
 		assert.Error(t, err)
 		assert.Equal(t, `error parsing query "", query must have either statistic or statistics field`, err.Error())
 
@@ -1120,7 +1106,7 @@ func Test_ParseMetricDataQueries_statistics_and_query_type_validation_and_MatchE
 				{
 					JSON: []byte(`{"type":"some other type", "statistic":"Average", "matchExact":false}`),
 				},
-			}, time.Now(), time.Now(), "us-east-2", logger, false, false)
+			}, time.Now(), time.Now(), "us-east-2", logger, false)
 		assert.NoError(t, err)
 
 		assert.Empty(t, actual)
@@ -1132,7 +1118,7 @@ func Test_ParseMetricDataQueries_statistics_and_query_type_validation_and_MatchE
 				{
 					JSON: []byte(`{"statistic":"Average"}`),
 				},
-			}, time.Now(), time.Now(), "us-east-2", logger, false, false)
+			}, time.Now(), time.Now(), "us-east-2", logger, false)
 		assert.NoError(t, err)
 
 		assert.NotEmpty(t, actual)
@@ -1144,7 +1130,7 @@ func Test_ParseMetricDataQueries_statistics_and_query_type_validation_and_MatchE
 				{
 					JSON: []byte(`{"statistic":"Average"}`),
 				},
-			}, time.Now(), time.Now(), "us-east-2", logger, false, false)
+			}, time.Now(), time.Now(), "us-east-2", logger, false)
 		assert.NoError(t, err)
 
 		assert.Len(t, actual, 1)
@@ -1158,7 +1144,7 @@ func Test_ParseMetricDataQueries_statistics_and_query_type_validation_and_MatchE
 				{
 					JSON: []byte(`{"statistic":"Average","matchExact":false}`),
 				},
-			}, time.Now(), time.Now(), "us-east-2", logger, false, false)
+			}, time.Now(), time.Now(), "us-east-2", logger, false)
 		assert.NoError(t, err)
 
 		assert.Len(t, actual, 1)
@@ -1174,7 +1160,7 @@ func Test_ParseMetricDataQueries_account_Id(t *testing.T) {
 				{
 					JSON: []byte(`{"accountId":"some account id", "statistic":"Average"}`),
 				},
-			}, time.Now(), time.Now(), "us-east-2", logger, false, true)
+			}, time.Now(), time.Now(), "us-east-2", logger, true)
 		assert.NoError(t, err)
 
 		require.Len(t, actual, 1)
@@ -1189,7 +1175,7 @@ func Test_ParseMetricDataQueries_account_Id(t *testing.T) {
 				{
 					JSON: []byte(`{"accountId":"some account id", "statistic":"Average"}`),
 				},
-			}, time.Now(), time.Now(), "us-east-2", logger, false, false)
+			}, time.Now(), time.Now(), "us-east-2", logger, false)
 		assert.NoError(t, err)
 
 		require.Len(t, actual, 1)
@@ -1221,7 +1207,7 @@ func Test_ParseMetricDataQueries_default_region(t *testing.T) {
 		}
 
 		region := "us-east-2"
-		res, err := ParseMetricDataQueries(query, time.Now().Add(-2*time.Hour), time.Now().Add(-time.Hour), region, logger, false, false)
+		res, err := ParseMetricDataQueries(query, time.Now().Add(-2*time.Hour), time.Now().Add(-time.Hour), region, logger, false)
 		assert.NoError(t, err)
 		require.Len(t, res, 1)
 		require.NotNil(t, res[0])
@@ -1259,7 +1245,6 @@ func Test_ParseMetricDataQueries_ApplyMacros(t *testing.T) {
 								"region":"us-east-1",
 								"namespace":"ec2",
 								"metricName":"CPUUtilization",
-								"alias":"{{period}} {{any_other_word}}",
 								"dimensions":{"InstanceId":["test"]},
 								"statistic":"Average",
 								"period":"600",
@@ -1269,7 +1254,7 @@ func Test_ParseMetricDataQueries_ApplyMacros(t *testing.T) {
 								"metricEditorMode": 1
 							 }`),
 						},
-					}, tc.startTime, time.Now(), "us-east-1", logger, false, false)
+					}, tc.startTime, time.Now(), "us-east-1", logger, false)
 				assert.NoError(t, err)
 				assert.Equal(t, fmt.Sprintf("SEARCH('{AWS/EC2,InstanceId}', 'Average', %s)", tc.expectedPeriod), actual[0].Expression)
 			})
@@ -1285,7 +1270,6 @@ func Test_ParseMetricDataQueries_ApplyMacros(t *testing.T) {
 						"region":"us-east-1",
 						"namespace":"ec2",
 						"metricName":"CPUUtilization",
-						"alias":"{{period}} {{any_other_word}}",
 						"dimensions":{"InstanceId":["test"]},
 						"statistic":"Average",
 						"period":"600",
@@ -1295,7 +1279,7 @@ func Test_ParseMetricDataQueries_ApplyMacros(t *testing.T) {
 						"metricEditorMode": 1
 					 }`),
 				},
-			}, time.Now(), time.Now(), "us-east-1", logger, false, false)
+			}, time.Now(), time.Now(), "us-east-1", logger, false)
 		assert.NoError(t, err)
 		assert.Equal(t, "SEARCH('{AWS/EC2,InstanceId}', 'Average', $__period_auto)", actual[0].Expression)
 	})
