@@ -4,12 +4,12 @@ import (
 	"encoding/json"
 	"os"
 	"path/filepath"
-	"strings"
 	"testing"
 	"time"
 
 	"github.com/aws/aws-sdk-go/aws"
 	"github.com/aws/aws-sdk-go/service/cloudwatch"
+	"github.com/grafana/grafana/pkg/tsdb/cloudwatch/features"
 	"github.com/grafana/grafana/pkg/tsdb/cloudwatch/models"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
@@ -27,8 +27,6 @@ func loadGetMetricDataOutputsFromFile(filePath string) ([]*cloudwatch.GetMetricD
 }
 
 func TestCloudWatchResponseParser(t *testing.T) {
-	startTime := time.Now()
-	endTime := startTime.Add(2 * time.Hour)
 	t.Run("when aggregating multi-outputs response", func(t *testing.T) {
 		getMetricDataOutputs, err := loadGetMetricDataOutputsFromFile("./testdata/multiple-outputs-query-a.json")
 		require.NoError(t, err)
@@ -138,13 +136,27 @@ func TestCloudWatchResponseParser(t *testing.T) {
 		})
 	})
 
-	t.Run("Expand dimension value using exact match", func(t *testing.T) {
+	t.Run("when receiving a permissions error should pass it to the user", func(t *testing.T) {
+		getMetricDataOutputs, err := loadGetMetricDataOutputsFromFile("./testdata/permissions-error-output.json")
+		require.NoError(t, err)
+		aggregatedResponse := aggregateResponse(getMetricDataOutputs)
+
+		assert.True(t, aggregatedResponse["a"].HasPermissionError)
+		assert.Equal(t, "Access denied when getting data - please check that you have the pi:GetResourceMetrics permission", aggregatedResponse["a"].PermissionErrorMessage)
+	})
+}
+
+func Test_buildDataFrames_parse_label_to_name_and_labels(t *testing.T) {
+	startTime := time.Now()
+	endTime := startTime.Add(2 * time.Hour)
+
+	t.Run("using multi filter", func(t *testing.T) {
 		timestamp := time.Unix(0, 0)
 		response := &models.QueryRowResponse{
 			Metrics: []*cloudwatch.MetricDataResult{
 				{
 					Id:    aws.String("id1"),
-					Label: aws.String("lb1"),
+					Label: aws.String("lb1|&|lb1"),
 					Timestamps: []*time.Time{
 						aws.Time(timestamp),
 						aws.Time(timestamp.Add(time.Minute)),
@@ -159,7 +171,7 @@ func TestCloudWatchResponseParser(t *testing.T) {
 				},
 				{
 					Id:    aws.String("id2"),
-					Label: aws.String("lb2"),
+					Label: aws.String("lb2|&|lb2"),
 					Timestamps: []*time.Time{
 						aws.Time(timestamp),
 						aws.Time(timestamp.Add(time.Minute)),
@@ -186,92 +198,33 @@ func TestCloudWatchResponseParser(t *testing.T) {
 			},
 			Statistic:        "Average",
 			Period:           60,
-			Alias:            "{{LoadBalancer}} Expanded",
 			MetricQueryType:  models.MetricQueryTypeSearch,
 			MetricEditorMode: models.MetricEditorModeBuilder,
+			MatchExact:       true,
 		}
-		frames, err := buildDataFrames(startTime, endTime, *response, query, false)
+		frames, err := buildDataFrames(contextWithFeaturesEnabled(features.FlagCloudWatchNewLabelParsing), startTime, endTime, *response, query)
 		require.NoError(t, err)
 
 		frame1 := frames[0]
-		assert.Equal(t, "lb1 Expanded", frame1.Name)
+		assert.Equal(t, "lb1", frame1.Name)
+		assert.Equal(t, "lb1", frame1.Fields[1].Labels["Series"])
 		assert.Equal(t, "lb1", frame1.Fields[1].Labels["LoadBalancer"])
+		assert.Equal(t, "tg", frame1.Fields[1].Labels["TargetGroup"])
 
 		frame2 := frames[1]
-		assert.Equal(t, "lb2 Expanded", frame2.Name)
+		assert.Equal(t, "lb2", frame2.Name)
+		assert.Equal(t, "lb2", frame2.Fields[1].Labels["Series"])
 		assert.Equal(t, "lb2", frame2.Fields[1].Labels["LoadBalancer"])
+		assert.Equal(t, "tg", frame2.Fields[1].Labels["TargetGroup"])
 	})
 
-	t.Run("Expand dimension value using substring", func(t *testing.T) {
-		timestamp := time.Unix(0, 0)
-		response := &models.QueryRowResponse{
-			Metrics: []*cloudwatch.MetricDataResult{
-				{
-					Id:    aws.String("id1"),
-					Label: aws.String("lb1 Sum"),
-					Timestamps: []*time.Time{
-						aws.Time(timestamp),
-						aws.Time(timestamp.Add(time.Minute)),
-						aws.Time(timestamp.Add(3 * time.Minute)),
-					},
-					Values: []*float64{
-						aws.Float64(10),
-						aws.Float64(20),
-						aws.Float64(30),
-					},
-					StatusCode: aws.String("Complete"),
-				},
-				{
-					Id:    aws.String("id2"),
-					Label: aws.String("lb2 Average"),
-					Timestamps: []*time.Time{
-						aws.Time(timestamp),
-						aws.Time(timestamp.Add(time.Minute)),
-						aws.Time(timestamp.Add(3 * time.Minute)),
-					},
-					Values: []*float64{
-						aws.Float64(10),
-						aws.Float64(20),
-						aws.Float64(30),
-					},
-					StatusCode: aws.String("Complete"),
-				},
-			}}
-
-		query := &models.CloudWatchQuery{
-			RefId:      "refId1",
-			Region:     "us-east-1",
-			Namespace:  "AWS/ApplicationELB",
-			MetricName: "TargetResponseTime",
-			Dimensions: map[string][]string{
-				"LoadBalancer": {"lb1", "lb2"},
-				"TargetGroup":  {"tg"},
-			},
-			Statistic:        "Average",
-			Period:           60,
-			Alias:            "{{LoadBalancer}} Expanded",
-			MetricQueryType:  models.MetricQueryTypeSearch,
-			MetricEditorMode: models.MetricEditorModeBuilder,
-		}
-		frames, err := buildDataFrames(startTime, endTime, *response, query, false)
-		require.NoError(t, err)
-
-		frame1 := frames[0]
-		assert.Equal(t, "lb1 Expanded", frame1.Name)
-		assert.Equal(t, "lb1", frame1.Fields[1].Labels["LoadBalancer"])
-
-		frame2 := frames[1]
-		assert.Equal(t, "lb2 Expanded", frame2.Name)
-		assert.Equal(t, "lb2", frame2.Fields[1].Labels["LoadBalancer"])
-	})
-
-	t.Run("Expand dimension value using wildcard", func(t *testing.T) {
+	t.Run("using multiple wildcard filters", func(t *testing.T) {
 		timestamp := time.Unix(0, 0)
 		response := &models.QueryRowResponse{
 			Metrics: []*cloudwatch.MetricDataResult{
 				{
 					Id:    aws.String("lb3"),
-					Label: aws.String("lb3"),
+					Label: aws.String("some label lb3|&|inst1|&|balancer 1"),
 					Timestamps: []*time.Time{
 						aws.Time(timestamp),
 						aws.Time(timestamp.Add(time.Minute)),
@@ -286,7 +239,7 @@ func TestCloudWatchResponseParser(t *testing.T) {
 				},
 				{
 					Id:    aws.String("lb4"),
-					Label: aws.String("lb4"),
+					Label: aws.String("some label lb4|&|inst2|&|balancer 2"),
 					Timestamps: []*time.Time{
 						aws.Time(timestamp),
 						aws.Time(timestamp.Add(time.Minute)),
@@ -309,28 +262,38 @@ func TestCloudWatchResponseParser(t *testing.T) {
 			MetricName: "TargetResponseTime",
 			Dimensions: map[string][]string{
 				"LoadBalancer": {"*"},
+				"InstanceType": {"*"},
 				"TargetGroup":  {"tg"},
 			},
 			Statistic:        "Average",
 			Period:           60,
-			Alias:            "{{LoadBalancer}} Expanded",
 			MetricQueryType:  models.MetricQueryTypeSearch,
 			MetricEditorMode: models.MetricEditorModeBuilder,
 		}
-		frames, err := buildDataFrames(startTime, endTime, *response, query, false)
+		frames, err := buildDataFrames(contextWithFeaturesEnabled(features.FlagCloudWatchNewLabelParsing), startTime, endTime, *response, query)
 		require.NoError(t, err)
 
-		assert.Equal(t, "lb3 Expanded", frames[0].Name)
-		assert.Equal(t, "lb4 Expanded", frames[1].Name)
+		assert.Equal(t, "some label lb3", frames[0].Name)
+		assert.Equal(t, "some label lb3", frames[0].Fields[1].Labels["Series"])
+		assert.Equal(t, "balancer 1", frames[0].Fields[1].Labels["LoadBalancer"])
+		assert.Equal(t, "inst1", frames[0].Fields[1].Labels["InstanceType"])
+		assert.Equal(t, "tg", frames[0].Fields[1].Labels["TargetGroup"])
+
+		assert.Equal(t, "some label lb4", frames[1].Name)
+		assert.Equal(t, "some label lb4", frames[1].Fields[1].Labels["Series"])
+		assert.Equal(t, "balancer 2", frames[1].Fields[1].Labels["LoadBalancer"])
+		assert.Equal(t, "inst2", frames[1].Fields[1].Labels["InstanceType"])
+		assert.Equal(t, "tg", frames[1].Fields[1].Labels["TargetGroup"])
 	})
 
-	t.Run("Expand dimension value when no values are returned and a multi-valued template variable is used", func(t *testing.T) {
+	t.Run("when no values are returned and a multi-valued template variable is used", func(t *testing.T) {
 		timestamp := time.Unix(0, 0)
+		// When there are no results, CloudWatch sets the label values to --
 		response := &models.QueryRowResponse{
 			Metrics: []*cloudwatch.MetricDataResult{
 				{
 					Id:    aws.String("lb3"),
-					Label: aws.String("lb3"),
+					Label: aws.String("some label|&|--"),
 					Timestamps: []*time.Time{
 						aws.Time(timestamp),
 						aws.Time(timestamp.Add(time.Minute)),
@@ -351,25 +314,27 @@ func TestCloudWatchResponseParser(t *testing.T) {
 			},
 			Statistic:        "Average",
 			Period:           60,
-			Alias:            "{{LoadBalancer}} Expanded",
 			MetricQueryType:  models.MetricQueryTypeSearch,
 			MetricEditorMode: models.MetricEditorModeBuilder,
 		}
-		frames, err := buildDataFrames(startTime, endTime, *response, query, false)
+		frames, err := buildDataFrames(contextWithFeaturesEnabled(features.FlagCloudWatchNewLabelParsing), startTime, endTime, *response, query)
 		require.NoError(t, err)
 
 		assert.Len(t, frames, 2)
-		assert.Equal(t, "lb1 Expanded", frames[0].Name)
-		assert.Equal(t, "lb2 Expanded", frames[1].Name)
+		assert.Equal(t, "some label", frames[0].Name)
+		assert.Equal(t, "lb1", frames[0].Fields[1].Labels["LoadBalancer"])
+		assert.Equal(t, "some label", frames[1].Name)
+		assert.Equal(t, "lb2", frames[1].Fields[1].Labels["LoadBalancer"])
 	})
 
-	t.Run("Expand dimension value when no values are returned and a multi-valued template variable and two single-valued dimensions are used", func(t *testing.T) {
+	t.Run("when no values are returned and a multi-valued template variable and two single-valued dimensions are used", func(t *testing.T) {
 		timestamp := time.Unix(0, 0)
+		// When there are no results, CloudWatch sets the label values to --
 		response := &models.QueryRowResponse{
 			Metrics: []*cloudwatch.MetricDataResult{
 				{
 					Id:    aws.String("lb3"),
-					Label: aws.String("lb3"),
+					Label: aws.String("some label|&|--"),
 					Timestamps: []*time.Time{
 						aws.Time(timestamp),
 						aws.Time(timestamp.Add(time.Minute)),
@@ -393,25 +358,31 @@ func TestCloudWatchResponseParser(t *testing.T) {
 			},
 			Statistic:        "Average",
 			Period:           60,
-			Alias:            "{{LoadBalancer}} Expanded {{InstanceType}} - {{Resource}}",
 			MetricQueryType:  models.MetricQueryTypeSearch,
 			MetricEditorMode: models.MetricEditorModeBuilder,
 		}
-		frames, err := buildDataFrames(startTime, endTime, *response, query, false)
+		frames, err := buildDataFrames(contextWithFeaturesEnabled(features.FlagCloudWatchNewLabelParsing), startTime, endTime, *response, query)
 		require.NoError(t, err)
 
 		assert.Len(t, frames, 2)
-		assert.Equal(t, "lb1 Expanded micro - res", frames[0].Name)
-		assert.Equal(t, "lb2 Expanded micro - res", frames[1].Name)
+		assert.Equal(t, "some label", frames[0].Name)
+		assert.Equal(t, "lb1", frames[0].Fields[1].Labels["LoadBalancer"])
+		assert.Equal(t, "micro", frames[0].Fields[1].Labels["InstanceType"])
+		assert.Equal(t, "res", frames[0].Fields[1].Labels["Resource"])
+
+		assert.Equal(t, "some label", frames[1].Name)
+		assert.Equal(t, "lb2", frames[1].Fields[1].Labels["LoadBalancer"])
+		assert.Equal(t, "micro", frames[1].Fields[1].Labels["InstanceType"])
+		assert.Equal(t, "res", frames[1].Fields[1].Labels["Resource"])
 	})
 
-	t.Run("Should only expand certain fields when using SQL queries", func(t *testing.T) {
+	t.Run("when not using multi-value dimension filters on a `MetricSearch` query", func(t *testing.T) {
 		timestamp := time.Unix(0, 0)
 		response := &models.QueryRowResponse{
 			Metrics: []*cloudwatch.MetricDataResult{
 				{
 					Id:    aws.String("lb3"),
-					Label: aws.String("lb3"),
+					Label: aws.String("some label"),
 					Timestamps: []*time.Time{
 						aws.Time(timestamp),
 					},
@@ -433,20 +404,250 @@ func TestCloudWatchResponseParser(t *testing.T) {
 			},
 			Statistic:        "Average",
 			Period:           60,
-			Alias:            "{{LoadBalancer}} {{InstanceType}} {{metric}} {{namespace}} {{stat}} {{region}} {{period}}",
-			MetricQueryType:  models.MetricQueryTypeQuery,
-			MetricEditorMode: models.MetricEditorModeRaw,
+			MetricQueryType:  models.MetricQueryTypeSearch,
+			MetricEditorMode: models.MetricEditorModeBuilder,
 		}
-		frames, err := buildDataFrames(startTime, endTime, *response, query, false)
+		frames, err := buildDataFrames(contextWithFeaturesEnabled(features.FlagCloudWatchNewLabelParsing), startTime, endTime, *response, query)
 		require.NoError(t, err)
 
-		assert.False(t, strings.Contains(frames[0].Name, "AWS/ApplicationELB"))
-		assert.False(t, strings.Contains(frames[0].Name, "lb1"))
-		assert.False(t, strings.Contains(frames[0].Name, "micro"))
-		assert.False(t, strings.Contains(frames[0].Name, "AWS/ApplicationELB"))
+		assert.Equal(t, "some label", frames[0].Name)
+		assert.Equal(t, "lb1", frames[0].Fields[1].Labels["LoadBalancer"])
+		assert.Equal(t, "micro", frames[0].Fields[1].Labels["InstanceType"])
+		assert.Equal(t, "res", frames[0].Fields[1].Labels["Resource"])
+	})
 
-		assert.True(t, strings.Contains(frames[0].Name, "us-east-1"))
-		assert.True(t, strings.Contains(frames[0].Name, "60"))
+	t.Run("when non-static label set on a `MetricSearch` query", func(t *testing.T) {
+		timestamp := time.Unix(0, 0)
+		response := &models.QueryRowResponse{
+			Metrics: []*cloudwatch.MetricDataResult{
+				{
+					Id:    aws.String("lb3"),
+					Label: aws.String("some label|&|res"),
+					Timestamps: []*time.Time{
+						aws.Time(timestamp),
+					},
+					Values:     []*float64{aws.Float64(23)},
+					StatusCode: aws.String("Complete"),
+				},
+			},
+		}
+
+		query := &models.CloudWatchQuery{
+			RefId:      "refId1",
+			Region:     "us-east-1",
+			Namespace:  "AWS/ApplicationELB",
+			MetricName: "TargetResponseTime",
+			Dimensions: map[string][]string{
+				"LoadBalancer": {"lb1"},
+				"InstanceType": {"micro"},
+				"Resource":     {"*"},
+			},
+			Statistic:        "Average",
+			Period:           60,
+			MetricQueryType:  models.MetricQueryTypeSearch,
+			MetricEditorMode: models.MetricEditorModeBuilder,
+			Label:            "set ${AVG} label",
+		}
+		frames, err := buildDataFrames(contextWithFeaturesEnabled(features.FlagCloudWatchNewLabelParsing), startTime, endTime, *response, query)
+		require.NoError(t, err)
+
+		assert.Equal(t, "some label", frames[0].Name)
+		assert.Equal(t, "lb1", frames[0].Fields[1].Labels["LoadBalancer"])
+		assert.Equal(t, "micro", frames[0].Fields[1].Labels["InstanceType"])
+		assert.Equal(t, "res", frames[0].Fields[1].Labels["Resource"])
+	})
+
+	t.Run("when static label set on a `MetricSearch` query", func(t *testing.T) {
+		timestamp := time.Unix(0, 0)
+		response := &models.QueryRowResponse{
+			Metrics: []*cloudwatch.MetricDataResult{
+				{
+					Id:    aws.String("lb3"),
+					Label: aws.String("some label|&|res"),
+					Timestamps: []*time.Time{
+						aws.Time(timestamp),
+					},
+					Values:     []*float64{aws.Float64(23)},
+					StatusCode: aws.String("Complete"),
+				},
+			},
+		}
+
+		query := &models.CloudWatchQuery{
+			RefId:      "refId1",
+			Region:     "us-east-1",
+			Namespace:  "AWS/ApplicationELB",
+			MetricName: "TargetResponseTime",
+			Dimensions: map[string][]string{
+				"LoadBalancer": {"lb1"},
+				"InstanceType": {"micro"},
+				"Resource":     {"*"},
+			},
+			Statistic:        "Average",
+			Period:           60,
+			MetricQueryType:  models.MetricQueryTypeSearch,
+			MetricEditorMode: models.MetricEditorModeBuilder,
+			Label:            "actual",
+		}
+		frames, err := buildDataFrames(contextWithFeaturesEnabled(features.FlagCloudWatchNewLabelParsing), startTime, endTime, *response, query)
+		require.NoError(t, err)
+
+		assert.Equal(t, "actual", frames[0].Name)
+		assert.Equal(t, "lb1", frames[0].Fields[1].Labels["LoadBalancer"])
+		assert.Equal(t, "micro", frames[0].Fields[1].Labels["InstanceType"])
+		assert.Equal(t, "res", frames[0].Fields[1].Labels["Resource"])
+	})
+
+	t.Run("when code editor used for `MetricSearch` query add fallback label", func(t *testing.T) {
+		timestamp := time.Unix(0, 0)
+		response := &models.QueryRowResponse{
+			Metrics: []*cloudwatch.MetricDataResult{
+				{
+					Id:    aws.String("lb3"),
+					Label: aws.String("some label"),
+					Timestamps: []*time.Time{
+						aws.Time(timestamp),
+					},
+					Values:     []*float64{aws.Float64(23)},
+					StatusCode: aws.String("Complete"),
+				},
+			},
+		}
+
+		query := &models.CloudWatchQuery{
+			RefId:            "refId1",
+			Region:           "us-east-1",
+			Namespace:        "",
+			MetricName:       "",
+			Expression:       "SEARCH('MetricName=\"ResourceCount\" AND (\"AWS/Usage\") AND Resource=TargetsPer NOT QueueName=TargetsPerNetworkLoadBalancer', 'Average')",
+			Dimensions:       map[string][]string{},
+			Statistic:        "Average",
+			Period:           60,
+			MetricQueryType:  models.MetricQueryTypeSearch,
+			MetricEditorMode: models.MetricEditorModeRaw,
+			Label:            "actual",
+		}
+		frames, err := buildDataFrames(contextWithFeaturesEnabled(features.FlagCloudWatchNewLabelParsing), startTime, endTime, *response, query)
+		require.NoError(t, err)
+
+		assert.Equal(t, "actual", frames[0].Name)
+		assert.Equal(t, "some label", frames[0].Fields[1].Labels["Series"])
+	})
+
+	t.Run("when `MetricQuery` query has no label set and `GROUP BY` clause has multiple fields", func(t *testing.T) {
+		timestamp := time.Unix(0, 0)
+		response := &models.QueryRowResponse{
+			Metrics: []*cloudwatch.MetricDataResult{
+				{
+					Id:    aws.String("query1"),
+					Label: aws.String("EC2 vCPU"),
+					Timestamps: []*time.Time{
+						aws.Time(timestamp),
+					},
+					Values:     []*float64{aws.Float64(23)},
+					StatusCode: aws.String("Complete"),
+				},
+				{
+					Id:    aws.String("query2"),
+					Label: aws.String("Elastic Loading Balancing ApplicationLoadBalancersPerRegion"),
+					Timestamps: []*time.Time{
+						aws.Time(timestamp),
+					},
+					Values:     []*float64{aws.Float64(23)},
+					StatusCode: aws.String("Complete"),
+				},
+			},
+		}
+
+		query := &models.CloudWatchQuery{
+			RefId:            "refId1",
+			Region:           "us-east-1",
+			Statistic:        "Average",
+			Period:           60,
+			MetricQueryType:  models.MetricQueryTypeQuery,
+			MetricEditorMode: models.MetricEditorModeBuilder,
+			Dimensions:       map[string][]string{"Service": {"EC2", "Elastic Loading Balancing"}, "Resource": {"vCPU", "ApplicationLoadBalancersPerRegion"}},
+			SqlExpression:    "SELECT AVG(ResourceCount) FROM SCHEMA(\"AWS/Usage\", Class, Resource, Service, Type) GROUP BY Service, Resource",
+		}
+		frames, err := buildDataFrames(contextWithFeaturesEnabled(features.FlagCloudWatchNewLabelParsing), startTime, endTime, *response, query)
+		require.NoError(t, err)
+
+		assert.Equal(t, "EC2 vCPU", frames[0].Name)
+		assert.Equal(t, "EC2", frames[0].Fields[1].Labels["Service"])
+		assert.Equal(t, "vCPU", frames[0].Fields[1].Labels["Resource"])
+		assert.Equal(t, "Elastic Loading Balancing ApplicationLoadBalancersPerRegion", frames[1].Name)
+		assert.Equal(t, "Elastic Loading Balancing", frames[1].Fields[1].Labels["Service"])
+		assert.Equal(t, "ApplicationLoadBalancersPerRegion", frames[1].Fields[1].Labels["Resource"])
+	})
+
+	t.Run("when `MetricQuery` query has no `GROUP BY` clause", func(t *testing.T) {
+		timestamp := time.Unix(0, 0)
+		response := &models.QueryRowResponse{
+			Metrics: []*cloudwatch.MetricDataResult{
+				{
+					Id:    aws.String("query1"),
+					Label: aws.String("cloudwatch-default-label"),
+					Timestamps: []*time.Time{
+						aws.Time(timestamp),
+					},
+					Values:     []*float64{aws.Float64(23)},
+					StatusCode: aws.String("Complete"),
+				},
+			},
+		}
+
+		query := &models.CloudWatchQuery{
+			RefId:            "refId1",
+			Region:           "us-east-1",
+			Statistic:        "Average",
+			Period:           60,
+			MetricQueryType:  models.MetricQueryTypeQuery,
+			MetricEditorMode: models.MetricEditorModeBuilder,
+			SqlExpression:    "SELECT AVG(ResourceCount) FROM SCHEMA(\"AWS/Usage\", Class, Resource, Service, Type)",
+		}
+		frames, err := buildDataFrames(contextWithFeaturesEnabled(features.FlagCloudWatchNewLabelParsing), startTime, endTime, *response, query)
+		require.NoError(t, err)
+
+		assert.Equal(t, "cloudwatch-default-label", frames[0].Name)
+		assert.Equal(t, "cloudwatch-default-label", frames[0].Fields[1].Labels["Series"])
+	})
+
+	t.Run("ignore dimensions for raw mode query", func(t *testing.T) {
+		timestamp := time.Unix(0, 0)
+		response := &models.QueryRowResponse{
+			Metrics: []*cloudwatch.MetricDataResult{
+				{
+					Id:    aws.String("lb3"),
+					Label: aws.String("some label"),
+					Timestamps: []*time.Time{
+						aws.Time(timestamp),
+					},
+					Values:     []*float64{aws.Float64(23)},
+					StatusCode: aws.String("Complete"),
+				},
+			},
+		}
+
+		query := &models.CloudWatchQuery{
+			RefId:      "refId1",
+			Region:     "us-east-1",
+			Namespace:  "AWS/ApplicationELB",
+			MetricName: "TargetResponseTime",
+			Dimensions: map[string][]string{
+				"LoadBalancer": {"*"},
+			},
+			Expression:       "SEARCH('MetricName=\"ResourceCount\" AND (\"AWS/Usage\") AND Resource=TargetsPer NOT QueueName=TargetsPerNetworkLoadBalancer', 'Average')",
+			Statistic:        "Average",
+			Period:           60,
+			MetricQueryType:  models.MetricQueryTypeSearch,
+			MetricEditorMode: models.MetricEditorModeRaw,
+		}
+		frames, err := buildDataFrames(contextWithFeaturesEnabled(features.FlagCloudWatchNewLabelParsing), startTime, endTime, *response, query)
+		require.NoError(t, err)
+
+		assert.Equal(t, "some label", frames[0].Name)
+		assert.Len(t, frames[0].Fields[1].Labels, 1)
+		assert.Equal(t, "some label", frames[0].Fields[1].Labels["Series"])
 	})
 
 	t.Run("Parse cloudwatch response", func(t *testing.T) {
@@ -455,7 +656,7 @@ func TestCloudWatchResponseParser(t *testing.T) {
 			Metrics: []*cloudwatch.MetricDataResult{
 				{
 					Id:    aws.String("id1"),
-					Label: aws.String("lb"),
+					Label: aws.String("some label"),
 					Timestamps: []*time.Time{
 						aws.Time(timestamp),
 						aws.Time(timestamp.Add(time.Minute)),
@@ -482,15 +683,14 @@ func TestCloudWatchResponseParser(t *testing.T) {
 			},
 			Statistic:        "Average",
 			Period:           60,
-			Alias:            "{{namespace}}_{{metric}}_{{stat}}",
 			MetricQueryType:  models.MetricQueryTypeSearch,
 			MetricEditorMode: models.MetricEditorModeBuilder,
 		}
-		frames, err := buildDataFrames(startTime, endTime, *response, query, false)
+		frames, err := buildDataFrames(contextWithFeaturesEnabled(features.FlagCloudWatchNewLabelParsing), startTime, endTime, *response, query)
 		require.NoError(t, err)
 
 		frame := frames[0]
-		assert.Equal(t, "AWS/ApplicationELB_TargetResponseTime_Average", frame.Name)
+		assert.Equal(t, "some label", frame.Name)
 		assert.Equal(t, "Time", frame.Fields[0].Name)
 		assert.Equal(t, "lb", frame.Fields[1].Labels["LoadBalancer"])
 		assert.Equal(t, 10.0, *frame.Fields[1].At(0).(*float64))
@@ -499,23 +699,4 @@ func TestCloudWatchResponseParser(t *testing.T) {
 		assert.Equal(t, "Value", frame.Fields[1].Name)
 		assert.Equal(t, "", frame.Fields[1].Config.DisplayName)
 	})
-
-	t.Run("buildDataFrames should use response label as frame name when dynamic label is enabled", func(t *testing.T) {
-		response := &models.QueryRowResponse{
-			Metrics: []*cloudwatch.MetricDataResult{
-				{
-					Label:      aws.String("some response label"),
-					Timestamps: []*time.Time{},
-					Values:     []*float64{aws.Float64(10)},
-					StatusCode: aws.String("Complete"),
-				},
-			},
-		}
-
-		frames, err := buildDataFrames(startTime, endTime, *response, &models.CloudWatchQuery{}, true)
-
-		assert.NoError(t, err)
-		require.Len(t, frames, 1)
-		assert.Equal(t, "some response label", frames[0].Name)
-	})
 }
