@@ -14,9 +14,11 @@ import (
 	"github.com/grafana/grafana-plugin-sdk-go/backend"
 	"github.com/grafana/grafana-plugin-sdk-go/backend/datasource"
 	"github.com/grafana/grafana-plugin-sdk-go/backend/instancemgmt"
+	"github.com/grafana/grafana-plugin-sdk-go/backend/log"
 	"github.com/stretchr/testify/mock"
 
-	"github.com/grafana/grafana/pkg/services/featuremgmt"
+	"github.com/grafana/grafana/pkg/tsdb/cloudwatch/features"
+	"github.com/grafana/grafana/pkg/tsdb/cloudwatch/kinds/dataquery"
 	"github.com/grafana/grafana/pkg/tsdb/cloudwatch/mocks"
 	"github.com/grafana/grafana/pkg/tsdb/cloudwatch/models"
 
@@ -25,7 +27,7 @@ import (
 )
 
 func TestTimeSeriesQuery(t *testing.T) {
-	executor := newExecutor(nil, newTestConfig(), &fakeSessionCache{}, featuremgmt.WithFeatures())
+	executor := newExecutor(defaultTestInstanceManager(), log.NewNullLogger())
 	now := time.Now()
 
 	origNewCWClient := NewCWClient
@@ -49,11 +51,9 @@ func TestTimeSeriesQuery(t *testing.T) {
 					StatusCode: aws.String("Complete"), Id: aws.String("b"), Label: aws.String("NetworkIn"), Values: []*float64{aws.Float64(1.0)}, Timestamps: []*time.Time{&now},
 				}}}, nil)
 
-		im := datasource.NewInstanceManager(func(s backend.DataSourceInstanceSettings) (instancemgmt.Instance, error) {
-			return DataSource{Settings: models.CloudWatchSettings{}}, nil
-		})
+		im := defaultTestInstanceManager()
 
-		executor := newExecutor(im, newTestConfig(), &fakeSessionCache{}, featuremgmt.WithFeatures())
+		executor := newExecutor(im, log.NewNullLogger())
 		resp, err := executor.QueryData(context.Background(), &backend.QueryDataRequest{
 			PluginContext: backend.PluginContext{
 				DataSourceInstanceSettings: &backend.DataSourceInstanceSettings{},
@@ -76,7 +76,6 @@ func TestTimeSeriesQuery(t *testing.T) {
 						},
 						"region": "us-east-2",
 						"id": "a",
-						"alias": "NetworkOut",
 						"statistics": [
 						  "Maximum"
 						],
@@ -103,7 +102,6 @@ func TestTimeSeriesQuery(t *testing.T) {
 						},
 						"region": "us-east-2",
 						"id": "b",
-						"alias": "NetworkIn",
 						"statistics": [
 						"Maximum"
 						],
@@ -120,7 +118,7 @@ func TestTimeSeriesQuery(t *testing.T) {
 	})
 
 	t.Run("End time before start time should result in error", func(t *testing.T) {
-		_, err := executor.executeTimeSeriesQuery(context.Background(), logger, &backend.QueryDataRequest{Queries: []backend.DataQuery{{TimeRange: backend.TimeRange{
+		_, err := executor.executeTimeSeriesQuery(context.Background(), &backend.QueryDataRequest{Queries: []backend.DataQuery{{TimeRange: backend.TimeRange{
 			From: now.Add(time.Hour * -1),
 			To:   now.Add(time.Hour * -2),
 		}}}})
@@ -128,7 +126,7 @@ func TestTimeSeriesQuery(t *testing.T) {
 	})
 
 	t.Run("End time equals start time should result in error", func(t *testing.T) {
-		_, err := executor.executeTimeSeriesQuery(context.Background(), logger, &backend.QueryDataRequest{Queries: []backend.DataQuery{{TimeRange: backend.TimeRange{
+		_, err := executor.executeTimeSeriesQuery(context.Background(), &backend.QueryDataRequest{Queries: []backend.DataQuery{{TimeRange: backend.TimeRange{
 			From: now.Add(time.Hour * -1),
 			To:   now.Add(time.Hour * -1),
 		}}}})
@@ -149,19 +147,20 @@ func Test_executeTimeSeriesQuery_getCWClient_is_called_once_per_region_and_GetMe
 		return &mockMetricClient
 	}
 
-	im := datasource.NewInstanceManager(func(s backend.DataSourceInstanceSettings) (instancemgmt.Instance, error) {
-		return DataSource{Settings: models.CloudWatchSettings{}}, nil
-	})
-
-	t.Run("Queries with the same region should call GetSession with that region 1 time and call GetMetricDataWithContext 1 time", func(t *testing.T) {
+	t.Run("Queries with the same region should call GetSessionWithAuthSettings with that region 1 time and call GetMetricDataWithContext 1 time", func(t *testing.T) {
 		mockSessionCache := &mockSessionCache{}
-		mockSessionCache.On("GetSession", mock.MatchedBy(
-			func(config awsds.SessionConfig) bool { return config.Settings.Region == "us-east-1" })). // region from queries is asserted here
+		mockSessionCache.On("GetSessionWithAuthSettings", mock.MatchedBy(
+			func(config awsds.GetSessionConfig) bool {
+				return config.Settings.Region == "us-east-1"
+			})). // region from queries is asserted here
 			Return(&session.Session{Config: &aws.Config{}}, nil).Once()
+		im := datasource.NewInstanceManager(func(ctx context.Context, s backend.DataSourceInstanceSettings) (instancemgmt.Instance, error) {
+			return DataSource{Settings: models.CloudWatchSettings{}, sessions: mockSessionCache}, nil
+		})
 		mockMetricClient = mocks.MetricsAPI{}
 		mockMetricClient.On("GetMetricDataWithContext", mock.Anything, mock.Anything, mock.Anything).Return(nil, nil)
 
-		executor := newExecutor(im, newTestConfig(), mockSessionCache, featuremgmt.WithFeatures())
+		executor := newExecutor(im, log.NewNullLogger())
 		_, err := executor.QueryData(context.Background(), &backend.QueryDataRequest{
 			PluginContext: backend.PluginContext{
 				DataSourceInstanceSettings: &backend.DataSourceInstanceSettings{},
@@ -201,18 +200,27 @@ func Test_executeTimeSeriesQuery_getCWClient_is_called_once_per_region_and_GetMe
 		// GetMetricData is asserted to have been called 1 time for the 1 region present in the queries
 	})
 
-	t.Run("3 queries with 2 regions calls GetSession 2 times and calls GetMetricDataWithContext 2 times", func(t *testing.T) {
+	t.Run("3 queries with 2 regions calls GetSessionWithAuthSettings 2 times and calls GetMetricDataWithContext 2 times", func(t *testing.T) {
 		sessionCache := &mockSessionCache{}
-		sessionCache.On("GetSession", mock.MatchedBy(
-			func(config awsds.SessionConfig) bool { return config.Settings.Region == "us-east-1" })).
-			Return(&session.Session{Config: &aws.Config{}}, nil, nil).Once()
-		sessionCache.On("GetSession", mock.MatchedBy(
-			func(config awsds.SessionConfig) bool { return config.Settings.Region == "us-east-2" })).
-			Return(&session.Session{Config: &aws.Config{}}, nil, nil).Once()
+		sessionCache.On("GetSessionWithAuthSettings", mock.MatchedBy(
+			func(config awsds.GetSessionConfig) bool {
+				return config.Settings.Region == "us-east-1"
+			})).
+			Return(&session.Session{Config: &aws.Config{}}, nil).Once()
+		sessionCache.On("GetSessionWithAuthSettings", mock.MatchedBy(
+			func(config awsds.GetSessionConfig) bool {
+				return config.Settings.Region == "us-east-2"
+			})).
+			Return(&session.Session{Config: &aws.Config{}}, nil).Once()
+
+		im := datasource.NewInstanceManager(func(ctx context.Context, s backend.DataSourceInstanceSettings) (instancemgmt.Instance, error) {
+			return DataSource{Settings: models.CloudWatchSettings{}, sessions: sessionCache}, nil
+		})
+
 		mockMetricClient = mocks.MetricsAPI{}
 		mockMetricClient.On("GetMetricDataWithContext", mock.Anything, mock.Anything, mock.Anything).Return(nil, nil)
 
-		executor := newExecutor(im, newTestConfig(), sessionCache, featuremgmt.WithFeatures())
+		executor := newExecutor(im, log.NewNullLogger())
 		_, err := executor.QueryData(context.Background(), &backend.QueryDataRequest{
 			PluginContext: backend.PluginContext{
 				DataSourceInstanceSettings: &backend.DataSourceInstanceSettings{},
@@ -270,16 +278,15 @@ type queryDimensions struct {
 }
 
 type queryParameters struct {
-	MetricQueryType  models.MetricQueryType  `json:"metricQueryType"`
-	MetricEditorMode models.MetricEditorMode `json:"metricEditorMode"`
-	Dimensions       queryDimensions         `json:"dimensions"`
-	Expression       string                  `json:"expression"`
-	Alias            string                  `json:"alias"`
-	Label            *string                 `json:"label"`
-	Statistic        string                  `json:"statistic"`
-	Period           string                  `json:"period"`
-	MatchExact       bool                    `json:"matchExact"`
-	MetricName       string                  `json:"metricName"`
+	MetricQueryType  dataquery.MetricQueryType  `json:"metricQueryType"`
+	MetricEditorMode dataquery.MetricEditorMode `json:"metricEditorMode"`
+	Dimensions       queryDimensions            `json:"dimensions"`
+	Expression       string                     `json:"expression"`
+	Label            *string                    `json:"label"`
+	Statistic        string                     `json:"statistic"`
+	Period           string                     `json:"period"`
+	MatchExact       bool                       `json:"matchExact"`
+	MetricName       string                     `json:"metricName"`
 }
 
 var queryId = "query id"
@@ -288,18 +295,17 @@ func newTestQuery(t testing.TB, p queryParameters) json.RawMessage {
 	t.Helper()
 
 	tsq := struct {
-		Type             string                  `json:"type"`
-		MetricQueryType  models.MetricQueryType  `json:"metricQueryType"`
-		MetricEditorMode models.MetricEditorMode `json:"metricEditorMode"`
-		Namespace        string                  `json:"namespace"`
-		MetricName       string                  `json:"metricName"`
+		Type             string                     `json:"type"`
+		MetricQueryType  dataquery.MetricQueryType  `json:"metricQueryType"`
+		MetricEditorMode dataquery.MetricEditorMode `json:"metricEditorMode"`
+		Namespace        string                     `json:"namespace"`
+		MetricName       string                     `json:"metricName"`
 		Dimensions       struct {
 			InstanceID []string `json:"InstanceId,omitempty"`
 		} `json:"dimensions"`
 		Expression string  `json:"expression"`
 		Region     string  `json:"region"`
 		ID         string  `json:"id"`
-		Alias      string  `json:"alias"`
 		Label      *string `json:"label"`
 		Statistic  string  `json:"statistic"`
 		Period     string  `json:"period"`
@@ -316,7 +322,6 @@ func newTestQuery(t testing.TB, p queryParameters) json.RawMessage {
 		MetricEditorMode: p.MetricEditorMode,
 		Dimensions:       p.Dimensions,
 		Expression:       p.Expression,
-		Alias:            p.Alias,
 		Label:            p.Label,
 		Statistic:        p.Statistic,
 		Period:           p.Period,
@@ -341,14 +346,14 @@ func Test_QueryData_timeSeriesQuery_GetMetricDataWithContext(t *testing.T) {
 		return &api
 	}
 
-	im := datasource.NewInstanceManager(func(s backend.DataSourceInstanceSettings) (instancemgmt.Instance, error) {
-		return DataSource{Settings: models.CloudWatchSettings{}}, nil
+	im := datasource.NewInstanceManager(func(ctx context.Context, s backend.DataSourceInstanceSettings) (instancemgmt.Instance, error) {
+		return DataSource{Settings: models.CloudWatchSettings{}, sessions: &fakeSessionCache{}}, nil
 	})
 
-	t.Run("passes query label as GetMetricData label when dynamic labels feature toggle is enabled", func(t *testing.T) {
+	t.Run("passes query label as GetMetricData label", func(t *testing.T) {
 		api = mocks.MetricsAPI{}
 		api.On("GetMetricDataWithContext", mock.Anything, mock.Anything, mock.Anything).Return(&cloudwatch.GetMetricDataOutput{}, nil)
-		executor := newExecutor(im, newTestConfig(), &fakeSessionCache{}, featuremgmt.WithFeatures(featuremgmt.FlagCloudWatchDynamicLabels))
+		executor := newExecutor(im, log.NewNullLogger())
 		query := newTestQuery(t, queryParameters{
 			Label: aws.String("${PROP('Period')} some words ${PROP('Dim.InstanceId')}"),
 		})
@@ -377,27 +382,17 @@ func Test_QueryData_timeSeriesQuery_GetMetricDataWithContext(t *testing.T) {
 	})
 
 	testCases := map[string]struct {
-		feature    *featuremgmt.FeatureManager
 		parameters queryParameters
 	}{
-		"should not pass GetMetricData label when query label is empty, dynamic labels is enabled": {
-			feature: featuremgmt.WithFeatures(featuremgmt.FlagCloudWatchDynamicLabels),
-		},
-		"should not pass GetMetricData label when query label is empty string, dynamic labels is enabled": {
-			feature:    featuremgmt.WithFeatures(featuremgmt.FlagCloudWatchDynamicLabels),
-			parameters: queryParameters{Label: aws.String("")},
-		},
-		"should not pass GetMetricData label when dynamic labels is disabled": {
-			feature:    featuremgmt.WithFeatures(),
-			parameters: queryParameters{Label: aws.String("${PROP('Period')} some words ${PROP('Dim.InstanceId')}")},
-		},
+		"should not pass GetMetricData label when query label is empty":        {},
+		"should not pass GetMetricData label when query label is empty string": {parameters: queryParameters{Label: aws.String("")}},
 	}
 
 	for name, tc := range testCases {
 		t.Run(name, func(t *testing.T) {
 			api = mocks.MetricsAPI{}
 			api.On("GetMetricDataWithContext", mock.Anything, mock.Anything, mock.Anything).Return(&cloudwatch.GetMetricDataOutput{}, nil)
-			executor := newExecutor(im, newTestConfig(), &fakeSessionCache{}, tc.feature)
+			executor := newExecutor(im, log.NewNullLogger())
 
 			_, err := executor.QueryData(context.Background(), &backend.QueryDataRequest{
 				PluginContext: backend.PluginContext{DataSourceInstanceSettings: &backend.DataSourceInstanceSettings{}},
@@ -424,12 +419,16 @@ func Test_QueryData_timeSeriesQuery_GetMetricDataWithContext(t *testing.T) {
 	}
 }
 
-func Test_QueryData_response_data_frame_names(t *testing.T) {
+func Test_QueryData_response_data_frame_name_is_always_response_label(t *testing.T) {
 	origNewCWClient := NewCWClient
 	t.Cleanup(func() {
 		NewCWClient = origNewCWClient
 	})
-	var api mocks.MetricsAPI
+
+	api := mocks.MetricsAPI{Metrics: []*cloudwatch.Metric{
+		{MetricName: aws.String(""), Dimensions: []*cloudwatch.Dimension{{Name: aws.String("InstanceId"), Value: aws.String("i-00645d91ed77d87ac")}}},
+	}}
+	api.On("ListMetricsPagesWithContext").Return(nil)
 
 	NewCWClient = func(sess *session.Session) cloudwatchiface.CloudWatchAPI {
 		return &api
@@ -443,16 +442,13 @@ func Test_QueryData_response_data_frame_names(t *testing.T) {
 					Values: []*float64{aws.Float64(1.0)}, Timestamps: []*time.Time{{}}},
 			}}, nil)
 
-	im := datasource.NewInstanceManager(func(s backend.DataSourceInstanceSettings) (instancemgmt.Instance, error) {
-		return DataSource{Settings: models.CloudWatchSettings{}}, nil
-	})
-	executor := newExecutor(im, newTestConfig(), &fakeSessionCache{}, featuremgmt.WithFeatures())
+	im := defaultTestInstanceManager()
+	executor := newExecutor(im, log.NewNullLogger())
 
-	t.Run("where user defines search expression and alias is defined, then frame name prioritizes period and stat from expression over input", func(t *testing.T) {
+	t.Run("where user defines search expression", func(t *testing.T) {
 		query := newTestQuery(t, queryParameters{
-			MetricQueryType:  models.MetricQueryTypeSearch, // contributes to isUserDefinedSearchExpression = true
-			MetricEditorMode: models.MetricEditorModeRaw,   // contributes to isUserDefinedSearchExpression = true
-			Alias:            "{{period}} {{stat}}",
+			MetricQueryType:  models.MetricQueryTypeSearch,                                                 // contributes to isUserDefinedSearchExpression = true
+			MetricEditorMode: models.MetricEditorModeRaw,                                                   // contributes to isUserDefinedSearchExpression = true
 			Expression:       `SEARCH('{AWS/EC2,InstanceId} MetricName="CPUUtilization"', 'Average', 300)`, // period 300 and stat 'Average' parsed from this expression
 			Statistic:        "Maximum",                                                                    // stat parsed from expression takes precedence over 'Maximum'
 			Period:           "1200",                                                                       // period parsed from expression takes precedence over 1200
@@ -470,10 +466,10 @@ func Test_QueryData_response_data_frame_names(t *testing.T) {
 		})
 
 		assert.NoError(t, err)
-		assert.Equal(t, "300 Average", resp.Responses["A"].Frames[0].Name)
+		assert.Equal(t, labelFromGetMetricData, resp.Responses["A"].Frames[0].Name)
 	})
 
-	t.Run("where no alias is provided and query is math expression, then frame name is queryId", func(t *testing.T) {
+	t.Run("where query is math expression", func(t *testing.T) {
 		query := newTestQuery(t, queryParameters{
 			MetricQueryType:  models.MetricQueryTypeSearch,
 			MetricEditorMode: models.MetricEditorModeRaw,
@@ -491,10 +487,10 @@ func Test_QueryData_response_data_frame_names(t *testing.T) {
 		})
 
 		assert.NoError(t, err)
-		assert.Equal(t, queryId, resp.Responses["A"].Frames[0].Name)
+		assert.Equal(t, labelFromGetMetricData, resp.Responses["A"].Frames[0].Name)
 	})
 
-	t.Run("where no alias provided and query type is MetricQueryTypeQuery, then frame name is label", func(t *testing.T) {
+	t.Run("where query type is MetricQueryTypeQuery", func(t *testing.T) {
 		query := newTestQuery(t, queryParameters{
 			MetricQueryType: models.MetricQueryTypeQuery,
 		})
@@ -514,7 +510,7 @@ func Test_QueryData_response_data_frame_names(t *testing.T) {
 		assert.Equal(t, labelFromGetMetricData, resp.Responses["A"].Frames[0].Name)
 	})
 
-	// where query is inferred search expression and not multivalued dimension expression, then frame name is label
+	// where query is inferred search expression and not multivalued dimension expression
 	testCasesReturningLabel := map[string]queryParameters{
 		"with specific dimensions, matchExact false": {Dimensions: queryDimensions{[]string{"some-instance"}}, MatchExact: false},
 		"with wildcard dimensions, matchExact false": {Dimensions: queryDimensions{[]string{"*"}}, MatchExact: false},
@@ -541,7 +537,7 @@ func Test_QueryData_response_data_frame_names(t *testing.T) {
 		})
 	}
 
-	// complementary test cases to above return default of "metricName_stat"
+	// complementary test cases to above
 	testCasesReturningMetricStat := map[string]queryParameters{
 		"with specific dimensions, matchExact true": {
 			Dimensions: queryDimensions{[]string{"some-instance"}},
@@ -584,7 +580,7 @@ func Test_QueryData_response_data_frame_names(t *testing.T) {
 			})
 
 			assert.NoError(t, err)
-			assert.Equal(t, "CPUUtilization_Maximum", resp.Responses["A"].Frames[0].Name)
+			assert.Equal(t, labelFromGetMetricData, resp.Responses["A"].Frames[0].Name)
 		})
 	}
 }
@@ -599,16 +595,14 @@ func TestTimeSeriesQuery_CrossAccountQuerying(t *testing.T) {
 	NewCWClient = func(sess *session.Session) cloudwatchiface.CloudWatchAPI {
 		return &api
 	}
-	im := datasource.NewInstanceManager(func(s backend.DataSourceInstanceSettings) (instancemgmt.Instance, error) {
-		return DataSource{Settings: models.CloudWatchSettings{}}, nil
-	})
+	im := defaultTestInstanceManager()
 
 	t.Run("should call GetMetricDataInput with AccountId nil when no AccountId is provided", func(t *testing.T) {
 		api = mocks.MetricsAPI{}
 		api.On("GetMetricDataWithContext", mock.Anything, mock.Anything, mock.Anything).Return(&cloudwatch.GetMetricDataOutput{}, nil)
-		executor := newExecutor(im, newTestConfig(), &fakeSessionCache{}, featuremgmt.WithFeatures(featuremgmt.FlagCloudWatchCrossAccountQuerying))
+		executor := newExecutor(im, log.NewNullLogger())
 
-		_, err := executor.QueryData(context.Background(), &backend.QueryDataRequest{
+		_, err := executor.QueryData(contextWithFeaturesEnabled(features.FlagCloudWatchCrossAccountQuerying), &backend.QueryDataRequest{
 			PluginContext: backend.PluginContext{
 				DataSourceInstanceSettings: &backend.DataSourceInstanceSettings{},
 			},
@@ -626,7 +620,6 @@ func TestTimeSeriesQuery_CrossAccountQuerying(t *testing.T) {
 						},
 						"region": "us-east-2",
 						"id": "a",
-						"alias": "NetworkOut",
 						"statistic": "Maximum",
 						"period": "300",
 						"hide": false,
@@ -648,7 +641,7 @@ func TestTimeSeriesQuery_CrossAccountQuerying(t *testing.T) {
 	t.Run("should call GetMetricDataInput with AccountId nil when feature flag is false", func(t *testing.T) {
 		api = mocks.MetricsAPI{}
 		api.On("GetMetricDataWithContext", mock.Anything, mock.Anything, mock.Anything).Return(&cloudwatch.GetMetricDataOutput{}, nil)
-		executor := newExecutor(im, newTestConfig(), &fakeSessionCache{}, featuremgmt.WithFeatures())
+		executor := newExecutor(im, log.NewNullLogger())
 		_, err := executor.QueryData(context.Background(), &backend.QueryDataRequest{
 			PluginContext: backend.PluginContext{
 				DataSourceInstanceSettings: &backend.DataSourceInstanceSettings{},
@@ -667,7 +660,6 @@ func TestTimeSeriesQuery_CrossAccountQuerying(t *testing.T) {
 						},
 						"region": "us-east-2",
 						"id": "a",
-						"alias": "NetworkOut",
 						"statistic": "Maximum",
 						"period": "300",
 						"hide": false,
@@ -690,8 +682,8 @@ func TestTimeSeriesQuery_CrossAccountQuerying(t *testing.T) {
 	t.Run("should call GetMetricDataInput with AccountId in a MetricStat query", func(t *testing.T) {
 		api = mocks.MetricsAPI{}
 		api.On("GetMetricDataWithContext", mock.Anything, mock.Anything, mock.Anything).Return(&cloudwatch.GetMetricDataOutput{}, nil)
-		executor := newExecutor(im, newTestConfig(), &fakeSessionCache{}, featuremgmt.WithFeatures(featuremgmt.FlagCloudWatchCrossAccountQuerying))
-		_, err := executor.QueryData(context.Background(), &backend.QueryDataRequest{
+		executor := newExecutor(im, log.NewNullLogger())
+		_, err := executor.QueryData(contextWithFeaturesEnabled(features.FlagCloudWatchCrossAccountQuerying), &backend.QueryDataRequest{
 			PluginContext: backend.PluginContext{
 				DataSourceInstanceSettings: &backend.DataSourceInstanceSettings{},
 			},
@@ -709,7 +701,6 @@ func TestTimeSeriesQuery_CrossAccountQuerying(t *testing.T) {
 						},
 						"region": "us-east-2",
 						"id": "a",
-						"alias": "NetworkOut",
 						"statistic": "Maximum",
 						"period": "300",
 						"hide": false,
@@ -732,8 +723,8 @@ func TestTimeSeriesQuery_CrossAccountQuerying(t *testing.T) {
 	t.Run("should GetMetricDataInput with AccountId in an inferred search expression query", func(t *testing.T) {
 		api = mocks.MetricsAPI{}
 		api.On("GetMetricDataWithContext", mock.Anything, mock.Anything, mock.Anything).Return(&cloudwatch.GetMetricDataOutput{}, nil)
-		executor := newExecutor(im, newTestConfig(), &fakeSessionCache{}, featuremgmt.WithFeatures(featuremgmt.FlagCloudWatchCrossAccountQuerying))
-		_, err := executor.QueryData(context.Background(), &backend.QueryDataRequest{
+		executor := newExecutor(im, log.NewNullLogger())
+		_, err := executor.QueryData(contextWithFeaturesEnabled(features.FlagCloudWatchCrossAccountQuerying), &backend.QueryDataRequest{
 			PluginContext: backend.PluginContext{
 				DataSourceInstanceSettings: &backend.DataSourceInstanceSettings{},
 			},
@@ -751,7 +742,6 @@ func TestTimeSeriesQuery_CrossAccountQuerying(t *testing.T) {
 						},
 						"region": "us-east-2",
 						"id": "a",
-						"alias": "NetworkOut",
 						"statistic": "Maximum",
 						"period": "300",
 						"hide": false,
