@@ -1,11 +1,13 @@
 package elasticsearch
 
 import (
+	"context"
 	"encoding/json"
 	"testing"
 	"time"
 
 	"github.com/grafana/grafana-plugin-sdk-go/backend"
+	"github.com/grafana/grafana-plugin-sdk-go/backend/log"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 
@@ -328,7 +330,7 @@ func TestExecuteElasticsearchDataQuery(t *testing.T) {
 			require.Equal(t, percentilesAgg.Aggregation.Type, "percentiles")
 			metricAgg := percentilesAgg.Aggregation.Aggregation.(*es.MetricAggregation)
 			require.Equal(t, metricAgg.Field, "@load_time")
-			percents := metricAgg.Settings["percents"].([]interface{})
+			percents := metricAgg.Settings["percents"].([]any)
 			require.Len(t, percents, 4)
 			require.Equal(t, percents[0], "1")
 			require.Equal(t, percents[1], "2")
@@ -426,7 +428,7 @@ func TestExecuteElasticsearchDataQuery(t *testing.T) {
 			require.Equal(t, sr.Size, defaultSize)
 			require.Equal(t, sr.Sort["@timestamp"], map[string]string{"order": "desc", "unmapped_type": "boolean"})
 			require.Equal(t, sr.Sort["_doc"], map[string]string{"order": "desc"})
-			require.Equal(t, sr.CustomProps["script_fields"], map[string]interface{}{})
+			require.Equal(t, sr.CustomProps["script_fields"], map[string]any{})
 		})
 
 		t.Run("With raw data metric query (from frontend tests)", func(t *testing.T) {
@@ -447,7 +449,7 @@ func TestExecuteElasticsearchDataQuery(t *testing.T) {
 			require.Equal(t, sr.Size, defaultSize)
 			require.Equal(t, sr.Sort["@timestamp"], map[string]string{"order": "desc", "unmapped_type": "boolean"})
 			require.Equal(t, sr.Sort["_doc"], map[string]string{"order": "desc"})
-			require.Equal(t, sr.CustomProps["script_fields"], map[string]interface{}{})
+			require.Equal(t, sr.CustomProps["script_fields"], map[string]any{})
 		})
 
 		t.Run("With raw document metric size set", func(t *testing.T) {
@@ -612,7 +614,7 @@ func TestExecuteElasticsearchDataQuery(t *testing.T) {
 						"id": "3",
 						"type": "geohash_grid",
 						"field": "@location",
-						"settings": { "precision": 3 }
+						"settings": { "precision": "6" }
 					}
 				],
 				"metrics": [{"type": "count", "id": "1" }]
@@ -625,6 +627,56 @@ func TestExecuteElasticsearchDataQuery(t *testing.T) {
 			require.Equal(t, firstLevel.Aggregation.Type, "geohash_grid")
 			ghGridAgg := firstLevel.Aggregation.Aggregation.(*es.GeoHashGridAggregation)
 			require.Equal(t, ghGridAgg.Field, "@location")
+			require.Equal(t, ghGridAgg.Precision, 6)
+		})
+
+		t.Run("With geo hash grid agg with invalid int precision", func(t *testing.T) {
+			c := newFakeClient()
+			_, err := executeElasticsearchDataQuery(c, `{
+				"bucketAggs": [
+					{
+						"id": "3",
+						"type": "geohash_grid",
+						"field": "@location",
+						"settings": { "precision": 7 }
+					}
+				],
+				"metrics": [{"type": "count", "id": "1" }]
+			}`, from, to)
+			require.NoError(t, err)
+			sr := c.multisearchRequests[0].Requests[0]
+
+			firstLevel := sr.Aggs[0]
+			require.Equal(t, firstLevel.Key, "3")
+			require.Equal(t, firstLevel.Aggregation.Type, "geohash_grid")
+			ghGridAgg := firstLevel.Aggregation.Aggregation.(*es.GeoHashGridAggregation)
+			require.Equal(t, ghGridAgg.Field, "@location")
+			// It should default to 3
+			require.Equal(t, ghGridAgg.Precision, 3)
+		})
+
+		t.Run("With geo hash grid agg with no precision", func(t *testing.T) {
+			c := newFakeClient()
+			_, err := executeElasticsearchDataQuery(c, `{
+				"bucketAggs": [
+					{
+						"id": "3",
+						"type": "geohash_grid",
+						"field": "@location",
+						"settings": {}
+					}
+				],
+				"metrics": [{"type": "count", "id": "1" }]
+			}`, from, to)
+			require.NoError(t, err)
+			sr := c.multisearchRequests[0].Requests[0]
+
+			firstLevel := sr.Aggs[0]
+			require.Equal(t, firstLevel.Key, "3")
+			require.Equal(t, firstLevel.Aggregation.Type, "geohash_grid")
+			ghGridAgg := firstLevel.Aggregation.Aggregation.(*es.GeoHashGridAggregation)
+			require.Equal(t, ghGridAgg.Field, "@location")
+			// It should default to 3
 			require.Equal(t, ghGridAgg.Precision, 3)
 		})
 
@@ -1159,7 +1211,7 @@ func TestExecuteElasticsearchDataQuery(t *testing.T) {
 			bucketScriptAgg := firstLevel.Aggregation.Aggs[2]
 			require.Equal(t, bucketScriptAgg.Key, "4")
 			plAgg := bucketScriptAgg.Aggregation.Aggregation.(*es.PipelineAggregation)
-			require.Equal(t, plAgg.BucketPath.(map[string]interface{}), map[string]interface{}{
+			require.Equal(t, plAgg.BucketPath.(map[string]any), map[string]any{
 				"var1": "1",
 				"var2": "3",
 			})
@@ -1195,7 +1247,7 @@ func TestExecuteElasticsearchDataQuery(t *testing.T) {
 			bucketScriptAgg := firstLevel.Aggregation.Aggs[2]
 			require.Equal(t, bucketScriptAgg.Key, "2")
 			plAgg := bucketScriptAgg.Aggregation.Aggregation.(*es.PipelineAggregation)
-			require.Equal(t, plAgg.BucketPath.(map[string]interface{}), map[string]interface{}{
+			require.Equal(t, plAgg.BucketPath.(map[string]any), map[string]any{
 				"var1": "3",
 				"var2": "5",
 			})
@@ -1229,7 +1281,7 @@ func TestExecuteElasticsearchDataQuery(t *testing.T) {
 			bucketScriptAgg := firstLevel.Aggregation.Aggs[0]
 			require.Equal(t, bucketScriptAgg.Key, "2")
 			plAgg := bucketScriptAgg.Aggregation.Aggregation.(*es.PipelineAggregation)
-			require.Equal(t, plAgg.BucketPath.(map[string]interface{}), map[string]interface{}{
+			require.Equal(t, plAgg.BucketPath.(map[string]any), map[string]any{
 				"var1": "_count",
 			})
 		})
@@ -1262,7 +1314,7 @@ func TestExecuteElasticsearchDataQuery(t *testing.T) {
 			bucketScriptAgg := firstLevel.Aggregation.Aggs[0]
 			require.Equal(t, bucketScriptAgg.Key, "4")
 			plAgg := bucketScriptAgg.Aggregation.Aggregation.(*es.PipelineAggregation)
-			require.Equal(t, plAgg.BucketPath.(map[string]interface{}), map[string]interface{}{
+			require.Equal(t, plAgg.BucketPath.(map[string]any), map[string]any{
 				"var1": "_count",
 			})
 		})
@@ -1312,7 +1364,7 @@ func TestExecuteElasticsearchDataQuery(t *testing.T) {
 
 			require.Equal(t, sr.Sort["@timestamp"], map[string]string{"order": "desc", "unmapped_type": "boolean"})
 			require.Equal(t, sr.Sort["_doc"], map[string]string{"order": "desc"})
-			require.Equal(t, sr.CustomProps["script_fields"], map[string]interface{}{})
+			require.Equal(t, sr.CustomProps["script_fields"], map[string]any{})
 
 			firstLevel := sr.Aggs[0]
 			require.Equal(t, firstLevel.Key, "1")
@@ -1344,9 +1396,9 @@ func TestExecuteElasticsearchDataQuery(t *testing.T) {
 			}`, from, to)
 			require.NoError(t, err)
 			sr := c.multisearchRequests[0].Requests[0]
-			require.Equal(t, sr.CustomProps["highlight"], map[string]interface{}{
-				"fields": map[string]interface{}{
-					"*": map[string]interface{}{},
+			require.Equal(t, sr.CustomProps["highlight"], map[string]any{
+				"fields": map[string]any{
+					"*": map[string]any{},
 				},
 				"fragment_size": 2147483647,
 				"post_tags":     []string{"@/HIGHLIGHT@"},
@@ -1354,12 +1406,33 @@ func TestExecuteElasticsearchDataQuery(t *testing.T) {
 			})
 		})
 
-		t.Run("With invalid query should return error", (func(t *testing.T) {
+		t.Run("With log context query with sortDirection and searchAfter should return correct query", func(t *testing.T) {
 			c := newFakeClient()
 			_, err := executeElasticsearchDataQuery(c, `{
+			"metrics": [{ "type": "logs", "id": "1", "settings": { "limit": "1000", "sortDirection": "asc", "searchAfter": [1, "2"] }}]
+		}`, from, to)
+			require.NoError(t, err)
+			sr := c.multisearchRequests[0].Requests[0]
+			require.Equal(t, sr.Sort["@timestamp"], map[string]string{"order": "asc", "unmapped_type": "boolean"})
+			require.Equal(t, sr.Sort["_doc"], map[string]string{"order": "asc"})
+
+			searchAfter := sr.CustomProps["search_after"].([]any)
+			firstSearchAfter, err := searchAfter[0].(json.Number).Int64()
+			require.NoError(t, err)
+			require.Equal(t, firstSearchAfter, int64(1))
+			secondSearchAfter := searchAfter[1].(string)
+			require.NoError(t, err)
+			require.Equal(t, secondSearchAfter, "2")
+		})
+
+		t.Run("With invalid query should return error", (func(t *testing.T) {
+			c := newFakeClient()
+			res, err := executeElasticsearchDataQuery(c, `{
 				"query": "foo",
 			}`, from, to)
-			require.Error(t, err)
+			require.NoError(t, err)
+			require.Equal(t, res.Responses["A"].ErrorSource, backend.ErrorSourcePlugin)
+			require.Equal(t, res.Responses["A"].Error.Error(), "invalid character '}' looking for beginning of object key string")
 		}))
 	})
 }
@@ -1387,7 +1460,7 @@ func TestSettingsCasting(t *testing.T) {
 								"gamma": "3",
 								"period": "4"
 							}
-						} 
+						}
 					}
 				],
 				"bucketAggs": [{"type": "date_histogram", "field": "@timestamp", "id": "1"}]
@@ -1399,7 +1472,7 @@ func TestSettingsCasting(t *testing.T) {
 		assert.Equal(t, movingAvgSettings["window"], 5.0)
 		assert.Equal(t, movingAvgSettings["predict"], 10.0)
 
-		modelSettings := movingAvgSettings["settings"].(map[string]interface{})
+		modelSettings := movingAvgSettings["settings"].(map[string]any)
 
 		assert.Equal(t, modelSettings["alpha"], 1.0)
 		assert.Equal(t, modelSettings["beta"], 2.0)
@@ -1443,7 +1516,7 @@ func TestSettingsCasting(t *testing.T) {
 		assert.Equal(t, 10., movingAvgSettings["window"])
 		assert.Equal(t, 5., movingAvgSettings["predict"])
 
-		modelSettings := movingAvgSettings["settings"].(map[string]interface{})
+		modelSettings := movingAvgSettings["settings"].(map[string]any)
 
 		assert.Equal(t, .5, modelSettings["alpha"])
 		assert.Equal(t, .7, modelSettings["beta"])
@@ -1597,6 +1670,31 @@ func TestSettingsCasting(t *testing.T) {
 
 				assert.NotZero(t, dateHistogramAgg.FixedInterval)
 			})
+
+			t.Run("Uses calendar_interval", func(t *testing.T) {
+				c := newFakeClient()
+				_, err := executeElasticsearchDataQuery(c, `{
+					"bucketAggs": [
+						{
+							"type": "date_histogram",
+							"field": "@timestamp",
+							"id": "2",
+							"settings": {
+								"interval": "1M"
+							}
+						}
+					],
+					"metrics": [
+						{ "id": "1", "type": "average", "field": "@value" }
+					]
+				}`, from, to)
+				assert.Nil(t, err)
+				sr := c.multisearchRequests[0].Requests[0]
+
+				dateHistogramAgg := sr.Aggs[0].Aggregation.Aggregation.(*es.DateHistogramAgg)
+
+				assert.NotZero(t, dateHistogramAgg.CalendarInterval)
+			})
 		})
 	})
 
@@ -1683,6 +1781,21 @@ func TestSettingsCasting(t *testing.T) {
 			dateHistogramAgg := sr.Aggs[0].Aggregation.Aggregation.(*es.DateHistogramAgg)
 			assert.Equal(t, dateHistogramAgg.FixedInterval, "1d")
 		})
+
+		t.Run("Should use calendar_interval", func(t *testing.T) {
+			c := newFakeClient()
+			_, err := executeElasticsearchDataQuery(c, `{
+				"metrics": [{ "type": "count", "id": "1" }],
+				"bucketAggs": [
+					{ "type": "date_histogram", "id": "2", "field": "@time", "settings": { "min_doc_count": "1", "interval": "1w" } }
+				]
+			}`, from, to)
+
+			assert.Nil(t, err)
+			sr := c.multisearchRequests[0].Requests[0]
+			dateHistogramAgg := sr.Aggs[0].Aggregation.Aggregation.(*es.DateHistogramAgg)
+			assert.Equal(t, dateHistogramAgg.CalendarInterval, "1w")
+		})
 	})
 }
 
@@ -1744,9 +1857,10 @@ func executeElasticsearchDataQuery(c es.Client, body string, from, to time.Time)
 			{
 				JSON:      json.RawMessage(body),
 				TimeRange: timeRange,
+				RefID:     "A",
 			},
 		},
 	}
-	query := newElasticsearchDataQuery(c, dataRequest.Queries)
+	query := newElasticsearchDataQuery(context.Background(), c, &dataRequest, log.New())
 	return query.execute()
 }
