@@ -1,16 +1,5 @@
 // Copyright (c) 2021 The Jaeger Authors.
-//
-// Licensed under the Apache License, Version 2.0 (the "License");
-// you may not use this file except in compliance with the License.
-// You may obtain a copy of the License at
-//
-// http://www.apache.org/licenses/LICENSE-2.0
-//
-// Unless required by applicable law or agreed to in writing, software
-// distributed under the License is distributed on an "AS IS" BASIS,
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-// See the License for the specific language governing permissions and
-// limitations under the License.
+// SPDX-License-Identifier: Apache-2.0
 
 package metricsstore
 
@@ -38,6 +27,7 @@ import (
 	"github.com/jaegertracing/jaeger/pkg/bearertoken"
 	"github.com/jaegertracing/jaeger/pkg/config/tlscfg"
 	"github.com/jaegertracing/jaeger/pkg/prometheus/config"
+	"github.com/jaegertracing/jaeger/pkg/testutils"
 	"github.com/jaegertracing/jaeger/proto-gen/api_v2/metrics"
 	"github.com/jaegertracing/jaeger/storage/metricsstore"
 )
@@ -60,9 +50,8 @@ const defaultTimeout = 30 * time.Second
 
 // defaultConfig should consist of the default values for the prometheus.query.* command line options.
 var defaultConfig = config.Configuration{
-	SupportSpanmetricsConnector: false,
-	MetricNamespace:             "",
-	LatencyUnit:                 "ms",
+	MetricNamespace: "",
+	LatencyUnit:     "ms",
 }
 
 func tracerProvider(t *testing.T) (trace.TracerProvider, *tracetest.InMemoryExporter, func()) {
@@ -137,7 +126,7 @@ func TestMetricsServerError(t *testing.T) {
 		},
 	}
 
-	mockPrometheus := httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
+	mockPrometheus := httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, _ *http.Request) {
 		http.Error(w, "internal server error", http.StatusInternalServerError)
 	}))
 	defer mockPrometheus.Close()
@@ -171,7 +160,7 @@ func TestGetLatencies(t *testing.T) {
 			wantLabels: map[string]string{
 				"service_name": "emailservice",
 			},
-			wantPromQlQuery: `histogram_quantile(0.95, sum(rate(latency_bucket{service_name =~ "emailservice", ` +
+			wantPromQlQuery: `histogram_quantile(0.95, sum(rate(duration_bucket{service_name =~ "emailservice", ` +
 				`span_kind =~ "SPAN_KIND_SERVER"}[10m])) by (service_name,le))`,
 		},
 		{
@@ -185,8 +174,8 @@ func TestGetLatencies(t *testing.T) {
 				"operation":    "/OrderResult",
 				"service_name": "emailservice",
 			},
-			wantPromQlQuery: `histogram_quantile(0.95, sum(rate(latency_bucket{service_name =~ "emailservice", ` +
-				`span_kind =~ "SPAN_KIND_SERVER"}[10m])) by (service_name,operation,le))`,
+			wantPromQlQuery: `histogram_quantile(0.95, sum(rate(duration_bucket{service_name =~ "emailservice", ` +
+				`span_kind =~ "SPAN_KIND_SERVER"}[10m])) by (service_name,span_name,le))`,
 		},
 		{
 			name:             "two services and span kinds result in regex 'or' symbol in query",
@@ -198,16 +187,15 @@ func TestGetLatencies(t *testing.T) {
 			wantLabels: map[string]string{
 				"service_name": "emailservice",
 			},
-			wantPromQlQuery: `histogram_quantile(0.95, sum(rate(latency_bucket{service_name =~ "frontend|emailservice", ` +
+			wantPromQlQuery: `histogram_quantile(0.95, sum(rate(duration_bucket{service_name =~ "frontend|emailservice", ` +
 				`span_kind =~ "SPAN_KIND_SERVER|SPAN_KIND_CLIENT"}[10m])) by (service_name,le))`,
 		},
 		{
-			name:             "enable support for spanmetrics connector with normalized metric name",
+			name:             "enable support for spanmetrics connector with a namespace",
 			serviceNames:     []string{"emailservice"},
 			spanKinds:        []string{"SPAN_KIND_SERVER"},
 			groupByOperation: true,
 			updateConfig: func(cfg config.Configuration) config.Configuration {
-				cfg.SupportSpanmetricsConnector = true
 				cfg.MetricNamespace = "span_metrics"
 				cfg.LatencyUnit = "s"
 				return cfg
@@ -215,6 +203,7 @@ func TestGetLatencies(t *testing.T) {
 			wantName:        "service_operation_latencies",
 			wantDescription: "0.95th quantile latency, grouped by service & operation",
 			wantLabels: map[string]string{
+				"operation":    "/OrderResult",
 				"service_name": "emailservice",
 			},
 			wantPromQlQuery: `histogram_quantile(0.95, sum(rate(span_metrics_duration_bucket{service_name =~ "emailservice", ` +
@@ -226,7 +215,6 @@ func TestGetLatencies(t *testing.T) {
 			spanKinds:        []string{"SPAN_KIND_SERVER"},
 			groupByOperation: true,
 			updateConfig: func(cfg config.Configuration) config.Configuration {
-				cfg.SupportSpanmetricsConnector = true
 				cfg.NormalizeDuration = true
 				cfg.LatencyUnit = "s"
 				return cfg
@@ -234,6 +222,7 @@ func TestGetLatencies(t *testing.T) {
 			wantName:        "service_operation_latencies",
 			wantDescription: "0.95th quantile latency, grouped by service & operation",
 			wantLabels: map[string]string{
+				"operation":    "/OrderResult",
 				"service_name": "emailservice",
 			},
 			wantPromQlQuery: `histogram_quantile(0.95, sum(rate(duration_seconds_bucket{service_name =~ "emailservice", ` +
@@ -289,7 +278,7 @@ func TestGetCallRates(t *testing.T) {
 				"service_name": "emailservice",
 			},
 			wantPromQlQuery: `sum(rate(calls{service_name =~ "emailservice", ` +
-				`span_kind =~ "SPAN_KIND_SERVER"}[10m])) by (service_name,operation)`,
+				`span_kind =~ "SPAN_KIND_SERVER"}[10m])) by (service_name,span_name)`,
 		},
 		{
 			name:             "two services and span kinds result in regex 'or' symbol in query",
@@ -310,13 +299,13 @@ func TestGetCallRates(t *testing.T) {
 			spanKinds:        []string{"SPAN_KIND_SERVER"},
 			groupByOperation: true,
 			updateConfig: func(cfg config.Configuration) config.Configuration {
-				cfg.SupportSpanmetricsConnector = true
 				cfg.MetricNamespace = "span_metrics"
 				return cfg
 			},
 			wantName:        "service_operation_call_rate",
 			wantDescription: "calls/sec, grouped by service & operation",
 			wantLabels: map[string]string{
+				"operation":    "/OrderResult",
 				"service_name": "emailservice",
 			},
 			wantPromQlQuery: `sum(rate(span_metrics_calls{service_name =~ "emailservice", ` +
@@ -328,13 +317,13 @@ func TestGetCallRates(t *testing.T) {
 			spanKinds:        []string{"SPAN_KIND_SERVER"},
 			groupByOperation: true,
 			updateConfig: func(cfg config.Configuration) config.Configuration {
-				cfg.SupportSpanmetricsConnector = true
 				cfg.NormalizeCalls = true
 				return cfg
 			},
 			wantName:        "service_operation_call_rate",
 			wantDescription: "calls/sec, grouped by service & operation",
 			wantLabels: map[string]string{
+				"operation":    "/OrderResult",
 				"service_name": "emailservice",
 			},
 			wantPromQlQuery: `sum(rate(calls_total{service_name =~ "emailservice", ` +
@@ -390,8 +379,8 @@ func TestGetErrorRates(t *testing.T) {
 				"service_name": "emailservice",
 			},
 			wantPromQlQuery: `sum(rate(calls{service_name =~ "emailservice", status_code = "STATUS_CODE_ERROR", ` +
-				`span_kind =~ "SPAN_KIND_SERVER"}[10m])) by (service_name,operation) / ` +
-				`sum(rate(calls{service_name =~ "emailservice", span_kind =~ "SPAN_KIND_SERVER"}[10m])) by (service_name,operation)`,
+				`span_kind =~ "SPAN_KIND_SERVER"}[10m])) by (service_name,span_name) / ` +
+				`sum(rate(calls{service_name =~ "emailservice", span_kind =~ "SPAN_KIND_SERVER"}[10m])) by (service_name,span_name)`,
 		},
 		{
 			name:             "two services and span kinds result in regex 'or' symbol in query",
@@ -413,7 +402,6 @@ func TestGetErrorRates(t *testing.T) {
 			spanKinds:        []string{"SPAN_KIND_SERVER"},
 			groupByOperation: false,
 			updateConfig: func(cfg config.Configuration) config.Configuration {
-				cfg.SupportSpanmetricsConnector = false
 				cfg.MetricNamespace = "span_metrics"
 				cfg.NormalizeCalls = true
 				return cfg
@@ -423,9 +411,9 @@ func TestGetErrorRates(t *testing.T) {
 			wantLabels: map[string]string{
 				"service_name": "emailservice",
 			},
-			wantPromQlQuery: `sum(rate(calls{service_name =~ "emailservice", status_code = "STATUS_CODE_ERROR", ` +
+			wantPromQlQuery: `sum(rate(span_metrics_calls_total{service_name =~ "emailservice", status_code = "STATUS_CODE_ERROR", ` +
 				`span_kind =~ "SPAN_KIND_SERVER"}[10m])) by (service_name) / ` +
-				`sum(rate(calls{service_name =~ "emailservice", span_kind =~ "SPAN_KIND_SERVER"}[10m])) by (service_name)`,
+				`sum(rate(span_metrics_calls_total{service_name =~ "emailservice", span_kind =~ "SPAN_KIND_SERVER"}[10m])) by (service_name)`,
 		},
 		{
 			name:             "enable support for spanmetrics connector with a metric namespace",
@@ -433,13 +421,13 @@ func TestGetErrorRates(t *testing.T) {
 			spanKinds:        []string{"SPAN_KIND_SERVER"},
 			groupByOperation: true,
 			updateConfig: func(cfg config.Configuration) config.Configuration {
-				cfg.SupportSpanmetricsConnector = true
 				cfg.MetricNamespace = "span_metrics"
 				return cfg
 			},
 			wantName:        "service_operation_error_rate",
 			wantDescription: "error rate, computed as a fraction of errors/sec over calls/sec, grouped by service & operation",
 			wantLabels: map[string]string{
+				"operation":    "/OrderResult",
 				"service_name": "emailservice",
 			},
 			wantPromQlQuery: `sum(rate(span_metrics_calls{service_name =~ "emailservice", status_code = "STATUS_CODE_ERROR", ` +
@@ -452,13 +440,13 @@ func TestGetErrorRates(t *testing.T) {
 			spanKinds:        []string{"SPAN_KIND_SERVER"},
 			groupByOperation: true,
 			updateConfig: func(cfg config.Configuration) config.Configuration {
-				cfg.SupportSpanmetricsConnector = true
 				cfg.NormalizeCalls = true
 				return cfg
 			},
 			wantName:        "service_operation_error_rate",
 			wantDescription: "error rate, computed as a fraction of errors/sec over calls/sec, grouped by service & operation",
 			wantLabels: map[string]string{
+				"operation":    "/OrderResult",
 				"service_name": "emailservice",
 			},
 			wantPromQlQuery: `sum(rate(calls_total{service_name =~ "emailservice", status_code = "STATUS_CODE_ERROR", ` +
@@ -513,13 +501,13 @@ func TestGetErrorRatesZero(t *testing.T) {
 		defer r.Body.Close()
 
 		u, err := url.Parse("http://" + r.Host + r.RequestURI + "?" + string(body))
-		require.NoError(t, err)
-
-		q := u.Query()
-		promQuery := q.Get("query")
-		assert.Equal(t, wantPromQLQueries[callCount], promQuery)
-		sendResponse(t, w, responses[callCount])
-		callCount++
+		if assert.NoError(t, err) {
+			q := u.Query()
+			promQuery := q.Get("query")
+			assert.Equal(t, wantPromQLQueries[callCount], promQuery)
+			sendResponse(t, w, responses[callCount])
+			callCount++
+		}
 	}))
 
 	logger := zap.NewNop()
@@ -576,7 +564,7 @@ func TestGetErrorRatesNull(t *testing.T) {
 		defer r.Body.Close()
 
 		u, err := url.Parse("http://" + r.Host + r.RequestURI + "?" + string(body))
-		require.NoError(t, err)
+		assert.NoError(t, err)
 
 		q := u.Query()
 		promQuery := q.Get("query")
@@ -647,7 +635,7 @@ func TestGetErrorRatesErrors(t *testing.T) {
 				defer r.Body.Close()
 
 				u, err := url.Parse("http://" + r.Host + r.RequestURI + "?" + string(body))
-				require.NoError(t, err)
+				assert.NoError(t, err)
 
 				q := u.Query()
 				promQuery := q.Get("query")
@@ -698,9 +686,8 @@ func TestInvalidLatencyUnit(t *testing.T) {
 	tracer, _, closer := tracerProvider(t)
 	defer closer()
 	cfg := config.Configuration{
-		SupportSpanmetricsConnector: true,
-		NormalizeDuration:           true,
-		LatencyUnit:                 "something invalid",
+		NormalizeDuration: true,
+		LatencyUnit:       "something invalid",
 	}
 	_, _ = NewMetricsReader(cfg, zap.NewNop(), tracer)
 }
@@ -729,7 +716,7 @@ func newFakePromServer(t *testing.T) *fakePromServer {
 	s := &fakePromServer{}
 	s.Server = httptest.NewServer(
 		http.HandlerFunc(
-			func(w http.ResponseWriter, r *http.Request) {
+			func(_ http.ResponseWriter, r *http.Request) {
 				t.Logf("Request to fake Prometheus server %+v", r)
 				h := r.Header.Get("Authorization")
 				s.authReceived.Store(&h)
@@ -753,13 +740,15 @@ func TestGetRoundTripperTLSConfig(t *testing.T) {
 	} {
 		t.Run(tc.name, func(t *testing.T) {
 			logger := zap.NewNop()
-			rt, err := getHTTPRoundTripper(&config.Configuration{
+			config := &config.Configuration{
 				ConnectTimeout: 9 * time.Millisecond,
 				TLS: tlscfg.Options{
 					Enabled: tc.tlsEnabled,
 				},
 				TokenOverrideFromContext: true,
-			}, logger)
+			}
+			defer config.TLS.Close()
+			rt, err := getHTTPRoundTripper(config, logger)
 			require.NoError(t, err)
 
 			server := newFakePromServer(t)
@@ -890,15 +879,15 @@ func startMockPrometheusServer(t *testing.T, wantPromQlQuery string, wantWarning
 		defer r.Body.Close()
 
 		u, err := url.Parse("http://" + r.Host + r.RequestURI + "?" + string(body))
-		require.NoError(t, err)
+		assert.NoError(t, err)
 
 		q := u.Query()
 		promQuery := q.Get("query")
 		assert.Equal(t, wantPromQlQuery, promQuery)
 
 		mockResponsePayloadFile := "testdata/service_datapoint_response.json"
-		if strings.Contains(promQuery, "by (service_name,operation") {
-			mockResponsePayloadFile = "testdata/service_operation_datapoint_response.json"
+		if strings.Contains(promQuery, "by (service_name,span_name") {
+			mockResponsePayloadFile = "testdata/service_span_name_datapoint_response.json"
 		}
 		sendResponse(t, w, mockResponsePayloadFile)
 	}))
@@ -951,6 +940,10 @@ func assertMetrics(t *testing.T, gotMetrics *metrics.MetricFamily, wantLabels ma
 	mps := gotMetrics.Metrics[0].MetricPoints
 	assert.Len(t, mps, 1)
 
+	// logging for expected and actual labels
+	t.Logf("Expected labels: %v\n", wantLabels)
+	t.Logf("Actual labels: %v\n", gotMetrics.Metrics[0].Labels)
+
 	// There is no guaranteed order of labels, so we need to take the approach of using a map of expected values.
 	labels := gotMetrics.Metrics[0].Labels
 	assert.Equal(t, len(wantLabels), len(labels))
@@ -960,8 +953,17 @@ func assertMetrics(t *testing.T, gotMetrics *metrics.MetricFamily, wantLabels ma
 		delete(wantLabels, l.Name)
 	}
 	assert.Empty(t, wantLabels)
+
+	// Additional logging to show that all expected labels were found and matched
+	t.Logf("Remaining expected labels after matching: %v\n", wantLabels)
+	t.Logf("\n")
+
 	assert.Equal(t, int64(1620351786), mps[0].Timestamp.GetSeconds())
 
 	actualVal := mps[0].Value.(*metrics.MetricPoint_GaugeValue).GaugeValue.Value.(*metrics.GaugeValue_DoubleValue).DoubleValue
-	assert.Equal(t, float64(9223372036854), actualVal)
+	assert.InDelta(t, float64(9223372036854), actualVal, 0.01)
+}
+
+func TestMain(m *testing.M) {
+	testutils.VerifyGoLeaks(m)
 }
