@@ -1,20 +1,10 @@
 // Copyright (c) 2018 The Jaeger Authors.
-//
-// Licensed under the Apache License, Version 2.0 (the "License");
-// you may not use this file except in compliance with the License.
-// You may obtain a copy of the License at
-//
-// http://www.apache.org/licenses/LICENSE-2.0
-//
-// Unless required by applicable law or agreed to in writing, software
-// distributed under the License is distributed on an "AS IS" BASIS,
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-// See the License for the specific language governing permissions and
-// limitations under the License.
+// SPDX-License-Identifier: Apache-2.0
 
 package badger
 
 import (
+	"context"
 	"errors"
 	"expvar"
 	"flag"
@@ -23,7 +13,7 @@ import (
 	"strings"
 	"time"
 
-	"github.com/dgraph-io/badger/v3"
+	"github.com/dgraph-io/badger/v4"
 	"github.com/spf13/viper"
 	"go.uber.org/zap"
 
@@ -50,6 +40,7 @@ var ( // interface comformance checks
 	_ storage.Factory     = (*Factory)(nil)
 	_ io.Closer           = (*Factory)(nil)
 	_ plugin.Configurable = (*Factory)(nil)
+	_ storage.Purger      = (*Factory)(nil)
 
 	// TODO badger could implement archive storage
 	// _ storage.ArchiveFactory       = (*Factory)(nil)
@@ -59,10 +50,10 @@ var ( // interface comformance checks
 
 // Factory implements storage.Factory for Badger backend.
 type Factory struct {
-	Options *Options
-	store   *badger.DB
-	cache   *badgerStore.CacheStore
-	logger  *zap.Logger
+	Config *Config
+	store  *badger.DB
+	cache  *badgerStore.CacheStore
+	logger *zap.Logger
 
 	tmpDir          string
 	maintenanceDone chan bool
@@ -86,18 +77,18 @@ type Factory struct {
 // NewFactory creates a new Factory.
 func NewFactory() *Factory {
 	return &Factory{
-		Options:         NewOptions("badger"),
+		Config:          DefaultConfig(),
 		maintenanceDone: make(chan bool),
 	}
 }
 
 func NewFactoryWithConfig(
-	cfg NamespaceConfig,
+	cfg Config,
 	metricsFactory metrics.Factory,
 	logger *zap.Logger,
 ) (*Factory, error) {
 	f := NewFactory()
-	f.InitFromOptions(Options{Primary: cfg})
+	f.configure(&cfg)
 	err := f.Initialize(metricsFactory, logger)
 	if err != nil {
 		return nil, err
@@ -107,17 +98,18 @@ func NewFactoryWithConfig(
 
 // AddFlags implements plugin.Configurable
 func (f *Factory) AddFlags(flagSet *flag.FlagSet) {
-	f.Options.AddFlags(flagSet)
+	f.Config.AddFlags(flagSet)
 }
 
 // InitFromViper implements plugin.Configurable
 func (f *Factory) InitFromViper(v *viper.Viper, logger *zap.Logger) {
-	f.Options.InitFromViper(v, logger)
+	f.Config.InitFromViper(v, logger)
+	f.configure(f.Config)
 }
 
-// InitFromOptions initializes Factory from supplied options
-func (f *Factory) InitFromOptions(opts Options) {
-	f.Options = &opts
+// configure initializes Factory from supplied Config.
+func (f *Factory) configure(config *Config) {
+	f.Config = config
 }
 
 // Initialize implements storage.Factory
@@ -126,7 +118,7 @@ func (f *Factory) Initialize(metricsFactory metrics.Factory, logger *zap.Logger)
 
 	opts := badger.DefaultOptions("")
 
-	if f.Options.Primary.Ephemeral {
+	if f.Config.Ephemeral {
 		opts.SyncWrites = false
 		// Error from TempDir is ignored to satisfy Codecov
 		dir, _ := os.MkdirTemp("", "badger")
@@ -134,19 +126,19 @@ func (f *Factory) Initialize(metricsFactory metrics.Factory, logger *zap.Logger)
 		opts.Dir = f.tmpDir
 		opts.ValueDir = f.tmpDir
 
-		f.Options.Primary.KeyDirectory = f.tmpDir
-		f.Options.Primary.ValueDirectory = f.tmpDir
+		f.Config.Directories.Keys = f.tmpDir
+		f.Config.Directories.Values = f.tmpDir
 	} else {
 		// Errors are ignored as they're caught in the Open call
-		initializeDir(f.Options.Primary.KeyDirectory)
-		initializeDir(f.Options.Primary.ValueDirectory)
+		initializeDir(f.Config.Directories.Keys)
+		initializeDir(f.Config.Directories.Values)
 
-		opts.SyncWrites = f.Options.Primary.SyncWrites
-		opts.Dir = f.Options.Primary.KeyDirectory
-		opts.ValueDir = f.Options.Primary.ValueDirectory
+		opts.SyncWrites = f.Config.SyncWrites
+		opts.Dir = f.Config.Directories.Keys
+		opts.ValueDir = f.Config.Directories.Values
 
 		// These options make no sense with ephemeral data
-		opts.ReadOnly = f.Options.Primary.ReadOnly
+		opts.ReadOnly = f.Config.ReadOnly
 	}
 
 	store, err := badger.Open(opts)
@@ -155,7 +147,7 @@ func (f *Factory) Initialize(metricsFactory metrics.Factory, logger *zap.Logger)
 	}
 	f.store = store
 
-	f.cache = badgerStore.NewCacheStore(f.store, f.Options.Primary.SpanStoreTTL, true)
+	f.cache = badgerStore.NewCacheStore(f.store, f.Config.TTL.Spans, true)
 
 	f.metrics.ValueLogSpaceAvailable = metricsFactory.Gauge(metrics.Options{Name: valueLogSpaceAvailableName})
 	f.metrics.KeyLogSpaceAvailable = metricsFactory.Gauge(metrics.Options{Name: keyLogSpaceAvailableName})
@@ -186,7 +178,7 @@ func (f *Factory) CreateSpanReader() (spanstore.Reader, error) {
 
 // CreateSpanWriter implements storage.Factory
 func (f *Factory) CreateSpanWriter() (spanstore.Writer, error) {
-	return badgerStore.NewSpanWriter(f.store, f.cache, f.Options.Primary.SpanStoreTTL), nil
+	return badgerStore.NewSpanWriter(f.store, f.cache, f.Config.TTL.Spans), nil
 }
 
 // CreateDependencyReader implements storage.Factory
@@ -196,12 +188,12 @@ func (f *Factory) CreateDependencyReader() (dependencystore.Reader, error) {
 }
 
 // CreateSamplingStore implements storage.SamplingStoreFactory
-func (f *Factory) CreateSamplingStore(maxBuckets int) (samplingstore.Store, error) {
+func (f *Factory) CreateSamplingStore(int /* maxBuckets */) (samplingstore.Store, error) {
 	return badgerSampling.NewSamplingStore(f.store), nil
 }
 
 // CreateLock implements storage.SamplingStoreFactory
-func (f *Factory) CreateLock() (distributedlock.Lock, error) {
+func (*Factory) CreateLock() (distributedlock.Lock, error) {
 	return &lock{}, nil
 }
 
@@ -214,7 +206,7 @@ func (f *Factory) Close() error {
 	err := f.store.Close()
 
 	// Remove tmp files if this was ephemeral storage
-	if f.Options.Primary.Ephemeral {
+	if f.Config.Ephemeral {
 		errSecondary := os.RemoveAll(f.tmpDir)
 		if err == nil {
 			err = errSecondary
@@ -226,7 +218,7 @@ func (f *Factory) Close() error {
 
 // Maintenance starts a background maintenance job for the badger K/V store, such as ValueLogGC
 func (f *Factory) maintenance() {
-	maintenanceTicker := time.NewTicker(f.Options.Primary.MaintenanceInterval)
+	maintenanceTicker := time.NewTicker(f.Config.MaintenanceInterval)
 	defer maintenanceTicker.Stop()
 	for {
 		select {
@@ -252,7 +244,7 @@ func (f *Factory) maintenance() {
 }
 
 func (f *Factory) metricsCopier() {
-	metricsTicker := time.NewTicker(f.Options.Primary.MetricsUpdateInterval)
+	metricsTicker := time.NewTicker(f.Config.MetricsUpdateInterval)
 	defer metricsTicker.Stop()
 	for {
 		select {
@@ -303,3 +295,12 @@ func (f *Factory) registerBadgerExpvarMetrics(metricsFactory metrics.Factory) {
 		}
 	})
 }
+
+// Purge removes all data from the Factory's underlying Badger store.
+// This function is intended for testing purposes only and should not be used in production environments.
+// Calling Purge in production will result in permanent data loss.
+func (f *Factory) Purge(_ context.Context) error {
+	return f.store.Update(func(_ *badger.Txn) error {
+		return f.store.DropAll()
+	})
+}
