@@ -1,16 +1,5 @@
 // Copyright (c) 2018 The Jaeger Authors.
-//
-// Licensed under the Apache License, Version 2.0 (the "License");
-// you may not use this file except in compliance with the License.
-// You may obtain a copy of the License at
-//
-// http://www.apache.org/licenses/LICENSE-2.0
-//
-// Unless required by applicable law or agreed to in writing, software
-// distributed under the License is distributed on an "AS IS" BASIS,
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-// See the License for the specific language governing permissions and
-// limitations under the License.
+// SPDX-License-Identifier: Apache-2.0
 
 package spanstore_test
 
@@ -37,7 +26,7 @@ import (
 )
 
 func TestWriteReadBack(t *testing.T) {
-	runFactoryTest(t, func(tb testing.TB, sw spanstore.Writer, sr spanstore.Reader) {
+	runFactoryTest(t, func(_ testing.TB, sw spanstore.Writer, sr spanstore.Reader) {
 		tid := time.Now()
 		traces := 40
 		spans := 3
@@ -91,7 +80,7 @@ func TestWriteReadBack(t *testing.T) {
 }
 
 func TestValidation(t *testing.T) {
-	runFactoryTest(t, func(tb testing.TB, sw spanstore.Writer, sr spanstore.Reader) {
+	runFactoryTest(t, func(_ testing.TB, _ spanstore.Writer, sr spanstore.Reader) {
 		tid := time.Now()
 		params := &spanstore.TraceQueryParameters{
 			StartTimeMin: tid,
@@ -130,7 +119,7 @@ func TestValidation(t *testing.T) {
 }
 
 func TestIndexSeeks(t *testing.T) {
-	runFactoryTest(t, func(tb testing.TB, sw spanstore.Writer, sr spanstore.Reader) {
+	runFactoryTest(t, func(_ testing.TB, sw spanstore.Writer, sr spanstore.Reader) {
 		startT := time.Now()
 		traces := 60
 		spans := 3
@@ -290,7 +279,7 @@ func TestIndexSeeks(t *testing.T) {
 }
 
 func TestFindNothing(t *testing.T) {
-	runFactoryTest(t, func(tb testing.TB, sw spanstore.Writer, sr spanstore.Reader) {
+	runFactoryTest(t, func(_ testing.TB, _ spanstore.Writer, sr spanstore.Reader) {
 		startT := time.Now()
 		params := &spanstore.TraceQueryParameters{
 			StartTimeMin: startT,
@@ -309,7 +298,7 @@ func TestFindNothing(t *testing.T) {
 }
 
 func TestWriteDuplicates(t *testing.T) {
-	runFactoryTest(t, func(tb testing.TB, sw spanstore.Writer, sr spanstore.Reader) {
+	runFactoryTest(t, func(_ testing.TB, sw spanstore.Writer, _ spanstore.Reader) {
 		tid := time.Now()
 		times := 40
 		spans := 3
@@ -336,7 +325,7 @@ func TestWriteDuplicates(t *testing.T) {
 }
 
 func TestMenuSeeks(t *testing.T) {
-	runFactoryTest(t, func(tb testing.TB, sw spanstore.Writer, sr spanstore.Reader) {
+	runFactoryTest(t, func(_ testing.TB, sw spanstore.Writer, sr spanstore.Reader) {
 		tid := time.Now()
 		traces := 40
 		services := 4
@@ -384,8 +373,8 @@ func TestPersist(t *testing.T) {
 			require.NoError(t, f.Close())
 		}()
 
-		opts := badger.NewOptions("badger")
-		v, command := config.Viperize(opts.AddFlags)
+		cfg := badger.DefaultConfig()
+		v, command := config.Viperize(cfg.AddFlags)
 
 		keyParam := fmt.Sprintf("--badger.directory-key=%s", dir)
 		valueParam := fmt.Sprintf("--badger.directory-value=%s", dir)
@@ -410,7 +399,7 @@ func TestPersist(t *testing.T) {
 		test(t, sw, sr)
 	}
 
-	p(t, dir, func(t *testing.T, sw spanstore.Writer, sr spanstore.Reader) {
+	p(t, dir, func(t *testing.T, sw spanstore.Writer, _ spanstore.Reader) {
 		s := model.Span{
 			TraceID: model.TraceID{
 				Low:  uint64(1),
@@ -428,7 +417,7 @@ func TestPersist(t *testing.T) {
 		require.NoError(t, err)
 	})
 
-	p(t, dir, func(t *testing.T, sw spanstore.Writer, sr spanstore.Reader) {
+	p(t, dir, func(t *testing.T, _ spanstore.Writer, sr spanstore.Reader) {
 		trace, err := sr.GetTrace(context.Background(), model.TraceID{
 			Low:  uint64(1),
 			High: 1,
@@ -449,8 +438,8 @@ func runFactoryTest(tb testing.TB, test func(tb testing.TB, sw spanstore.Writer,
 		require.NoError(tb, f.Close())
 	}()
 
-	opts := badger.NewOptions("badger")
-	v, command := config.Viperize(opts.AddFlags)
+	cfg := badger.DefaultConfig()
+	v, command := config.Viperize(cfg.AddFlags)
 	command.ParseFlags([]string{
 		"--badger.ephemeral=true",
 		"--badger.consistency=false",
@@ -494,7 +483,7 @@ func writeSpans(sw spanstore.Writer, tags []model.KeyValue, services, operations
 }
 
 func BenchmarkWrites(b *testing.B) {
-	runFactoryTest(b, func(tb testing.TB, sw spanstore.Writer, sr spanstore.Reader) {
+	runFactoryTest(b, func(_ testing.TB, sw spanstore.Writer, _ spanstore.Reader) {
 		tid := time.Now()
 		traces := 1000
 		spans := 32
@@ -538,8 +527,8 @@ func makeWriteSupports(tagsCount, spans int) ([]model.KeyValue, []string, []stri
 	return tags, services, operations
 }
 
-func makeReadBenchmark(b *testing.B, tid time.Time, params *spanstore.TraceQueryParameters, outputFile string) {
-	runLargeFactoryTest(b, func(tb testing.TB, sw spanstore.Writer, sr spanstore.Reader) {
+func makeReadBenchmark(b *testing.B, _ time.Time, params *spanstore.TraceQueryParameters, outputFile string) {
+	runLargeFactoryTest(b, func(_ testing.TB, sw spanstore.Writer, sr spanstore.Reader) {
 		tid := time.Now()
 
 		// Total amount of traces is traces * tracesTimes
@@ -609,8 +598,8 @@ func BenchmarkServiceIndexLimitFetch(b *testing.B) {
 func runLargeFactoryTest(tb testing.TB, test func(tb testing.TB, sw spanstore.Writer, sr spanstore.Reader)) {
 	assertion := require.New(tb)
 	f := badger.NewFactory()
-	opts := badger.NewOptions("badger")
-	v, command := config.Viperize(opts.AddFlags)
+	cfg := badger.DefaultConfig()
+	v, command := config.Viperize(cfg.AddFlags)
 
 	dir := filepath.Join(tb.TempDir(), "badger-testRun")
 	err := os.MkdirAll(dir, 0o700)
@@ -646,7 +635,7 @@ func runLargeFactoryTest(tb testing.TB, test func(tb testing.TB, sw spanstore.Wr
 
 // TestRandomTraceID from issue #1808
 func TestRandomTraceID(t *testing.T) {
-	runFactoryTest(t, func(tb testing.TB, sw spanstore.Writer, sr spanstore.Reader) {
+	runFactoryTest(t, func(_ testing.TB, sw spanstore.Writer, sr spanstore.Reader) {
 		s1 := model.Span{
 			TraceID: model.TraceID{
 				Low:  uint64(14767110704788176287),
