@@ -1,17 +1,6 @@
 // Copyright (c) 2019 The Jaeger Authors.
 // Copyright (c) 2017 Uber Technologies, Inc.
-//
-// Licensed under the Apache License, Version 2.0 (the "License");
-// you may not use this file except in compliance with the License.
-// You may obtain a copy of the License at
-//
-// http://www.apache.org/licenses/LICENSE-2.0
-//
-// Unless required by applicable law or agreed to in writing, software
-// distributed under the License is distributed on an "AS IS" BASIS,
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-// See the License for the specific language governing permissions and
-// limitations under the License.
+// SPDX-License-Identifier: Apache-2.0
 
 package storage
 
@@ -24,6 +13,7 @@ import (
 	"github.com/spf13/viper"
 	"go.uber.org/zap"
 
+	"github.com/jaegertracing/jaeger/internal/safeexpvar"
 	"github.com/jaegertracing/jaeger/pkg/metrics"
 	"github.com/jaegertracing/jaeger/plugin"
 	"github.com/jaegertracing/jaeger/plugin/storage/badger"
@@ -44,7 +34,7 @@ const (
 	elasticsearchStorageType = "elasticsearch"
 	memoryStorageType        = "memory"
 	kafkaStorageType         = "kafka"
-	grpcPluginStorageType    = "grpc-plugin"
+	grpcStorageType          = "grpc"
 	badgerStorageType        = "badger"
 	blackholeStorageType     = "blackhole"
 
@@ -67,7 +57,7 @@ var AllStorageTypes = []string{
 	kafkaStorageType,
 	badgerStorageType,
 	blackholeStorageType,
-	grpcPluginStorageType,
+	grpcStorageType,
 }
 
 // AllSamplingStorageTypes returns all storage backends that implement adaptive sampling
@@ -123,7 +113,7 @@ func NewFactory(config FactoryConfig) (*Factory, error) {
 	return f, nil
 }
 
-func (f *Factory) getFactoryOfType(factoryType string) (storage.Factory, error) {
+func (*Factory) getFactoryOfType(factoryType string) (storage.Factory, error) {
 	switch factoryType {
 	case cassandraStorageType:
 		return cassandra.NewFactory(), nil
@@ -135,7 +125,7 @@ func (f *Factory) getFactoryOfType(factoryType string) (storage.Factory, error)
 		return kafka.NewFactory(), nil
 	case badgerStorageType:
 		return badger.NewFactory(), nil
-	case grpcPluginStorageType:
+	case grpcStorageType:
 		return grpc.NewFactory(), nil
 	case blackholeStorageType:
 		return blackhole.NewFactory(), nil
@@ -338,9 +328,6 @@ func (f *Factory) Close() error {
 }
 
 func (f *Factory) publishOpts() {
-	internalFactory := f.metricsFactory.Namespace(metrics.NSOptions{Name: "internal"})
-	internalFactory.Gauge(metrics.Options{Name: downsamplingRatio}).
-		Update(int64(f.FactoryConfig.DownsamplingRatio))
-	internalFactory.Gauge(metrics.Options{Name: spanStorageType + "-" + f.SpanReaderType}).
-		Update(1)
+	safeexpvar.SetInt(downsamplingRatio, int64(f.FactoryConfig.DownsamplingRatio))
+	safeexpvar.SetInt(spanStorageType+"-"+f.FactoryConfig.SpanReaderType, 1)
 }
