@@ -1,37 +1,24 @@
 // Copyright (c) 2019 The Jaeger Authors.
 // Copyright (c) 2018 Uber Technologies, Inc.
-//
-// Licensed under the Apache License, Version 2.0 (the "License");
-// you may not use this file except in compliance with the License.
-// You may obtain a copy of the License at
-//
-// http://www.apache.org/licenses/LICENSE-2.0
-//
-// Unless required by applicable law or agreed to in writing, software
-// distributed under the License is distributed on an "AS IS" BASIS,
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-// See the License for the specific language governing permissions and
-// limitations under the License.
+// SPDX-License-Identifier: Apache-2.0
 
 package storage
 
 import (
 	"errors"
+	"expvar"
 	"flag"
 	"fmt"
 	"io"
 	"reflect"
 	"strings"
 	"testing"
-	"time"
 
 	"github.com/spf13/viper"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 	"go.uber.org/zap"
 
-	"github.com/jaegertracing/jaeger/internal/metrics/fork"
-	"github.com/jaegertracing/jaeger/internal/metricstest"
 	"github.com/jaegertracing/jaeger/pkg/config"
 	"github.com/jaegertracing/jaeger/pkg/metrics"
 	"github.com/jaegertracing/jaeger/storage"
@@ -379,7 +366,7 @@ func TestParsingDownsamplingRatio(t *testing.T) {
 	require.NoError(t, err)
 	f.InitFromViper(v, zap.NewNop())
 
-	assert.Equal(t, 1.0, f.FactoryConfig.DownsamplingRatio)
+	assert.InDelta(t, 1.0, f.FactoryConfig.DownsamplingRatio, 0.01)
 	assert.Equal(t, "jaeger", f.FactoryConfig.DownsamplingHashSalt)
 
 	err = command.ParseFlags([]string{
@@ -387,7 +374,7 @@ func TestParsingDownsamplingRatio(t *testing.T) {
 	})
 	require.NoError(t, err)
 	f.InitFromViper(v, zap.NewNop())
-	assert.Equal(t, 0.5, f.FactoryConfig.DownsamplingRatio)
+	assert.InDelta(t, 0.5, f.FactoryConfig.DownsamplingRatio, 0.01)
 }
 
 func TestDefaultDownsamplingWithAddFlags(t *testing.T) {
@@ -397,7 +384,7 @@ func TestDefaultDownsamplingWithAddFlags(t *testing.T) {
 	require.NoError(t, err)
 	f.InitFromViper(v, zap.NewNop())
 
-	assert.Equal(t, defaultDownsamplingRatio, f.FactoryConfig.DownsamplingRatio)
+	assert.InDelta(t, defaultDownsamplingRatio, f.FactoryConfig.DownsamplingRatio, 0.01)
 	assert.Equal(t, defaultDownsamplingHashSalt, f.FactoryConfig.DownsamplingHashSalt)
 
 	err = command.ParseFlags([]string{
@@ -409,25 +396,10 @@ func TestDefaultDownsamplingWithAddFlags(t *testing.T) {
 func TestPublishOpts(t *testing.T) {
 	f, err := NewFactory(defaultCfg())
 	require.NoError(t, err)
-
-	baseMetrics := metricstest.NewFactory(time.Second)
-	baseMetrics.Stop()
-	forkFactory := metricstest.NewFactory(time.Second)
-	forkFactory.Stop()
-	metricsFactory := fork.New("internal", forkFactory, baseMetrics)
-	f.metricsFactory = metricsFactory
-
-	// This method is called inside factory.Initialize method
 	f.publishOpts()
 
-	forkFactory.AssertGaugeMetrics(t, metricstest.ExpectedMetric{
-		Name:  "internal." + downsamplingRatio,
-		Value: int(f.DownsamplingRatio),
-	})
-	forkFactory.AssertGaugeMetrics(t, metricstest.ExpectedMetric{
-		Name:  "internal." + spanStorageType + "-" + f.SpanReaderType,
-		Value: 1,
-	})
+	assert.EqualValues(t, 1, expvar.Get(downsamplingRatio).(*expvar.Int).Value())
+	assert.EqualValues(t, 1, expvar.Get(spanStorageType+"-"+f.SpanReaderType).(*expvar.Int).Value())
 }
 
 type errorFactory struct {
@@ -439,19 +411,19 @@ var (
 	_ io.Closer       = (*errorFactory)(nil)
 )
 
-func (e errorFactory) Initialize(metricsFactory metrics.Factory, logger *zap.Logger) error {
+func (errorFactory) Initialize(metrics.Factory, *zap.Logger) error {
 	panic("implement me")
 }
 
-func (e errorFactory) CreateSpanReader() (spanstore.Reader, error) {
+func (errorFactory) CreateSpanReader() (spanstore.Reader, error) {
 	panic("implement me")
 }
 
-func (e errorFactory) CreateSpanWriter() (spanstore.Writer, error) {
+func (errorFactory) CreateSpanWriter() (spanstore.Writer, error) {
 	panic("implement me")
 }
 
-func (e errorFactory) CreateDependencyReader() (dependencystore.Reader, error) {
+func (errorFactory) CreateDependencyReader() (dependencystore.Reader, error) {
 	panic("implement me")
 }
 
