@@ -9,9 +9,11 @@
 set -euo pipefail
 
 NO_TEST_FILE_DIRS=""
+total_pkgs=0
 
 # shellcheck disable=SC2048
 for dir in $*; do
+  ((total_pkgs+=1))
   mainFile=$(find "${dir}" -maxdepth 1 -name 'main.go')
   testFiles=$(find "${dir}" -maxdepth 1 -name '*_test.go')
   if [ -z "${testFiles}" ]; then
@@ -42,4 +44,6 @@ if [ -n "${NO_TEST_FILE_DIRS}" ]; then
   echo "error: at least one *_test.go file must be in all directories with go files so that they are counted for code coverage" >&2
   echo "       if no tests are possible for a package (e.g. it only defines types), create empty_test.go" >&2
   exit 1
+else
+  echo "✅ Info(check-test-files): no issues after scanning ${total_pkgs} package(s)."
 fi
