@@ -27,15 +27,15 @@ bool CollectionIterator::next() {
 }
 
 bool CollectionIterator::decode_value() {
-  DataType::ConstPtr data_type;
   if (collection_->value_type() == CASS_VALUE_TYPE_MAP) {
-    data_type =
+    const DataType::ConstPtr& data_type =
         (index_ % 2 == 0) ? collection_->primary_data_type() : collection_->secondary_data_type();
+    value_ = decoder_.decode_value(data_type);
   } else {
-    data_type = collection_->primary_data_type();
+    value_ = decoder_.decode_value(collection_->primary_data_type());
   }
 
-  return decoder_.decode_value(data_type, value_, true);
+  return value_.is_valid();
 }
 
 bool TupleIterator::next() {
@@ -43,5 +43,7 @@ bool TupleIterator::next() {
     return false;
   }
   current_ = next_++;
-  return decoder_.decode_value(*current_, value_);
+
+  value_ = decoder_.decode_value(*current_);
+  return value_.is_valid();
 }
