@@ -1,6 +1,7 @@
 /*
  * Copyright 2014-2017 Spotify AB
  * Copyright 2016-2019 The Last Pickle Ltd
+ * Copyright 2020-2020 DataStax, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -18,47 +19,48 @@
 package io.cassandrareaper;
 
 import io.cassandrareaper.ReaperApplicationConfiguration.DatacenterAvailability;
-import io.cassandrareaper.ReaperApplicationConfiguration.JmxCredentials;
 import io.cassandrareaper.core.Cluster;
 import io.cassandrareaper.core.Node;
-import io.cassandrareaper.jmx.ClusterFacade;
-import io.cassandrareaper.jmx.JmxConnectionFactory;
-import io.cassandrareaper.jmx.JmxConnectionsInitializer;
+import io.cassandrareaper.crypto.Cryptograph;
+import io.cassandrareaper.crypto.NoopCrypotograph;
+import io.cassandrareaper.management.ClusterFacade;
+import io.cassandrareaper.management.http.HttpManagementConnectionFactory;
+import io.cassandrareaper.management.jmx.JmxManagementConnectionFactory;
+import io.cassandrareaper.metrics.PrometheusMetricsFilter;
 import io.cassandrareaper.resources.ClusterResource;
+import io.cassandrareaper.resources.CryptoResource;
 import io.cassandrareaper.resources.DiagEventSseResource;
 import io.cassandrareaper.resources.DiagEventSubscriptionResource;
 import io.cassandrareaper.resources.NodeStatsResource;
 import io.cassandrareaper.resources.PingResource;
 import io.cassandrareaper.resources.ReaperHealthCheck;
+import io.cassandrareaper.resources.ReaperResource;
 import io.cassandrareaper.resources.RepairRunResource;
 import io.cassandrareaper.resources.RepairScheduleResource;
+import io.cassandrareaper.resources.RequestUtils;
 import io.cassandrareaper.resources.SnapshotResource;
 import io.cassandrareaper.resources.auth.LoginResource;
 import io.cassandrareaper.resources.auth.ShiroExceptionMapper;
 import io.cassandrareaper.resources.auth.ShiroJwtProvider;
 import io.cassandrareaper.service.AutoSchedulingManager;
+import io.cassandrareaper.service.Heart;
 import io.cassandrareaper.service.PurgeService;
 import io.cassandrareaper.service.RepairManager;
 import io.cassandrareaper.service.SchedulingManager;
-import io.cassandrareaper.storage.CassandraStorage;
 import io.cassandrareaper.storage.IDistributedStorage;
-import io.cassandrareaper.storage.IStorage;
-import io.cassandrareaper.storage.MemoryStorage;
-import io.cassandrareaper.storage.PostgresStorage;
+import io.cassandrareaper.storage.InitializeStorage;
 
+import java.nio.file.Files;
+import java.nio.file.Paths;
 import java.util.EnumSet;
-import java.util.Map;
 import java.util.Optional;
 import java.util.concurrent.ScheduledExecutorService;
 import java.util.concurrent.TimeUnit;
-
 import javax.servlet.DispatcherType;
 import javax.servlet.FilterRegistration;
 
 import com.codahale.metrics.InstrumentedScheduledExecutorService;
-import com.codahale.metrics.MetricRegistry;
-import com.codahale.metrics.Timer;
-import com.datastax.driver.core.policies.EC2MultiRegionAddressTranslator;
+import com.fasterxml.jackson.datatype.joda.JodaModule;
 import com.fasterxml.jackson.datatype.jsr310.JavaTimeModule;
 import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Preconditions;
@@ -67,26 +69,23 @@ import io.dropwizard.assets.AssetsBundle;
 import io.dropwizard.client.HttpClientBuilder;
 import io.dropwizard.configuration.EnvironmentVariableSubstitutor;
 import io.dropwizard.configuration.SubstitutingSourceProvider;
-import io.dropwizard.db.DataSourceFactory;
-import io.dropwizard.jdbi.DBIFactory;
-import io.dropwizard.jetty.BiDiGzipHandler;
 import io.dropwizard.setup.Bootstrap;
 import io.dropwizard.setup.Environment;
 import io.prometheus.client.CollectorRegistry;
 import io.prometheus.client.dropwizard.DropwizardExports;
 import io.prometheus.client.exporter.MetricsServlet;
-import org.apache.commons.lang3.StringUtils;
 import org.apache.http.client.HttpClient;
 import org.eclipse.jetty.server.Handler;
+import org.eclipse.jetty.server.handler.gzip.GzipHandler;
 import org.eclipse.jetty.server.session.SessionHandler;
 import org.eclipse.jetty.servlets.CrossOriginFilter;
-import org.flywaydb.core.Flyway;
 import org.joda.time.DateTimeZone;
 import org.secnod.dropwizard.shiro.ShiroBundle;
 import org.secnod.dropwizard.shiro.ShiroConfiguration;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
-import sun.misc.Signal;
+
+import static io.cassandrareaper.metrics.PrometheusMetricsConfiguration.getCustomSampleMethodBuilder;
 
 public final class ReaperApplication extends Application<ReaperApplicationConfiguration> {
 
@@ -99,22 +98,29 @@ public final class ReaperApplication extends Application<ReaperApplicationConfig
     this.context = new AppContext();
   }
 
-  @VisibleForTesting
-  public ReaperApplication(AppContext context) {
-    super();
-    LOG.info("ReaperApplication constructor called with custom AppContext");
-    this.context = context;
-  }
-
   public static void main(String[] args) throws Exception {
     new ReaperApplication().run(args);
   }
 
+  private static void setupSse(Environment environment) {
+    // Enabling gzip buffering will prevent flushing of server-side-events, so we disable compression for SSE
+    environment.lifecycle().addServerLifecycleListener(server -> {
+      for (Handler handler : server.getChildHandlersByClass(GzipHandler.class)) {
+        ((GzipHandler) handler).addExcludedMimeTypes("text/event-stream");
+      }
+    });
+  }
+
   @Override
   public String getName() {
     return "cassandra-reaper";
   }
 
+  @VisibleForTesting
+  public AppContext getContext() {
+    return context;
+  }
+
   /**
    * Before a Dropwizard application can provide the command-line interface, parse a configuration file, or run as a
    * server, it must first go through a bootstrapping phase. You can add Bundles, Commands, or register Jackson modules
@@ -122,8 +128,9 @@ public final class ReaperApplication extends Application<ReaperApplicationConfig
    */
   @Override
   public void initialize(Bootstrap<ReaperApplicationConfiguration> bootstrap) {
+    bootstrap.addCommand(new ReaperDbMigrationCommand("schema-migration", "Performs database schema migrations"));
     bootstrap.addBundle(new AssetsBundle("/assets/", "/webui", "index.html"));
-    bootstrap.getObjectMapper().registerModule(new JavaTimeModule());
+    bootstrap.getObjectMapper().registerModule(new JavaTimeModule()).registerModule(new JodaModule());
 
     // enable using environment variables in yml files
     final SubstitutingSourceProvider envSourceProvider = new SubstitutingSourceProvider(
@@ -148,7 +155,6 @@ public final class ReaperApplication extends Application<ReaperApplicationConfig
     bootstrap.setConfigurationSourceProvider(
         new SubstitutingSourceProvider(
             bootstrap.getConfigurationSourceProvider(), new EnvironmentVariableSubstitutor()));
-    bootstrap.getObjectMapper().registerModule(new JavaTimeModule());
   }
 
   @Override
@@ -157,9 +163,9 @@ public final class ReaperApplication extends Application<ReaperApplicationConfig
     DateTimeZone.setDefault(DateTimeZone.UTC);
     checkConfiguration(config);
     context.config = config;
-    addSignalHandlers(); // SIGHUP, etc.
     context.metricRegistry = environment.metrics();
-    CollectorRegistry.defaultRegistry.register(new DropwizardExports(environment.metrics()));
+    CollectorRegistry.defaultRegistry.register(new DropwizardExports(environment.metrics(),
+        new PrometheusMetricsFilter(), getCustomSampleMethodBuilder()));
 
     environment
         .admin()
@@ -167,51 +173,32 @@ public final class ReaperApplication extends Application<ReaperApplicationConfig
         .addMapping("/prometheusMetrics");
 
     int repairThreads = config.getRepairRunThreadCount();
-    LOG.info("initializing runner thread pool with {} threads", repairThreads);
+    int maxParallelRepairs = config.getMaxParallelRepairs();
+    LOG.info("initializing runner thread pool with {} threads and {} repair runners",
+        repairThreads, maxParallelRepairs);
 
     tryInitializeStorage(config, environment);
 
-    if (context.jmxConnectionFactory == null) {
-      LOG.info("no JMX connection factory given in context, creating default");
-      context.jmxConnectionFactory = new JmxConnectionFactory(context);
+    Cryptograph cryptograph = context.config == null || context.config.getCryptograph() == null
+        ? new NoopCrypotograph() : context.config.getCryptograph().create();
 
-      // read jmx host/port mapping from config and provide to jmx con.factory
-      Map<String, Integer> jmxPorts = config.getJmxPorts();
-      if (jmxPorts != null) {
-        LOG.debug("using JMX ports mapping: {}", jmxPorts);
-        context.jmxConnectionFactory.setJmxPorts(jmxPorts);
-      }
-      if (config.useAddressTranslator()) {
-        context.jmxConnectionFactory.setAddressTranslator(new EC2MultiRegionAddressTranslator());
-      }
-    }
-
-    JmxCredentials jmxAuth = config.getJmxAuth();
-    if (jmxAuth != null) {
-      LOG.debug("using specified JMX credentials for authentication");
-      context.jmxConnectionFactory.setJmxAuth(jmxAuth);
-    }
-
-    Map<String, JmxCredentials> jmxCredentials = config.getJmxCredentials();
-    if (jmxCredentials != null) {
-      LOG.debug("using specified JMX credentials per cluster for authentication");
-      context.jmxConnectionFactory.setJmxCredentials(jmxCredentials);
-    }
+    initializeManagement(context, environment, cryptograph);
 
     context.repairManager = RepairManager.create(
         context,
         environment.lifecycle().scheduledExecutorService("RepairRunner").threads(repairThreads).build(),
-        config.getHangingRepairTimeoutMins(),
-        TimeUnit.MINUTES,
         config.getRepairManagerSchedulingIntervalSeconds(),
-        TimeUnit.SECONDS);
+        TimeUnit.SECONDS,
+        maxParallelRepairs,
+        context.storage.getRepairRunDao());
 
+    RequestUtils.setCorsEnabled(config.isEnableCrossOrigin());
     // Enable cross-origin requests for using external GUI applications.
     if (config.isEnableCrossOrigin() || System.getProperty("enableCrossOrigin") != null) {
       FilterRegistration.Dynamic co = environment.servlets().addFilter("crossOriginRequests", CrossOriginFilter.class);
       co.setInitParameter("allowedOrigins", "*");
-      co.setInitParameter("allowedHeaders", "X-Requested-With,Content-Type,Accept,Origin");
-      co.setInitParameter("allowedMethods", "OPTIONS,GET,PUT,POST,DELETE,HEAD");
+      co.setInitParameter("allowedHeaders", "X-Requested-With,Content-Type,Accept,Origin,Authorization");
+      co.setInitParameter("allowedMethods", "OPTIONS,GET,PUT,POST,DELETE,HEAD,PATCH");
       co.addMappingForUrlPatterns(EnumSet.allOf(DispatcherType.class), true, "/*");
     }
 
@@ -225,31 +212,44 @@ public final class ReaperApplication extends Application<ReaperApplicationConfig
     final PingResource pingResource = new PingResource(healthCheck);
     environment.jersey().register(pingResource);
 
-    final ClusterResource addClusterResource = new ClusterResource(
-        context,
-        environment.lifecycle().executorService("ClusterResource").minThreads(6).maxThreads(6).build());
-
+    final ClusterResource addClusterResource = ClusterResource.create(context, cryptograph,
+        context.storage.getEventsDao(),
+        context.storage.getRepairRunDao());
     environment.jersey().register(addClusterResource);
-    final RepairRunResource addRepairRunResource = new RepairRunResource(context);
+    final RepairRunResource addRepairRunResource = new RepairRunResource(context,
+        context.storage.getRepairRunDao());
     environment.jersey().register(addRepairRunResource);
-    final RepairScheduleResource addRepairScheduleResource = new RepairScheduleResource(context);
+    final RepairScheduleResource addRepairScheduleResource = new RepairScheduleResource(context,
+        context.storage.getRepairRunDao());
     environment.jersey().register(addRepairScheduleResource);
-    final SnapshotResource snapshotResource = new SnapshotResource(context, environment);
+    final SnapshotResource snapshotResource = new SnapshotResource(context, environment,
+        context.storage.getSnapshotDao());
     environment.jersey().register(snapshotResource);
+    final ReaperResource reaperResource = new ReaperResource(context);
+    environment.jersey().register(reaperResource);
 
     final NodeStatsResource nodeStatsResource = new NodeStatsResource(context);
     environment.jersey().register(nodeStatsResource);
 
+    final CryptoResource addCryptoResource = new CryptoResource(cryptograph);
+    environment.jersey().register(addCryptoResource);
+
     HttpClient httpClient = createHttpClient(config, environment);
-    ScheduledExecutorService ses = environment.lifecycle().scheduledExecutorService("Diagnostics").threads(6).build();
-    final DiagEventSubscriptionResource eventsResource = new DiagEventSubscriptionResource(context, httpClient, ses);
-    environment.jersey().register(eventsResource);
-    final DiagEventSseResource diagEvents = new DiagEventSseResource(context, httpClient, ses);
-    environment.jersey().register(diagEvents);
+
+    if (config.getHttpManagement() == null || !config.getHttpManagement().isEnabled()) {
+      ScheduledExecutorService ses = environment.lifecycle().scheduledExecutorService("Diagnostics").threads(6).build();
+      final DiagEventSubscriptionResource eventsResource = new DiagEventSubscriptionResource(context, httpClient, ses,
+          context.storage.getEventsDao());
+      environment.jersey().register(eventsResource);
+      final DiagEventSseResource diagEvents = new DiagEventSseResource(context, httpClient, ses,
+          context.storage.getEventsDao());
+      environment.jersey().register(diagEvents);
+    }
 
     if (config.isAccessControlEnabled()) {
       SessionHandler sessionHandler = new SessionHandler();
       sessionHandler.setMaxInactiveInterval((int) config.getAccessControl().getSessionTimeout().getSeconds());
+      RequestUtils.setSessionTimeout(config.getAccessControl().getSessionTimeout());
       environment.getApplicationContext().setSessionHandler(sessionHandler);
       environment.servlets().setSessionHandler(sessionHandler);
       environment.jersey().register(new ShiroExceptionMapper());
@@ -258,15 +258,16 @@ public final class ReaperApplication extends Application<ReaperApplicationConfig
     }
 
     Thread.sleep(1000);
-    context.schedulingManager = SchedulingManager.create(context);
+    context.schedulingManager = SchedulingManager.create(context,
+        context.storage.getRepairRunDao());
     context.schedulingManager.start();
 
     if (config.hasAutoSchedulingEnabled()) {
       LOG.debug("using specified configuration for auto scheduling: {}", config.getAutoScheduling());
-      AutoSchedulingManager.start(context);
+      AutoSchedulingManager.start(context, context.storage.getRepairRunDao());
     }
 
-    initializeJmxSeedsForAllClusters();
+
     maybeInitializeSidecarMode(addClusterResource);
     LOG.info("resuming pending repair runs");
 
@@ -281,53 +282,45 @@ public final class ReaperApplication extends Application<ReaperApplicationConfig
         "Cassandra backend storage is the only one allowing EACH datacenter availability modes.");
 
     ScheduledExecutorService scheduler = new InstrumentedScheduledExecutorService(
-            environment.lifecycle().scheduledExecutorService("ReaperApplication-scheduler").threads(3).build(),
-            context.metricRegistry);
+        environment.lifecycle().scheduledExecutorService("ReaperApplication-scheduler").threads(3).build(),
+        context.metricRegistry);
 
-    if (context.storage instanceof IDistributedStorage) {
-      // Allowing multiple Reaper instances to work concurrently requires
-      // us to poll the database for running repairs regularly
-      // only with Cassandra storage
-      scheduleRepairManager(scheduler);
-      scheduleHandleMetricsRequest(scheduler);
-    } else {
-      // Storage is different than Cassandra, assuming we have a single instance
-      context.repairManager.resumeRunningRepairRuns();
-    }
+    // SIDECAR mode must be distributed. ALL|EACH|LOCAL are lazy: we wait until we see multiple reaper instances
+    context.isDistributed
+        .compareAndSet(false, DatacenterAvailability.SIDECAR == context.config.getDatacenterAvailability());
 
+    // Allowing multiple Reaper instances require concurrent database polls for repair and metrics statuses
+    scheduleRepairManager(scheduler);
+    scheduleHeartbeat(scheduler);
+
+
+    context.repairManager.resumeRunningRepairRuns();
     schedulePurge(scheduler);
+
     LOG.info("Initialization complete!");
     LOG.warn("Reaper is ready to get things done!");
   }
 
-  private void tryInitializeStorage(ReaperApplicationConfiguration config, Environment environment)
-      throws ReaperException, InterruptedException {
-    if (context.storage == null) {
-      LOG.info("initializing storage of type: {}", config.getStorageType());
-      int storageFailures = 0;
-      while (true) {
-        try {
-          context.storage = initializeStorage(config, environment);
-          break;
-        } catch (RuntimeException e) {
-          LOG.error("Storage is not ready yet, trying again to connect shortly...", e);
-          storageFailures++;
-          if (storageFailures > 60) {
-            LOG.error("Too many failures when trying to connect storage. Exiting :'(");
-            System.exit(1);
-          }
-          Thread.sleep(10000);
-        }
+
+  private void initializeManagement(AppContext context, Environment environment, Cryptograph cryptograph) {
+    if (context.managementConnectionFactory == null) {
+      LOG.info("no management connection factory given in context, creating default");
+      if (context.config.getHttpManagement() == null || !context.config.getHttpManagement().isEnabled()) {
+        LOG.info("HTTP management connection config not set, or set disabled. Creating JMX connection factory instead");
+        context.managementConnectionFactory = new JmxManagementConnectionFactory(context, cryptograph);
+      } else {
+        ScheduledExecutorService jobStatusPollerExecutor = environment.lifecycle()
+            .scheduledExecutorService("JobStatusPoller")
+            .threads(2).build();
+        context.managementConnectionFactory = new HttpManagementConnectionFactory(context, jobStatusPollerExecutor);
       }
-    } else {
-      LOG.info("storage already given in context, not initializing a new one");
     }
   }
 
   /**
    * If Reaper is in sidecar mode, grab the local host id and the associated broadcast address
-   * @param addClusterResource
    *
+   * @param addClusterResource a cluster resource instance
    * @throws ReaperException any caught runtime exception
    */
   private void maybeInitializeSidecarMode(ClusterResource addClusterResource) throws ReaperException {
@@ -336,8 +329,8 @@ public final class ReaperApplication extends Application<ReaperApplicationConfig
       Node host = Node.builder().withHostname(context.config.getEnforcedLocalNode().orElse("127.0.0.1")).build();
       try {
         context.localNodeAddress = context.config
-                .getEnforcedLocalNode()
-                .orElse(clusterFacade.getLocalEndpoint(host));
+            .getEnforcedLocalNode()
+            .orElse(clusterFacade.getLocalEndpoint(host));
 
         LOG.info("Sidecar mode. Local node is : {}", context.localNodeAddress);
         selfRegisterClusterForSidecar(addClusterResource, context.config.getEnforcedLocalNode().orElse("127.0.0.1"));
@@ -350,30 +343,24 @@ public final class ReaperApplication extends Application<ReaperApplicationConfig
 
   private boolean selfRegisterClusterForSidecar(ClusterResource addClusterResource, String seedHost)
       throws ReaperException {
-    final Optional<Cluster> cluster = addClusterResource.findClusterWithSeedHost(seedHost, Optional.empty());
+    final Optional<Cluster> cluster = addClusterResource.findClusterWithSeedHost(
+        seedHost, Optional.empty(),Optional.empty()
+    );
     if (!cluster.isPresent()) {
       return false;
     }
 
-    if (context.storage.getClusters().stream().noneMatch(c -> c.getName().equals(cluster.get().getName()))) {
+    if (context.storage.getClusterDao().getClusters().stream()
+        .noneMatch(c -> c.getName().equals(cluster.get().getName()))) {
       LOG.info("registering new cluster : {}", cluster.get().getName());
 
       // it is ok for this be called in parallel by different sidecars on the same cluster,
       // so long we're not split-brain
-      context.storage.addCluster(cluster.get());
+      context.storage.getClusterDao().addCluster(cluster.get());
     }
     return true;
   }
 
-  private static void setupSse(Environment environment) {
-    // Enabling gzip buffering will prevent flushing of server-side-events, so we disable compression for SSE
-    environment.lifecycle().addServerLifecycleListener(server -> {
-      for (Handler handler : server.getChildHandlersByClass(BiDiGzipHandler.class)) {
-        ((BiDiGzipHandler) handler).addExcludedMimeTypes("text/event-stream");
-      }
-    });
-  }
-
   private HttpClient createHttpClient(ReaperApplicationConfiguration config, Environment environment) {
     return new HttpClientBuilder(environment).using(config.getHttpClientConfiguration()).build(getName());
   }
@@ -381,29 +368,33 @@ public final class ReaperApplication extends Application<ReaperApplicationConfig
   private void scheduleRepairManager(ScheduledExecutorService scheduler) {
     scheduler.scheduleWithFixedDelay(
         () -> {
-          try {
-            context.repairManager.resumeRunningRepairRuns();
-          } catch (ReaperException | RuntimeException e) {
-            // test-pollution: grim_reaper trashes this log error
-            //if (!Boolean.getBoolean("grim.reaper.running")) {
-            LOG.error("Couldn't resume running repair runs", e);
-            //}
+          if (context.isDistributed.get()) {
+            try {
+              context.repairManager.resumeRunningRepairRuns();
+            } catch (ReaperException | RuntimeException e) {
+              // test-pollution: grim_reaper trashes this log error
+              //if (!Boolean.getBoolean("grim.reaper.running")) {
+              LOG.error("Couldn't resume running repair runs", e);
+              //}
+            }
           }
         },
-        0,
+        10,
         10,
         TimeUnit.SECONDS);
   }
 
-  private void scheduleHandleMetricsRequest(ScheduledExecutorService scheduler) {
-    scheduler.scheduleWithFixedDelay(
+  private void scheduleHeartbeat(ScheduledExecutorService scheduler) throws ReaperException {
+    final Heart heart = Heart.create(context);
+
+    scheduler.scheduleAtFixedRate(
         () -> {
           try {
-            context.repairManager.handleMetricsRequests();
-          } catch (ReaperException | RuntimeException e) {
+            heart.beat();
+          } catch (RuntimeException e) {
             // test-pollution: grim_reaper trashes this log error
             //if (!Boolean.getBoolean("grim.reaper.running")) {
-            LOG.error("Couldn't handle metrics requests", e);
+            LOG.error("Couldn't heartbeat", e);
             //}
           }
         },
@@ -413,7 +404,8 @@ public final class ReaperApplication extends Application<ReaperApplicationConfig
   }
 
   private void schedulePurge(ScheduledExecutorService scheduler) {
-    final PurgeService purgeManager = PurgeService.create(context);
+    final PurgeService purgeManager = PurgeService.create(context,
+        context.storage.getRepairRunDao());
 
     scheduler.scheduleWithFixedDelay(
         () -> {
@@ -429,105 +421,54 @@ public final class ReaperApplication extends Application<ReaperApplicationConfig
         TimeUnit.HOURS);
   }
 
-  private IStorage initializeStorage(ReaperApplicationConfiguration config, Environment environment)
-      throws ReaperException {
-    IStorage storage;
-
-    if ("memory".equalsIgnoreCase(config.getStorageType())) {
-      storage = new MemoryStorage();
-    } else if ("cassandra".equalsIgnoreCase(config.getStorageType())) {
-      storage = new CassandraStorage(context.reaperInstanceId, config, environment);
-    } else if ("postgres".equalsIgnoreCase(config.getStorageType())
-        || "h2".equalsIgnoreCase(config.getStorageType())
-        || "database".equalsIgnoreCase(config.getStorageType())) {
-      // create DBI instance
-      final DBIFactory factory = new DBIFactory();
-      if (StringUtils.isEmpty(config.getDataSourceFactory().getDriverClass())
-          && "postgres".equalsIgnoreCase(config.getStorageType())) {
-        config.getDataSourceFactory().setDriverClass("org.postgresql.Driver");
-      } else if (StringUtils.isEmpty(config.getDataSourceFactory().getDriverClass())
-          && "h2".equalsIgnoreCase(config.getStorageType())) {
-        config.getDataSourceFactory().setDriverClass("org.h2.Driver");
-      }
-      // instantiate store
-      storage = new PostgresStorage(
-          context.reaperInstanceId,
-          factory.build(environment, config.getDataSourceFactory(), "postgresql")
-      );
-      initDatabase(config);
-    } else {
-      LOG.error("invalid storageType: {}", config.getStorageType());
-      throw new ReaperException("invalid storage type: " + config.getStorageType());
-    }
-    Preconditions.checkState(storage.isStorageConnected(), "Failed to connect storage");
-    return storage;
-  }
-
   private void checkConfiguration(ReaperApplicationConfiguration config) {
     LOG.debug("repairIntensity: {}", config.getRepairIntensity());
     LOG.debug("incrementalRepair: {}", config.getIncrementalRepair());
+    LOG.debug("subrangeIncrementalRepair: {}", config.getSubrangeIncrementalRepair());
     LOG.debug("repairRunThreadCount: {}", config.getRepairRunThreadCount());
     LOG.debug("segmentCount: {}", config.getSegmentCount());
     LOG.debug("repairParallelism: {}", config.getRepairParallelism());
     LOG.debug("hangingRepairTimeoutMins: {}", config.getHangingRepairTimeoutMins());
     LOG.debug("jmxPorts: {}", config.getJmxPorts());
-  }
 
-  void reloadConfiguration() {
-    // TODO: reload configuration, but how?
-    LOG.warn("SIGHUP signal dropped, missing implementation for configuration reload");
-  }
-
-  private void addSignalHandlers() {
-    if (!System.getProperty("os.name").toLowerCase().contains("win")) {
-      LOG.debug("adding signal handler for SIGHUP");
-      Signal.handle(new Signal("HUP"), signal -> {
-        LOG.info("received SIGHUP signal: {}", signal);
-        reloadConfiguration();
-      });
+    if (config.getHttpManagement() != null) {
+      if (config.getHttpManagement().isEnabled()) {
+        if (config.getHttpManagement().getTruststoresDir() != null) {
+          if (!Files.exists(Paths.get(config.getHttpManagement().getTruststoresDir()))) {
+            throw new RuntimeException(String.format(
+                "HttpManagement truststores directory is configured as %s but it does not exist",
+                config.getHttpManagement().getTruststoresDir()
+            ));
+          }
+        }
+      }
     }
   }
 
-  private void initDatabase(ReaperApplicationConfiguration config) throws ReaperException {
-    Flyway flyway = new Flyway();
-    DataSourceFactory dsfactory = config.getDataSourceFactory();
-    flyway.setDataSource(
-        dsfactory.getUrl(),
-        dsfactory.getUser(),
-        dsfactory.getPassword());
+  private void tryInitializeStorage(ReaperApplicationConfiguration config, Environment environment)
+      throws ReaperException, InterruptedException {
+    if (context.storage == null) {
+      LOG.info("initializing storage of type: {}", config.getStorageType());
+      int storageFailures = 0;
+      while (true) {
+        try {
+          context.storage = InitializeStorage.initializeStorage(
+              config, environment, context.reaperInstanceId).initializeStorageBackend();
 
-    if ("database".equals(config.getStorageType())) {
-      LOG.warn("!!!!!!!!!!    USAGE 'database' AS STORAGE TYPE IS NOW DEPRECATED   !!!!!!!!!!!!!!");
-      LOG.warn("!!!!!!!!!!    PLEASE USE EITHER 'postgres' OR 'h2' FROM NOW ON     !!!!!!!!!!!!!!");
-      if (config.getDataSourceFactory().getUrl().contains("h2")) {
-        flyway.setLocations("/db/h2");
-      } else {
-        flyway.setLocations("/db/postgres");
+          // Allows to execute cleanup queries as shutdown hooks
+          environment.lifecycle().manage(context.storage);
+          break;
+        } catch (RuntimeException e) {
+          LOG.error("Storage is not ready yet, trying again to connect shortly...", e);
+          storageFailures++;
+          if (storageFailures > 60) {
+            throw new ReaperException("Too many failures when trying to connect storage. Exiting :'(");
+          }
+          Thread.sleep(10000);
+        }
       }
     } else {
-      flyway.setLocations("/db/".concat(config.getStorageType().toLowerCase()));
-    }
-    flyway.setBaselineOnMigrate(true);
-    flyway.repair();
-    flyway.migrate();
-  }
-
-  private void initializeJmxSeedsForAllClusters() {
-    LOG.info("Initializing JMX seed list for all clusters...");
-    try (JmxConnectionsInitializer jmxConnectionsIntializer = JmxConnectionsInitializer.create(context);
-        Timer.Context cxt = context
-                .metricRegistry
-                .timer(MetricRegistry.name(JmxConnectionFactory.class, "jmxConnectionsIntializer"))
-                .time()) {
-
-      context
-          .storage
-          .getClusters()
-          .parallelStream()
-          .sorted()
-          .forEach(cluster -> jmxConnectionsIntializer.on(cluster));
-
-      LOG.info("Initialized JMX seed list for all clusters.");
+      LOG.info("storage already given in context, not initializing a new one");
     }
   }
-}
+}
\ No newline at end of file
