@@ -20,6 +20,8 @@ package io.cassandrareaper.resources;
 import io.cassandrareaper.AppContext;
 import io.cassandrareaper.ReaperException;
 import io.cassandrareaper.core.Cluster;
+import io.cassandrareaper.core.EditableRepairSchedule;
+import io.cassandrareaper.core.PercentRepairedMetric;
 import io.cassandrareaper.core.RepairSchedule;
 import io.cassandrareaper.core.RepairUnit;
 import io.cassandrareaper.resources.view.RepairRunStatus;
@@ -27,6 +29,7 @@ import io.cassandrareaper.resources.view.RepairScheduleStatus;
 import io.cassandrareaper.service.RepairRunService;
 import io.cassandrareaper.service.RepairScheduleService;
 import io.cassandrareaper.service.RepairUnitService;
+import io.cassandrareaper.storage.repairrun.IRepairRunDao;
 
 import java.net.URI;
 import java.util.ArrayList;
@@ -35,7 +38,9 @@ import java.util.List;
 import java.util.Optional;
 import java.util.Set;
 import java.util.UUID;
-
+import javax.validation.Valid;
+import javax.validation.constraints.NotNull;
+import javax.ws.rs.Consumes;
 import javax.ws.rs.DELETE;
 import javax.ws.rs.GET;
 import javax.ws.rs.POST;
@@ -50,7 +55,10 @@ import javax.ws.rs.core.Response;
 import javax.ws.rs.core.UriInfo;
 
 import com.google.common.base.Preconditions;
+import com.google.common.collect.ImmutableList;
 import com.google.common.collect.Lists;
+import io.dropwizard.jersey.PATCH;
+import io.dropwizard.jersey.validation.ValidationErrorMessage;
 import org.apache.cassandra.repair.RepairParallelism;
 import org.joda.time.DateTime;
 import org.slf4j.Logger;
@@ -68,11 +76,132 @@ public final class RepairScheduleResource {
   private final RepairScheduleService repairScheduleService;
   private final RepairRunService repairRunService;
 
-  public RepairScheduleResource(AppContext context) {
+  private final IRepairRunDao repairRunDao;
+
+  public RepairScheduleResource(AppContext context, IRepairRunDao repairRunDao) {
     this.context = context;
     this.repairUnitService = RepairUnitService.create(context);
-    this.repairScheduleService = RepairScheduleService.create(context);
-    this.repairRunService = RepairRunService.create(context);
+    this.repairScheduleService = RepairScheduleService.create(context, repairRunDao);
+    this.repairRunService = RepairRunService.create(context, repairRunDao);
+    this.repairRunDao = repairRunDao;
+  }
+
+  private static boolean isPausing(RepairSchedule.State oldState, RepairSchedule.State newState) {
+    return oldState == RepairSchedule.State.ACTIVE && newState == RepairSchedule.State.PAUSED;
+  }
+
+  private static boolean isResuming(RepairSchedule.State oldState, RepairSchedule.State newState) {
+    return oldState == RepairSchedule.State.PAUSED && newState == RepairSchedule.State.ACTIVE;
+  }
+
+  /**
+   * Crafts an URI used to identify given repair schedule.
+   *
+   * @return The created resource URI.
+   */
+  private static URI buildRepairScheduleUri(UriInfo uriInfo, RepairSchedule repairSchedule) {
+    return uriInfo.getBaseUriBuilder().path("repair_schedule").path(repairSchedule.getId().toString()).build();
+  }
+
+  /**
+   * Utility method to apply any valid parameters to an existing RepairSchedule.
+   * This method assumes that any non-null parameter provided is valid and should
+   * be applied.
+   *
+   * @param repairSchedule             - The schedule object to be updated
+   * @param owner                      - The owner value to be used in the update
+   * @param repairParallelism          - the parallelism value to be used in the update
+   * @param intensity                  - The intensity value to be used in the update
+   * @param scheduleDaysBetween        - The days between value to be used in the update
+   * @param segmentCountPerNode        - The segments per node value to be used in the update
+   * @param adaptive                   - Whether or not the schedule is adaptive
+   * @param percentUnrepairedThreshold - Threshold of unrepaired percentage that triggers a repair
+   */
+  protected static RepairSchedule applyRepairPatchParams(
+      final RepairSchedule repairSchedule,
+      final String owner,
+      final RepairParallelism repairParallelism,
+      final Double intensity,
+      final Integer scheduleDaysBetween,
+      final Integer segmentCountPerNode,
+      final Boolean adaptive,
+      final Integer percentUnrepairedThreshold
+  ) {
+    if (repairSchedule == null) {
+      return null;
+    }
+
+    // Apply any valid incoming values to the schedule
+    return repairSchedule.with()
+        .owner(owner != null ? owner.trim() : repairSchedule.getOwner())
+        .repairParallelism(repairParallelism != null ? repairParallelism : repairSchedule.getRepairParallelism())
+        .intensity(intensity != null ? intensity : repairSchedule.getIntensity())
+        .daysBetween(scheduleDaysBetween != null ? scheduleDaysBetween : repairSchedule.getDaysBetween())
+        .segmentCountPerNode(segmentCountPerNode != null
+            ? segmentCountPerNode
+            : repairSchedule.getSegmentCountPerNode())
+        .percentUnrepairedThreshold(percentUnrepairedThreshold)
+        .adaptive(adaptive != null ? adaptive : false)
+        .build(repairSchedule.getId());
+  }
+
+  @PATCH
+  @Produces(MediaType.APPLICATION_JSON)
+  @Consumes(MediaType.APPLICATION_JSON)
+  @Path("/{id}")
+  public Response patchRepairSchedule(
+      @Context UriInfo uriInfo,
+      @PathParam("id") UUID repairScheduleId,
+      @NotNull @Valid EditableRepairSchedule editableRepairSchedule
+  ) {
+    if (repairScheduleId == null) {
+      ValidationErrorMessage errorMessage = new ValidationErrorMessage(
+          ImmutableList.copyOf(
+              Lists.newArrayList("id must not be null or empty")
+          )
+      );
+      return Response.status(400).entity(errorMessage).build();
+    }
+
+    // When executed through DropWizard the validation will prevent this from ever being reached
+    // but to protect against an NPE if the behavior is ever changed, do a quick check of the param
+    if (editableRepairSchedule == null) {
+      ValidationErrorMessage errorMessage = new ValidationErrorMessage(
+          ImmutableList.copyOf(
+              Lists.newArrayList("request body must not be null or empty")
+          )
+      );
+      return Response.status(400).entity(errorMessage).build();
+    }
+
+    // Try to find the schedule to be updated
+    Optional<RepairSchedule> repairScheduleWrapper = context.storage.getRepairScheduleDao()
+        .getRepairSchedule(repairScheduleId);
+    // See if we found the schedule
+    RepairSchedule repairSchedule = repairScheduleWrapper.orElse(null);
+    if (repairSchedule == null) {
+      return Response.status(Response.Status.NOT_FOUND).build();
+    }
+
+    // Apply any valid incoming values to the schedule
+    RepairSchedule patchedRepairSchedule = applyRepairPatchParams(
+        repairSchedule,
+        editableRepairSchedule.getOwner(),
+        editableRepairSchedule.getRepairParallelism(),
+        editableRepairSchedule.getIntensity(),
+        editableRepairSchedule.getDaysBetween(),
+        editableRepairSchedule.getSegmentCountPerNode(),
+        editableRepairSchedule.getAdaptive(),
+        editableRepairSchedule.getPercentUnrepairedThreshold()
+    );
+
+    // Attempt to update the schedule
+    boolean updated = context.storage.getRepairScheduleDao().updateRepairSchedule(patchedRepairSchedule);
+    if (updated) {
+      return Response.status(Response.Status.OK).entity(getRepairScheduleStatus(patchedRepairSchedule)).build();
+    } else {
+      return Response.status(Response.Status.INTERNAL_SERVER_ERROR).build();
+    }
   }
 
   /**
@@ -96,12 +225,17 @@ public final class RepairScheduleResource {
       @QueryParam("repairParallelism") Optional<String> repairParallelism,
       @QueryParam("intensity") Optional<String> intensityStr,
       @QueryParam("incrementalRepair") Optional<String> incrementalRepairStr,
+      @QueryParam("subrangeIncrementalRepair") Optional<String> subrangeIncrementalRepairStr,
       @QueryParam("scheduleDaysBetween") Optional<Integer> scheduleDaysBetween,
       @QueryParam("scheduleTriggerTime") Optional<String> scheduleTriggerTime,
       @QueryParam("nodes") Optional<String> nodesToRepairParam,
       @QueryParam("datacenters") Optional<String> datacentersToRepairParam,
       @QueryParam("blacklistedTables") Optional<String> blacklistedTableNamesParam,
-      @QueryParam("repairThreadCount") Optional<Integer> repairThreadCountParam) {
+      @QueryParam("repairThreadCount") Optional<Integer> repairThreadCountParam,
+      @QueryParam("force") Optional<String> forceParam,
+      @QueryParam("timeout") Optional<Integer> timeoutParam,
+      @QueryParam("adaptive") Optional<String> adaptiveParam,
+      @QueryParam("percentUnrepairedThreshold") Optional<Integer> percentUnrepairedParam) {
 
     try {
       Response possibleFailResponse = RepairRunResource.checkRequestForAddRepair(
@@ -114,10 +248,13 @@ public final class RepairScheduleResource {
           repairParallelism,
           intensityStr,
           incrementalRepairStr,
+          subrangeIncrementalRepairStr,
           nodesToRepairParam,
           datacentersToRepairParam,
           blacklistedTableNamesParam,
-          repairThreadCountParam);
+          repairThreadCountParam,
+          forceParam,
+          timeoutParam);
 
       if (null != possibleFailResponse) {
         return possibleFailResponse;
@@ -129,7 +266,7 @@ public final class RepairScheduleResource {
         if (nextActivation.isBefore(DateTime.now().minusMinutes(15))) {
           return Response.status(Response.Status.BAD_REQUEST)
               .entity("given schedule_trigger_time is too far in the past: "
-                      + RepairRunStatus.dateTimeToIso8601(nextActivation))
+                  + RepairRunStatus.dateTimeToIso8601(nextActivation))
               .build();
         }
       } catch (IllegalArgumentException ex) {
@@ -143,12 +280,8 @@ public final class RepairScheduleResource {
             .build();
       }
 
-      Cluster cluster ;
-      try {
-        cluster = context.storage.getCluster(Cluster.toSymbolicName(clusterName.get()));
-      } catch (IllegalArgumentException ex) {
-        return Response.status(Response.Status.NOT_FOUND).entity(ex.getMessage()).build();
-      }
+      Cluster cluster = context.storage.getClusterDao().getCluster(Cluster.toSymbolicName(clusterName.get()));
+
       Set<String> tableNames;
       try {
         tableNames = repairRunService.getTableNamesBasedOnParam(cluster, keyspace.get(), tableNamesParam);
@@ -174,16 +307,11 @@ public final class RepairScheduleResource {
         return Response.status(Response.Status.NOT_FOUND).entity(ex.getMessage()).build();
       }
 
-      final Set<String> datacentersToRepair;
-      try {
-        datacentersToRepair = RepairRunService
-            .getDatacentersToRepairBasedOnParam(cluster, datacentersToRepairParam);
-      } catch (final IllegalArgumentException ex) {
-        LOG.error(ex.getMessage(), ex);
-        return Response.status(Response.Status.NOT_FOUND).entity(ex.getMessage()).build();
-      }
+      final Set<String> datacentersToRepair = RepairRunService
+          .getDatacentersToRepairBasedOnParam(datacentersToRepairParam);
 
-      boolean incremental = isIncrementalRepair(incrementalRepairStr);
+      boolean incremental
+          = isIncrementalRepair(incrementalRepairStr) || isIncrementalRepair(subrangeIncrementalRepairStr);
       RepairParallelism parallelism = context.config.getRepairParallelism();
       if (repairParallelism.isPresent()) {
         LOG.debug("using given repair parallelism {} over configured value {}", repairParallelism.get(), parallelism);
@@ -192,19 +320,33 @@ public final class RepairScheduleResource {
 
       if (!parallelism.equals(RepairParallelism.PARALLEL) && incremental) {
         return Response.status(Response.Status.BAD_REQUEST)
-            .entity("Can't mix sequential repair and incremental repairs")
+            .entity("Incremental repairs only supports PARALLEL parallelism mode.")
             .build();
       }
 
+      if (percentUnrepairedParam.orElse(-1) > 0 && !incremental) {
+        return Response.status(Response.Status.BAD_REQUEST)
+            .entity("Triggering schedules on % unrepaired threshold is only allowed for incremental repairs.")
+            .build();
+      }
+
+      // explicitly force a schedule even if the schedule conflicts
+      boolean force = (forceParam.isPresent() ? Boolean.parseBoolean(forceParam.get()) : false);
+
+      int timeout = timeoutParam.orElse(context.config.getHangingRepairTimeoutMins());
+      boolean adaptive = (adaptiveParam.isPresent() ? Boolean.parseBoolean(adaptiveParam.get()) : false);
+      boolean subrangeIncremental = isIncrementalRepair(subrangeIncrementalRepairStr) ;
       RepairUnit.Builder unitBuilder = RepairUnit.builder()
           .clusterName(cluster.getName())
           .keyspaceName(keyspace.get())
           .columnFamilies(tableNames)
           .incrementalRepair(incremental)
+          .subrangeIncrementalRepair(subrangeIncremental)
           .nodes(nodesToRepair)
           .datacenters(datacentersToRepair)
           .blacklistedTables(blacklistedTableNames)
-          .repairThreadCount(repairThreadCountParam.orElse(context.config.getRepairThreadCount()));
+          .repairThreadCount(repairThreadCountParam.orElse(context.config.getRepairThreadCount()))
+          .timeout(timeout);
 
       return addRepairSchedule(
           cluster,
@@ -216,7 +358,10 @@ public final class RepairScheduleResource {
           incremental,
           nextActivation,
           getSegmentCount(segmentCountPerNode),
-          getIntensity(intensityStr));
+          getIntensity(intensityStr),
+          force,
+          adaptive,
+          percentUnrepairedParam.orElse(-1));
 
     } catch (ReaperException e) {
       LOG.error(e.getMessage(), e);
@@ -234,10 +379,19 @@ public final class RepairScheduleResource {
       boolean incremental,
       DateTime next,
       int segments,
-      Double intensity) {
+      Double intensity,
+      boolean force,
+      boolean adaptive,
+      int percentUnrepairedThreshold) {
 
     Optional<RepairSchedule> conflictingRepairSchedule
-        = repairScheduleService.conflictingRepairSchedule(cluster, unitBuilder);
+        = repairScheduleService.identicalRepairUnit(cluster, unitBuilder);
+
+    if (conflictingRepairSchedule.isPresent()) {
+      return Response.noContent().location(buildRepairScheduleUri(uriInfo, conflictingRepairSchedule.get())).build();
+    }
+
+    conflictingRepairSchedule = repairScheduleService.conflictingRepairSchedule(cluster, unitBuilder);
 
     if (conflictingRepairSchedule.isPresent()) {
       RepairSchedule existingSchedule = conflictingRepairSchedule.get();
@@ -249,29 +403,45 @@ public final class RepairScheduleResource {
         return Response.noContent().location(buildRepairScheduleUri(uriInfo, existingSchedule)).build();
       }
 
-      String msg = String.format(
-          "A repair schedule already exists for cluster \"%s\", keyspace \"%s\", and column families: %s",
-          cluster.getName(),
-          unitBuilder.keyspaceName,
-          unitBuilder.columnFamilies);
+      if (!force) {
+        String msg = String.format(
+            "A repair schedule already exists for cluster \"%s\", keyspace \"%s\", and column families: %s",
+            cluster.getName(),
+            unitBuilder.keyspaceName,
+            unitBuilder.columnFamilies);
 
-      return Response
-          .status(Response.Status.CONFLICT)
-          .location(buildRepairScheduleUri(uriInfo, existingSchedule))
-          .entity(msg)
-          .build();
-    } else {
+        return Response
+            .status(Response.Status.CONFLICT)
+            .location(buildRepairScheduleUri(uriInfo, existingSchedule))
+            .entity(msg)
+            .build();
+      }
+    }
 
-      RepairUnit unit = repairUnitService.getOrCreateRepairUnit(cluster, unitBuilder);
+    Optional<RepairUnit> maybeUnit = repairUnitService.getOrCreateRepairUnit(cluster, unitBuilder, force);
 
+    if (maybeUnit.isPresent()) {
+      RepairUnit unit = maybeUnit.get();
       Preconditions
-          .checkState(unit.getIncrementalRepair() == incremental, "%s!=%s", unit.getIncrementalRepair(), incremental);
+          .checkState(unit.getIncrementalRepair() == incremental
+              || unit.getSubrangeIncrementalRepair() == incremental,
+              "%s!=%s", unit.getIncrementalRepair(), incremental);
+      Preconditions
+          .checkState((percentUnrepairedThreshold > 0 && incremental) || percentUnrepairedThreshold <= 0,
+              "Setting a % repaired threshold can only be done on incremental schedules");
 
       RepairSchedule newRepairSchedule = repairScheduleService
-          .storeNewRepairSchedule(cluster, unit, days, next, owner, segments, parallel, intensity);
+          .storeNewRepairSchedule(
+              cluster, unit, days, next, owner, segments,
+              parallel, intensity, force, adaptive, percentUnrepairedThreshold);
 
       return Response.created(buildRepairScheduleUri(uriInfo, newRepairSchedule)).build();
     }
+
+    return Response
+        .status(Response.Status.NO_CONTENT)
+        .entity("Repair schedule couldn't be created as an existing repair unit seems to conflict with it.")
+        .build();
   }
 
   private int getDaysBetween(Optional<Integer> scheduleDaysBetween) {
@@ -338,14 +508,15 @@ public final class RepairScheduleResource {
       return Response.status(Response.Status.BAD_REQUEST.getStatusCode()).entity("\"state\" argument missing").build();
     }
 
-    Optional<RepairSchedule> repairSchedule = context.storage.getRepairSchedule(repairScheduleId);
+    Optional<RepairSchedule> repairSchedule = context.storage.getRepairScheduleDao()
+        .getRepairSchedule(repairScheduleId);
     if (!repairSchedule.isPresent()) {
       return Response.status(Response.Status.NOT_FOUND)
           .entity("repair schedule with id " + repairScheduleId + " not found")
           .build();
     }
 
-    RepairUnit repairUnit = context.storage.getRepairUnit(repairSchedule.get().getRepairUnitId());
+    RepairUnit repairUnit = context.storage.getRepairUnitDao().getRepairUnit(repairSchedule.get().getRepairUnitId());
     RepairSchedule.State newState;
     try {
       newState = RepairSchedule.State.valueOf(state.get().toUpperCase());
@@ -371,14 +542,6 @@ public final class RepairScheduleResource {
     }
   }
 
-  private static boolean isPausing(RepairSchedule.State oldState, RepairSchedule.State newState) {
-    return oldState == RepairSchedule.State.ACTIVE && newState == RepairSchedule.State.PAUSED;
-  }
-
-  private static boolean isResuming(RepairSchedule.State oldState, RepairSchedule.State newState) {
-    return oldState == RepairSchedule.State.PAUSED && newState == RepairSchedule.State.ACTIVE;
-  }
-
   private Response pauseSchedule(RepairSchedule repairSchedule, UriInfo uriInfo) {
     LOG.info("Pausing schedule {}", repairSchedule.getId());
     context.schedulingManager.pauseRepairSchedule(repairSchedule);
@@ -399,7 +562,8 @@ public final class RepairScheduleResource {
   public Response getRepairSchedule(
       @PathParam("id") UUID repairScheduleId) {
     LOG.debug("get repair_schedule called with: id = {}", repairScheduleId);
-    Optional<RepairSchedule> repairSchedule = context.storage.getRepairSchedule(repairScheduleId);
+    Optional<RepairSchedule> repairSchedule = context.storage.getRepairScheduleDao()
+        .getRepairSchedule(repairScheduleId);
     if (repairSchedule.isPresent()) {
       return Response.ok().entity(getRepairScheduleStatus(repairSchedule.get())).build();
     } else {
@@ -416,14 +580,15 @@ public final class RepairScheduleResource {
   @Path("/start/{id}")
   public Response startRepairSchedule(@PathParam("id") UUID repairScheduleId) {
     LOG.debug("start repair_schedule called with: id = {}", repairScheduleId);
-    Optional<RepairSchedule> repairSchedule = context.storage.getRepairSchedule(repairScheduleId);
+    Optional<RepairSchedule> repairSchedule = context.storage.getRepairScheduleDao()
+        .getRepairSchedule(repairScheduleId);
     if (repairSchedule.isPresent()) {
       RepairSchedule newSchedule = repairSchedule.get()
           .with()
           .nextActivation(DateTime.now())
           .build(repairScheduleId);
 
-      context.storage.updateRepairSchedule(newSchedule);
+      context.storage.getRepairScheduleDao().updateRepairSchedule(newSchedule);
       return Response.ok().entity(getRepairScheduleStatus(newSchedule)).build();
     } else {
       return Response.status(404)
@@ -441,7 +606,8 @@ public final class RepairScheduleResource {
   public Response getRepairSchedulesForCluster(
       @PathParam("cluster_name") String clusterName) {
     LOG.debug("get repair schedules for cluster called with: cluster_name = {}", clusterName);
-    Collection<RepairSchedule> repairSchedules = context.storage.getRepairSchedulesForCluster(clusterName);
+    Collection<RepairSchedule> repairSchedules = context.storage.getRepairScheduleDao()
+        .getRepairSchedulesForCluster(clusterName);
     Collection<RepairScheduleStatus> repairScheduleViews = new ArrayList<>();
     for (RepairSchedule repairSchedule : repairSchedules) {
       repairScheduleViews.add(getRepairScheduleStatus(repairSchedule));
@@ -453,24 +619,15 @@ public final class RepairScheduleResource {
    * @return RepairSchedule status for viewing
    */
   private RepairScheduleStatus getRepairScheduleStatus(RepairSchedule repairSchedule) {
-    RepairUnit repairUnit = context.storage.getRepairUnit(repairSchedule.getRepairUnitId());
+    RepairUnit repairUnit = context.storage.getRepairUnitDao().getRepairUnit(repairSchedule.getRepairUnitId());
     return new RepairScheduleStatus(repairSchedule, repairUnit);
   }
 
   /**
-   * Crafts an URI used to identify given repair schedule.
-   *
-   * @return The created resource URI.
-   */
-  private static URI buildRepairScheduleUri(UriInfo uriInfo, RepairSchedule repairSchedule) {
-    return uriInfo.getBaseUriBuilder().path("repair_schedule").path(repairSchedule.getId().toString()).build();
-  }
-
-  /**
-   * @param clusterName The cluster name to list the schedules for. If not given, will list all schedules for all
-   *        clusters.
+   * @param clusterName  The cluster name to list the schedules for. If not given, will list all schedules for all
+   *                     clusters.
    * @param keyspaceName The keyspace name to list schedules for. Limits the returned list and works whether the cluster
-   *        name is given or not.
+   *                     name is given or not.
    * @return All schedules in the system.
    */
   @GET
@@ -480,7 +637,7 @@ public final class RepairScheduleResource {
 
     List<RepairScheduleStatus> scheduleStatuses = Lists.newArrayList();
     getScheduleList(clusterName, keyspaceName).forEach((schedule) -> {
-      RepairUnit unit = context.storage.getRepairUnit(schedule.getRepairUnitId());
+      RepairUnit unit = context.storage.getRepairUnitDao().getRepairUnit(schedule.getRepairUnitId());
       scheduleStatuses.add(new RepairScheduleStatus(schedule, unit));
     });
     return Response.ok().entity(scheduleStatuses).build();
@@ -489,13 +646,14 @@ public final class RepairScheduleResource {
   private Collection<RepairSchedule> getScheduleList(Optional<String> clusterName, Optional<String> keyspaceName) {
     Collection<RepairSchedule> schedules;
     if (clusterName.isPresent() && keyspaceName.isPresent()) {
-      schedules = context.storage.getRepairSchedulesForClusterAndKeyspace(clusterName.get(), keyspaceName.get());
+      schedules = context.storage.getRepairScheduleDao()
+          .getRepairSchedulesForClusterAndKeyspace(clusterName.get(), keyspaceName.get());
     } else if (clusterName.isPresent()) {
-      schedules = context.storage.getRepairSchedulesForCluster(clusterName.get());
+      schedules = context.storage.getRepairScheduleDao().getRepairSchedulesForCluster(clusterName.get());
     } else if (keyspaceName.isPresent()) {
-      schedules = context.storage.getRepairSchedulesForKeyspace(keyspaceName.get());
+      schedules = context.storage.getRepairScheduleDao().getRepairSchedulesForKeyspace(keyspaceName.get());
     } else {
-      schedules = context.storage.getAllRepairSchedules();
+      schedules = context.storage.getRepairScheduleDao().getAllRepairSchedules();
     }
     return schedules;
   }
@@ -507,7 +665,7 @@ public final class RepairScheduleResource {
    * Repair schedule can only be deleted when it is not active, so you must stop it first.
    *
    * @param repairScheduleId The id for the RepairSchedule instance to delete.
-   * @param owner The assigned owner of the deleted resource. Must match the stored one.
+   * @param owner            The assigned owner of the deleted resource. Must match the stored one.
    * @return 202 response code if the delete has been accepted, 409 if schedule can't be stopped.
    */
   @DELETE
@@ -522,7 +680,8 @@ public final class RepairScheduleResource {
           .entity("required query parameter \"owner\" is missing")
           .build();
     }
-    Optional<RepairSchedule> scheduleToDelete = context.storage.getRepairSchedule(repairScheduleId);
+    Optional<RepairSchedule> scheduleToDelete = context.storage.getRepairScheduleDao()
+        .getRepairSchedule(repairScheduleId);
     if (scheduleToDelete.isPresent()) {
       if (RepairSchedule.State.ACTIVE == scheduleToDelete.get().getState()) {
         String msg = String.format("Repair schedule %s currently running. Must be first stopped", repairScheduleId);
@@ -532,7 +691,7 @@ public final class RepairScheduleResource {
         String msg = String.format("Repair schedule %s is not owned by %s", repairScheduleId, owner.get());
         return Response.status(Response.Status.CONFLICT).entity(msg).build();
       }
-      context.storage.deleteRepairSchedule(repairScheduleId);
+      repairScheduleService.deleteRepairSchedule(repairScheduleId);
       return Response.accepted().build();
     }
     return Response.status(Response.Status.NOT_FOUND)
@@ -554,4 +713,13 @@ public final class RepairScheduleResource {
     }
     return nextActivation;
   }
-}
+
+  @GET
+  @Path("/{clusterName}/{id}/percent_repaired")
+  public List<PercentRepairedMetric> getPercentRepairedMetricsForSchedule(
+      @PathParam("clusterName") String clusterName,
+      @PathParam("id") UUID repairScheduleId) throws IllegalArgumentException {
+    long since = DateTime.now().minusHours(1).getMillis();
+    return context.storage.getPercentRepairedMetrics(clusterName, repairScheduleId, since);
+  }
+}
\ No newline at end of file
