@@ -22,15 +22,17 @@ import io.cassandrareaper.ReaperException;
 import io.cassandrareaper.core.RepairRun;
 import io.cassandrareaper.core.RepairSegment;
 import io.cassandrareaper.core.RepairUnit;
-import io.cassandrareaper.jmx.ClusterFacade;
-import io.cassandrareaper.jmx.JmxProxy;
+import io.cassandrareaper.management.ClusterFacade;
 import io.cassandrareaper.storage.IDistributedStorage;
+import io.cassandrareaper.storage.repairrun.IRepairRunDao;
 
 import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.List;
 import java.util.Map;
+import java.util.Optional;
+import java.util.Set;
 import java.util.UUID;
 import java.util.concurrent.ScheduledExecutorService;
 import java.util.concurrent.TimeUnit;
@@ -61,28 +63,30 @@ public final class RepairManager implements AutoCloseable {
 
   private final AppContext context;
   private final ClusterFacade clusterFacade;
-  private final Heart heart;
   private final ListeningScheduledExecutorService executor;
-  private final long repairTimeoutMillis;
   private final long retryDelayMillis;
+  private final int maxParallelRepairs;
+
+  private final IRepairRunDao repairRunDao;
 
   private RepairManager(
       AppContext context,
       ClusterFacade clusterFacade,
       ScheduledExecutorService executor,
-      long repairTimeout,
-      TimeUnit repairTimeoutTimeUnit,
       long retryDelay,
-      TimeUnit retryDelayTimeUnit) throws ReaperException {
+      TimeUnit retryDelayTimeUnit,
+      int maxParallelRepairs,
+      IRepairRunDao repairRunDao
+  ) throws ReaperException {
 
     this.context = context;
     this.clusterFacade = clusterFacade;
-    this.heart = Heart.create(context);
-    this.repairTimeoutMillis = repairTimeoutTimeUnit.toMillis(repairTimeout);
     this.retryDelayMillis = retryDelayTimeUnit.toMillis(retryDelay);
 
     this.executor = MoreExecutors.listeningDecorator(
         new InstrumentedScheduledExecutorService(executor, context.metricRegistry));
+    this.maxParallelRepairs = maxParallelRepairs;
+    this.repairRunDao = repairRunDao;
   }
 
   @VisibleForTesting
@@ -90,41 +94,78 @@ public final class RepairManager implements AutoCloseable {
       AppContext context,
       ClusterFacade clusterFacadeSupplier,
       ScheduledExecutorService executor,
-      long repairTimeout,
-      TimeUnit repairTimeoutTimeUnit,
       long retryDelay,
-      TimeUnit retryDelayTimeUnit) throws ReaperException {
+      TimeUnit retryDelayTimeUnit,
+      int maxParallelRepairs,
+      IRepairRunDao repairRunDao
+  ) throws ReaperException {
 
     return new RepairManager(
         context,
         clusterFacadeSupplier,
         executor,
-        repairTimeout,
-        repairTimeoutTimeUnit,
         retryDelay,
-        retryDelayTimeUnit);
+        retryDelayTimeUnit,
+        maxParallelRepairs,
+        repairRunDao);
   }
 
   public static RepairManager create(
       AppContext context,
       ScheduledExecutorService executor,
-      long repairTimeout,
-      TimeUnit repairTimeoutTimeUnit,
       long retryDelay,
-      TimeUnit retryDelayTimeUnit) throws ReaperException {
+      TimeUnit retryDelayTimeUnit,
+      int maxParallelRepairs,
+      IRepairRunDao repairRunDao
+  ) throws ReaperException {
 
     return create(
         context,
         ClusterFacade.create(context),
         executor,
-        repairTimeout,
-        repairTimeoutTimeUnit,
         retryDelay,
-        retryDelayTimeUnit);
+        retryDelayTimeUnit,
+        maxParallelRepairs,
+        repairRunDao);
   }
 
-  long getRepairTimeoutMillis() {
-    return repairTimeoutMillis;
+  private static boolean takeLead(AppContext context, UUID leaderElectionId) {
+    try (Timer.Context cx
+             = context.metricRegistry.timer(MetricRegistry.name(RepairManager.class, "takeLead")).time()) {
+
+      boolean result = context.storage instanceof IDistributedStorage
+          ? ((IDistributedStorage) context.storage).takeLead(leaderElectionId)
+          : true;
+
+      if (!result) {
+        context.metricRegistry.counter(MetricRegistry.name(RepairManager.class, "takeLead", "failed")).inc();
+      }
+      return result;
+    }
+  }
+
+  private static boolean renewLead(AppContext context, UUID leaderElectionId) {
+    try (Timer.Context cx
+             = context.metricRegistry.timer(MetricRegistry.name(RepairManager.class, "renewLead")).time()) {
+
+      boolean result = context.storage instanceof IDistributedStorage
+          ? ((IDistributedStorage) context.storage).renewLead(leaderElectionId)
+          : true;
+
+      if (!result) {
+        context.metricRegistry.counter(MetricRegistry.name(RepairManager.class, "renewLead", "failed")).inc();
+      }
+      return result;
+    }
+  }
+
+  private static void releaseLead(AppContext context, UUID leaderElectionId) {
+    try (Timer.Context cx
+             = context.metricRegistry.timer(MetricRegistry.name(RepairManager.class, "releaseLead")).time()) {
+      if (context.storage instanceof IDistributedStorage) {
+        ((IDistributedStorage) context.storage).releaseLead(leaderElectionId);
+      }
+    }
   }
 
   /**
@@ -132,45 +173,41 @@ public final class RepairManager implements AutoCloseable {
    */
   public void resumeRunningRepairRuns() throws ReaperException {
     try {
-      heart.beat();
-      Collection<RepairRun> runningRepairRuns = context.storage.getRepairRunsWithState(RepairRun.RunState.RUNNING);
-      Collection<RepairRun> pausedRepairRuns = context.storage.getRepairRunsWithState(RepairRun.RunState.PAUSED);
+      Collection<RepairRun> runningRepairRuns = repairRunDao.getRepairRunsWithState(RepairRun.RunState.RUNNING);
+      Collection<RepairRun> pausedRepairRuns = repairRunDao.getRepairRunsWithState(RepairRun.RunState.PAUSED);
       abortAllRunningSegmentsWithNoLeader(runningRepairRuns);
       abortAllRunningSegmentsInKnownPausedRepairRuns(pausedRepairRuns);
-      resumeUnkownRunningRepairRuns(runningRepairRuns);
+      resumeUnknownRunningRepairRuns(runningRepairRuns);
       resumeUnknownPausedRepairRuns(pausedRepairRuns);
     } catch (RuntimeException e) {
       throw new ReaperException(e);
     }
   }
 
-  public void handleMetricsRequests() throws ReaperException {
-    try {
-      heart.beatMetrics();
-    } catch (RuntimeException e) {
-      throw new ReaperException(e);
-    }
-  }
-
   private void abortAllRunningSegmentsWithNoLeader(Collection<RepairRun> runningRepairRuns) {
     runningRepairRuns
         .forEach((repairRun) -> {
-          Collection<RepairSegment> runningSegments
-              = context.storage.getSegmentsWithState(repairRun.getId(), RepairSegment.State.RUNNING);
+          Collection<RepairSegment> runningSegments = context.storage.getRepairSegmentDao().getSegmentsWithState(
+              repairRun.getId(),
+              RepairSegment.State.RUNNING);
+          Collection<RepairSegment> startedSegments = context.storage.getRepairSegmentDao().getSegmentsWithState(
+              repairRun.getId(),
+              RepairSegment.State.STARTED);
 
           abortSegmentsWithNoLeader(repairRun, runningSegments);
+          abortSegmentsWithNoLeader(repairRun, startedSegments);
         });
   }
 
-  private void resumeUnkownRunningRepairRuns(Collection<RepairRun> runningRepairRuns) throws ReaperException {
+  private void resumeUnknownRunningRepairRuns(Collection<RepairRun> runningRepairRuns) throws ReaperException {
     try {
       repairRunnersLock.lock();
       for (RepairRun repairRun : runningRepairRuns) {
         if (!repairRunners.containsKey(repairRun.getId())) {
           LOG.info("Restarting run id {} that has no runner", repairRun.getId());
           // it may be that this repair is already "running" actively on other reaper instances
-          //  nonetheless we need to make it actively running on this reaper instance as well
-          //   so to help in running the queued segments
+          // nonetheless we need to make it actively running on this reaper instance as well
+          // so to help in running the queued segments
           startRepairRun(repairRun);
         }
       }
@@ -187,11 +224,16 @@ public final class RepairManager implements AutoCloseable {
           .stream()
           .filter((pausedRepairRun) -> repairRunners.containsKey(pausedRepairRun.getId()))
           .forEach((pausedRepairRun) -> {
-            // Abort all running segments for paused repair runs
-            Collection<RepairSegment> runningSegments
-                = context.storage.getSegmentsWithState(pausedRepairRun.getId(), RepairSegment.State.RUNNING);
+            // Abort all running and started segments for paused repair runs
+            Collection<RepairSegment> runningSegments = context.storage.getRepairSegmentDao().getSegmentsWithState(
+                pausedRepairRun.getId(),
+                RepairSegment.State.RUNNING);
+            Collection<RepairSegment> startedSegments = context.storage.getRepairSegmentDao().getSegmentsWithState(
+                pausedRepairRun.getId(),
+                RepairSegment.State.STARTED);
 
             abortSegments(runningSegments, pausedRepairRun);
+            abortSegments(startedSegments, pausedRepairRun);
           });
     } finally {
       repairRunnersLock.unlock();
@@ -205,14 +247,27 @@ public final class RepairManager implements AutoCloseable {
       pausedRepairRuns
           .stream()
           .filter((pausedRepairRun) -> (!repairRunners.containsKey(pausedRepairRun.getId())))
-          // add "paused" repair run to this reaper instance, so it can be visualised in UI
-          .forEachOrdered((pausedRepairRun) -> startRunner(pausedRepairRun.getId()));
+          // add "paused" repair run to this reaper instance, so it can be visualized in UI
+          .forEachOrdered((pausedRepairRun) -> startRunner(pausedRepairRun));
     } finally {
       repairRunnersLock.unlock();
     }
   }
 
   private void abortSegmentsWithNoLeader(RepairRun repairRun, Collection<RepairSegment> runningSegments) {
+    RepairUnit repairUnit = context.storage.getRepairUnitDao().getRepairUnit(repairRun.getRepairUnitId());
+    if (repairUnitIsNonSubrangeIncremental(repairUnit)) {
+      abortSegmentsWithNoLeaderIncremental(repairRun, runningSegments);
+    } else {
+      abortSegmentsWithNoLeaderNonIncremental(repairRun, runningSegments);
+    }
+  }
+
+  private boolean repairUnitIsNonSubrangeIncremental(RepairUnit repairUnit) {
+    return repairUnit.getIncrementalRepair() && !repairUnit.getSubrangeIncrementalRepair();
+  }
+
+  private void abortSegmentsWithNoLeaderIncremental(RepairRun repairRun, Collection<RepairSegment> runningSegments) {
 
     if (LOG.isDebugEnabled()) {
       LOG.debug(
@@ -223,10 +278,10 @@ public final class RepairManager implements AutoCloseable {
       repairRunnersLock.lock();
       if (context.storage instanceof IDistributedStorage || !repairRunners.containsKey(repairRun.getId())) {
         // When multiple Reapers are in use, we can get stuck segments when one instance is rebooted
-        // Any segment in RUNNING state but with no leader should be killed
+        // Any segment in RUNNING or STARTED state but with no leader should be killed
         List<UUID> leaders = context.storage instanceof IDistributedStorage
-                ? ((IDistributedStorage) context.storage).getLeaders()
-                : Collections.emptyList();
+            ? ((IDistributedStorage) context.storage).getLeaders()
+            : Collections.emptyList();
 
         Collection<RepairSegment> orphanedSegments = runningSegments
             .stream()
@@ -241,12 +296,44 @@ public final class RepairManager implements AutoCloseable {
     }
   }
 
+  private void abortSegmentsWithNoLeaderNonIncremental(RepairRun repairRun, Collection<RepairSegment> runningSegments) {
+
+    if (LOG.isDebugEnabled()) {
+      LOG.debug(
+          "Checking leadership on the following segments : {}",
+          runningSegments.stream().map(seg -> seg.getId()).collect(Collectors.toList()));
+    }
+    try {
+      repairRunnersLock.lock();
+      if (context.storage instanceof IDistributedStorage || !repairRunners.containsKey(repairRun.getId())) {
+        // When multiple Reapers are in use, we can get stuck segments when one instance is rebooted
+        // Any segment in RUNNING or STARTED state but with no leader should be killed
+        Set<UUID> leaders = context.storage instanceof IDistributedStorage
+            ? ((IDistributedStorage) context.storage).getLockedSegmentsForRun(repairRun.getId())
+            : Collections.emptySet();
+
+        Collection<RepairSegment> orphanedSegments = runningSegments
+            .stream()
+            .filter(segment -> !leaders.contains(segment.getId()))
+            .collect(Collectors.toSet());
+
+        LOG.debug("No leader on the following segments : {}", orphanedSegments);
+        abortSegments(orphanedSegments, repairRun);
+      }
+    } finally {
+      repairRunnersLock.unlock();
+    }
+  }
+
   public RepairSegment abortSegment(UUID runId, UUID segmentId) throws ReaperException {
-    RepairSegment segment = context.storage.getRepairSegment(runId, segmentId).get();
+    RepairSegment segment = context.storage.getRepairSegmentDao().getRepairSegment(runId, segmentId).get();
     try {
       if (null == segment.getCoordinatorHost() || RepairSegment.State.DONE == segment.getState()) {
-        RepairUnit repairUnit = context.storage.getRepairUnit(segment.getRepairUnitId());
-        UUID leaderElectionId = repairUnit.getIncrementalRepair() ? runId : segmentId;
+        RepairUnit repairUnit = context.storage.getRepairUnitDao().getRepairUnit(segment.getRepairUnitId());
+        // Incremental non subrange repairs will use the run id as leader election id
+        // to prevent multiple segments to run at once. Subrange incremental will allow multiple segments.
+        UUID leaderElectionId = repairUnit.getIncrementalRepair() && !repairUnit.getSubrangeIncrementalRepair()
+            ? runId : segmentId;
         boolean tookLead;
         if (tookLead = takeLead(context, leaderElectionId) || renewLead(context, leaderElectionId)) {
           try {
@@ -258,43 +345,19 @@ public final class RepairManager implements AutoCloseable {
           }
         }
       } else {
-        abortSegments(Arrays.asList(segment), context.storage.getRepairRun(runId).get());
+        abortSegments(Arrays.asList(segment), repairRunDao.getRepairRun(runId).get());
       }
-      return context.storage.getRepairSegment(runId, segmentId).get();
+      return context.storage.getRepairSegmentDao().getRepairSegment(runId, segmentId).get();
     } catch (AssertionError error) {
       throw new ReaperException("lead is already taken on " + runId + ":" + segmentId, new Exception(error));
     }
   }
 
   void abortSegments(Collection<RepairSegment> runningSegments, RepairRun repairRun) {
-    RepairUnit repairUnit = context.storage.getRepairUnit(repairRun.getRepairUnitId());
+    RepairUnit repairUnit = context.storage.getRepairUnitDao().getRepairUnit(repairRun.getRepairUnitId());
     for (RepairSegment segment : runningSegments) {
       LOG.debug("Trying to abort stuck segment {} in repair run {}", segment.getId(), repairRun.getId());
-      UUID leaderElectionId = repairUnit.getIncrementalRepair() ? repairRun.getId() : segment.getId();
-      boolean tookLead;
-      if (tookLead = takeLead(context, leaderElectionId) || renewLead(context, leaderElectionId)) {
-        try {
-          // refresh segment once we're inside leader-election
-          segment = context.storage.getRepairSegment(repairRun.getId(), segment.getId()).get();
-          if (RepairSegment.State.RUNNING == segment.getState()) {
-            JmxProxy jmxProxy = ClusterFacade.create(context).connect(
-                      context.storage.getCluster(repairRun.getClusterName()),
-                      Arrays.asList(segment.getCoordinatorHost()));
-
-            SegmentRunner.abort(context, segment, jmxProxy);
-          }
-        } catch (ReaperException | NumberFormatException e) {
-          String msg = "Tried to abort repair on segment {} marked as RUNNING, but the "
-              + "host was down (so abortion won't be needed). Postponing the segment.";
-
-          LOG.debug(msg, segment.getId(), e);
-          SegmentRunner.postponeSegment(context, segment);
-        } finally {
-          if (tookLead) {
-            releaseLead(context, leaderElectionId);
-          }
-        }
-      }
+      SegmentRunner.postponeSegment(context, segment);
     }
   }
 
@@ -309,10 +372,10 @@ public final class RepairManager implements AutoCloseable {
             .runState(RepairRun.RunState.RUNNING)
             .startTime(DateTime.now())
             .build(runToBeStarted.getId());
-        if (!context.storage.updateRepairRun(updatedRun)) {
+        if (!repairRunDao.updateRepairRun(updatedRun)) {
           throw new ReaperException("failed updating repair run " + updatedRun.getId());
         }
-        startRunner(runId);
+        startRunner(updatedRun);
         return updatedRun;
       }
       case PAUSED: {
@@ -321,22 +384,22 @@ public final class RepairManager implements AutoCloseable {
             .pauseTime(null)
             .build(runToBeStarted.getId());
 
-        if (!context.storage.updateRepairRun(updatedRun)) {
+        if (!repairRunDao.updateRepairRun(updatedRun)) {
           throw new ReaperException("failed updating repair run " + updatedRun.getId());
         }
         return updatedRun;
       }
       case RUNNING:
         LOG.info("re-trigger a running run after restart, with id {}", runId);
-        startRunner(runId);
+        startRunner(runToBeStarted);
         return runToBeStarted;
       case ERROR: {
-        RepairRun updatedRun
-            = runToBeStarted.with().runState(RepairRun.RunState.RUNNING).endTime(null).build(runToBeStarted.getId());
-        if (!context.storage.updateRepairRun(updatedRun)) {
+        RepairRun updatedRun = runToBeStarted.with().runState(RepairRun.RunState.RUNNING).endTime(null).build(
+            runToBeStarted.getId());
+        if (!repairRunDao.updateRepairRun(updatedRun)) {
           throw new ReaperException("failed updating repair run " + updatedRun.getId());
         }
-        startRunner(runId);
+        startRunner(updatedRun);
         return updatedRun;
       }
       default:
@@ -346,40 +409,52 @@ public final class RepairManager implements AutoCloseable {
 
   public RepairRun updateRepairRunIntensity(RepairRun repairRun, Double intensity) throws ReaperException {
     RepairRun updatedRun = repairRun.with().intensity(intensity).build(repairRun.getId());
-    if (!context.storage.updateRepairRun(updatedRun)) {
+    if (!repairRunDao.updateRepairRun(updatedRun, Optional.of(false))) {
       throw new ReaperException("failed updating repair run " + updatedRun.getId());
     }
     return updatedRun;
   }
 
-  private void startRunner(UUID runId) {
+  @SuppressWarnings("CheckReturnValue")
+  private void startRunner(RepairRun run) {
     try {
       repairRunnersLock.lock();
 
       Preconditions.checkState(
-          !repairRunners.containsKey(runId),
-          "there is already a repair runner for run with id " + runId + ". This should not happen.");
+          !repairRunners.containsKey(run.getId()),
+          "there is already a repair runner for run with id " + run.getId() + ". This should not happen.");
 
-      LOG.info("scheduling repair for repair run #{}", runId);
+      LOG.debug("scheduling repair for repair run #{}", run.getId());
       try {
-        RepairRunner newRunner = RepairRunner.create(context, runId, clusterFacade);
-        repairRunners.put(runId, newRunner);
+        RepairRunner newRunner = RepairRunner.create(context, run.getId(), clusterFacade, repairRunDao);
+        repairRunners.put(run.getId(), newRunner);
         executor.submit(newRunner);
       } catch (ReaperException e) {
-        LOG.warn("Failed to schedule repair for repair run #" + runId, e);
+        LOG.warn("Failed to schedule repair for repair run #" + run.getId(), e);
       }
     } finally {
       repairRunnersLock.unlock();
     }
   }
 
+  @VisibleForTesting
+  public int countRepairRunnersForCluster(String clusterName) {
+    Long repairRunnersForCluster
+        = repairRunners.entrySet()
+        .stream()
+        .map(entry -> entry.getValue())
+        .filter(runner -> runner.getCluster().getName().equals(clusterName))
+        .count();
+    return repairRunnersForCluster.intValue();
+  }
+
   public RepairRun pauseRepairRun(RepairRun runToBePaused) throws ReaperException {
     RepairRun updatedRun = runToBePaused.with()
         .runState(RepairRun.RunState.PAUSED)
         .pauseTime(DateTime.now())
         .build(runToBePaused.getId());
 
-    if (!context.storage.updateRepairRun(updatedRun)) {
+    if (!repairRunDao.updateRepairRun(updatedRun)) {
       throw new ReaperException("failed updating repair run " + updatedRun.getId());
     }
     return updatedRun;
@@ -392,12 +467,17 @@ public final class RepairManager implements AutoCloseable {
         .endTime(DateTime.now())
         .build(runToBeAborted.getId());
 
-    if (!context.storage.updateRepairRun(updatedRun)) {
+    if (!repairRunDao.updateRepairRun(updatedRun)) {
       throw new ReaperException("failed updating repair run " + updatedRun.getId());
     }
+
+    context.metricRegistry.counter(
+        MetricRegistry.name(RepairManager.class, "repairDone", RepairRun.RunState.ABORTED.toString())).inc();
+
     return updatedRun;
   }
 
+  @SuppressWarnings("CheckReturnValue")
   void scheduleRetry(RepairRunner runner) {
     executor.schedule(runner, retryDelayMillis, TimeUnit.MILLISECONDS);
   }
@@ -415,48 +495,8 @@ public final class RepairManager implements AutoCloseable {
     }
   }
 
-  private static boolean takeLead(AppContext context, UUID leaderElectionId) {
-    try (Timer.Context cx
-        = context.metricRegistry.timer(MetricRegistry.name(RepairManager.class, "takeLead")).time()) {
-
-      boolean result = context.storage instanceof IDistributedStorage
-          ? ((IDistributedStorage) context.storage).takeLead(leaderElectionId)
-          : true;
-
-      if (!result) {
-        context.metricRegistry.counter(MetricRegistry.name(RepairManager.class, "takeLead", "failed")).inc();
-      }
-      return result;
-    }
-  }
-
-  private static boolean renewLead(AppContext context, UUID leaderElectionId) {
-    try (Timer.Context cx
-        = context.metricRegistry.timer(MetricRegistry.name(RepairManager.class, "renewLead")).time()) {
-
-      boolean result = context.storage instanceof IDistributedStorage
-          ? ((IDistributedStorage) context.storage).renewLead(leaderElectionId)
-          : true;
-
-      if (!result) {
-        context.metricRegistry.counter(MetricRegistry.name(RepairManager.class, "renewLead", "failed")).inc();
-      }
-      return result;
-    }
-  }
-
-  private static void releaseLead(AppContext context, UUID leaderElectionId) {
-    try (Timer.Context cx
-        = context.metricRegistry.timer(MetricRegistry.name(RepairManager.class, "releaseLead")).time()) {
-      if (context.storage instanceof IDistributedStorage) {
-        ((IDistributedStorage) context.storage).releaseLead(leaderElectionId);
-      }
-    }
-  }
-
   @Override
   public void close() {
-    heart.close();
     executor.shutdownNow();
   }
-}
+}
\ No newline at end of file
