@@ -21,10 +21,14 @@ import io.cassandrareaper.AppContext;
 import io.cassandrareaper.ReaperException;
 import io.cassandrareaper.core.Cluster;
 import io.cassandrareaper.core.Table;
-import io.cassandrareaper.jmx.JmxConnectionFactory;
-import io.cassandrareaper.jmx.JmxProxy;
+import io.cassandrareaper.crypto.Cryptograph;
+import io.cassandrareaper.crypto.NoopCrypotograph;
+import io.cassandrareaper.management.ClusterFacade;
+import io.cassandrareaper.management.ICassandraManagementProxy;
+import io.cassandrareaper.management.jmx.JmxCassandraManagementProxy;
+import io.cassandrareaper.management.jmx.JmxManagementConnectionFactory;
 import io.cassandrareaper.service.TestRepairConfiguration;
-import io.cassandrareaper.storage.MemoryStorage;
+import io.cassandrareaper.storage.MemoryStorageFacade;
 
 import java.net.URI;
 import java.time.Duration;
@@ -32,9 +36,6 @@ import java.util.Arrays;
 import java.util.List;
 import java.util.Optional;
 import java.util.Set;
-import java.util.concurrent.ExecutorService;
-import java.util.concurrent.Executors;
-
 import javax.ws.rs.core.Response;
 import javax.ws.rs.core.UriBuilder;
 import javax.ws.rs.core.UriInfo;
@@ -42,14 +43,17 @@ import javax.ws.rs.core.UriInfo;
 import com.google.common.collect.ImmutableSet;
 import com.google.common.collect.Lists;
 import com.google.common.collect.Sets;
+import org.apache.commons.lang3.RandomStringUtils;
 import org.assertj.core.api.Assertions;
 import org.eclipse.jetty.http.HttpStatus;
 import org.junit.Test;
 import org.mockito.Mockito;
 
 import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertNotEquals;
 import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertTrue;
+import static org.mockito.Mockito.any;
 import static org.mockito.Mockito.doReturn;
 import static org.mockito.Mockito.mock;
 import static org.mockito.Mockito.when;
@@ -62,34 +66,44 @@ public final class ClusterResourceTest {
   static final URI SAMPLE_URI = URI.create("http://reaper_host/cluster/");
   static final String I_DO_EXIST = "i_do_exist";
   static final String I_DONT_EXIST = "i_dont_exist";
+  static final String JMX_USERNAME = "foo";
+  static final String JMX_PASSWORD = "bar";
 
   private static final String STCS = "SizeTieredCompactionStrategy";
-  private static final ExecutorService EXECUTOR = Executors.newSingleThreadExecutor();
 
   @Test
   public void testAddCluster() throws Exception {
     final MockObjects mocks = initMocks();
 
-    ClusterResource clusterResource = new ClusterResource(mocks.context, EXECUTOR);
+    ClusterResource clusterResource
+        = ClusterResource.create(mocks.context, mocks.cryptograph, mocks.context.storage.getEventsDao(),
+        mocks.context.storage.getRepairRunDao());
 
     Response response = clusterResource
-        .addOrUpdateCluster(mocks.uriInfo, Optional.of(SEED_HOST), Optional.of(Cluster.DEFAULT_JMX_PORT));
+        .addOrUpdateCluster(mocks.uriInfo, Optional.of(SEED_HOST),
+            Optional.of(Cluster.DEFAULT_JMX_PORT),
+            Optional.of(JMX_USERNAME),
+            Optional.of(JMX_PASSWORD));
 
     Assertions.assertThat(response.getStatus()).isEqualTo(HttpStatus.CREATED_201);
-    assertEquals(1, mocks.context.storage.getClusters().size());
+    assertEquals(1, mocks.context.storage.getClusterDao().getClusters().size());
 
-    Cluster cluster = mocks.context.storage.getCluster(CLUSTER_NAME);
+    Cluster cluster = mocks.context.storage.getClusterDao().getCluster(CLUSTER_NAME);
     assertNotNull("Did not find expected cluster", cluster);
-    assertEquals(0, mocks.context.storage.getRepairRunsForCluster(cluster.getName(), Optional.of(1)).size());
+    assertEquals(0,
+        mocks.context.storage.getRepairRunDao().getRepairRunsForCluster(cluster.getName(), Optional.of(1)).size());
     assertEquals(CLUSTER_NAME, cluster.getName());
     assertEquals(1, cluster.getSeedHosts().size());
     assertEquals(SEED_HOST, cluster.getSeedHosts().iterator().next());
+    assertTrue(cluster.getJmxCredentials().isPresent());
+    assertEquals(JMX_USERNAME, cluster.getJmxCredentials().get().getUsername());
+    assertNotEquals(JMX_PASSWORD, cluster.getJmxCredentials().get().getPassword());
   }
 
   @Test
   public void testAddExistingCluster() throws Exception {
     final MockObjects mocks = initMocks();
-    when(mocks.jmxProxy.getLiveNodes()).thenReturn(Arrays.asList(SEED_HOST));
+    when(mocks.cassandraManagementProxy.getLiveNodes()).thenReturn(Arrays.asList(SEED_HOST));
 
     Cluster cluster = Cluster.builder()
         .withName(CLUSTER_NAME)
@@ -98,20 +112,26 @@ public final class ClusterResourceTest {
         .withState(Cluster.State.ACTIVE)
         .build();
 
-    mocks.context.storage.addCluster(cluster);
+    mocks.context.storage.getClusterDao().addCluster(cluster);
 
-    ClusterResource clusterResource = new ClusterResource(mocks.context, EXECUTOR);
+    ClusterResource clusterResource = ClusterResource.create(mocks.context, mocks.cryptograph,
+        mocks.context.storage.getEventsDao(),
+        mocks.context.storage.getRepairRunDao());
 
     Response response = clusterResource
-        .addOrUpdateCluster(mocks.uriInfo, Optional.of(SEED_HOST), Optional.of(Cluster.DEFAULT_JMX_PORT));
+        .addOrUpdateCluster(mocks.uriInfo, Optional.of(SEED_HOST),
+            Optional.of(Cluster.DEFAULT_JMX_PORT),
+            Optional.of(JMX_USERNAME),
+            Optional.of(JMX_PASSWORD));
 
     Assertions.assertThat(response.getStatus()).isEqualTo(HttpStatus.NO_CONTENT_204);
     assertTrue(response.getLocation().toString().endsWith("/cluster/" + cluster.getName()));
-    assertEquals(1, mocks.context.storage.getClusters().size());
+    assertEquals(1, mocks.context.storage.getClusterDao().getClusters().size());
 
-    cluster = mocks.context.storage.getCluster(CLUSTER_NAME);
+    cluster = mocks.context.storage.getClusterDao().getCluster(CLUSTER_NAME);
     assertNotNull("Did not find expected cluster", cluster);
-    assertEquals(0, mocks.context.storage.getRepairRunsForCluster(cluster.getName(), Optional.of(1)).size());
+    assertEquals(0,
+        mocks.context.storage.getRepairRunDao().getRepairRunsForCluster(cluster.getName(), Optional.of(1)).size());
     assertEquals(CLUSTER_NAME, cluster.getName());
     assertEquals(1, cluster.getSeedHosts().size());
     assertEquals(SEED_HOST, cluster.getSeedHosts().iterator().next());
@@ -120,7 +140,7 @@ public final class ClusterResourceTest {
   @Test
   public void testAddExistingClusterWithClusterName() throws Exception {
     final MockObjects mocks = initMocks();
-    when(mocks.jmxProxy.getLiveNodes()).thenReturn(Arrays.asList(SEED_HOST));
+    when(mocks.cassandraManagementProxy.getLiveNodes()).thenReturn(Arrays.asList(SEED_HOST));
 
     Cluster cluster = Cluster.builder()
         .withName(CLUSTER_NAME)
@@ -129,34 +149,61 @@ public final class ClusterResourceTest {
         .withState(Cluster.State.ACTIVE)
         .build();
 
-    mocks.context.storage.addCluster(cluster);
+    mocks.context.storage.getClusterDao().addCluster(cluster);
 
-    ClusterResource clusterResource = new ClusterResource(mocks.context, EXECUTOR);
+    ClusterResource clusterResource = ClusterResource.create(mocks.context, mocks.cryptograph,
+        mocks.context.storage.getEventsDao(),
+        mocks.context.storage.getRepairRunDao());
 
     Response response = clusterResource.addOrUpdateCluster(
-            mocks.uriInfo,
-            CLUSTER_NAME,
-            Optional.of(SEED_HOST),
-            Optional.of(Cluster.DEFAULT_JMX_PORT));
+        mocks.uriInfo,
+        CLUSTER_NAME,
+        Optional.of(SEED_HOST),
+        Optional.of(Cluster.DEFAULT_JMX_PORT),
+        Optional.of(JMX_USERNAME),
+        Optional.of(JMX_PASSWORD));
 
     Assertions.assertThat(response.getStatus()).isEqualTo(HttpStatus.NO_CONTENT_204);
     assertTrue(response.getLocation().toString().endsWith("/cluster/" + cluster.getName()));
-    assertEquals(1, mocks.context.storage.getClusters().size());
+    assertEquals(1, mocks.context.storage.getClusterDao().getClusters().size());
 
-    cluster = mocks.context.storage.getCluster(CLUSTER_NAME);
+    cluster = mocks.context.storage.getClusterDao().getCluster(CLUSTER_NAME);
     assertNotNull("Did not find expected cluster", cluster);
-    assertEquals(0, mocks.context.storage.getRepairRunsForCluster(cluster.getName(), Optional.of(1)).size());
+    assertEquals(0,
+        mocks.context.storage.getRepairRunDao().getRepairRunsForCluster(cluster.getName(), Optional.of(1)).size());
     assertEquals(CLUSTER_NAME, cluster.getName());
     assertEquals(1, cluster.getSeedHosts().size());
     assertEquals(SEED_HOST, cluster.getSeedHosts().iterator().next());
   }
 
+  @Test
+  public void testFailAddingJmxCredentialsWithoutEncryptionConfigured() throws Exception {
+    final MockObjects mocks = initMocks();
+    ClusterFacade clusterFacade = mock(ClusterFacade.class);
+
+    ClusterResource clusterResource
+        = ClusterResource.create(mocks.context, new NoopCrypotograph(), () -> clusterFacade,
+        mocks.context.storage.getEventsDao(),
+        mocks.context.storage.getRepairRunDao());
+
+    Response response = clusterResource.addOrUpdateCluster(mocks.uriInfo,
+        Optional.of(SEED_HOST),
+        Optional.of(Cluster.DEFAULT_JMX_PORT),
+        Optional.of(JMX_USERNAME),
+        Optional.of(JMX_PASSWORD));
+
+    assertEquals(Response.Status.BAD_REQUEST.getStatusCode(), response.getStatus());
+  }
+
   @Test
   public void testGetNonExistingCluster() throws ReaperException {
     final MockObjects mocks = initMocks();
-    when(mocks.jmxProxy.getLiveNodes()).thenReturn(Arrays.asList(SEED_HOST));
+    when(mocks.cassandraManagementProxy.getLiveNodes()).thenReturn(Arrays.asList(SEED_HOST));
 
-    ClusterResource clusterResource = new ClusterResource(mocks.context, EXECUTOR);
+    ClusterResource clusterResource
+        = ClusterResource.create(mocks.context, new NoopCrypotograph(), () -> mocks.clusterFacade,
+        mocks.context.storage.getEventsDao(),
+        mocks.context.storage.getRepairRunDao());
     Response response = clusterResource.getCluster(I_DONT_EXIST, Optional.<Integer>empty());
     Assertions.assertThat(response.getStatus()).isEqualTo(HttpStatus.NOT_FOUND_404);
   }
@@ -164,7 +211,7 @@ public final class ClusterResourceTest {
   @Test
   public void testGetExistingCluster() throws ReaperException {
     final MockObjects mocks = initMocks();
-    when(mocks.jmxProxy.getLiveNodes()).thenReturn(Arrays.asList(SEED_HOST));
+    when(mocks.cassandraManagementProxy.getLiveNodes()).thenReturn(Arrays.asList(SEED_HOST));
 
     Cluster cluster = Cluster.builder()
         .withName(I_DO_EXIST)
@@ -173,9 +220,12 @@ public final class ClusterResourceTest {
         .withState(Cluster.State.ACTIVE)
         .build();
 
-    mocks.context.storage.addCluster(cluster);
+    mocks.context.storage.getClusterDao().addCluster(cluster);
 
-    ClusterResource clusterResource = new ClusterResource(mocks.context, EXECUTOR);
+    ClusterResource clusterResource
+        = ClusterResource.create(mocks.context, new NoopCrypotograph(), () -> mocks.clusterFacade,
+        mocks.context.storage.getEventsDao(),
+        mocks.context.storage.getRepairRunDao());
     Response response = clusterResource.getCluster(I_DO_EXIST, Optional.<Integer>empty());
     Assertions.assertThat(response.getStatus()).isEqualTo(HttpStatus.OK_200);
   }
@@ -190,9 +240,12 @@ public final class ClusterResourceTest {
         .withState(Cluster.State.ACTIVE)
         .build();
 
-    mocks.context.storage.addCluster(cluster);
+    mocks.context.storage.getClusterDao().addCluster(cluster);
 
-    ClusterResource clusterResource = new ClusterResource(mocks.context, EXECUTOR);
+    ClusterResource clusterResource
+        = ClusterResource.create(mocks.context, new NoopCrypotograph(), () -> mocks.clusterFacade,
+        mocks.context.storage.getEventsDao(),
+        mocks.context.storage.getRepairRunDao());
     Response response = clusterResource.getClusterList(Optional.empty());
     Assertions.assertThat(response.getStatus()).isEqualTo(HttpStatus.OK_200);
     List<String> clusterNames = (List<String>) response.getEntity();
@@ -210,9 +263,12 @@ public final class ClusterResourceTest {
         .withState(Cluster.State.ACTIVE)
         .build();
 
-    mocks.context.storage.addCluster(cluster);
+    mocks.context.storage.getClusterDao().addCluster(cluster);
 
-    ClusterResource clusterResource = new ClusterResource(mocks.context, EXECUTOR);
+    ClusterResource clusterResource
+        = ClusterResource.create(mocks.context, new NoopCrypotograph(), () -> mocks.clusterFacade,
+        mocks.context.storage.getEventsDao(),
+        mocks.context.storage.getRepairRunDao());
     Response response = clusterResource.getClusterList(Optional.of(SEED_HOST));
     Assertions.assertThat(response.getStatus()).isEqualTo(HttpStatus.OK_200);
     List<String> clusterNames = (List<String>) response.getEntity();
@@ -230,7 +286,7 @@ public final class ClusterResourceTest {
         .withState(Cluster.State.ACTIVE)
         .build();
 
-    mocks.context.storage.addCluster(cluster);
+    mocks.context.storage.getClusterDao().addCluster(cluster);
 
     cluster = Cluster.builder()
         .withName("cluster2")
@@ -238,9 +294,12 @@ public final class ClusterResourceTest {
         .withState(Cluster.State.ACTIVE)
         .build();
 
-    mocks.context.storage.addCluster(cluster);
+    mocks.context.storage.getClusterDao().addCluster(cluster);
 
-    ClusterResource clusterResource = new ClusterResource(mocks.context, EXECUTOR);
+    ClusterResource clusterResource
+        = ClusterResource.create(mocks.context, new NoopCrypotograph(), () -> mocks.clusterFacade,
+        mocks.context.storage.getEventsDao(),
+        mocks.context.storage.getRepairRunDao());
     Response response = clusterResource.getClusterList(Optional.of(SEED_HOST));
     Assertions.assertThat(response.getStatus()).isEqualTo(HttpStatus.OK_200);
     List<String> clusterNames = (List<String>) response.getEntity();
@@ -258,7 +317,7 @@ public final class ClusterResourceTest {
         .withState(Cluster.State.ACTIVE)
         .build();
 
-    mocks.context.storage.addCluster(cluster);
+    mocks.context.storage.getClusterDao().addCluster(cluster);
 
     cluster = Cluster.builder()
         .withName("cluster2")
@@ -266,9 +325,12 @@ public final class ClusterResourceTest {
         .withState(Cluster.State.ACTIVE)
         .build();
 
-    mocks.context.storage.addCluster(cluster);
+    mocks.context.storage.getClusterDao().addCluster(cluster);
 
-    ClusterResource clusterResource = new ClusterResource(mocks.context, EXECUTOR);
+    ClusterResource clusterResource
+        = ClusterResource.create(mocks.context, new NoopCrypotograph(), () -> mocks.clusterFacade,
+        mocks.context.storage.getEventsDao(),
+        mocks.context.storage.getRepairRunDao());
     Response response = clusterResource.getClusterList(Optional.of("host2"));
     Assertions.assertThat(response.getStatus()).isEqualTo(HttpStatus.OK_200);
     List<String> clusterNames = (List<String>) response.getEntity();
@@ -286,7 +348,7 @@ public final class ClusterResourceTest {
         .withState(Cluster.State.ACTIVE)
         .build();
 
-    mocks.context.storage.addCluster(cluster);
+    mocks.context.storage.getClusterDao().addCluster(cluster);
 
     cluster = Cluster.builder()
         .withName("cluster2")
@@ -294,9 +356,12 @@ public final class ClusterResourceTest {
         .withState(Cluster.State.ACTIVE)
         .build();
 
-    mocks.context.storage.addCluster(cluster);
+    mocks.context.storage.getClusterDao().addCluster(cluster);
 
-    ClusterResource clusterResource = new ClusterResource(mocks.context, EXECUTOR);
+    ClusterResource clusterResource
+        = ClusterResource.create(mocks.context, new NoopCrypotograph(), () -> mocks.clusterFacade,
+        mocks.context.storage.getEventsDao(),
+        mocks.context.storage.getRepairRunDao());
     Response response = clusterResource.getClusterList(Optional.empty());
     Assertions.assertThat(response.getStatus()).isEqualTo(HttpStatus.OK_200);
     List<String> clusterNames = (List<String>) response.getEntity();
@@ -314,7 +379,7 @@ public final class ClusterResourceTest {
         .withState(Cluster.State.ACTIVE)
         .build();
 
-    mocks.context.storage.addCluster(cluster);
+    mocks.context.storage.getClusterDao().addCluster(cluster);
 
     cluster = Cluster.builder()
         .withName("abc")
@@ -322,9 +387,12 @@ public final class ClusterResourceTest {
         .withState(Cluster.State.ACTIVE)
         .build();
 
-    mocks.context.storage.addCluster(cluster);
+    mocks.context.storage.getClusterDao().addCluster(cluster);
 
-    ClusterResource clusterResource = new ClusterResource(mocks.context, EXECUTOR);
+    ClusterResource clusterResource
+        = ClusterResource.create(mocks.context, new NoopCrypotograph(), () -> mocks.clusterFacade,
+        mocks.context.storage.getEventsDao(),
+        mocks.context.storage.getRepairRunDao());
     Response response = clusterResource.getClusterList(Optional.empty());
     Assertions.assertThat(response.getStatus()).isEqualTo(HttpStatus.OK_200);
     List<String> clusterNames = (List<String>) response.getEntity();
@@ -342,7 +410,7 @@ public final class ClusterResourceTest {
         .withState(Cluster.State.UNREACHABLE)
         .build();
 
-    mocks.context.storage.addCluster(cluster);
+    mocks.context.storage.getClusterDao().addCluster(cluster);
 
     cluster = Cluster.builder()
         .withName("cluster2")
@@ -350,9 +418,12 @@ public final class ClusterResourceTest {
         .withState(Cluster.State.ACTIVE)
         .build();
 
-    mocks.context.storage.addCluster(cluster);
+    mocks.context.storage.getClusterDao().addCluster(cluster);
 
-    ClusterResource clusterResource = new ClusterResource(mocks.context, EXECUTOR);
+    ClusterResource clusterResource
+        = ClusterResource.create(mocks.context, new NoopCrypotograph(), () -> mocks.clusterFacade,
+        mocks.context.storage.getEventsDao(),
+        mocks.context.storage.getRepairRunDao());
     Response response = clusterResource.getClusterList(Optional.empty());
     Assertions.assertThat(response.getStatus()).isEqualTo(HttpStatus.OK_200);
     List<String> clusterNames = (List<String>) response.getEntity();
@@ -369,7 +440,7 @@ public final class ClusterResourceTest {
         .withSeedHosts(ImmutableSet.of(SEED_HOST))
         .build();
 
-    mocks.context.storage.addCluster(cluster);
+    mocks.context.storage.getClusterDao().addCluster(cluster);
   }
 
   @Test(expected = IllegalArgumentException.class)
@@ -382,7 +453,7 @@ public final class ClusterResourceTest {
         .withState(Cluster.State.ACTIVE)
         .build();
 
-    mocks.context.storage.addCluster(cluster);
+    mocks.context.storage.getClusterDao().addCluster(cluster);
 
     cluster = Cluster.builder()
         .withName(CLUSTER_NAME)
@@ -390,33 +461,49 @@ public final class ClusterResourceTest {
         .withState(Cluster.State.ACTIVE)
         .build();
 
-    mocks.context.storage.addCluster(cluster);
+    mocks.context.storage.getClusterDao().addCluster(cluster);
   }
 
   @Test
   public void testModifyClusterSeeds() throws ReaperException {
     final MockObjects mocks = initMocks();
 
-    ClusterResource clusterResource = new ClusterResource(mocks.context, Executors.newFixedThreadPool(2));
-    clusterResource.addOrUpdateCluster(mocks.uriInfo, Optional.of(SEED_HOST), Optional.of(Cluster.DEFAULT_JMX_PORT));
-    doReturn(Arrays.asList(SEED_HOST + 1, SEED_HOST)).when(mocks.jmxProxy).getLiveNodes();
+    ClusterResource clusterResource
+        = ClusterResource.create(mocks.context, mocks.cryptograph, mocks.context.storage.getEventsDao(),
+        mocks.context.storage.getRepairRunDao());
+
+    clusterResource.addOrUpdateCluster(
+        mocks.uriInfo,
+        Optional.of(SEED_HOST),
+        Optional.of(Cluster.DEFAULT_JMX_PORT),
+        Optional.of(JMX_USERNAME),
+        Optional.of(JMX_PASSWORD)
+    );
+
+    doReturn(Arrays.asList(SEED_HOST + 1, SEED_HOST)).when(mocks.cassandraManagementProxy).getLiveNodes();
 
     Response response = clusterResource
-        .addOrUpdateCluster(mocks.uriInfo, Optional.of(SEED_HOST + 1), Optional.of(Cluster.DEFAULT_JMX_PORT));
+        .addOrUpdateCluster(mocks.uriInfo, Optional.of(SEED_HOST + 1),
+            Optional.of(Cluster.DEFAULT_JMX_PORT),
+            Optional.of(JMX_USERNAME),
+            Optional.of(JMX_PASSWORD));
 
     assertEquals(HttpStatus.OK_200, response.getStatus());
     assertTrue(response.getLocation().toString().endsWith("/cluster/" + CLUSTER_NAME));
-    assertEquals(1, mocks.context.storage.getClusters().size());
+    assertEquals(1, mocks.context.storage.getClusterDao().getClusters().size());
 
-    Cluster cluster = mocks.context.storage.getCluster(CLUSTER_NAME);
+    Cluster cluster = mocks.context.storage.getClusterDao().getCluster(CLUSTER_NAME);
     Assertions.assertThat(cluster.getSeedHosts()).hasSize(2);
     Assertions.assertThat(cluster.getSeedHosts()).contains(SEED_HOST + 1);
 
     response = clusterResource.addOrUpdateCluster(
-            mocks.uriInfo,
-            CLUSTER_NAME,
-            Optional.of(SEED_HOST + 1),
-            Optional.of(Cluster.DEFAULT_JMX_PORT));
+        mocks.uriInfo,
+        CLUSTER_NAME,
+        Optional.of(SEED_HOST + 1),
+        Optional.of(Cluster.DEFAULT_JMX_PORT),
+        Optional.of(JMX_USERNAME),
+        Optional.of(JMX_PASSWORD)
+    );
 
     assertEquals(HttpStatus.NO_CONTENT_204, response.getStatus());
     assertTrue(response.getLocation().toString().endsWith("/cluster/" + cluster.getName()));
@@ -426,29 +513,44 @@ public final class ClusterResourceTest {
   public void testModifyClusterSeedsWithClusterName() throws ReaperException {
     final MockObjects mocks = initMocks();
 
-    ClusterResource clusterResource = new ClusterResource(mocks.context, Executors.newFixedThreadPool(2));
-    clusterResource.addOrUpdateCluster(mocks.uriInfo, Optional.of(SEED_HOST), Optional.of(Cluster.DEFAULT_JMX_PORT));
-    doReturn(Arrays.asList(SEED_HOST + 1, SEED_HOST)).when(mocks.jmxProxy).getLiveNodes();
+    ClusterResource clusterResource
+        = ClusterResource.create(mocks.context, mocks.cryptograph,
+        mocks.context.storage.getEventsDao(),
+        mocks.context.storage.getRepairRunDao());
+    ;
+    clusterResource.addOrUpdateCluster(
+        mocks.uriInfo,
+        Optional.of(SEED_HOST),
+        Optional.of(Cluster.DEFAULT_JMX_PORT),
+        Optional.of(JMX_USERNAME),
+        Optional.of(JMX_PASSWORD)
+    );
+
+    doReturn(Arrays.asList(SEED_HOST + 1, SEED_HOST)).when(mocks.cassandraManagementProxy).getLiveNodes();
 
     Response response = clusterResource.addOrUpdateCluster(
-            mocks.uriInfo,
-            CLUSTER_NAME,
-            Optional.of(SEED_HOST + 1),
-            Optional.of(Cluster.DEFAULT_JMX_PORT));
+        mocks.uriInfo,
+        CLUSTER_NAME,
+        Optional.of(SEED_HOST + 1),
+        Optional.of(Cluster.DEFAULT_JMX_PORT),
+        Optional.of(JMX_USERNAME),
+        Optional.of(JMX_PASSWORD));
 
     assertEquals(HttpStatus.OK_200, response.getStatus());
     assertTrue(response.getLocation().toString().endsWith("/cluster/" + CLUSTER_NAME));
-    assertEquals(1, mocks.context.storage.getClusters().size());
+    assertEquals(1, mocks.context.storage.getClusterDao().getClusters().size());
 
-    Cluster cluster = mocks.context.storage.getCluster(CLUSTER_NAME);
+    Cluster cluster = mocks.context.storage.getClusterDao().getCluster(CLUSTER_NAME);
     Assertions.assertThat(cluster.getSeedHosts()).hasSize(2);
     Assertions.assertThat(cluster.getSeedHosts()).contains(SEED_HOST + 1);
 
     response = clusterResource.addOrUpdateCluster(
-            mocks.uriInfo,
-            CLUSTER_NAME,
-            Optional.of(SEED_HOST + 1),
-            Optional.of(Cluster.DEFAULT_JMX_PORT));
+        mocks.uriInfo,
+        CLUSTER_NAME,
+        Optional.of(SEED_HOST + 1),
+        Optional.of(Cluster.DEFAULT_JMX_PORT),
+        Optional.of(JMX_USERNAME),
+        Optional.of(JMX_PASSWORD));
     assertEquals(HttpStatus.NO_CONTENT_204, response.getStatus());
 
     assertTrue(response.getLocation().toString().endsWith("/cluster/" + cluster.getName()));
@@ -457,10 +559,10 @@ public final class ClusterResourceTest {
   @Test
   public void addingAClusterAutomaticallySetupSchedulingRepairsWhenEnabled() throws Exception {
     final MockObjects mocks = initMocks();
-    when(mocks.jmxProxy.getLiveNodes()).thenReturn(Arrays.asList(SEED_HOST));
-    when(mocks.jmxProxy.getKeyspaces()).thenReturn(Lists.newArrayList("keyspace1"));
+    when(mocks.cassandraManagementProxy.getLiveNodes()).thenReturn(Arrays.asList(SEED_HOST));
+    when(mocks.cassandraManagementProxy.getKeyspaces()).thenReturn(Lists.newArrayList("keyspace1"));
 
-    when(mocks.jmxProxy.getTablesForKeyspace("keyspace1"))
+    when(mocks.cassandraManagementProxy.getTablesForKeyspace("keyspace1"))
         .thenReturn(Sets.newHashSet(Table.builder().withName("table1").withCompactionStrategy(STCS).build()));
 
     mocks.context.config = TestRepairConfiguration.defaultConfigBuilder()
@@ -471,23 +573,34 @@ public final class ClusterResourceTest {
                 .build())
         .build();
 
-    ClusterResource clusterResource = new ClusterResource(mocks.context, Executors.newFixedThreadPool(2));
+    ClusterResource clusterResource
+        = ClusterResource.create(mocks.context, mocks.cryptograph,
+        mocks.context.storage.getEventsDao(),
+        mocks.context.storage.getRepairRunDao());
+    ;
 
-    Response response = clusterResource
-        .addOrUpdateCluster(mocks.uriInfo, Optional.of(SEED_HOST), Optional.of(Cluster.DEFAULT_JMX_PORT));
+    Response response = clusterResource.addOrUpdateCluster(
+        mocks.uriInfo,
+        Optional.of(SEED_HOST),
+        Optional.of(Cluster.DEFAULT_JMX_PORT),
+        Optional.of(JMX_USERNAME),
+        Optional.of(JMX_PASSWORD)
+    );
 
     assertEquals(HttpStatus.CREATED_201, response.getStatus());
-    assertEquals(1, mocks.context.storage.getAllRepairSchedules().size());
-    assertEquals(1, mocks.context.storage.getRepairSchedulesForClusterAndKeyspace(CLUSTER_NAME, "keyspace1").size());
+    assertEquals(1, mocks.context.storage.getRepairScheduleDao().getAllRepairSchedules().size());
+    assertEquals(1,
+        mocks.context.storage.getRepairScheduleDao().getRepairSchedulesForClusterAndKeyspace(CLUSTER_NAME, "keyspace1")
+            .size());
   }
 
   @Test
   public void testClusterDeleting() throws Exception {
     final MockObjects mocks = initMocks();
-    when(mocks.jmxProxy.getLiveNodes()).thenReturn(Arrays.asList(SEED_HOST));
-    when(mocks.jmxProxy.getKeyspaces()).thenReturn(Lists.newArrayList("keyspace1"));
+    when(mocks.cassandraManagementProxy.getLiveNodes()).thenReturn(Arrays.asList(SEED_HOST));
+    when(mocks.cassandraManagementProxy.getKeyspaces()).thenReturn(Lists.newArrayList("keyspace1"));
 
-    when(mocks.jmxProxy.getTablesForKeyspace("keyspace1"))
+    when(mocks.cassandraManagementProxy.getTablesForKeyspace("keyspace1"))
         .thenReturn(Sets.newHashSet(Table.builder().withName("table1").withCompactionStrategy(STCS).build()));
 
     mocks.context.config = TestRepairConfiguration.defaultConfigBuilder()
@@ -498,14 +611,25 @@ public final class ClusterResourceTest {
                 .build())
         .build();
 
-    ClusterResource clusterResource = new ClusterResource(mocks.context, Executors.newFixedThreadPool(2));
+    ClusterResource clusterResource
+        = ClusterResource.create(mocks.context, mocks.cryptograph,
+        mocks.context.storage.getEventsDao(),
+        mocks.context.storage.getRepairRunDao());
+    ;
 
-    Response response = clusterResource
-        .addOrUpdateCluster(mocks.uriInfo, Optional.of(SEED_HOST), Optional.of(Cluster.DEFAULT_JMX_PORT));
+    Response response = clusterResource.addOrUpdateCluster(
+        mocks.uriInfo,
+        Optional.of(SEED_HOST),
+        Optional.of(Cluster.DEFAULT_JMX_PORT),
+        Optional.of(JMX_USERNAME),
+        Optional.of(JMX_PASSWORD)
+    );
 
     assertEquals(HttpStatus.CREATED_201, response.getStatus());
-    assertEquals(1, mocks.context.storage.getAllRepairSchedules().size());
-    assertEquals(1, mocks.context.storage.getRepairSchedulesForClusterAndKeyspace(CLUSTER_NAME, "keyspace1").size());
+    assertEquals(1, mocks.context.storage.getRepairScheduleDao().getAllRepairSchedules().size());
+    assertEquals(1,
+        mocks.context.storage.getRepairScheduleDao().getRepairSchedulesForClusterAndKeyspace(CLUSTER_NAME, "keyspace1")
+            .size());
 
     assertEquals(HttpStatus.CONFLICT_409, clusterResource.deleteCluster(CLUSTER_NAME, Optional.empty()).getStatus());
 
@@ -547,36 +671,51 @@ public final class ClusterResourceTest {
 
   private MockObjects initMocks() throws ReaperException {
     AppContext context = new AppContext();
-    context.storage = new MemoryStorage();
+    context.storage = new MemoryStorageFacade();
     context.config = TestRepairConfiguration.defaultConfig();
 
     UriInfo uriInfo = mock(UriInfo.class);
     when(uriInfo.getBaseUriBuilder()).thenReturn(UriBuilder.fromUri(SAMPLE_URI));
 
-    JmxProxy jmxProxy = mock(JmxProxy.class);
-    when(jmxProxy.getClusterName()).thenReturn(CLUSTER_NAME);
-    when(jmxProxy.getPartitioner()).thenReturn(PARTITIONER);
+    JmxCassandraManagementProxy cassandraManagementProxy = mock(JmxCassandraManagementProxy.class);
+    when(cassandraManagementProxy.getClusterName()).thenReturn(CLUSTER_NAME);
+    when(cassandraManagementProxy.getPartitioner()).thenReturn(PARTITIONER);
 
-    context.jmxConnectionFactory = mock(JmxConnectionFactory.class);
+    context.managementConnectionFactory = mock(JmxManagementConnectionFactory.class);
 
-    when(context.jmxConnectionFactory.connectAny(Mockito.anyCollection())).thenReturn(jmxProxy);
+    when(((JmxManagementConnectionFactory) context.managementConnectionFactory).connectAny(Mockito.anyCollection()))
+            .thenReturn(cassandraManagementProxy);
 
-    return new MockObjects(context, uriInfo, jmxProxy);
+    Cryptograph cryptograph = mock(Cryptograph.class);
+    when(cryptograph.encrypt(any(String.class))).thenReturn(RandomStringUtils.randomNumeric(10));
+
+    ClusterFacade clusterFacade = mock(ClusterFacade.class);
+
+    return new MockObjects(context, cryptograph, uriInfo, cassandraManagementProxy, clusterFacade);
   }
 
   private static final class MockObjects {
 
     final AppContext context;
+    final Cryptograph cryptograph;
     final UriInfo uriInfo;
-    final JmxProxy jmxProxy;
+    final ICassandraManagementProxy cassandraManagementProxy;
+    final ClusterFacade clusterFacade;
 
-    MockObjects(AppContext context, UriInfo uriInfo, JmxProxy jmxProxy) {
+    MockObjects(
+        AppContext context,
+        Cryptograph cryptograph,
+        UriInfo uriInfo,
+        ICassandraManagementProxy cassandraManagementProxy,
+        ClusterFacade clusterFacade) {
       super();
       this.context = context;
+      this.cryptograph = cryptograph;
       this.uriInfo = uriInfo;
-      this.jmxProxy = jmxProxy;
+      this.cassandraManagementProxy = cassandraManagementProxy;
+      this.clusterFacade = clusterFacade;
     }
 
   }
 
-}
+}
\ No newline at end of file
