@@ -48,7 +48,6 @@ std::string Options::public_key_ = "public.key";
 std::string Options::private_key_ = "private.key";
 bool Options::is_verbose_ccm_ = false;
 bool Options::is_verbose_integration_ = false;
-bool Options::is_beta_protocol_ = true;
 
 // Static initialization is not guaranteed for the following types
 CCM::DseCredentialsType Options::dse_credentials_type_;
@@ -185,8 +184,6 @@ bool Options::initialize(int argc, char* argv[]) {
           is_verbose_ccm_ = true;
           is_verbose_integration_ = true;
         }
-      } else if (key == "--disable-beta-protocol") {
-        is_beta_protocol_ = false;
       }
 #ifdef CASS_USE_LIBSSH2
       else if (key == "--authentication") {
@@ -380,11 +377,6 @@ void Options::print_help() {
   std::cout << "  --verbose(=ccm,integration)" << std::endl
             << "      "
             << "Enable verbose output for component(s)." << std::endl;
-  std::cout << "  --disable-beta-protocol" << std::endl
-            << "      "
-            << "Disable beta protocol use by default." << std::endl
-            << "      "
-            << "NOTE: Individual tests may override this setting." << std::endl;
   std::cout << std::endl;
 }
 
@@ -522,8 +514,6 @@ bool Options::is_verbose_ccm() { return is_verbose_ccm_; }
 
 bool Options::is_verbose_integration() { return is_verbose_integration_; }
 
-bool Options::is_beta_protocol() { return is_beta_protocol_; }
-
 Options::Options() {}
 
 bool Options::bool_value(const std::string& value) {
