@@ -285,6 +285,12 @@ static void qed_cxt_qm_iids(struct qed_hwfn *p_hwfn, struct qed_qm_iids *iids)
 	iids->vf_cids = vf_cids;
 	iids->tids += vf_tids * p_mngr->vf_count;
 
+	/* in pqset mode, all cids are pf cids, and can go to any pq */
+	if (QED_PQSET_SUPPORTED(p_hwfn)) {
+		iids->cids += vf_cids * p_mngr->vf_count;
+		iids->vf_cids = 0;
+	}
+
 	DP_VERBOSE(p_hwfn, QED_MSG_ILT,
 		   "iids: CIDS %08x vf_cids %08x tids %08x vf_tids %08x\n",
 		   iids->cids, iids->vf_cids, iids->tids, vf_tids);
@@ -468,7 +474,7 @@ static void qed_ilt_cli_adv_line(struct qed_hwfn *p_hwfn,
 				 struct qed_ilt_cli_blk *p_blk,
 				 u32 * p_line, enum ilt_clients client_id)
 {
-	if (!p_blk->total_size)
+	if (!p_blk->total_size || !p_blk->real_size_in_page)
 		return;
 
 	if (!p_cli->active)
@@ -567,16 +573,17 @@ static void eocre_ilt_blk_reset(struct qed_hwfn *p_hwfn)
 
 int qed_cxt_cfg_ilt_compute(struct qed_hwfn *p_hwfn, u32 * line_count)
 {
-	u32 curr_line, total, i, task_size, line, total_size, elem_size;
+	u32 total, i, task_size, line, total_size, elem_size;
+	u16 num_pf_pqs, num_vf_pqs, total_lines;
 	struct qed_cxt_mngr *p_mngr = p_hwfn->p_cxt_mngr;
 	struct qed_qm_info *qm_info = &p_hwfn->qm_info;
+	u32 curr_line, prev_line;
 	struct qed_ilt_client_cfg *p_cli;
 	struct qed_ilt_cli_blk *p_blk;
 	struct qed_cdu_iids cdu_iids;
 	struct qed_qm_iids qm_iids;
 	struct qed_tm_iids tm_iids;
 	struct qed_tid_seg *p_seg;
-	u16 num_vf_pqs;
 
 	memset(&qm_iids, 0, sizeof(qm_iids));
 	memset(&cdu_iids, 0, sizeof(cdu_iids));
@@ -653,8 +660,39 @@ int qed_cxt_cfg_ilt_compute(struct qed_hwfn *p_hwfn, u32 * line_count)
 		qed_ilt_cli_blk_fill(p_cli, p_blk, curr_line, total,
 				     p_mngr->task_type_size[p_seg->type]);
 
+		prev_line = curr_line;
 		qed_ilt_cli_adv_line(p_hwfn, p_cli, p_blk, &curr_line,
 				     ILT_CLI_CDUT);
+		total_lines = (u16) (curr_line - prev_line);
+
+		switch (i) {
+		case QED_CXT_ISCSI_TID_SEG:
+			p_mngr->iscsi_task_pages = total_lines;
+			DP_VERBOSE(p_hwfn, QED_MSG_ILT,
+				   "CDUT ILT Info: iscsi_task_pages %hu\n",
+				   p_mngr->iscsi_task_pages);
+			break;
+		case QED_CXT_FCOE_TID_SEG:
+			p_mngr->fcoe_task_pages = total_lines;
+			DP_VERBOSE(p_hwfn, QED_MSG_ILT,
+				   "CDUT ILT Info: fcoe_task_pages %hu\n",
+				   p_mngr->fcoe_task_pages);
+			break;
+		case QED_CXT_ROCE_TID_SEG:
+			p_mngr->roce_task_pages = total_lines;
+			DP_VERBOSE(p_hwfn, QED_MSG_ILT,
+				   "CDUT ILT Info: roce_task_pages %hu\n",
+				   p_mngr->roce_task_pages);
+			break;
+		case QED_CXT_ETH_TID_SEG:
+			p_mngr->eth_task_pages = total_lines;
+			DP_VERBOSE(p_hwfn, QED_MSG_ILT,
+				   "CDUT ILT Info: eth_task_pages %hu\n",
+				   p_mngr->eth_task_pages);
+			break;
+		default:
+			break;
+		}
 	}
 
 	/* next the 'init' task memory (forced load memory) */
@@ -757,18 +795,22 @@ int qed_cxt_cfg_ilt_compute(struct qed_hwfn *p_hwfn, u32 * line_count)
 	/* At this stage, after the first QM configuration, the PF PQs amount
 	 * is the highest possible. Save this value at qm_info->ilt_pf_pqs to
 	 * detect overflows in the future.
-	 * Even though VF PQs amount can be larger than VF count, use vf_count
-	 * because each VF requires only the full amount of CIDs.
+	 * In case of VF rdma, amount of PQs per VF may be up to 2 X vf count,
+	 * but must be capped by the resource amount, hence using that.
+	 * In PQSET mode all cids of PF and all of its VFs can end up on any PQ
+	 * thus all are computed as belonging to PF.
 	 */
 	qm_info->ilt_pf_pqs = qm_info->num_pqs - qm_info->num_vf_pqs;
+
+	num_pf_pqs = qm_info->ilt_pf_pqs;
 	if (QED_IS_VF_RDMA(p_hwfn))
 		num_vf_pqs = RESC_NUM(p_hwfn, QED_PQ) - qm_info->ilt_pf_pqs;
 	else
 		num_vf_pqs = (u16) p_mngr->vf_count;
 
 	total = qed_qm_pf_mem_size(qm_iids.cids,
-				   qm_iids.vf_cids, qm_iids.tids,
-				   qm_info->ilt_pf_pqs, num_vf_pqs);
+				   qm_iids.vf_cids,
+				   qm_iids.tids, num_pf_pqs, num_vf_pqs);
 
 	DP_VERBOSE(p_hwfn,
 		   QED_MSG_ILT,
@@ -1615,23 +1657,33 @@ static void qed_cdu_init_pf(struct qed_hwfn *p_hwfn)
 void qed_qm_init_pf(struct qed_hwfn *p_hwfn,
 		    struct qed_ptt *p_ptt, bool is_pf_loading)
 {
+	u16 num_pf_pqs, num_vf_pqs;
 	struct qed_qm_info *qm_info = &p_hwfn->qm_info;
+	struct qed_mcp_link_state *p_link;
 	struct qed_qm_iids iids;
 
 	memset(&iids, 0, sizeof(iids));
 	qed_cxt_qm_iids(p_hwfn, &iids);
+
+	p_link = &QED_LEADING_HWFN(p_hwfn->cdev)->mcp_info->link_output;
+
+	num_pf_pqs = qm_info->num_pqs - qm_info->num_vf_pqs;
+	num_vf_pqs = qm_info->num_vf_pqs;
+
 	qed_qm_pf_rt_init(p_hwfn, p_ptt, p_hwfn->rel_pf_id,
 			  qm_info->max_phys_tcs_per_port,
 			  is_pf_loading,
 			  iids.cids, iids.vf_cids, iids.tids,
 			  qm_info->start_pq,
-			  qm_info->num_pqs - qm_info->num_vf_pqs,
-			  qm_info->num_vf_pqs,
-			  qm_info->start_vport,
-			  qm_info->num_vports, qm_info->pf_wfq,
-			  qm_info->pf_rl,
+			  num_pf_pqs,
+			  num_vf_pqs,
+			  qm_info->start_vport, qm_info->num_vports,
+			  qm_info->start_rl, qm_info->num_rls,
+			  qm_info->pf_wfq,
+			  qm_info->pf_rl, p_link->speed,
 			  p_hwfn->qm_info.qm_pq_params,
-			  p_hwfn->qm_info.qm_vport_params);
+			  p_hwfn->qm_info.qm_vport_params,
+			  p_hwfn->qm_info.qm_rl_params);
 }
 
 /* CM PF */
@@ -1698,18 +1750,22 @@ static void qed_dq_init_pf(struct qed_hwfn *p_hwfn)
 static void qed_ilt_bounds_init(struct qed_hwfn *p_hwfn)
 {
 	struct qed_ilt_client_cfg *ilt_clients;
+	bool active;
 	int i;
 
 	ilt_clients = p_hwfn->p_cxt_mngr->clients;
-	for_each_ilt_valid_client(i, ilt_clients) {
+	for (i = 0; i < ILT_CLI_RGFS; i++) {
+		active = ilt_clients[i].active;
+
 		STORE_RT_REG(p_hwfn,
 			     ilt_clients[i].first.reg,
-			     ilt_clients[i].first.val);
+			     active ? ilt_clients[i].first.val : 0);
 		STORE_RT_REG(p_hwfn,
-			     ilt_clients[i].last.reg, ilt_clients[i].last.val);
+			     ilt_clients[i].last.reg,
+			     active ? ilt_clients[i].last.val : 0);
 		STORE_RT_REG(p_hwfn,
 			     ilt_clients[i].p_size.reg,
-			     ilt_clients[i].p_size.val);
+			     active ? ilt_clients[i].p_size.val : 0);
 	}
 }
 
@@ -1900,7 +1956,7 @@ static void qed_tm_init_pf(struct qed_hwfn *p_hwfn)
 	u32 active_seg_mask = 0, tm_offset, rt_reg;
 	struct qed_tm_iids tm_iids;
 	u64 cfg_word;
-	u8 i;
+	u32 i;
 
 	memset(&tm_iids, 0, sizeof(tm_iids));
 	qed_cxt_tm_iids(p_hwfn, &tm_iids);
@@ -2067,7 +2123,7 @@ int _qed_cxt_acquire_cid(struct qed_hwfn *p_hwfn,
 	struct qed_cxt_mngr *p_mngr = p_hwfn->p_cxt_mngr;
 	struct qed_cid_acquired_map *p_map;
 
-	if (type >= MAX_CONN_TYPES) {
+	if (type >= MAX_CONN_TYPES || type < 0) {
 		DP_NOTICE(p_hwfn, "Invalid protocol type %d", type);
 		return -EINVAL;
 	}
@@ -2383,17 +2439,13 @@ int qed_cxt_set_pf_params(struct qed_hwfn *p_hwfn, u32 rdma_tasks)
 	case QED_PCI_ETH_RDMA:
 	case QED_PCI_ETH_IWARP:
 	case QED_PCI_ETH_ROCE:
-		/* Both RoCE and iWarp use FW task type 2 */
-		p_mngr->task_type_id = PROTOCOLID_ROCE;
-		p_mngr->task_ctx_size = TYPE1_TASK_CXT_SIZE(p_hwfn);
-
 		rc = qed_rdma_set_pf_params(p_hwfn,
 					    &p_hwfn->pf_params.rdma_pf_params,
 					    rdma_tasks);
 		if (rc)
 			return rc;
-
-		/* Fall through */
+		COMPAT_FALLTHROUGH;
+		/* fallthrough */
 	case QED_PCI_ETH:
 		{
 			u32 count = 0;
@@ -2401,6 +2453,8 @@ int qed_cxt_set_pf_params(struct qed_hwfn *p_hwfn, u32 rdma_tasks)
 			struct qed_eth_pf_params *p_params =
 			    &p_hwfn->pf_params.eth_pf_params;
 
+			p_mngr->task_ctx_size = TYPE1_TASK_CXT_SIZE(p_hwfn);
+
 			if (!p_params->num_vf_cons)
 				p_params->num_vf_cons =
 				    ETH_PF_PARAMS_VF_CONS_DEFAULT;
@@ -2420,7 +2474,6 @@ int qed_cxt_set_pf_params(struct qed_hwfn *p_hwfn, u32 rdma_tasks)
 		{
 			struct qed_fcoe_pf_params *p_params;
 
-			p_mngr->task_type_id = PROTOCOLID_FCOE;
 			p_mngr->task_ctx_size = TYPE0_TASK_CXT_SIZE(p_hwfn);
 
 			p_params = &p_hwfn->pf_params.fcoe_pf_params;
@@ -2444,7 +2497,6 @@ int qed_cxt_set_pf_params(struct qed_hwfn *p_hwfn, u32 rdma_tasks)
 		{
 			struct qed_iscsi_pf_params *p_params;
 
-			p_mngr->task_type_id = PROTOCOLID_ISCSI;
 			p_mngr->task_ctx_size = TYPE0_TASK_CXT_SIZE(p_hwfn);
 
 			p_params = &p_hwfn->pf_params.iscsi_pf_params;
@@ -2568,8 +2620,8 @@ qed_cxt_dynamic_ilt_alloc(struct qed_hwfn *p_hwfn,
 			 &p_cli->pf_blks[CDUT_SEG_BLK(QED_CXT_ROCE_TID_SEG)]);
 		break;
 	default:
-		DP_NOTICE(p_hwfn, "-EINVALID elem type = %d", elem_type);
-		return -EINVAL;
+		DP_NOTICE(p_hwfn, "-EOPNOTSUPP elem type = %d", elem_type);
+		return -EOPNOTSUPP;
 	}
 
 	/* Calculate line in ilt */
@@ -2586,6 +2638,14 @@ qed_cxt_dynamic_ilt_alloc(struct qed_hwfn *p_hwfn,
 
 	shadow_line = line - p_hwfn->p_cxt_mngr->pf_start_line;
 
+	DP_VERBOSE(p_hwfn, QED_MSG_ILT,
+		   "ilt line %d(%d) - iid 0x%x vf_id 0x%x elem_type %d "
+		   "elem_size 0x%x hw_p_size %d(0x%x) elems_per_p 0x%x "
+		   "vf_total_lines 0x%x.\n",
+		   line, shadow_line, iid, vf_id, elem_type, elem_size,
+		   hw_p_size, ILT_PAGE_IN_BYTES(hw_p_size), elems_per_p,
+		   is_vf ? p_cli->vf_total_lines : 0);
+
 	/* If line is already allocated, do nothing, otherwise allocate it and
 	 * write it to the PSWRQ2 registers.
 	 * This section can be run in parallel from different contexts and thus
@@ -2652,6 +2712,16 @@ qed_cxt_dynamic_ilt_alloc(struct qed_hwfn *p_hwfn,
 	qed_dmae_host2grc(p_hwfn, p_ptt, (u64) (uintptr_t) & ilt_hw_entry,
 			  reg_offset, sizeof(ilt_hw_entry) / sizeof(u32),
 			  NULL /* default parameters */ );
+
+	DP_VERBOSE(p_hwfn,
+		   QED_MSG_ILT,
+		   "line %d(%d) allocated - v_addr %p p_addr 0x%llx size 0x%x. "
+		   "elem_type %d iid 0x%x vf_id 0x%x.\n",
+		   line,
+		   shadow_line,
+		   p_virt,
+		   (u64) p_phys,
+		   p_blk->real_size_in_page, elem_type, iid, vf_id);
 out1:
 	qed_ptt_release(p_hwfn, p_ptt);
 out0:
@@ -2749,6 +2819,13 @@ qed_cxt_free_ilt_range(struct qed_hwfn *p_hwfn,
 		return -EBUSY;
 	}
 
+	DP_VERBOSE(p_hwfn, QED_MSG_ILT,
+		   "start_line %d(%d) start_iid 0x%x "
+		   "end_line %d(%d) end_iid 0x%x "
+		   "vf_id 0x%x elem_type %d count 0x%x.\n",
+		   start_line, shadow_start_line, start_iid,
+		   end_line, shadow_end_line, end_iid, vf_id, elem_type, count);
+
 	/* This piece of code takes care of freeing the ILT specific range, as
 	 * well as setting it to zero.
 	 * In case the start or the end lines of the range share other iids,
