@@ -43,6 +43,7 @@
 #include "qed_cxt.h"
 #include "qed_dcbx.h"
 #include "qed_hsi.h"
+#include "qed_iro_hsi.h"
 #include "qed_rdma.h"
 #include "qed_sp.h"
 #include "qed_sriov.h"
@@ -363,7 +364,7 @@ qed_dcbx_process_tlv(struct qed_hwfn *p_hwfn,
 	bool enable, ieee, eth_tlv;
 	u8 tc, priority_map;
 	u16 protocol_id;
-	u8 priority;
+	u8 priority = 0;
 	int rc = 0;
 	int i;
 
@@ -371,7 +372,10 @@ qed_dcbx_process_tlv(struct qed_hwfn *p_hwfn,
 		   "Num APP entries = %d pri_tc_tbl = 0x%x dcbx_version = %u\n",
 		   count, pri_tc_tbl, dcbx_version);
 
-	ieee = (dcbx_version == DCBX_CONFIG_VERSION_IEEE);
+	ieee = ((dcbx_version == DCBX_CONFIG_VERSION_IEEE) ||
+		((dcbx_version == DCBX_CONFIG_VERSION_DISABLED) &&
+		 p_hwfn->p_dcbx_info->is_admin_cfg_applied));
+
 	eth_tlv = false;
 	/* Parse APP TLV */
 	for (i = 0; i < count; i++) {
@@ -381,10 +385,10 @@ qed_dcbx_process_tlv(struct qed_hwfn *p_hwfn,
 		DP_VERBOSE(p_hwfn, QED_MSG_DCB, "Id = 0x%x pri_map = %u\n",
 			   protocol_id, priority_map);
 		rc = qed_dcbx_get_app_priority(priority_map, &priority);
-		if (rc == -EINVAL) {
-			DP_ERR(p_hwfn, "Invalid priority\n");
-			return -EINVAL;
-		}
+		if (rc == -EINVAL)
+			DP_ERR(p_hwfn,
+			       "Invalid priority Id = 0x%x pri_map = %u\n",
+			       protocol_id, priority_map);
 
 		tc = QED_DCBX_PRIO2TC(pri_tc_tbl, priority);
 		if (qed_dcbx_get_app_protocol_type(p_hwfn, p_tbl[i].entry,
@@ -575,6 +579,7 @@ qed_dcbx_get_app_data(struct qed_hwfn *p_hwfn,
 		      struct dcbx_app_priority_entry *p_tbl,
 		      struct qed_dcbx_params *p_params, bool ieee)
 {
+	int rc = 0;
 	struct qed_app_entry *entry;
 	u8 pri_map;
 	int i;
@@ -584,6 +589,8 @@ qed_dcbx_get_app_data(struct qed_hwfn *p_hwfn,
 	p_params->app_error = GET_MFW_FIELD(p_app->flags, DCBX_APP_ERROR);
 	p_params->num_app_entries = GET_MFW_FIELD(p_app->flags,
 						  DCBX_APP_NUM_ENTRIES);
+	if (p_params->num_app_entries > QED_DCBX_MAX_APP_PROTOCOL)
+		p_params->num_app_entries = QED_DCBX_MAX_APP_PROTOCOL;
 	for (i = 0; i < p_params->num_app_entries; i++) {
 		entry = &p_params->app_entry[i];
 		if (ieee) {
@@ -620,7 +627,11 @@ qed_dcbx_get_app_data(struct qed_hwfn *p_hwfn,
 		}
 
 		pri_map = GET_MFW_FIELD(p_tbl[i].entry, DCBX_APP_PRI_MAP);
-		qed_dcbx_get_app_priority(pri_map, &entry->prio);
+		rc = qed_dcbx_get_app_priority(pri_map, &entry->prio);
+		if (rc == -EINVAL)
+			DP_ERR(p_hwfn,
+			       "Invalid priority pri_map = %u\n", pri_map);
+
 		entry->proto_id = GET_MFW_FIELD(p_tbl[i].entry,
 						DCBX_APP_PROTOCOL_ID);
 		qed_dcbx_get_app_protocol_type(p_hwfn, p_tbl[i].entry,
@@ -787,12 +798,6 @@ qed_dcbx_get_operational_params(struct qed_hwfn *p_hwfn,
 	p_operational = &params->operational;
 	enabled = ! !(GET_MFW_FIELD(flags, DCBX_CONFIG_VERSION) !=
 		      DCBX_CONFIG_VERSION_DISABLED);
-	if (!enabled) {
-		p_operational->enabled = enabled;
-		p_operational->valid = false;
-		DP_VERBOSE(p_hwfn, QED_MSG_DCB, "Dcbx is disabled\n");
-		return;
-	}
 
 	p_feat = &p_hwfn->p_dcbx_info->operational.features;
 	p_results = &p_hwfn->p_dcbx_info->results;
@@ -1069,7 +1074,11 @@ qed_dcbx_mib_update_event(struct qed_hwfn *p_hwfn,
 
 	rc = qed_dcbx_read_mib(p_hwfn, p_ptt, type);
 	if (rc)
-		return rc;
+		goto dcbx_err;
+
+	DP_VERBOSE(p_hwfn, QED_MSG_DCB,
+		   "MFW DCBx: MIB Type:%d   Admin Notification:%d\n",
+		   type, p_hwfn->p_dcbx_info->is_admin_cfg_applied);
 
 	if (type == QED_DCBX_OPERATIONAL_MIB) {
 		qed_dcbx_get_dscp_params(p_hwfn, &p_hwfn->p_dcbx_info->get);
@@ -1127,6 +1136,11 @@ qed_dcbx_mib_update_event(struct qed_hwfn *p_hwfn,
 
 	qed_dcbx_aen(p_hwfn, type);
 
+dcbx_err:
+	if (type == QED_DCBX_OPERATIONAL_MIB &&
+	    p_hwfn->p_dcbx_info->is_admin_cfg_applied)
+		p_hwfn->p_dcbx_info->is_admin_cfg_applied = false;
+
 	return rc;
 }
 
@@ -1489,6 +1503,14 @@ int qed_dcbx_config_params(struct qed_hwfn *p_hwfn,
 			      data.size);
 	}
 
+	/* For Non static dcbx mode, send NOTIFY command to MFW. */
+	if (params->ver_num != DCBX_CONFIG_VERSION_STATIC &&
+	    params->override_flags & QED_DCBX_OVERRIDE_ADMIN_CFG) {
+		/* To receive notification from MFW */
+		SET_MFW_FIELD(drv_mb_param,
+			      DRV_MB_PARAM_DCBX_ADMIN_CFG_NOTIFY, 1);
+	}
+
 	SET_MFW_FIELD(drv_mb_param, DRV_MB_PARAM_LLDP_SEND, 1);
 	rc = qed_mcp_cmd(p_hwfn, p_ptt, DRV_MSG_CODE_SET_DCBX,
 			 drv_mb_param, &resp, &param);
@@ -1948,25 +1970,47 @@ static u8 qed_dcbnl_getstate(struct qed_dev *cdev)
 static u8 qed_dcbnl_setstate(struct qed_dev *cdev, u8 state)
 {
 	struct qed_hwfn *hwfn = QED_LEADING_HWFN(cdev);
+	struct qed_dcbx_get *dcbx_info;
 	struct qed_dcbx_set dcbx_set;
 	struct qed_ptt *ptt;
 	int rc;
 
 	DP_VERBOSE(hwfn, QED_MSG_DCB, "DCB state = %d\n", state);
 
+	if (qed_dcbnl_getstate(cdev) == state)
+		return 0;
+
+	if (!hwfn->p_dcbx_info->set.config.valid) {
+		dcbx_info = kmalloc(sizeof(*dcbx_info), GFP_KERNEL);
+		if (!dcbx_info)
+			return -ENOMEM;
+
+		memset(dcbx_info, 0, sizeof(*dcbx_info));
+		rc = qed_dcbx_query_params(hwfn, dcbx_info, QED_DCBX_LOCAL_MIB);
+		if (rc) {
+			kfree(dcbx_info);
+			return rc;
+		}
+
+		hwfn->p_dcbx_info->set.ver_num = DCBX_CONFIG_VERSION_STATIC;
+		memcpy(&hwfn->p_dcbx_info->set.dscp,
+		       &hwfn->p_dcbx_info->get.dscp,
+		       sizeof(struct qed_dcbx_dscp_params));
+		memcpy(&hwfn->p_dcbx_info->set.config.params,
+		       &dcbx_info->local.params,
+		       sizeof(hwfn->p_dcbx_info->set.config.params));
+		hwfn->p_dcbx_info->set.config.valid = true;
+
+		kfree(dcbx_info);
+	}
+
 	memset(&dcbx_set, 0, sizeof(dcbx_set));
-	rc = qed_dcbx_get_config_params(hwfn, &dcbx_set);
-	if (rc)
-		return 1;
-
-	dcbx_set.enabled = ! !state;
-
+	memcpy(&dcbx_set, &hwfn->p_dcbx_info->set, sizeof(struct qed_dcbx_set));
 	ptt = qed_ptt_acquire(hwfn);
 	if (!ptt)
 		return 1;
-
-	rc = qed_dcbx_config_params(hwfn, ptt, &dcbx_set, 0);
-
+	dcbx_set.enabled = ! !state;
+	rc = qed_dcbx_config_params(hwfn, ptt, &dcbx_set, 1);
 	qed_ptt_release(hwfn, ptt);
 
 	return rc ? 1 : 0;
@@ -2079,7 +2123,7 @@ static void qed_dcbnl_setpfccfg(struct qed_dev *cdev, int priority, u8 setting)
 	if (!ptt)
 		return;
 
-	rc = qed_dcbx_config_params(hwfn, ptt, &dcbx_set, 0);
+	rc = qed_dcbx_config_params(hwfn, ptt, &dcbx_set, 1);
 
 	qed_ptt_release(hwfn, ptt);
 }
@@ -2108,7 +2152,7 @@ static u8 qed_dcbnl_getcap(struct qed_dev *cdev, int capid, u8 * cap)
 		break;
 	case DCB_CAP_ATTR_DCBX:
 		*cap = (DCB_CAP_DCBX_VER_CEE | DCB_CAP_DCBX_VER_IEEE |
-			DCB_CAP_DCBX_STATIC);
+			DCB_CAP_DCBX_HOST);
 		break;
 	default:
 		*cap = false;
@@ -2176,12 +2220,7 @@ static u8 qed_dcbnl_getdcbx(struct qed_dev *cdev)
 	if (!dcbx_info)
 		return 0;
 
-	if (dcbx_info->operational.ieee)
-		mode |= DCB_CAP_DCBX_VER_IEEE;
-	if (dcbx_info->operational.cee)
-		mode |= DCB_CAP_DCBX_VER_CEE;
-	if (dcbx_info->operational.local)
-		mode |= DCB_CAP_DCBX_STATIC;
+	mode = DCB_CAP_DCBX_VER_IEEE | DCB_CAP_DCBX_VER_CEE | DCB_CAP_DCBX_HOST;
 
 	DP_VERBOSE(hwfn, QED_MSG_DCB, "dcb mode = %d\n", mode);
 	kfree(dcbx_info);
@@ -2219,7 +2258,7 @@ static void qed_dcbnl_setpgtccfgtx(struct qed_dev *cdev,
 	if (!ptt)
 		return;
 
-	rc = qed_dcbx_config_params(hwfn, ptt, &dcbx_set, 0);
+	rc = qed_dcbx_config_params(hwfn, ptt, &dcbx_set, 1);
 
 	qed_ptt_release(hwfn, ptt);
 }
@@ -2256,7 +2295,7 @@ static void qed_dcbnl_setpgbwgcfgtx(struct qed_dev *cdev, int pgid, u8 bw_pct)
 	if (!ptt)
 		return;
 
-	rc = qed_dcbx_config_params(hwfn, ptt, &dcbx_set, 0);
+	rc = qed_dcbx_config_params(hwfn, ptt, &dcbx_set, 1);
 
 	qed_ptt_release(hwfn, ptt);
 }
@@ -2282,6 +2321,7 @@ static u8 qed_dcbnl_setall(struct qed_dev *cdev)
 	if (!ptt)
 		return 1;
 
+	dcbx_set.ver_num = DCBX_CONFIG_VERSION_STATIC;
 	rc = qed_dcbx_config_params(hwfn, ptt, &dcbx_set, 1);
 
 	qed_ptt_release(hwfn, ptt);
@@ -2320,7 +2360,7 @@ static int qed_dcbnl_setnumtcs(struct qed_dev *cdev, int tcid, u8 num)
 	if (!ptt)
 		return -EINVAL;
 
-	rc = qed_dcbx_config_params(hwfn, ptt, &dcbx_set, 0);
+	rc = qed_dcbx_config_params(hwfn, ptt, &dcbx_set, 1);
 
 	qed_ptt_release(hwfn, ptt);
 
@@ -2348,7 +2388,7 @@ static void qed_dcbnl_setpfcstate(struct qed_dev *cdev, u8 state)
 	if (!ptt)
 		return;
 
-	rc = qed_dcbx_config_params(hwfn, ptt, &dcbx_set, 0);
+	rc = qed_dcbx_config_params(hwfn, ptt, &dcbx_set, 1);
 
 	qed_ptt_release(hwfn, ptt);
 }
@@ -2427,7 +2467,7 @@ static int qed_dcbnl_setapp(struct qed_dev *cdev,
 	if (!ptt)
 		return -EBUSY;
 
-	rc = qed_dcbx_config_params(hwfn, ptt, &dcbx_set, 0);
+	rc = qed_dcbx_config_params(hwfn, ptt, &dcbx_set, 1);
 
 	qed_ptt_release(hwfn, ptt);
 
@@ -2443,8 +2483,7 @@ static u8 qed_dcbnl_setdcbx(struct qed_dev *cdev, u8 mode)
 
 	DP_VERBOSE(hwfn, QED_MSG_DCB, "new mode = %x\n", mode);
 
-	if (!(mode & DCB_CAP_DCBX_VER_IEEE) &&
-	    !(mode & DCB_CAP_DCBX_VER_CEE) && !(mode & DCB_CAP_DCBX_STATIC)) {
+	if (!(mode & DCB_CAP_DCBX_VER_IEEE) && !(mode & DCB_CAP_DCBX_VER_CEE)) {
 		DP_INFO(hwfn, "Allowed modes are cee, ieee or static\n");
 		return 1;
 	}
@@ -2454,27 +2493,13 @@ static u8 qed_dcbnl_setdcbx(struct qed_dev *cdev, u8 mode)
 	if (rc)
 		return 1;
 
-	dcbx_set.ver_num = 0;
-	if (mode & DCB_CAP_DCBX_VER_CEE) {
-		dcbx_set.ver_num |= DCBX_CONFIG_VERSION_CEE;
-		dcbx_set.enabled = true;
-	}
-
-	if (mode & DCB_CAP_DCBX_VER_IEEE) {
-		dcbx_set.ver_num |= DCBX_CONFIG_VERSION_IEEE;
-		dcbx_set.enabled = true;
-	}
-
-	if (mode & DCB_CAP_DCBX_STATIC) {
-		dcbx_set.ver_num |= DCBX_CONFIG_VERSION_STATIC;
-		dcbx_set.enabled = true;
-	}
+	dcbx_set.ver_num = DCBX_CONFIG_VERSION_STATIC;
 
 	ptt = qed_ptt_acquire(hwfn);
 	if (!ptt)
 		return 1;
 
-	rc = qed_dcbx_config_params(hwfn, ptt, &dcbx_set, 0);
+	rc = qed_dcbx_config_params(hwfn, ptt, &dcbx_set, 1);
 
 	qed_ptt_release(hwfn, ptt);
 
@@ -2564,7 +2589,7 @@ static u8 qed_dcbnl_setfeatcfg(struct qed_dev *cdev, int featid, u8 flags)
 	if (!ptt)
 		return 1;
 
-	rc = qed_dcbx_config_params(hwfn, ptt, &dcbx_set, 0);
+	rc = qed_dcbx_config_params(hwfn, ptt, &dcbx_set, 1);
 
 	qed_ptt_release(hwfn, ptt);
 
@@ -2674,12 +2699,6 @@ static int qed_dcbnl_get_ieee_pfc(struct qed_dev *cdev,
 	if (!dcbx_info)
 		return -EINVAL;
 
-	if (!dcbx_info->operational.ieee) {
-		DP_INFO(hwfn, "DCBX is not enabled/operational in IEEE mode\n");
-		kfree(dcbx_info);
-		return -EINVAL;
-	}
-
 	if (remote) {
 		memset(dcbx_info, 0, sizeof(*dcbx_info));
 		rc = qed_dcbx_query_params(hwfn, dcbx_info,
@@ -2713,23 +2732,10 @@ static int qed_dcbnl_ieee_getpfc(struct qed_dev *cdev, struct ieee_pfc *pfc)
 static int qed_dcbnl_ieee_setpfc(struct qed_dev *cdev, struct ieee_pfc *pfc)
 {
 	struct qed_hwfn *hwfn = QED_LEADING_HWFN(cdev);
-	struct qed_dcbx_get *dcbx_info;
 	struct qed_dcbx_set dcbx_set;
 	struct qed_ptt *ptt;
 	int rc, i;
 
-	dcbx_info = qed_dcbnl_get_dcbx(hwfn, QED_DCBX_OPERATIONAL_MIB);
-	if (!dcbx_info)
-		return -EINVAL;
-
-	if (!dcbx_info->operational.ieee) {
-		DP_INFO(hwfn, "DCBX is not enabled/operational in IEEE mode\n");
-		kfree(dcbx_info);
-		return -EINVAL;
-	}
-
-	kfree(dcbx_info);
-
 	memset(&dcbx_set, 0, sizeof(dcbx_set));
 	rc = qed_dcbx_get_config_params(hwfn, &dcbx_set);
 	if (rc)
@@ -2745,7 +2751,7 @@ static int qed_dcbnl_ieee_setpfc(struct qed_dev *cdev, struct ieee_pfc *pfc)
 	if (!ptt)
 		return -EINVAL;
 
-	rc = qed_dcbx_config_params(hwfn, ptt, &dcbx_set, 0);
+	rc = qed_dcbx_config_params(hwfn, ptt, &dcbx_set, 1);
 
 	qed_ptt_release(hwfn, ptt);
 
@@ -2764,12 +2770,6 @@ static int qed_dcbnl_get_ieee_ets(struct qed_dev *cdev,
 	if (!dcbx_info)
 		return -EINVAL;
 
-	if (!dcbx_info->operational.ieee) {
-		DP_INFO(hwfn, "DCBX is not enabled/operational in IEEE mode\n");
-		kfree(dcbx_info);
-		return -EINVAL;
-	}
-
 	if (remote) {
 		memset(dcbx_info, 0, sizeof(*dcbx_info));
 		rc = qed_dcbx_query_params(hwfn, dcbx_info,
@@ -2803,23 +2803,10 @@ static int qed_dcbnl_ieee_getets(struct qed_dev *cdev, struct ieee_ets *ets)
 static int qed_dcbnl_ieee_setets(struct qed_dev *cdev, struct ieee_ets *ets)
 {
 	struct qed_hwfn *hwfn = QED_LEADING_HWFN(cdev);
-	struct qed_dcbx_get *dcbx_info;
 	struct qed_dcbx_set dcbx_set;
 	struct qed_ptt *ptt;
 	int rc;
 
-	dcbx_info = qed_dcbnl_get_dcbx(hwfn, QED_DCBX_OPERATIONAL_MIB);
-	if (!dcbx_info)
-		return -EINVAL;
-
-	if (!dcbx_info->operational.ieee) {
-		DP_INFO(hwfn, "DCBX is not enabled/operational in IEEE mode\n");
-		kfree(dcbx_info);
-		return -EINVAL;
-	}
-
-	kfree(dcbx_info);
-
 	memset(&dcbx_set, 0, sizeof(dcbx_set));
 	rc = qed_dcbx_get_config_params(hwfn, &dcbx_set);
 	if (rc)
@@ -2840,7 +2827,7 @@ static int qed_dcbnl_ieee_setets(struct qed_dev *cdev, struct ieee_ets *ets)
 	if (!ptt)
 		return -EINVAL;
 
-	rc = qed_dcbx_config_params(hwfn, ptt, &dcbx_set, 0);
+	rc = qed_dcbx_config_params(hwfn, ptt, &dcbx_set, 1);
 
 	qed_ptt_release(hwfn, ptt);
 
@@ -2904,12 +2891,6 @@ static int qed_dcbnl_ieee_getapp(struct qed_dev *cdev, struct dcb_app *app)
 	if (!dcbx_info)
 		return -EINVAL;
 
-	if (!dcbx_info->operational.ieee) {
-		DP_INFO(hwfn, "DCBX is not enabled/operational in IEEE mode\n");
-		kfree(dcbx_info);
-		return -EINVAL;
-	}
-
 	for (i = 0; i < QED_DCBX_MAX_APP_PROTOCOL; i++) {
 		entry = &dcbx_info->operational.params.app_entry[i];
 		if ((entry->sf_ieee == sf_ieee) &&
@@ -2936,7 +2917,6 @@ static int qed_dcbnl_ieee_getapp(struct qed_dev *cdev, struct dcb_app *app)
 static int qed_dcbnl_ieee_setapp(struct qed_dev *cdev, struct dcb_app *app)
 {
 	struct qed_hwfn *hwfn = QED_LEADING_HWFN(cdev);
-	struct qed_dcbx_get *dcbx_info;
 	struct qed_dcbx_set dcbx_set;
 	struct qed_app_entry *entry;
 	struct qed_ptt *ptt;
@@ -2956,18 +2936,6 @@ static int qed_dcbnl_ieee_setapp(struct qed_dev *cdev, struct dcb_app *app)
 		return -EINVAL;
 	}
 
-	dcbx_info = qed_dcbnl_get_dcbx(hwfn, QED_DCBX_OPERATIONAL_MIB);
-	if (!dcbx_info)
-		return -EINVAL;
-
-	if (!dcbx_info->operational.ieee) {
-		DP_INFO(hwfn, "DCBX is not enabled/operational in IEEE mode\n");
-		kfree(dcbx_info);
-		return -EINVAL;
-	}
-
-	kfree(dcbx_info);
-
 	memset(&dcbx_set, 0, sizeof(dcbx_set));
 	rc = qed_dcbx_get_config_params(hwfn, &dcbx_set);
 	if (rc)
@@ -2999,7 +2967,7 @@ static int qed_dcbnl_ieee_setapp(struct qed_dev *cdev, struct dcb_app *app)
 	if (!ptt)
 		return -EBUSY;
 
-	rc = qed_dcbx_config_params(hwfn, ptt, &dcbx_set, 0);
+	rc = qed_dcbx_config_params(hwfn, ptt, &dcbx_set, 1);
 
 	qed_ptt_release(hwfn, ptt);
 
