@@ -42,7 +42,7 @@
 #include <linux/spinlock.h>
 #include "qed.h"
 #include "qed_dev_api.h"
-#include "qed_hsi.h"
+#include "qed_mfw_hsi.h"
 #include "qed_fcoe_if.h"
 #include "qed_iscsi_if.h"
 
@@ -291,6 +291,17 @@ union qed_mfw_tlv_data {
 #define QED_NVM_CFG_OPTION_COMMIT       (1 << 2)
 #define QED_NVM_CFG_OPTION_FREE (1 << 3)
 #define QED_NVM_CFG_OPTION_ENTITY_SEL   (1 << 4)
+#define QED_NVM_CFG_GET_FLAGS           0xA
+#define QED_NVM_CFG_GET_PF_FLAGS        0x1A
+#define QED_NVM_CFG_MAX_ATTRS           50
+
+enum qed_nvm_flash_cmd {
+	QED_NVM_FLASH_CMD_FILE_DATA = 0x2,
+	QED_NVM_FLASH_CMD_FILE_START = 0x3,
+	QED_NVM_FLASH_CMD_NVM_CHANGE = 0x4,
+	QED_NVM_FLASH_CMD_NVM_CFG_ID = 0x5,
+	QED_NVM_FLASH_CMD_NVM_MAX,
+};
 
 /**
  * @brief - returns the link params of the hw function
@@ -420,7 +431,7 @@ int qed_mcp_get_board_config(struct qed_hwfn *p_hwfn,
 			     struct qed_ptt *p_ptt, u32 * p_board_config);
 
 /**
- * @brief - Sends a command to the MCP mailbox.
+ * @brief - Sends a command to the MCP mailbox from sleepable context.
  *
  * @param p_hwfn      - hw function
  * @param p_ptt       - PTT required for register access
@@ -437,6 +448,25 @@ int qed_mcp_cmd(struct qed_hwfn *p_hwfn,
 		struct qed_ptt *p_ptt,
 		u32 cmd, u32 param, u32 * o_mcp_resp, u32 * o_mcp_param);
 
+/**
+ * @brief - Sends a command to the MCP mailbox from non-sleepable context.
+ *
+ * @param p_hwfn      - hw function
+ * @param p_ptt       - PTT required for register access
+ * @param cmd         - command to be sent to the MCP
+ * @param param       - Optional param
+ * @param o_mcp_resp  - The MCP response code (exclude sequence)
+ * @param o_mcp_param - Optional parameter provided by the MCP response
+ *
+ * @return int -
+ *      0 - operation was successful
+ *      -EBUSY    - operation failed
+ */
+int qed_mcp_cmd_nosleep(struct qed_hwfn *p_hwfn,
+			struct qed_ptt *p_ptt,
+			u32 cmd,
+			u32 param, u32 * o_mcp_resp, u32 * o_mcp_param);
+
 /**
  * @brief - drains the nig, allowing completion to pass in case of pauses.
  *          (Should be called only from sleepable context)
@@ -570,7 +600,8 @@ int qed_mcp_ov_update_mtu(struct qed_hwfn *p_hwfn,
  * @return int - 0 - operation was successful.
  */
 int
-qed_mcp_ov_update_mac(struct qed_hwfn *p_hwfn, struct qed_ptt *p_ptt, u8 * mac);
+qed_mcp_ov_update_mac(struct qed_hwfn *p_hwfn,
+		      struct qed_ptt *p_ptt, const u8 * mac);
 
 /**
  * @brief Send WOL mode to MFW
@@ -965,6 +996,28 @@ int qed_mcp_mdump_clear_logs(struct qed_hwfn *p_hwfn, struct qed_ptt *p_ptt);
  */
 int qed_mcp_mdump_clr_retain(struct qed_hwfn *p_hwfn, struct qed_ptt *p_ptt);
 
+/**
+ * @brief - Get mdump2 offset and size.
+ *
+ * @param p_hwfn
+ * @param p_ptt
+ * @param buff_byte_size - pointer to get mdump2 buffer size in bytes
+ * @param buff_byte_addr - pointer to get mdump2 buffer buffer address.
+ * @param return 0 upon success.
+ */
+int qed_mdump2_req_offsize(struct qed_hwfn *p_hwfn,
+			   struct qed_ptt *p_ptt,
+			   u32 * buff_byte_size, u32 * buff_byte_addr);
+
+/**
+ * @brief - Request to free mdump2 buffer.
+ *
+ * @param p_hwfn
+ * @param p_ptt
+ * @param return 0 upon success.
+ */
+int qed_mdump2_req_free(struct qed_hwfn *p_hwfn, struct qed_ptt *p_ptt);
+
 /**
  * @brief - Gets the LLDP MAC address.
  *
@@ -1140,6 +1193,16 @@ int
 qed_mcp_get_esl_status(struct qed_hwfn *p_hwfn,
 		       struct qed_ptt *p_ptt, bool * active);
 
+/**
+ * @brief - Instruct mfw to collect idlechk and fw asserts.
+ *
+ * @param p_hwfn
+ * @param p_ptt
+ *
+ * @param return 0 upon success.
+ */
+int qed_mcp_gen_mdump_idlechk(struct qed_hwfn *p_hwfn, struct qed_ptt *p_ptt);
+
 /* Using hwfn number (and not pf_num) is required since in CMT mode,
  * same pf_num may be used by two different hwfn
  * TODO - this shouldn't really be in .h file, but until all fields
@@ -1170,6 +1233,11 @@ struct qed_mcp_info {
 	/* Flag to indicate whether MFW was halted */
 	bool b_halted;
 
+	/* Flag to indicate that driver discovered that mcp is running in
+	 * recovery mode.
+	 */
+	bool recovery_mode;
+
 	/* Spinlock used for syncing SW link-changes and link-changes
 	 * originating from attention context.
 	 */
@@ -1183,6 +1251,8 @@ struct qed_mcp_info {
 	u32 mfw_mb_addr;
 	/* Address of the port configuration (link) */
 	u32 port_addr;
+	/* size of the port configuration (for compatability) */
+	u32 port_size;
 
 	/* Current driver mailbox sequence */
 	u16 drv_mb_seq;
@@ -1212,6 +1282,188 @@ struct qed_mcp_info {
 #define QED_MCP_IN_PROCESSING_BIT       1
 };
 
+struct qed_mcp_name_table {
+	u32 value;
+	char name[64];
+};
+
+static const struct qed_mcp_name_table qed_mcp_cmd_name_table[] = {
+	{DRV_MSG_CODE_NVM_PUT_FILE_BEGIN,
+	 "DRV_MSG_CODE_NVM_PUT_FILE_BEGIN"},
+	{DRV_MSG_CODE_NVM_PUT_FILE_DATA,
+	 "DRV_MSG_CODE_NVM_PUT_FILE_DATA"},
+	{DRV_MSG_CODE_NVM_GET_FILE_ATT,
+	 "DRV_MSG_CODE_NVM_GET_FILE_ATT"},
+	{DRV_MSG_CODE_NVM_READ_NVRAM,
+	 "DRV_MSG_CODE_NVM_READ_NVRAM"},
+	{DRV_MSG_CODE_NVM_WRITE_NVRAM,
+	 "DRV_MSG_CODE_NVM_WRITE_NVRAM"},
+	{DRV_MSG_CODE_NVM_DEL_FILE, "DRV_MSG_CODE_NVM_DEL_FILE"},
+	{DRV_MSG_CODE_MCP_RESET, "DRV_MSG_CODE_MCP_RESET"},
+	{DRV_MSG_CODE_PHY_RAW_READ, "DRV_MSG_CODE_PHY_RAW_READ"},
+	{DRV_MSG_CODE_PHY_RAW_WRITE,
+	 "DRV_MSG_CODE_PHY_RAW_WRITE"},
+	{DRV_MSG_CODE_PHY_CORE_READ,
+	 "DRV_MSG_CODE_PHY_CORE_READ"},
+	{DRV_MSG_CODE_PHY_CORE_WRITE,
+	 "DRV_MSG_CODE_PHY_CORE_WRITE"},
+	{DRV_MSG_CODE_SET_VERSION, "DRV_MSG_CODE_SET_VERSION"},
+	{DRV_MSG_CODE_MCP_HALT, "DRV_MSG_CODE_MCP_HALT"},
+	{DRV_MSG_CODE_SET_VMAC, "DRV_MSG_CODE_SET_VMAC"},
+	{DRV_MSG_CODE_GET_VMAC, "DRV_MSG_CODE_GET_VMAC"},
+	{DRV_MSG_CODE_GET_STATS, "DRV_MSG_CODE_GET_STATS"},
+	{DRV_MSG_CODE_PMD_DIAG_DUMP,
+	 "DRV_MSG_CODE_PMD_DIAG_DUMP"},
+	{DRV_MSG_CODE_PMD_DIAG_EYE, "DRV_MSG_CODE_PMD_DIAG_EYE"},
+	{DRV_MSG_CODE_TRANSCEIVER_READ,
+	 "DRV_MSG_CODE_TRANSCEIVER_READ"},
+	{DRV_MSG_CODE_TRANSCEIVER_WRITE,
+	 "DRV_MSG_CODE_TRANSCEIVER_WRITE"},
+	{DRV_MSG_CODE_OCBB_DATA, "DRV_MSG_CODE_OCBB_DATA"},
+	{DRV_MSG_CODE_SET_BW, "DRV_MSG_CODE_SET_BW"},
+	{DRV_MSG_CODE_MASK_PARITIES,
+	 "DRV_MSG_CODE_MASK_PARITIES"},
+	{DRV_MSG_CODE_INDUCE_FAILURE,
+	 "DRV_MSG_CODE_INDUCE_FAILURE"},
+	{DRV_MSG_CODE_GPIO_READ, "DRV_MSG_CODE_GPIO_READ"},
+	{DRV_MSG_CODE_GPIO_WRITE, "DRV_MSG_CODE_GPIO_WRITE"},
+	{DRV_MSG_CODE_BIST_TEST, "DRV_MSG_CODE_BIST_TEST"},
+	{DRV_MSG_CODE_GET_TEMPERATURE,
+	 "DRV_MSG_CODE_GET_TEMPERATURE"},
+	{DRV_MSG_CODE_SET_LED_MODE, "DRV_MSG_CODE_SET_LED_MODE"},
+	{DRV_MSG_CODE_TIMESTAMP, "DRV_MSG_CODE_TIMESTAMP"},
+	{DRV_MSG_CODE_EMPTY_MB, "DRV_MSG_CODE_EMPTY_MB"},
+	{DRV_MSG_CODE_RESOURCE_CMD, "DRV_MSG_CODE_RESOURCE_CMD"},
+	{DRV_MSG_CODE_GET_MBA_VERSION,
+	 "DRV_MSG_CODE_GET_MBA_VERSION"},
+	{DRV_MSG_CODE_MDUMP_CMD, "DRV_MSG_CODE_MDUMP_CMD"},
+	{DRV_MSG_CODE_MEM_ECC_EVENTS,
+	 "DRV_MSG_CODE_MEM_ECC_EVENTS"},
+	{DRV_MSG_CODE_GPIO_INFO, "DRV_MSG_CODE_GPIO_INFO"},
+	{DRV_MSG_CODE_EXT_PHY_READ, "DRV_MSG_CODE_EXT_PHY_READ"},
+	{DRV_MSG_CODE_EXT_PHY_WRITE,
+	 "DRV_MSG_CODE_EXT_PHY_WRITE"},
+	{DRV_MSG_CODE_EXT_PHY_FW_UPGRADE,
+	 "DRV_MSG_CODE_EXT_PHY_FW_UPGRADE"},
+	{DRV_MSG_CODE_GET_PF_RDMA_PROTOCOL,
+	 "DRV_MSG_CODE_GET_PF_RDMA_PROTOCOL"},
+	{DRV_MSG_CODE_SET_LLDP_MAC, "DRV_MSG_CODE_SET_LLDP_MAC"},
+	{DRV_MSG_CODE_GET_LLDP_MAC, "DRV_MSG_CODE_GET_LLDP_MAC"},
+	{DRV_MSG_CODE_OS_WOL, "DRV_MSG_CODE_OS_WOL"},
+	{DRV_MSG_CODE_GET_TLV_DONE, "DRV_MSG_CODE_GET_TLV_DONE"},
+	{DRV_MSG_CODE_FEATURE_SUPPORT,
+	 "DRV_MSG_CODE_FEATURE_SUPPORT"},
+	{DRV_MSG_CODE_GET_MFW_FEATURE_SUPPORT,
+	 "DRV_MSG_CODE_GET_MFW_FEATURE_SUPPORT"},
+	{DRV_MSG_CODE_READ_WOL_REG, "DRV_MSG_CODE_READ_WOL_REG"},
+	{DRV_MSG_CODE_WRITE_WOL_REG,
+	 "DRV_MSG_CODE_WRITE_WOL_REG"},
+	{DRV_MSG_CODE_GET_WOL_BUFFER,
+	 "DRV_MSG_CODE_GET_WOL_BUFFER"},
+	{DRV_MSG_CODE_ATTRIBUTE, "DRV_MSG_CODE_ATTRIBUTE"},
+	{DRV_MSG_CODE_ENCRYPT_PASSWORD,
+	 "DRV_MSG_CODE_ENCRYPT_PASSWORD"},
+	{DRV_MSG_CODE_GET_ENGINE_CONFIG,
+	 "DRV_MSG_CODE_GET_ENGINE_CONFIG"},
+	{DRV_MSG_CODE_PMBUS_READ, "DRV_MSG_CODE_PMBUS_READ"},
+	{DRV_MSG_CODE_PMBUS_WRITE, "DRV_MSG_CODE_PMBUS_WRITE"},
+	{DRV_MSG_CODE_GENERIC_IDC, "DRV_MSG_CODE_GENERIC_IDC"},
+	{DRV_MSG_CODE_RESET_CHIP, "DRV_MSG_CODE_RESET_CHIP"},
+	{DRV_MSG_CODE_SET_RETAIN_VMAC,
+	 "DRV_MSG_CODE_SET_RETAIN_VMAC"},
+	{DRV_MSG_CODE_GET_RETAIN_VMAC,
+	 "DRV_MSG_CODE_GET_RETAIN_VMAC"},
+	{DRV_MSG_CODE_GET_NVM_CFG_OPTION,
+	 "DRV_MSG_CODE_GET_NVM_CFG_OPTION"},
+	{DRV_MSG_CODE_SET_NVM_CFG_OPTION,
+	 "DRV_MSG_CODE_SET_NVM_CFG_OPTION"},
+	{DRV_MSG_CODE_PCIE_STATS_START,
+	 "DRV_MSG_CODE_PCIE_STATS_START"},
+	{DRV_MSG_CODE_PCIE_STATS_GET,
+	 "DRV_MSG_CODE_PCIE_STATS_GET"},
+	{DRV_MSG_CODE_GET_ATTN_CONTROL,
+	 "DRV_MSG_CODE_GET_ATTN_CONTROL"},
+	{DRV_MSG_CODE_SET_ATTN_CONTROL,
+	 "DRV_MSG_CODE_SET_ATTN_CONTROL"},
+	{DRV_MSG_CODE_SET_TRACE_FILTER,
+	 "DRV_MSG_CODE_SET_TRACE_FILTER"},
+	{DRV_MSG_CODE_RESTORE_TRACE_FILTER,
+	 "DRV_MSG_CODE_RESTORE_TRACE_FILTER"},
+	{DRV_MSG_CODE_INITIATE_FLR_DEPRECATED,
+	 "DRV_MSG_CODE_INITIATE_FLR_DEPRECATED"},
+	{DRV_MSG_CODE_INITIATE_PF_FLR,
+	 "DRV_MSG_CODE_INITIATE_PF_FLR"},
+	{DRV_MSG_CODE_INITIATE_VF_FLR,
+	 "DRV_MSG_CODE_INITIATE_VF_FLR"},
+	{DRV_MSG_CODE_LOAD_REQ, "DRV_MSG_CODE_LOAD_REQ"},
+	{DRV_MSG_CODE_LOAD_DONE, "DRV_MSG_CODE_LOAD_DONE"},
+	{DRV_MSG_CODE_INIT_HW, "DRV_MSG_CODE_INIT_HW"},
+	{DRV_MSG_CODE_CANCEL_LOAD_REQ,
+	 "DRV_MSG_CODE_CANCEL_LOAD_REQ"},
+	{DRV_MSG_CODE_UNLOAD_REQ, "DRV_MSG_CODE_UNLOAD_REQ"},
+	{DRV_MSG_CODE_UNLOAD_DONE, "DRV_MSG_CODE_UNLOAD_DONE"},
+	{DRV_MSG_CODE_INIT_PHY, "DRV_MSG_CODE_INIT_PHY"},
+	{DRV_MSG_CODE_LINK_RESET, "DRV_MSG_CODE_LINK_RESET"},
+	{DRV_MSG_CODE_SET_LLDP, "DRV_MSG_CODE_SET_LLDP"},
+	{DRV_MSG_CODE_REGISTER_LLDP_TLVS_RX,
+	 "DRV_MSG_CODE_REGISTER_LLDP_TLVS_RX"},
+	{DRV_MSG_CODE_SET_DCBX, "DRV_MSG_CODE_SET_DCBX"},
+	{DRV_MSG_CODE_OV_UPDATE_CURR_CFG,
+	 "DRV_MSG_CODE_OV_UPDATE_CURR_CFG"},
+	{DRV_MSG_CODE_OV_UPDATE_BUS_NUM,
+	 "DRV_MSG_CODE_OV_UPDATE_BUS_NUM"},
+	{DRV_MSG_CODE_OV_UPDATE_BOOT_PROGRESS,
+	 "DRV_MSG_CODE_OV_UPDATE_BOOT_PROGRESS"},
+	{DRV_MSG_CODE_OV_UPDATE_STORM_FW_VER,
+	 "DRV_MSG_CODE_OV_UPDATE_STORM_FW_VER"},
+	{DRV_MSG_CODE_NIG_DRAIN, "DRV_MSG_CODE_NIG_DRAIN"},
+	{DRV_MSG_CODE_OV_UPDATE_DRIVER_STATE,
+	 "DRV_MSG_CODE_OV_UPDATE_DRIVER_STATE"},
+	{DRV_MSG_CODE_BW_UPDATE_ACK,
+	 "DRV_MSG_CODE_BW_UPDATE_ACK"},
+	{DRV_MSG_CODE_OV_UPDATE_MTU,
+	 "DRV_MSG_CODE_OV_UPDATE_MTU"},
+	{DRV_MSG_CODE_OV_UPDATE_WOL,
+	 "DRV_MSG_CODE_OV_UPDATE_WOL"},
+	{DRV_MSG_CODE_OV_UPDATE_ESWITCH_MODE,
+	 "DRV_MSG_CODE_OV_UPDATE_ESWITCH_MODE"},
+	{DRV_MSG_CODE_S_TAG_UPDATE_ACK,
+	 "DRV_MSG_CODE_S_TAG_UPDATE_ACK"},
+	{DRV_MSG_CODE_OEM_UPDATE_FCOE_CVID,
+	 "DRV_MSG_CODE_OEM_UPDATE_FCOE_CVID"},
+	{DRV_MSG_CODE_OEM_UPDATE_FCOE_FABRIC_NAME,
+	 "DRV_MSG_CODE_OEM_UPDATE_FCOE_FABRIC_NAME"},
+	{DRV_MSG_CODE_OEM_UPDATE_BOOT_CFG,
+	 "DRV_MSG_CODE_OEM_UPDATE_BOOT_CFG"},
+	{DRV_MSG_CODE_OEM_RESET_TO_DEFAULT,
+	 "DRV_MSG_CODE_OEM_RESET_TO_DEFAULT"},
+	{DRV_MSG_CODE_OV_GET_CURR_CFG,
+	 "DRV_MSG_CODE_OV_GET_CURR_CFG"},
+	{DRV_MSG_CODE_GET_OEM_UPDATES,
+	 "DRV_MSG_CODE_GET_OEM_UPDATES"},
+	{DRV_MSG_CODE_GET_LLDP_STATS,
+	 "DRV_MSG_CODE_GET_LLDP_STATS"},
+	{DRV_MSG_CODE_GET_PPFID_BITMAP,
+	 "DRV_MSG_CODE_GET_PPFID_BITMAP"},
+	{DRV_MSG_CODE_VF_DISABLED_DONE,
+	 "DRV_MSG_CODE_VF_DISABLED_DONE"},
+	{DRV_MSG_CODE_CFG_VF_MSIX, "DRV_MSG_CODE_CFG_VF_MSIX"},
+	{DRV_MSG_CODE_CFG_PF_VFS_MSIX,
+	 "DRV_MSG_CODE_CFG_PF_VFS_MSIX"},
+	{DRV_MSG_CODE_GET_PERM_MAC, "DRV_MSG_CODE_GET_PERM_MAC"},
+	{DRV_MSG_CODE_DEBUG_DATA_SEND,
+	 "DRV_MSG_CODE_DEBUG_DATA_SEND"},
+	{DRV_MSG_CODE_GET_FCOE_CAP, "DRV_MSG_CODE_GET_FCOE_CAP"},
+	{DRV_MSG_CODE_VF_WITH_MORE_16SB,
+	 "DRV_MSG_CODE_VF_WITH_MORE_16SB"},
+	{DRV_MSG_CODE_GET_MANAGEMENT_STATUS,
+	 "DRV_MSG_CODE_GET_MANAGEMENT_STATUS"},
+	{DRV_MSG_GET_RESOURCE_ALLOC_MSG,
+	 "DRV_MSG_GET_RESOURCE_ALLOC_MSG"},
+	{DRV_MSG_SET_RESOURCE_VALUE_MSG,
+	 "DRV_MSG_SET_RESOURCE_VALUE_MSG"},
+};				/* LOOKUP_TABLE_END */
+
 struct qed_mcp_mb_params {
 	u32 cmd;
 	u32 param;
@@ -1648,7 +1900,8 @@ qed_mcp_resc_unlock(struct qed_hwfn *p_hwfn,
  * @param resource - the requested resource
  * @paral b_is_permanent - disable retries & aging when set
  */
-void qed_mcp_resc_lock_default_init(struct qed_resc_lock_params *p_lock,
+void qed_mcp_resc_lock_default_init(struct qed_hwfn *p_hwfn,
+				    struct qed_resc_lock_params *p_lock,
 				    struct qed_resc_unlock_params *p_unlock,
 				    enum qed_resc_lock
 				    resource, bool b_is_permanent);
@@ -1725,6 +1978,14 @@ qed_mcp_drv_attribute(struct qed_hwfn *p_hwfn,
  */
 void qed_mcp_read_ufp_config(struct qed_hwfn *p_hwfn, struct qed_ptt *p_ptt);
 
+/**
+ * @brief Read qinq config from the shared memory.
+ *
+ * @param p_hwfn
+ * @param p_ptt
+ */
+void qed_mcp_read_qinq_config(struct qed_hwfn *p_hwfn, struct qed_ptt *p_ptt);
+
 /**
  * @brief Get the engine affinity configuration.
  *
@@ -1768,6 +2029,13 @@ int qed_mcp_ind_table_unlock(struct qed_hwfn *p_hwfn, struct qed_ptt *p_ptt);
  */
 int qed_mcp_nvm_info_populate(struct qed_hwfn *p_hwfn);
 
+/**
+ * @brief Delete nvm info shadow in the given hardware function
+ *
+ * @param p_hwfn
+ */
+void qed_mcp_nvm_info_free(struct qed_hwfn *p_hwfn);
+
 int
 qed_mcp_set_trace_filter(struct qed_hwfn *p_hwfn,
 			 u32 * dbg_level, u32 * dbg_modules);
