@@ -45,6 +45,7 @@
 #include "qed_hsi.h"
 #include "qed_hw.h"
 #include "qed_int.h"
+#include "qed_iro_hsi.h"
 #include "qed_reg_addr.h"
 #include "qed_sp.h"
 #include "qed_sriov.h"
@@ -78,7 +79,7 @@ int qed_sp_init_request(struct qed_hwfn *p_hwfn,
 	/* Fill the SPQ entry */
 	p_ent = *pp_ent;
 	p_ent->elem.hdr.cid = cpu_to_le32(opaque_cid);
-	p_ent->elem.hdr.cmd_id = cmd;
+	p_ent->elem.hdr.cmd_id.raw = cmd;
 	p_ent->elem.hdr.protocol_id = protocol;
 	p_ent->priority = QED_SPQ_PRIORITY_NORMAL;
 	p_ent->comp_mode = p_data->comp_mode;
@@ -111,9 +112,11 @@ int qed_sp_init_request(struct qed_hwfn *p_hwfn,
 
 	DP_VERBOSE(p_hwfn,
 		   QED_MSG_SPQ,
-		   "Initialized: CID %08x cmd %02x protocol %02x data_addr %llx comp_mode [%s]\n",
+		   "Initialized: CID %08x %s:[%02x] %s:%02x data_addr %llx comp_mode [%s]\n",
 		   opaque_cid,
+		   qed_get_ramrod_cmd_id_str(protocol, cmd),
 		   cmd,
+		   qed_get_protocol_type_str(protocol),
 		   protocol,
 		   (unsigned long long)(uintptr_t) & p_ent->ramrod,
 		   D_TRINE(p_ent->comp_mode, QED_SPQ_MODE_EBLOCK,
@@ -396,13 +399,21 @@ int qed_sp_pf_start(struct qed_hwfn *p_hwfn,
 		    cpu_to_le16(((u16) p_hwfn->ufp_info.tc << 13));
 	}
 
+	if (test_bit(QED_MF_QINQ_SPECIFIC, &p_hwfn->cdev->mf_bits))
+		p_ramrod->outer_tag_config.outer_tag.tci |=
+		    cpu_to_le16(((u16) p_hwfn->qinq_tc << 13));
+
 	/* Place EQ address in RAMROD */
 	DMA_REGPAIR_LE(p_ramrod->event_ring_pbl_addr,
-		       p_hwfn->p_eq->chain.pbl_sp.p_phys_table);
+		       qed_chain_get_pbl_phys(&p_hwfn->p_eq->chain));
 	page_cnt = (u8) qed_chain_get_page_cnt(&p_hwfn->p_eq->chain);
 	p_ramrod->event_ring_num_pages = page_cnt;
-	DMA_REGPAIR_LE(p_ramrod->consolid_q_pbl_addr,
-		       p_hwfn->p_consq->chain.pbl_sp.p_phys_table);
+
+	/* Place consolidation queue address in ramrod */
+	DMA_REGPAIR_LE(p_ramrod->consolid_q_pbl_base_addr,
+		       qed_chain_get_pbl_phys(&p_hwfn->p_consq->chain));
+	page_cnt = (u8) qed_chain_get_page_cnt(&p_hwfn->p_consq->chain);
+	p_ramrod->consolid_q_num_pages = page_cnt;
 
 	qed_tunn_set_pf_start_params(p_hwfn, p_tunn, &p_ramrod->tunnel_config);
 
