@@ -52,6 +52,7 @@
 #include "qed_hsi.h"
 #include "qed_hw.h"
 #include "qed_int.h"
+#include "qed_iro_hsi.h"
 #include "qed_iscsi.h"
 #include "qed_mcp.h"
 #include "qed_ooo.h"
@@ -111,6 +112,15 @@ static int __qed_spq_block(struct qed_hwfn *p_hwfn,
 #endif
 
 	while (iter_cnt--) {
+		if (QED_RECOV_IN_PROG(p_hwfn->cdev)) {
+			DP_VERBOSE(p_hwfn,
+				   QED_MSG_SPQ,
+				   "Recovery is in progress. Skip completion. iter_cnt %d\n",
+				   iter_cnt);
+			/* return success to skip comp time-out handling */
+			return 0;
+		}
+
 		smp_rmb();
 		if (comp_done->done == 1) {
 			if (p_fw_ret)
@@ -118,10 +128,14 @@ static int __qed_spq_block(struct qed_hwfn *p_hwfn,
 			return 0;
 		}
 
-		if (sleep_between_iter)
-			msleep(SPQ_BLOCK_SLEEP_MS);
-		else
-			udelay(SPQ_BLOCK_DELAY_US);
+		if (p_hwfn->pf_params.eth_pf_params.is_ens_mode_enabled) {
+			udelay(SPQ_BLOCK_SLEEP_MS * 1000);
+		} else {
+			if (sleep_between_iter)
+				msleep(SPQ_BLOCK_SLEEP_MS);
+			else
+				udelay(SPQ_BLOCK_DELAY_US);
+		}
 	}
 
 	return -EBUSY;
@@ -178,21 +192,23 @@ static int qed_spq_block(struct qed_hwfn *p_hwfn,
 
 	rc = -EBUSY;
 err:
-	snprintf((char *)str,
-		 QED_HW_ERR_MAX_STR_SIZE,
-		 "Ramrod is stuck [CID %08x cmd %02x protocol %02x echo %04x]\n",
-		 le32_to_cpu(p_ent->elem.hdr.cid),
-		 p_ent->elem.hdr.cmd_id,
-		 p_ent->elem.hdr.protocol_id,
-		 le16_to_cpu(p_ent->elem.hdr.echo));
+	scnprintf((char *)str, QED_HW_ERR_MAX_STR_SIZE,
+		  "Ramrod is stuck [CID %08x %s:0x%02x %s:0x%02x echo %04x]\n",
+		  le32_to_cpu(p_ent->elem.hdr.cid),
+		  qed_get_ramrod_cmd_id_str(p_ent->elem.hdr.protocol_id,
+					    p_ent->elem.hdr.cmd_id.raw),
+		  p_ent->elem.hdr.cmd_id.raw,
+		  qed_get_protocol_type_str(p_ent->elem.hdr.protocol_id),
+		  p_ent->elem.hdr.protocol_id,
+		  le16_to_cpu(p_ent->elem.hdr.echo));
 	DP_NOTICE(p_hwfn, "%s", str);
 
 	p_ptt = qed_ptt_acquire(p_hwfn);
 	if (!p_ptt)
 		return rc;
 
-	qed_hw_err_notify(p_hwfn, p_ptt, QED_HW_ERR_RAMROD_FAIL, str,
-			  strlen((char *)str) + 1);
+	qed_hw_err_notify(p_hwfn, p_ptt, QED_HW_ERR_RAMROD_FAIL, (char *)str);
+
 	qed_ptt_release(p_hwfn, p_ptt);
 
 	return rc;
@@ -227,9 +243,12 @@ static int qed_spq_fill_entry(struct qed_hwfn *p_hwfn,
 
 	DP_VERBOSE(p_hwfn,
 		   QED_MSG_SPQ,
-		   "Ramrod header: [CID 0x%08x CMD 0x%02x protocol 0x%02x] Data pointer: [%08x:%08x] Completion Mode: %s\n",
+		   "Ramrod header: [CID 0x%08x %s:0x%02x %s:0x%02x] Data pointer: [%08x:%08x] Completion Mode: %s\n",
 		   p_ent->elem.hdr.cid,
-		   p_ent->elem.hdr.cmd_id,
+		   qed_get_ramrod_cmd_id_str(p_ent->elem.hdr.protocol_id,
+					     p_ent->elem.hdr.cmd_id.raw),
+		   p_ent->elem.hdr.cmd_id.raw,
+		   qed_get_protocol_type_str(p_ent->elem.hdr.protocol_id),
 		   p_ent->elem.hdr.protocol_id,
 		   p_ent->elem.data_ptr.hi,
 		   p_ent->elem.data_ptr.lo,
@@ -257,7 +276,6 @@ static void qed_spq_hw_initialize(struct qed_hwfn *p_hwfn,
 				  struct qed_spq *p_spq)
 {
 	__le32 *p_spq_base_lo, *p_spq_base_hi;
-	struct regpair *p_consolid_base_addr;
 	u8 *p_flags1, *p_flags9, *p_flags10;
 	struct core_conn_context *p_cxt;
 	struct qed_cxt_info cxt_info;
@@ -281,9 +299,8 @@ static void qed_spq_hw_initialize(struct qed_hwfn *p_hwfn,
 	p_flags9 = &p_cxt->xstorm_ag_context.flags9;
 	p_flags10 = &p_cxt->xstorm_ag_context.flags10;
 	p_physical_q0 = &p_cxt->xstorm_ag_context.physical_q0;
-	p_spq_base_lo = &p_cxt->xstorm_st_context.spq_base_lo;
-	p_spq_base_hi = &p_cxt->xstorm_st_context.spq_base_hi;
-	p_consolid_base_addr = &p_cxt->xstorm_st_context.consolid_base_addr;
+	p_spq_base_lo = &p_cxt->xstorm_st_context.spq_base_addr.lo;
+	p_spq_base_hi = &p_cxt->xstorm_st_context.spq_base_addr.hi;
 
 	/* @@@TBD we zero the context until we have ilt_reset implemented. */
 	memset(p_cxt, 0, core_conn_context_size);
@@ -300,9 +317,6 @@ static void qed_spq_hw_initialize(struct qed_hwfn *p_hwfn,
 
 	*p_spq_base_lo = DMA_LO_LE(p_spq->chain.p_phys_addr);
 	*p_spq_base_hi = DMA_HI_LE(p_spq->chain.p_phys_addr);
-
-	DMA_REGPAIR_LE(*p_consolid_base_addr,
-		       p_hwfn->p_consq->chain.p_phys_addr);
 }
 
 static int qed_spq_hw_post(struct qed_hwfn *p_hwfn,
@@ -327,7 +341,9 @@ static int qed_spq_hw_post(struct qed_hwfn *p_hwfn,
 	/* Make sure the SPQE is updated before the doorbell */
 	wmb();
 
-	DOORBELL(p_hwfn, p_spq->db_addr_offset, *(u32 *) p_db_data);
+	writel((u32) * (u32 *) p_db_data,
+	       (void __iomem *)((u8 __iomem *) (p_hwfn->doorbells) +
+				(p_spq->db_addr_offset)));
 
 	/* Make sure doorbell was rung */
 	wmb();
@@ -354,7 +370,9 @@ qed_async_event_completion(struct qed_hwfn *p_hwfn,
 	int rc;
 
 	if (p_eqe->protocol_id >= MAX_PROTOCOL_TYPE) {
-		DP_ERR(p_hwfn, "Wrong protocol: %d\n", p_eqe->protocol_id);
+		DP_ERR(p_hwfn, "Wrong protocol: %s:%d\n",
+		       qed_get_protocol_type_str(p_eqe->protocol_id),
+		       p_eqe->protocol_id);
 		return -EINVAL;
 	}
 
@@ -365,24 +383,27 @@ qed_async_event_completion(struct qed_hwfn *p_hwfn,
 	if (IS_PF(p_hwfn->cdev) && p_eqe->vf_id != QED_CXT_PF_CID &&
 	    p_eqe->protocol_id != PROTOCOLID_COMMON &&
 	    !(p_eqe->protocol_id == PROTOCOLID_ROCE &&
-	      p_eqe->opcode == ROCE_ASYNC_EVENT_DESTROY_QP_DONE))
+	      p_eqe->opcode.roce == ROCE_ASYNC_EVENT_DESTROY_QP_DONE))
 		return qed_iov_async_event_completion(p_hwfn, p_eqe);
 
 	cb = p_hwfn->p_spq->async_comp_cb[p_eqe->protocol_id];
 	if (!cb) {
 		DP_NOTICE(p_hwfn,
-			  "Unknown Async completion for protocol: %d\n",
+			  "Unknown Async completion for %s:%d\n",
+			  qed_get_protocol_type_str(p_eqe->protocol_id),
 			  p_eqe->protocol_id);
 		return -EINVAL;
 	}
 
-	rc = cb(p_hwfn, p_eqe->opcode, p_eqe->echo,
-		&p_eqe->data, p_eqe->fw_return_code, p_eqe->vf_id);
+	rc = cb(p_hwfn, p_eqe->opcode.raw, p_eqe->echo,
+		&p_eqe->data, p_eqe->fw_return_code.raw, p_eqe->vf_id);
 	if (rc)
 		DP_NOTICE(p_hwfn,
-			  "Async completion callback failed, rc = %d [opcode %x, echo %x, fw_return_code %x]",
+			  "Async completion callback failed, rc = %d [%s:0x%02x, echo %x, fw_return_code %x]",
 			  rc,
-			  p_eqe->opcode, p_eqe->echo, p_eqe->fw_return_code);
+			  qed_get_event_ring_entry_opcode_str(p_eqe),
+			  p_eqe->opcode.raw,
+			  p_eqe->echo, p_eqe->fw_return_code.raw);
 
 	return rc;
 }
@@ -392,7 +413,8 @@ qed_spq_register_async_cb(struct qed_hwfn *p_hwfn,
 			  enum protocol_type protocol_id,
 			  qed_spq_async_comp_cb cb)
 {
-	if (!p_hwfn->p_spq || (protocol_id >= MAX_PROTOCOL_TYPE))
+	if (!p_hwfn->p_spq || (protocol_id >= MAX_PROTOCOL_TYPE) ||
+	    (protocol_id < 0))
 		return -EINVAL;
 
 	p_hwfn->p_spq->async_comp_cb[protocol_id] = cb;
@@ -403,7 +425,8 @@ void
 qed_spq_unregister_async_cb(struct qed_hwfn *p_hwfn,
 			    enum protocol_type protocol_id)
 {
-	if (!p_hwfn->p_spq || (protocol_id >= MAX_PROTOCOL_TYPE))
+	if (!p_hwfn->p_spq || (protocol_id >= MAX_PROTOCOL_TYPE) ||
+	    (protocol_id < 0))
 		return;
 
 	p_hwfn->p_spq->async_comp_cb[protocol_id] = NULL;
@@ -414,23 +437,26 @@ qed_spq_unregister_async_cb(struct qed_hwfn *p_hwfn,
 ***************************************************************************/
 void qed_eq_prod_update(struct qed_hwfn *p_hwfn, u16 prod)
 {
-	u32 addr = GTT_BAR0_MAP_REG_USDM_RAM +
-	    USTORM_EQE_CONS_OFFSET(p_hwfn->rel_pf_id);
+	u32 addr = GET_GTT_REG_ADDR(GTT_BAR0_MAP_REG_USDM_RAM,
+				    USTORM_EQE_CONS, p_hwfn->rel_pf_id);
 
 	REG_WR16(p_hwfn, addr, prod);
-
-	/* keep prod updates ordered */
-	mmiowb();
 }
 
 void qed_event_ring_entry_dp(struct qed_hwfn *p_hwfn,
 			     struct event_ring_entry *p_eqe)
 {
-	DP_VERBOSE(p_hwfn, QED_MSG_SPQ,
-		   "op %x prot %x res0 %x vfid %x echo %x fwret %x flags %x\n",
-		   p_eqe->opcode, p_eqe->protocol_id, p_eqe->reserved0,
-		   p_eqe->vf_id, le16_to_cpu(p_eqe->echo),
-		   p_eqe->fw_return_code, p_eqe->flags);
+	DP_VERBOSE(p_hwfn,
+		   QED_MSG_SPQ,
+		   "%s:0x%02x %s:0x%02x res0 %x vfid %x echo %x fwret %x flags %x\n",
+		   qed_get_event_ring_entry_opcode_str(p_eqe),
+		   p_eqe->opcode.raw,
+		   qed_get_protocol_type_str(p_eqe->protocol_id),
+		   p_eqe->protocol_id,
+		   p_eqe->reserved0,
+		   p_eqe->vf_id,
+		   le16_to_cpu(p_eqe->echo),
+		   p_eqe->fw_return_code.raw, p_eqe->flags);
 }
 
 int qed_eq_completion(struct qed_hwfn *p_hwfn, void *cookie)
@@ -473,7 +499,8 @@ int qed_eq_completion(struct qed_hwfn *p_hwfn, void *cookie)
 		} else {
 			qed_spq_completion(p_hwfn,
 					   p_eqe->echo,
-					   p_eqe->fw_return_code, &p_eqe->data);
+					   p_eqe->fw_return_code.raw,
+					   &p_eqe->data);
 		}
 
 		qed_chain_recycle_consumed(p_chain);
@@ -934,6 +961,7 @@ int qed_spq_post(struct qed_hwfn *p_hwfn,
 {
 	int rc = 0;
 	struct qed_spq *p_spq = p_hwfn ? p_hwfn->p_spq : NULL;
+	struct ramrod_header hdr;
 	bool b_ret_ent = true;
 	bool eblock;
 
@@ -945,11 +973,17 @@ int qed_spq_post(struct qed_hwfn *p_hwfn,
 		return -EINVAL;
 	}
 
-	if (p_hwfn->cdev->recov_in_prog) {
+	hdr = p_ent->elem.hdr;
+
+	if (QED_RECOV_IN_PROG(p_hwfn->cdev)) {
 		DP_VERBOSE(p_hwfn,
 			   QED_MSG_SPQ,
-			   "Recovery is in progress. Skip spq post [cmd %02x protocol %02x]\n",
-			   p_ent->elem.hdr.cmd_id, p_ent->elem.hdr.protocol_id);
+			   "Recovery is in progress. Skip spq post [%s:0x%02x %s:0x%02x]\n",
+			   qed_get_ramrod_cmd_id_str(hdr.protocol_id,
+						     hdr.cmd_id.raw),
+			   hdr.cmd_id.raw,
+			   qed_get_protocol_type_str(hdr.protocol_id),
+			   hdr.protocol_id);
 
 		/* Let the flow complete w/o any error handling */
 		qed_spq_recov_set_ret_code(p_ent, fw_return_code);
@@ -1015,6 +1049,9 @@ int qed_spq_post(struct qed_hwfn *p_hwfn,
 spq_post_fail2:
 	spin_lock_bh(&p_spq->lock);
 	list_del(&p_ent->list);
+	if (SPQ_COMP_BMAP_TEST_BIT(p_spq, p_ent->elem.hdr.echo))
+		DP_ERR(p_hwfn, "SPQ Entry %p (echo %x) already completed\n",
+		       p_ent, p_ent->elem.hdr.echo);
 	qed_spq_comp_bmap_update(p_hwfn, p_ent->elem.hdr.echo);
 
 spq_post_fail:
