@@ -91,6 +91,7 @@ enum qed_iov_pf_to_vf_status {
 	PFVF_STATUS_FORCED,
 	PFVF_STATUS_MALICIOUS,
 	PFVF_STATUS_ACQUIRED,
+	PFVF_STATUS_MAX,
 };
 
 struct qed_mcp_link_params;
@@ -123,16 +124,16 @@ struct qed_public_vf_info {
 	 */
 	bool is_trusted_configured;
 	bool is_trusted_request;
+	u8 rx_accept_mode;
+	u8 tx_accept_mode;
 	u8 mac[ETH_ALEN];
+	bool accept_any_vlan;
 
 	/* IFLA_VF_LINK_STATE_<X> */
 	int link_state;
 
 	/* Currently configured Tx rate in MB/sec. 0 if unconfigured */
 	int tx_rate;
-
-	u8 rx_accept_mode;
-	u8 tx_accept_mode;
 };
 
 struct qed_iov_vf_init_params {
@@ -446,6 +447,16 @@ struct qed_vf_info {
 
 	u8 vport_id;
 	u8 rss_eng_id;
+	bool rss_enabled;
+	bool rss_ind_tbl_valid;
+
+	/* indirection table size when the table was updated */
+	u16 rss_ind_tbl_size;
+
+	/* current configured rss table size which can be samller than
+	 * rss_ind_tbl_size but not larger.
+	 */
+	u16 rss_tbl_size;
 	u8 relative_vf_id;
 	u8 abs_vf_id;
 #define QED_VF_ABS_ID(p_hwfn, p_vf)     (QED_PATH_ID(p_hwfn) ?		      \
@@ -585,6 +596,33 @@ void *qed_add_tlv(u8 ** offset, u16 type, u16 length);
  */
 void qed_dp_tlv_list(struct qed_hwfn *p_hwfn, void *tlvs_list);
 
+/**
+ * @brief qed_sriov_vfpf_malicious - handle malicious VF/PF
+ *
+ * @param p_fwfn
+ * @param p_data
+ */
+void qed_sriov_vfpf_malicious(struct qed_hwfn *p_hwfn,
+			      struct fw_err_data *p_data);
+
+/**
+ * @brief qed_sriov_eqe_event - callback for SRIOV events
+ *
+ * @param p_hwfn
+ * @param opcode
+ * @param echo
+ * @param data
+ * @param fw_return_code
+ * @param vf_id
+ *
+ * @return int
+ */
+int qed_sriov_eqe_event(struct qed_hwfn *p_hwfn,
+			u8 opcode,
+			u16 __maybe_unused echo,
+			union event_ring_data *data,
+			u8 __maybe_unused fw_return_code, u8 vf_id);
+
 /**
  * @brief qed_iov_alloc - allocate sriov related resources
  *
@@ -652,14 +690,6 @@ struct qed_vf_info *qed_iov_get_vf_info(struct qed_hwfn *p_hwfn,
 					u16 relative_vf_id,
 					bool b_enabled_only);
 
-/**
- * @brief init RDMA for VF
- *
- * @param cdev
- * @param p_ptt
- */
-void qed_iov_init_vf_rdma(struct qed_hwfn *p_hwfn);
-
 /**
  * @brief get the VF doorbell bar size.
  *
@@ -695,6 +725,37 @@ qed_iov_async_event_completion(struct qed_hwfn *p_hwfn,
 int
 qed_iov_init_vf_doorbell_bar(struct qed_hwfn *p_hwfn, struct qed_ptt *p_ptt);
 
+enum qed_iov_validate_q_mode {
+	QED_IOV_VALIDATE_Q_NA,
+	QED_IOV_VALIDATE_Q_ENABLE,
+	QED_IOV_VALIDATE_Q_DISABLE,
+};
+
+bool qed_iov_validate_queue_mode(struct qed_vf_info *p_vf,
+				 u16 qid,
+				 enum qed_iov_validate_q_mode mode,
+				 bool b_is_tx);
+
+bool qed_iov_validate_rxq(struct qed_hwfn *p_hwfn,
+			  struct qed_vf_info *p_vf,
+			  u16 rx_qid, enum qed_iov_validate_q_mode mode);
+/**
+ * @brief qed_iov_handle_trust_change(struct qed_hwfn *hwfn) - handles trust mode
+ *
+ * @param p_hwfn
+ */
+void qed_iov_handle_trust_change(struct qed_hwfn *hwfn);
+
+/**
+ * @brief qed_disable_channel_for_all_vfs(struct qed_hwfn *p_hwfn,
+ *                                          qed_hwfn *hwfn) - mark channel indication
+ *                                          for all vf as disable
+ *
+ * @param p_hwfn
+ */
+void qed_disable_channel_for_all_vfs(struct qed_hwfn *p_hwfn,
+				     struct qed_ptt *p_ptt);
+
 #else
 static inline int qed_iov_hw_info(struct qed_hwfn __maybe_unused * p_hwfn)
 {
@@ -713,6 +774,26 @@ static inline void qed_dp_tlv_list(struct qed_hwfn __maybe_unused * p_hwfn,
 {
 }
 
+static inline void qed_sriov_vfpf_malicious(struct qed_hwfn __maybe_unused *
+					    p_hwfn,
+					    struct fw_err_data __maybe_unused *
+					    p_data)
+{
+}
+
+static inline int qed_sriov_eqe_event(struct qed_hwfn __maybe_unused * p_hwfn,
+				      u8 __maybe_unused
+				      opcode,
+				      u16 __maybe_unused
+				      echo,
+				      union event_ring_data __maybe_unused
+				      * data,
+				      u8 __maybe_unused
+				      fw_return_code, u8 __maybe_unused vf_id)
+{
+	return 0;
+}
+
 static inline int qed_iov_alloc(struct qed_hwfn __maybe_unused * p_hwfn)
 {
 	return 0;
@@ -760,10 +841,6 @@ static inline struct qed_vf_info *qed_iov_get_vf_info(struct qed_hwfn
 	return NULL;
 }
 
-static inline void qed_iov_init_vf_rdma(struct qed_hwfn __maybe_unused * p_hwfn)
-{
-}
-
 static inline u32 qed_iov_vf_db_bar_size(struct qed_hwfn __maybe_unused *
 					 p_hwfn,
 					 struct qed_ptt __maybe_unused * p_ptt)
@@ -792,6 +869,18 @@ static inline int qed_iov_init_vf_doorbell_bar(struct qed_hwfn __maybe_unused *
 {
 	return 0;
 }
+
+static inline void qed_iov_handle_trust_change(struct qed_hwfn
+					       __maybe_unused * hwfn)
+{
+}
+
+static inline void qed_disable_channel_for_all_vfs(struct qed_hwfn
+						   __maybe_unused * hwfn,
+						   struct qed_ptt __maybe_unused
+						   * p_ptt)
+{
+}
 #endif
 
 enum qed_iov_wq_flag {
@@ -828,6 +917,7 @@ int qed_pf_validate_modify_tunn_config(struct qed_hwfn *p_hwfn,
 				       u16 * feature_mask,
 				       bool * update,
 				       struct qed_tunnel_info *info);
+
 #ifdef CONFIG_QED_SRIOV
 void qed_iov_wq_stop(struct qed_dev *cdev, bool schedule_first);
 int qed_iov_wq_start(struct qed_dev *cdev);
