@@ -47,10 +47,25 @@
 #include "qed_selftest.h"
 #include "qed_rdma.h"
 #include "qed_dcbx.h"
+#include "qed_hsi.h"
 
 extern int qed_qm_reconf(struct qed_hwfn *p_hwfn, struct qed_ptt *p_ptt);
 extern void qed_dp_init_qm_params(struct qed_hwfn *p_hwfn);
- 
+
+int qed_dmae_err_test(struct qed_hwfn *p_hwfn, struct qed_ptt *p_ptt)
+{
+	qed_dmae_err(p_hwfn, p_ptt);
+
+	return 0;
+}
+
+int qed_fw_assert_test(struct qed_hwfn *p_hwfn, struct qed_ptt *p_ptt)
+{
+	qed_sp_fw_assert(p_hwfn);
+
+	return 0;
+}
+
 int qed_qm_reconf_test(struct qed_hwfn *p_hwfn, struct qed_ptt *p_ptt)
 {
 
@@ -61,7 +76,7 @@ int qed_qm_reconf_test(struct qed_hwfn *p_hwfn, struct qed_ptt *p_ptt)
 
 int qed_ets_test(struct qed_hwfn *p_hwfn, struct qed_ptt *p_ptt)
 {
-	struct init_ets_req ets;
+	struct init_ets_req ets = {0};
 	int i;
 
 	for (i = 0; i < NUM_OF_TCS; i++){
@@ -130,25 +145,14 @@ int qed_coal_vf_test(struct qed_hwfn *p_hwfn, struct qed_ptt *p_ptt,
 int qed_gen_process_kill_test(struct qed_hwfn *p_hwfn, struct qed_ptt *p_ptt,
 			      u8 is_common_block)
 {
-	if (is_common_block) {
-		/* The DCBX nvm option should be set to CEE (2) to enable LLDP traffic */
-		qed_wr(p_hwfn, p_ptt,
-		       0x540420 /* BMB_REG_MEM001_RF_ECC_ERROR_CONNECT */,
-		       0x20300);
-	} else {
-		/* Traffic should be run after this register write */
-		qed_wr(p_hwfn, p_ptt,
-		       0x340420 /* BRB_REG_MEM001_RF_ECC_ERROR_CONNECT */,
-		       0x20300);
-	}
+	qed_gen_process_kill(p_hwfn, p_ptt, is_common_block);
 
 	return 0;
 }
 
 int qed_gen_system_kill_test(struct qed_hwfn *p_hwfn, struct qed_ptt *p_ptt)
 {
-	qed_wr(p_hwfn, p_ptt, 0x2b0204 /* ATC_REG_PRTY_MASK_H_0 */, 0x0);
-	qed_rd(p_hwfn, p_ptt, 0x2b4800 /* first DWORD of ATC_REG_ATC_GPA_ARRAY_ACCESS_STATE */);
+	qed_gen_system_kill(p_hwfn, p_ptt);
 
 	return 0;
 }
@@ -174,7 +178,12 @@ int qed_msix_vector_mask_test(struct qed_hwfn *p_hwfn, struct qed_ptt *p_ptt, u8
 	if (!entry)
 		return -EINVAL;
 
+#ifdef _PCI_MSI_DESC_HAS_MASK_BASE /* QED_UPSTREAM */
+	ptr = entry->pci.mask_base + vector * 4 * sizeof(u32);
+#else
 	ptr = entry->mask_base + vector * 4 * sizeof(u32);
+#endif
+
 	val = ptr[3];
 	new_val = ptr[3] & 0xfffffffe;
 
@@ -184,7 +193,7 @@ int qed_msix_vector_mask_test(struct qed_hwfn *p_hwfn, struct qed_ptt *p_ptt, u8
 	printk(KERN_ERR "MSI-X[%d]: %08x:%08x:%08x:%08x -> %08x:%08x:%08x:%08x\n",
 	       vector, ptr[0], ptr[1], ptr[2], val, ptr[0], ptr[1], ptr[2], new_val);
 
-	ptr[3] = new_val;		
+	ptr[3] = new_val;
 #endif
 	return 0;
 }
@@ -237,13 +246,13 @@ int qed_config_obff_fsm_test(struct qed_hwfn *p_hwfn, struct qed_ptt *p_ptt)
 	u32 val;
 
 	/* Read register CPMU_REG_OBFF_MODE_CONTROL, modify bit 1 (OBFF_ENGINE_IDLE_EN) to 0 */
-	val = qed_rd(p_hwfn, p_ptt, CPMU_REG_OBFF_MODE_CONTROL); 
+	val = qed_rd(p_hwfn, p_ptt, CPMU_REG_OBFF_MODE_CONTROL);
 	val &= ~(0x1 << 1);
 
 	/* When the OBFF state changes to OBFF/IDLE, are there already VQs which are not empty?
 	 * If yes, these VQs prevent the FSM from getting into the STALL state.
 	 * Write value of 0 to bits 8 and 9 of the CPMU_REG_OBFF_MODE_CONTROL register to not to
-	 * condition the FSM transfer to stall state with VOs not empty. 
+	 * condition the FSM transfer to stall state with VOs not empty.
 	 */
 	val &= ~(0x11 << 9);
 
@@ -282,8 +291,8 @@ int qed_config_obff_fsm_test(struct qed_hwfn *p_hwfn, struct qed_ptt *p_ptt)
 
 	/* Read register BRB_REG_OUT_IF_ENABLE (Offset: 0xF2C; Width: 32), modify bit 25
 	 *  (PM_OUT_IF_EN) to 0 [Disables BRB above threshold indication].
-	 */ 
-	val = qed_rd(p_hwfn, p_ptt, BRB_REG_OUT_IF_ENABLE); 
+	 */
+	val = qed_rd(p_hwfn, p_ptt, BRB_REG_OUT_IF_ENABLE);
 	val &= ~(0x1 << 25);
 	qed_wr(p_hwfn, p_ptt, BRB_REG_OUT_IF_ENABLE, val);
 
@@ -310,7 +319,7 @@ int qed_set_obff_state_test(struct qed_hwfn *p_hwfn, struct qed_ptt *p_ptt, u8 s
 	u32 val;
 
 	/* Read register CPMU_REG_OBFF_MODE_CONTROL */
-	val = qed_rd(p_hwfn, p_ptt, CPMU_REG_OBFF_MODE_CONTROL); 
+	val = qed_rd(p_hwfn, p_ptt, CPMU_REG_OBFF_MODE_CONTROL);
 
 	switch (state) {
 	case 0:
@@ -482,8 +491,8 @@ int qed_mcp_resc_lock_test(struct qed_hwfn *p_hwfn, struct qed_ptt *p_ptt,
 	struct qed_resc_lock_params resc_lock_params;
 	int rc;
 
-	qed_mcp_resc_lock_default_init(&resc_lock_params, NULL, resource,
-				       false);
+	qed_mcp_resc_lock_default_init(p_hwfn, &resc_lock_params, NULL,
+				       resource, false);
 	resc_lock_params.timeout = timeout;
 	rc = qed_mcp_resc_lock(p_hwfn, p_ptt, &resc_lock_params);
 	if (rc)
@@ -503,8 +512,8 @@ int qed_mcp_resc_unlock_test(struct qed_hwfn *p_hwfn, struct qed_ptt *p_ptt,
 	struct qed_resc_unlock_params resc_unlock_params;
 	int rc;
 
-	qed_mcp_resc_lock_default_init(NULL, &resc_unlock_params, resource,
-				       false);
+	qed_mcp_resc_lock_default_init(p_hwfn, NULL, &resc_unlock_params,
+				       resource, false);
 	resc_unlock_params.b_force = !!force;
 	rc = qed_mcp_resc_unlock(p_hwfn, p_ptt, &resc_unlock_params);
 	if (rc)
@@ -1123,7 +1132,7 @@ int qed_dcbx_app_tlv_set_app_test(struct qed_hwfn *p_hwfn,
 		}
 	}
 
-	if (i == QED_DCBX_MAX_APP_PROTOCOL) {
+	if (i >= QED_DCBX_MAX_APP_PROTOCOL) {
 		DP_ERR(p_hwfn, "App table is full\n");
 		return -EBUSY;
 	}
@@ -1710,6 +1719,8 @@ int qed_monitored_hw_addr_test(struct qed_hwfn *p_hwfn, struct qed_ptt *p_ptt, u
 
 int qed_get_phys_port_test(struct qed_hwfn *p_hwfn, struct qed_ptt *p_ptt)
 {
+	if (IS_VF(p_hwfn->cdev))
+		return 1;
 	return MFW_PORT(p_hwfn);
 }
 
@@ -1927,3 +1938,14 @@ int qed_esl_active_test(struct qed_hwfn *p_hwfn, struct qed_ptt *p_ptt)
 
 	return esl_active;
 }
+
+int qed_gen_mdump_idlechk_test(struct qed_hwfn *p_hwfn, struct qed_ptt *p_ptt)
+{
+	return qed_mcp_gen_mdump_idlechk(p_hwfn, p_ptt);
+}
+
+int qed_set_vf_stats_bin_id_test(struct qed_hwfn *p_hwfn, struct qed_ptt *p_ptt,
+				 u16 vf_id)
+{
+	return qed_set_vf_stats_bin_id(p_hwfn->cdev, vf_id);
+}
